#<profile4xi profile = "12" />
# =================================================================================================
#<program4xi program = "dragon2image4xi.py">
#<abstract4xi abstract = "DRAGON 2D image management"/>
#<refer4xi reference = "image2d4xi.py"/>
#<verify4xi verification = "10" target = "dragon2image4xi5test.py">
#<state4xi status = "ACK4review; ACK4tagging; ACK4moduletesting; ACK4systemtesting; [Rel 7.21]"/>
# ================================================================================================= 

# -------------------------------------------------------------------------------------------------
# Public API
# -------------------------------------------------------------------------------------------------
#<interface4xi interface = " 
__all__ = [ 'DRAGON2Image4Xi',                        # ACK 5.31  p76 
            
            'copy2image4dragon',                      # ACK 5.3.3 p88
            'copy2imagetobuffer4dragon',              # ACK 5.3.4 p90
            'copy2buffertoimage4dragon',              # ACK 5.3.4 p92
            'create2image4dragon',                    # ACK 5.31  p76            

            'get2imageinfo4dragon',                   # ACK 5.3.6 p97 nb: modernized so that you can also access only selected elements, if you want           
            'get2supportedimageformats4dragon',       # ACK 5.3.2 p82 nb: a separate function to get format whereas using get2imageinfo4dragon you can access any element by key such as 'FORMAT' 
                              
            'manoeuvre2copyimage4dragon',             # ACK modernized 
            'manoeuvre2copyimagetobuffer4dragon',     # ACK modernized        
            'manoeuvre2copybuffertoimage4dragon',     # ACK modernized
            'manoeuvre2mapimage4dragon',              # ACK modernized                                           
            'manoeuvre2readimage4dragon',             # ACK modernized
            'manoeuvre2writeimage4dragon',            # ACK modernized            
            'map2image4dragon',                       # ACK 5.3.5 p94 
            
            'read2image4dragon',                      # ACK 5.3.3 p84  
            'retain2image4dragon',                    # ACK 5.4.1 p99
            'release2image4dragon',                   # ACK 5.4.1 p99

            'set2imagecallback4dragon',               # ACK modernized
            'set2image4dragon',                       # ACK modernized nb: with memorymap features you can process 2D images as files!                    

            'trace2image4dragon',                     # ACK modernized 
                                    
            'wait2image4dragon',                      # ACK modernized nb: checks Queue object and thereby causes delay doing something useful
            'write2image4dragon'                      # ACK 5.3.3 p84                 
          ] 
#"
#/>


# -------------------------------------------------------------------------------------------------
# Standard Imports                          
# -------------------------------------------------------------------------------------------------
#<standardize4xi standard = "
import os
from platform import processor
import sys
import threading                
import _thread   
from decimal import *
#"
#/>


# -------------------------------------------------------------------------------------------------
# Scope4Xi imports                               
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
import version4xi

# -------------------------------------------------------------------------------------------------
# Buffer module                           
# -------------------------------------------------------------------------------------------------
from dragon2buffer4xi import DRAGON2Buffer4Xi
from dragon2buffer4xi import create2buffer4dragon
from dragon2buffer4xi import create2subbuffer4dragon
from dragon2buffer4xi import get2bufferinfo4dragon
from dragon2buffer4xi import release2buffer4dragon
from dragon2buffer4xi import retain2buffer4dragon
from dragon2buffer4xi import set2buffer4dragon
from dragon2buffer4xi import trace2buffer4dragon

# -------------------------------------------------------------------------------------------------
# Command queue module                           
# -------------------------------------------------------------------------------------------------
from dragon2commandqueue4xi import DRAGON2CommandQueue4Xi
from dragon2commandqueue4xi import create2commandqueue4dragon 
from dragon2commandqueue4xi import flush2commandqueue4dragon
from dragon2commandqueue4xi import finish2commandqueue4dragon
from dragon2commandqueue4xi import get2commandqueueinfo4dragon
from dragon2commandqueue4xi import release2commandqueue4dragon
from dragon2commandqueue4xi import retain2commandqueue4dragon
from dragon2commandqueue4xi import set2commandqueue4dragon
from dragon2commandqueue4xi import trace2commandqueue4dragon

# -------------------------------------------------------------------------------------------------
# Context module                           
# -------------------------------------------------------------------------------------------------
from dragon2context4xi import DRAGON2Context4Xi 
from dragon2context4xi import create2context4dragon
from dragon2context4xi import get2context4dragon
from dragon2context4xi import release2context4dragon
from dragon2context4xi import retain2context4dragon
from dragon2context4xi import set2context4dragon


# -------------------------------------------------------------------------------------------------
# Device module                        
# -------------------------------------------------------------------------------------------------
from dragon2device4xi import DRAGON4Xi
from dragon2device4xi import init2dragon4xi 
from dragon2device4xi import set2dragon4xi
from dragon2device4xi import get2dragon4xi

# -------------------------------------------------------------------------------------------------
# Event module                           
# -------------------------------------------------------------------------------------------------
from dragon2event4xi import DRAGON2Event4Xi
from dragon2event4xi import create2event4dragon
from dragon2event4xi import get2eventinfo4dragon
from dragon2event4xi import release2event4dragon
from dragon2event4xi import retain2event4dragon
from dragon2event4xi import set2eventcallback4dragon 
from dragon2event4xi import set2event4dragon
from dragon2event4xi import trace2event4dragon
from dragon2event4xi import wait2event4dragon


# -------------------------------------------------------------------------------------------------
# Platform module                           
# -------------------------------------------------------------------------------------------------
from platformids4xi  import get2platformids4xi
from platforminfo4xi import PlatformInfo4Xi                                                                        
from platforminfo4xi import init2platforminfo4xi
from platforminfo4xi import set2platforminfo4xi
from platforminfo4xi import get2platforminfo4xi
#"
#/>

# -------------------------------------------------------------------------------------------------
# Multiprocessing Imports                          
# -------------------------------------------------------------------------------------------------
#<process4xi process = "
import _multiprocessing
#"
#/>


# -------------------------------------------------------------------------------------------------
# DragonXi imports                              
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
from ace_object4xi import ace_object4xi
from arena4xi import Arena4Xi
from cpu_corecount4xi import cpu_corecount4xi
from dummyprocess4xi import DummyProcess4Xi
from print4xi import print4xi
from pipe4xi import Pipe4Xi
from queue4xi import Queue4Xi  
from time4xi import time4xi
from reason_object4xi import reason_object4xi
from sense_object4xi import sense_object4xi


#"
#/>

# -------------------------------------------------------------------------------------------------
# Global constants                            
# ------------------------------------------------------------------------------------------------
#<globalize4xi globalization = "
global dragon2image4xi                                                     # 2D-IMAGES FOR DRAGON

global dragon2trace4xi                                                     # TRACE

global dragon2buffer4xi                                                    # BUFFER FOR DRAGON

global control2readimage4dragon                                            # COMMAND AND CONTROL TO READ  2D IMAGES WITH DRAGON  
global state2read4dragon                                                   # STATE FOR COMMAND AND CONTROL
global max2readstate4dragon                                                # MAX FOR STATE
global ack2readinfo4dragon                                                 # ACK FOR IMAGE INFO
global message2readget4dragon                                              # QUEUE GET
global message2readput4dragon                                              # QUEUE PUT
global message2readreceive4dragon                                          # PIPE RECEIVE
global message2readsend4dragon                                             # PIPE SEND

global control2writeimage4dragon                                           # COMMAND AND CONTROL TO WRITE 2D IMAGES WITH DRAGON 
global state2write4dragon                                                  # STATE FOR COMMAND AND CONTROL
global max2writestate4dragon                                               # MAX FOR STATE
global ack2writeinfo4dragon                                                # ACK FOR IMAGE INFO
global message2writeget4dragon                                             # QUEUE GET
global message2writeput4dragon                                             # QUEUE PUT
global message2writereceive4dragon                                         # PIPE RECEIVE
global message2writesend4dragon                                            # PIPE SEND

global control2copyimage4dragon                                            # COMMAND AND CONTROL TO COPY 2D IMAGES WITH DRAGON 
global state2copyimage4dragon                                              # STATE FOR COMMAND AND CONTROL
global max2copyimagestate4dragon                                           # MAX FOR STATE
global ack2copyimageinfo4dragon                                            # ACK FOR IMAGE INFO
global message2copyimageget4dragon                                         # QUEUE GET
global message2copyimageput4dragon                                         # QUEUE PUT
global message2copyimagereceive4dragon                                     # PIPE RECEIVE
global message2copyimagesend4dragon                                        # PIPE SEND

global control2copyimagetobuffer4dragon                                    # COMMAND AND CONTROL TO COPY 2D IMAGES TO BUFFER WITH DRAGON 
global state2copyimagetobuffer4dragon                                      # STATE FOR COMMAND AND CONTROL
global max2copyimagetobufferstate4dragon                                   # MAX FOR STATE
global ack2copyimagetobufferinfo4dragon                                    # ACK FOR IMAGE INFO
global message2copyimagetobufferget4dragon                                 # QUEUE GET
global message2copyimagetobufferput4dragon                                 # QUEUE PUT
global message2copyimagetobufferreceive4dragon                             # PIPE RECEIVE
global message2copyimagetobuffersend4dragon                                # PIPE SEND

global control2copybuffertoimage4dragon                                    # COMMAND AND CONTROL TO COPY BUFFER TO 2D IMAGE WITH DRAGON 
global state2copybuffertoimage4dragon                                      # STATE FOR COMMAND AND CONTROL
global max2copybuffertoimagestate4dragon                                   # MAX FOR STATE
global ack2copybuffertoimageinfo4dragon                                    # ACK FOR IMAGE INFO
global message2copybuffertoimageget4dragon                                 # QUEUE GET
global message2copybuffertoimageput4dragon                                 # QUEUE PUT
global message2copybuffertoimagereceive4dragon                             # PIPE RECEIVE
global message2copybuffertoimagesend4dragon                                # PIPE SEND                      

global control2mapimage4dragon                                            # COMMAND AND CONTROL TO READ  2D IMAGES WITH DRAGON  
global state2map4dragon                                                   # STATE FOR COMMAND AND CONTROL
global max2mapstate4dragon                                                # MAX FOR STATE
global ack2mapinfo4dragon                                                 # ACK FOR IMAGE INFO
global message2mapget4dragon                                              # QUEUE GET
global message2mapput4dragon                                              # QUEUE PUT
global message2mapreceive4dragon                                          # PIPE RECEIVE
global message2mapsend4dragon                                             # PIPE SEND
            
#"
#/>

# -------------------------------------------------------------------------------------------------
# Settings
# -------------------------------------------------------------------------------------------------
#<set4xi setting = "

#"
#"/>

# -------------------------------------------------------------------------------------------------
#<function4xi function = "trace2image4dragon" usage = "
#
#   print('=' *33)          
#   print('(Step 15A/25) OF DRAGON2IMAGE4XI5TEST - TRYING TO TRACE DRAGON: ', time4xi())      
#   print('=' *33)      
#   trace2dragon = None
#   try:
#       trace2dragon = trace2image4dragon() 
#       print('(Step 15B/25) OF DRAGON2IMAGE4XI5TEST - TRACE2DRAGON: ', trace2dragon)  
#       # {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4dragon-failed {}; time: 01:39:15 10/06/11 Pacific Daylight Time}', 'CLASS': 'trace2image4dragon', 'METHOD': 'trace2image4dragon'}
#   except:
#       print('(Step 15Z1/25) OF DRAGON2IMAGE4XI5TEST *** FAILED TO TRACE DRAGON: ', trace2dragon)       
#">

# -------------------------------------------------------------------------------------------------   
def trace2image4dragon(task4trace2dragon=None):  

    '''
    creates callback to dragon2image4xi
    '''
    
    global dragon2trace4xi
    
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33)         
    # -----------------------------------------------------------------------------------------          
    class2print = "trace2image4dragon" 
    method2print = "trace2image4dragon"

    message2print = "{(Step 1A/33) started" + "; trace2image4dragon-started: " + str(trace2image4dragon)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                              
    # <function trace2image4dragon at 0x0000000003AF0348> 
  
             
    message2print = "{(Step 1B/33) passed" + "; task4trace2dragon-to-trace2image4dragon: " + str(task4trace2dragon) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              # 
    # None 

    code2ack  = ' '
    code2nack = ' '    
    
    dragon2trace4xi = {}                                                                                               # INITIALIZES TO EMPTY DICTIONARY
      
    if task4trace2dragon is None:
        # -----------------------------------------------------------------------------------------
        # NO PARAMETERS GIVEN BY DRAGONIZED CALLBACK !
        # -----------------------------------------------------------------------------------------    
        task4trace2dragon = {}                                                                                            
            
        task4trace2dragon['CLASS']   = "trace2image4dragon"                                                               # DRAGONIZE ZPU TRACE!
        task4trace2dragon['METHOD']  = "trace2image4dragon" 
        task4trace2dragon['MESSAGE'] = "{(Step 2Z0/33) failed" + "; trace2image4dragon-failed " + str(dragon2trace4xi)  + "; time: " + time4xi() + "}"               
  
        
        message2print = "{(Step 2A/33) set" + "; task4trace2dragon-set-by-trace2image4dragon to: " + str(task4trace2dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4dragon-failed {}; time: 08:14:33 09/06/11 Pacific Daylight Time}',
        #  'CLASS': 'trace2image4dragon',
        #  'METHOD': 'trace2image4dragon'}
        #    ; time: 08:14:33 09/06/11 Pacific Daylight Time}
 
    
        code2ack = code2ack + '2A-'            


    # -----------------------------------------------------------------------------------------
    # PARAMETER GIVEN BY DRAGONIZED CALLBACK !
    # -----------------------------------------------------------------------------------------          
    message2print = "{(Step 3A/33) try" + "; task4trace2dragon-try-by-trace2image4dragon: " + str(task4trace2dragon)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)    
    #  { trace2image4dragon : trace2image4dragon ; (Step 3A/33) try; task4trace2dragon-try-by-trace2image4dragon: 
    # {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4dragon-failed {}; time: 08:14:33 09/06/11 Pacific Daylight Time}',
    #  'CLASS': 'trace2image4dragon',
    #  'METHOD': 'trace2image4dragon'}
    #    ; time: 08:14:33 09/06/11 Pacific Daylight Time}

    try:
        dragon2trace4xi ['CLASS']  = task4trace2dragon['CLASS']                                                      # DRAGONIZE DRAGON TRACE!
        dragon2trace4xi ['METHOD']  = task4trace2dragon['METHOD']
        dragon2trace4xi ['MESSAGE'] = task4trace2dragon['MESSAGE']
        
        message2print = "{(Step 3B/33) set" + "; dragon2trace4xi-set-by-trace2image4dragon to: " + str(dragon2trace4xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # { trace2image4dragon : trace2image4dragon ; (Step 3B/33) set; dragon2trace4xi-set-by-trace2image4dragon to: 
        #                     {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4dragon-failed {}; time: 23:41:37 09/06/11 Pacific Daylight Time}',
        #                      'CLASS': 'trace2image4dragon',
        #                      'METHOD': 'trace2image4dragon'}
        #                     ; time: 23:41:37 09/06/11 Pacific Daylight Time}

        code2ack = code2ack + '3B-'         
    except:
        message2print = "{(Step 3Z1/33) failed" + "; dragon-trace-failed-by-trace2image4dragon: " + str(task4trace2dragon) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                    
        code2nack = code2nack + '3Z1-' 
        
    try:
        # =================================
        # SIMULATE DRAGON TRACE
        # =================================
        class2print   = task4trace2dragon['CLASS']                                                                # DRAGONIZE DRAGON TRACE!
        method2print  = task4trace2dragon['METHOD']
        
        message2print = task4trace2dragon['MESSAGE']
        print4xi(class2print, method2print, message2print)                                                     # DRAGONIZED TRACE FOR OPENCL CALLBACK!
        # { trace2image4dragon : trace2image4dragon ; (Step 2Z0/33) failed; trace2image4dragon-failed {}; time: 23:41:37 09/06/11 Pacific Daylight Time}
        
        code2ack = code2ack + '3C-'         
    except:
        class2print = "trace2image4dragon"                                                                        # SET TRACE BACK TO THIS FUNCTION!
        method2print = "trace2image4dragon"   
        message2print = "{(Step 3Z2/33) failed" + "; simulation-of-dragon-trace-failed-by-trace2image4dragon: " + str(task4trace2dragon) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                    
        code2nack = code2nack + '3Z2-' 
        
    try:
        # =================================
        # NORMAL TRACE
        # =================================                     
        class2print = "trace2image4dragon"                                                                        # SET TRACE BACK TO THIS FUNCTION!
        method2print = "trace2image4dragon"                
        message2print = "{(Step 3D/33) performed" + "; dragon-trace-performed-by-trace2image4dragon: " + str(task4trace2dragon) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # { trace2image4dragon : trace2image4dragon ; (Step 3D/33) performed; dragon-trace-performed-by-trace2image4dragon: 
        # {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4dragon-failed {}; time: 23:41:37 09/06/11 Pacific Daylight Time}', 
        # 'CLASS': 'trace2image4dragon', 'METHOD': 'trace2image4dragon'}; time: 23:41:37 09/06/11 Pacific Daylight Time}

        code2ack = code2ack + '3D-' 
    except:
        message2print = "{(Step 3Z3/33) failed" + "; dragon-trace-failed-by-trace2image4dragon: " + str(task4trace2dragon) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                    
        code2nack = code2nack + '3Z3-' 

    # -----------------------------------------------------------------------------------------
    # (Step 33/3)           
    # -----------------------------------------------------------------------------------------
    message2print = "{(Step 33/33) returning" + "; dragon2trace4xi-returning-by-trace2image4dragon: " + str(dragon2trace4xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4dragon-failed {}; time: 23:41:37 09/06/11 Pacific Daylight Time}', 
    #  'CLASS': 'trace2image4dragon', 
    #  'METHOD': 'trace2image4dragon'}; time: 23:41:37 09/06/11 Pacific Daylight Time}

    
    return dragon2trace4xi
            
#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "set2imagecallback4dragon" usage = "
#
#   print('(Step 14A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO SET 2D IMAGE CALLBACK OF DRAGON DEVICE ', time4xi())  
#   try:                               
#       response4setcallback2image = set2imagecallback4dragon()                                                                                                 
#       print('(Step 14B/700) OF DRAGON2IMAGE4XI5TEST - SET 2D IMAGE CALLBACK OF DRAGON DEVICE TO: ', response4setcallback2image)   
#       # <function trace2image4dragon at 0x0000000004E2CAC8>
#   except:
#       print('(Step 14Z1/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO SET 2D IMAGE CALLBACK OF DRAGON DEVICE ', time4xi())  
# 
#">

# ------------------------------------------------------------------------------------------------- 
 
def set2imagecallback4dragon(task4set2imagecallback=None):

    '''
    registers callback function to dragon2image4xi
    '''
    global dragon2image4xi
    
    class2print = "set2imagecallback4dragon" 
    method2print = "set2imagecallback4dragon"
    
    message2print = "{(Step 1A/70) started" + "; set2imagecallback4dragon-started: " + str(set2imagecallback4dragon)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # <function set2imagecallback4dragon at 0x000000000390B748>
    
    try:
    
        dragon2image4xi ['CALLBACK'] = trace2image4dragon
        
        message2print = "{(Step 1B/70) set callback2dragon" + "; imagecallback4dragon-set-by-set2imagecallback4dragon: " + str(dragon2image4xi ['CALLBACK']) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # <function trace2image4dragon at 0x0000000003A9E348> 
        
        message2print = "{(Step 1C/70) returning" + "; callback-of-dragon2image4xi-returning-by-set2imagecallback4dragon: " + str(dragon2image4xi ['CALLBACK']) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print) 
        # <function trace2image4dragon at 0x0000000003864A48> 
        
        return dragon2image4xi ['CALLBACK']      
        
    except:
        message2print = "{(Step 1Z1/70) failed" + "; callback2image-failed-by-set2imagecallback4dragon: " + str(callback2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         

        message2print = "{(Step 1Z2/70) returning" + "; trace2image4dragon-returning-by-set2imagecallback4dragon: " + str(trace2image4dragon) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)      
            
        return trace2image4dragon
            
#</function4xi> 




# -------------------------------------------------------------------------------------------------
#<function4xi function = "create2image4dragon" usage = "
#   #1     
#   print('=' *33)          
#   print('(Step 2A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO CREATE 2D IMAGE4DRAGON ', time4xi())      
#   print('=' *33)        
#   # 
#   try:
#       response4create2Dimage = create2image4dragon()                                                                 
#       print('(Step 2B/700) OF DRAGON2IMAGE4XI5TEST - CREATED 2D IMAGE4DRAGON ', time4xi())
#       keys2response = response4create2Dimage.keys()
#       for key2response in sorted(keys2response):
#           print(key2response, ' : ', response4create2Dimage[key2response])
#           # 
#   except:
#       print('(Step 2Z1/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO CREATE 2D IMAGE4DRAGON *** ', time4xi())
#   #2
#   print('(Step 123A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO CREATE 2D IMAGE for DRAGON ', time4xi()) 
#   # 
#   try:
#       xi2dragon = DRAGON2Image4Xi()                            
#       response4create2Dimage = xi2dragon.create2image()                                  
#       keys2response = response4create2Dimage.keys()
#       print('(Step 122B/700) OF DRAGON2IMAGE4XI5TEST - CREATED 2D IMAGE ', time4xi())  
#       for key2response in sorted(keys2response):
#           print(key2response, ' : ', response4create2Dimage[key2response])
#           # 
#   except:
#       print('(Step 123Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO CREATE 2D IMAGE for DRAGON ', time4xi())    
#">

# -------------------------------------------------------------------------------------------------   
def create2image4dragon(task4imagecreate2dragon=None):

    '''
    creates 2D image for DRAGON
    '''
    
    global dragon2image4xi                                                                                              
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "create2image4dragon" 
    method2print = "create2image4dragon"

    message2print = "{(Step 1A/70) started" + "; create2image4dragon-started: " + str(create2image4dragon)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # <function create2image4dragon at 0x00000000038D97C8> 
  
             
    message2print = "{(Step 1B/70) passed" + "; task4imagecreate2dragon-to-create2image4dragon: " + str(task4imagecreate2dragon) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None 

    code2ack  = ' '
    code2nack = ' '    
    
    dragon2image4xi = {}                                                                                               # INITIALIZES TO EMPTY DICTIONARY
               
    try:             
        if task4imagecreate2dragon is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER FOR CREATION - BUILD TASK TO DRAGON! 
            # -----------------------------------------------------------------------------------------    
            
            task4imagecreate2dragon = {}                                                                                                                                                                                       
            
            message2print = "{(Step 1C/70) initialized" + "; task4imagecreate2dragon-created-by-create2image4dragon to: " + str(task4imagecreate2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {}
            
            function2callback4dragon = trace2image4dragon                                                                 # CALLBACK FUNCTION FOR DRAGON
            
            message2print = "{(Step 1D/70) initialized" + "; function2callback4dragon-created-by-create2image4dragon to: " + str(function2callback4dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # <function trace2image4dragon at 0x00000000038D9348>
            
            code2ack  = code2ack + '1D-'
           
            args2callback = {}                                                                                      # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "create2image4dragon"                                                          # MODULE OF DRAGONIZED OPENCL MODULE
            args2callback['METHOD']  = "create2image4dragon"                                                          # FUNCTION OF DRAGONIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; create2image4dragon-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR DRAGONIZED OPENCL TO RETURN    

            message2print = "{(Step 1E/70) initialized" + "; args2callback-created-by-create2image4dragon to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {'MESSAGE': '{(Step 1Z1/33) failed; create2image4dragon-failed ; time: 00:54:15 09/06/11 Pacific Daylight Time}',
            #  'CLASS': 'create2image4dragon',
            #  'METHOD': 'create2image4dragon'}; time: 00:54:15 09/06/11 Pacific Daylight Time}

            code2ack  = code2ack + '1E-'
            
            # ---------------------------------------------------------------------------------------------
            # (1) CALLBACKS - EXTRA                      
            # ---------------------------------------------------------------------------------------------              
            task4imagecreate2dragon['DATA2CALLBACK'] = args2callback                                                   # USER DATA PASSED TO DRAGONIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagecreate2dragon['FUNCTION2CALLBACK'] = function2callback4dragon                                       # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            code2ack  = code2ack + '2C-'
            message2print = "{(Step 1F/70) initialized" + "; task4imagecreate2dragon-created-by-create2image4dragon to: " + str(task4imagecreate2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # {'FUNCTION2CALLBACK': <function trace2image4dragon at 0x00000000038D9348>,
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4dragon-failed ; time: 00:54:15 09/06/11 Pacific Daylight Time}',
            #  'CLASS': 'create2image4dragon',
            #  'METHOD': 'create2image4dragon'}}; time: 00:54:15 09/06/11 Pacific Daylight Time}
            
            # ---------------------------------------------------------------------------------------------
            # (2) PLATFORM - EXTRA                      
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-create2image4dragon to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # {}
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-create2image4dragon: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}; time: 00:54:15 09/06/11 Pacific Daylight Time}
                 
                code2ack  = code2ack + '2B-'
            except:
                message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-create2image4dragon: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '2Z-'
                 
            task4imagecreate2dragon['PLATFORM'] = platform2xi                                                          # PLATFORM OF SYSTEM 
              
            
            # ---------------------------------------------------------------------------------------------
            # (3) CONTEXT - REQUIRED                         
            # ---------------------------------------------------------------------------------------------    
            context2dragon = {}
            message2print = "{(Step 3A/70) trying" + "; context2dragon-trying-by-create2image4dragon: " + str(context2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2dragon = get2context4dragon()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 3B/70) got" + "; context2dragon-got-by-create2image4dragon: " + str(context2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'COUNT2REFERENCE': 0,
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4dragon-failed ; time: 00:54:15 09/06/11 Pacific Daylight Time}',
                #  'CLASS': 'create2context4dragon',
                #  'METHOD': 'create2context4dragon'},
                #  'FUNCTION2CALLBACK': <function trace2context4dragon at 0x00000000037D0148>,
                #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-',
                #  'INFO2PRIVATE': None,
                #  'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1},
                #  'NACK2RETURN': ' ',
                #  'SIZE2PRIVATE': 0, 'PROPERTIES': None}; time: 00:54:15 09/06/11 Pacific Daylight Time}

                
                code2ack  = code2ack + '3B-'         
            except:
                message2print = "{(Step 3Z/70) failed" + "; context2dragon-failed-by-create2image4dragon: " + str(context2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '3Z-'
                
            task4imagecreate2dragon ['CONTEXT'] = context2dragon           
            
            # ---------------------------------------------------------------------------------------------
            # (4) DEVICE - EXTRA                            
            # ---------------------------------------------------------------------------------------------    
            device2dragon = {}
            
            message2print = "{(Step 4A/70) trying" + "; device2dragon-trying-by-create2image4dragon: " + str(device2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2dragon = get2dragon4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 4B/70) got" + "; device2dragon-got-by-create2image4dragon: " + str(device2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'RESOLUTION2TIMER4PROFILING': 318,
                #  'ID2PLATFORM': '0000000001509568',
                #  'PROFILING2QUEUE': 'ACK',
                #  'EXECUTE2KERNELS4OPENCL': 'ACK',
                #  'NATIVE2WIDTHLONG4VECTOR': 2,
                #  'MAX2UNITS4COMPUTE': 4,
                #  'SIZE2MEMORY4GLOBAL': 3221225472,
                #  'NATIVE2WIDTHFLOAT4VECTOR': 4,
                #  'MAX2SIZE4WORKGROUP': 1024,
                #  'PREFERRED2WIDTHCHAR4VECTOR': 16,
                #  'SIZE2CACHE': 65536,
                #  'IEEE754-2008FUSED2MULTIPLY-ADD': 'NACK',
                #  'TYPE4CACHE': 'Read/Write',
                #  'PREFERRED2WIDTHINT4VECTOR': 4,
                #  'SIZE2PREFERREDWORKGROUPMULTIPLE4KERNEL': 1,
                #  'NATIVE2WIDTHINT4VECTOR': 4,
                #  'SIZE2BUFFER4CONSTANT': 65536,
                #  'EXECUTE2FUNCTION4NATIVE': 'ACK',
                #  'PREFERRED2WIDTHDOUBLE4VECTOR': 0,
                #  'MAX2NUMBER4CONSTANTARGS': 8,
                #  'VERSION2DRIVER': '2.0',
                #  'EXTENSIONS': 'cl_amd_fp64 cl_khr_global_int32_base_atomics cl_khr_global_int32_extended_atomics cl_khr_local_int32_base_atomics 
              
                #  'AVAILABLE2COMPILER': 'ACK',
                #  'MAX2DIMENSIONS4WORKITEMS': 3,
                #  'SUPPORT2CORRECTION4ERROR': 0,
                #  'BITS2ADDRESS': 64,
                #  'AVAILABLE': 'ACK',
                #  'SIZE2MEMORY4LOCAL': 32768,
                #  'VENDOR': 'AuthenticAMD',
                #  'SUPPORT2IMAGE': 'NACK',
                #  'MAX2SIZE4KERNELARGUMENT': 4096,
                #  'ACK2RETURN': ' 1C-1D-1E-1F-1D-2A-
                #  'OUT-OF-ORDER2QUEUE': 'NACK',
                #  'ALIGNMENT2ADDRESS4BASE': 1024,
                #  'PROFILE': 'FULL_PROFILE',
                #  'TYPE2MEMORY4LOCAL': 'Global',
                #  'PREFERRED2WIDTHFLOAT4VECTOR': 4,
                #  'PREFERRED2WIDTHLONG4VECTOR': 2,
                #  'MAX2FREQUENCY4CLOCK': 3210,
                #  'PREFERRED2WIDTHSHORT4VECTOR': 8,
                #  'ID': 4033,
                #  'MAX2WORKITEM2': 1024,
                #  'NATIVE2WIDTHDOUBLE4VECTOR': 0,
                #  'ROUND2ZERO': 'ACK',
                #  'NAME': 'AMD64 Family 16 Model 4 Stepping 2, AuthenticAMD',
                #  'MAX2WORKITEM1': 1024,
                #  'MAX2WORKITEM0': 1024,
                #  'SIZE2LINE4CACHE': 64,
                #  'MAX2ALLOCATION4MEMORY': 1073741824,
                #  'NATIVE2WIDTHCHAR4VECTOR': 16,
                #  'QUIET2NANS': 'ACK',
                #  'NATIVE2WIDTHSHORT4VECTOR': 8,
                #  'ROUND2NEAREST4EVEN': 'ACK',
                #  'VERSION': 'OpenCL 1.1 ATI-Stream-v2.3 (451)',
                #  'NACK2RETURN': ' ',
                #  'MEMORY2UNIFIED4HOSTANDDEVICE': 1,
                #  'ENDIANESS2DEVICE': 'Little',
                #  'MINIMUM2ALIGNMENT4DATATYPE': 128,
                #  'ROUND2+VE3INFINITY': 'ACK',
                #  'TYPE': 'CL_DEVICE_TYPE_DRAGON',
                #  'DENORMS2SINGLEPRECISION4FLOATINGPOINTCAPABILITY': 'ACK'}; time: 00:54:15 09/06/11 Pacific Daylight Time}

                
                code2ack  = code2ack + '4B-'  
            except:
                message2print = "{(Step 4Z/70) failed" + "; device2dragon-failed-by-create2image4dragon: " + str(device2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '4Z-'
                
            task4imagecreate2dragon ['DEVICE'] = device2dragon
            
                   
            # ---------------------------------------------------------------------------------------------
            # (5) ERROR CODE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            task4imagecreate2dragon ['CODE2ERROR'] = 0             
            code2ack  = code2ack + '5B-'              

            # ---------------------------------------------------------------------------------------------
            # (6) QUEUE - EXTRA                          
            # ---------------------------------------------------------------------------------------------         
            queue2dragon = None
            
            message2print = "{(Step 6A/70) trying" + "; queue2dragon-trying-by-create2image4dragon: " + str(queue2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2dragon = Queue4Xi()                                                                                  # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 6B/70) got" + "; queue2dragon-got-by-create2image4dragon: " + str(queue2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # <queue4xi.Queue4Xi object at 0x0000000003991B70> 
                
                code2ack  = code2ack + '6B-'  
            except:
                message2print = "{(Step 6Z/70) failed" + "; queue2dragon-failed-by-create2image4dragon: " + str(queue2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '6Z-'
                
            task4imagecreate2dragon ['QUEUE'] = queue2dragon

            
            # ---------------------------------------------------------------------------------------------
            # (7) PIPE - EXTRA                              
            # ---------------------------------------------------------------------------------------------         
            pipe2dragon = None
           
            message2print = "{(Step 7A/70) trying" + "; pipe2dragon-trying-by-create2image4dragon: " + str(pipe2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2dragon = Pipe4Xi()                                                                                    # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 7B/70) got" + "; pipe2dragon-got-by-create2image4dragon: " + str(pipe2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # (<read-write PipeConnection, handle 388>, <read-write PipeConnection, handle 392>)
                
                code2ack  = code2ack + '7B-'  
            except:
                message2print = "{(Step 7Z/70) failed" + "; pipe2dragon-failed-by-create2image4dragon: " + str(pipe2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '7Z-'
                
            task4imagecreate2dragon ['PIPE'] = pipe2dragon 
            
            # ---------------------------------------------------------------------------------------------
            # (8) COMMAND QUEUE INFO - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2dragon = {}
           
            message2print = "{(Step 8A/70) trying" + "; commandqueueinfo2dragon-trying-by-create2image4dragon: " + str(commandqueueinfo2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2dragon = get2commandqueueinfo4dragon()                                                    # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 8B/70) got" + "; commandqueueinfo2dragon-got-by-create2image4dragon: " + str(commandqueueinfo2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0,
                #  'COUNT2REFERENCE': 0,
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4dragon-failed ; time: 05:04:51 09/06/11 Pacific Daylight Time}',
                #  'CLASS': 'create2commandqueue4dragon',
                #  'METHOD': 'create2commandqueue4dragon'},
                #  'FUNCTION2CALLBACK': <function trace2commandqueue4dragon at 0x000000000372C848>,
                #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-8A-9B-10B-9B-10B-11B-12B-13B-14B-15B-16B-17B-18B-',
                #  'INFO2PRIVATE': None,
                #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003A7A6D8>,
                #  'NACK2RETURN': ' ',
                #  'CONTEXT': {'COUNT2REFERENCE': 0 ... 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4dragon-failed ; time: 05:04:51 09/06/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2context4dragon', 
                #  'METHOD': 'create2context4dragon'}, 
                #  'FUNCTION2CALLBACK': <function trace2context4dragon at 0x00000000036E0148>, 
                #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-', 
                #  'INFO2PRIVATE': None, 
                #  'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
                #  'NACK2RETURN': ' ', 
                #  'SIZE2PRIVATE': 0, 
                #  'PROPERTIES': None}, 
                #  'DEVICE': {'RESOLUTION2TIMER4PROFILING': 318, ...
                
                code2ack  = code2ack + '8B-'  
            except:
                message2print = "{(Step 8Z/70) failed" + "; commandqueueinfo2dragon-failed-by-create2image4dragon: " + str(commandqueueinfo2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '8Z-'
        
            task4imagecreate2dragon ['COMMANDQUEUE'] = commandqueueinfo2dragon
            
            
            
            # ---------------------------------------------------------------------------------------------
            # (9) BUFFER INFO - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2dragon = {}
           
            message2print = "{(Step 9A/70) trying" + "; bufferinfo2dragon-trying-by-create2image4dragon: " + str(bufferinfo2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2dragon = get2bufferinfo4dragon()                                                                   # SYSTEM CREATES BUFFER FOR DRAGON, IF NECESSARY
                message2print = "{(Step 9B/70) got" + "; bufferinfo2dragon-got-by-create2image4dragon: " + str(bufferinfo2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0,
                #  'COUNT2REFERENCE': 0,
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4dragon-failed ; time: 05:04:51 09/06/11 Pacific Daylight Time}',
                #                    'CLASS': 'create2buffer4dragon',
                #                    'METHOD': 'create2buffer4dragon'},
                #  'FUNCTION2CALLBACK': <function trace2buffer4dragon at 0x0000000003B2B348>,
                #  'POINTER2BUFFER': <mmap.mmap object at 0x0000000003ADD2F0>,
                #  'FLAGS': 0,
                #  'CONTEXT': {'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4dragon-failed ; time: 05:04:51 09/06/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2context4dragon', 
                #  'METHOD': 'create2context4dragon'}, 
                #  'FUNCTION2CALLBACK': <function trace2context4dragon at 0x00000000036E0148>, 
                #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-', 
                #  'INFO2PRIVATE': None, 
                #  'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
                #  'NACK2RETURN': ' ', 
                #  'SIZE2PRIVATE': 0, 
                #  'PROPERTIES': None}, 
                #  'DEVICE': {'RESOLUTION2TIMER4PROFILING': 318, ...
                
                
                code2ack  = code2ack + '9B-' 
            except:
                message2print = "{(Step 9Z/70) failed" + "; bufferinfo2dragon-failed-by-create2image4dragon: " + str(bufferinfo2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '9Z-'
        
            task4imagecreate2dragon ['BUFFER'] = bufferinfo2dragon 
            
            # ---------------------------------------------------------------------------------------------
            # (10) EVENT INFO - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
            eventinfo2dragon = {}
           
            message2print = "{(Step 10A/70) trying" + "; eventinfo2dragon-trying-by-create2image4dragon: " + str(eventinfo2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2dragon = get2eventinfo4dragon()                                                                     # SYSTEM CREATES EVENT IF NECESSAY   
                message2print = "{(Step 10B/70) got" + "; eventinfo2dragon-got-by-create2image4dragon: " + str(eventinfo2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0,
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2event4dragon-failed ; time: 05:10:51 09/06/11 Pacific Daylight Time}',
                #                    'CLASS': 'create2event4dragon',
                #                    'METHOD': 'create2event4dragon'},
                #  'FUNCTION2CALLBACK': <function trace2event4dragon at 0x0000000003747D48>,
                #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-6B-7B-8B-9B-10B-11B-12B-14B-15B-16B-',
                #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003A02898>,
                #  'NACK2RETURN': ' ',
                #  'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4dragon-failed ; time: 05:10:51 09/06/11 Pacific Daylight Time}', 
                #                                                      'CLASS': 'create2context4dragon', 'METHOD': 'create2context4dragon'}, 
                #  'FUNCTION2CALLBACK': <function trace2context4dragon at 0x00000000036F0148>,
                #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-',
                #  'INFO2PRIVATE': None,
                #  'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
                #  'NACK2RETURN': ' ',
                #  'SIZE2PRIVATE': 0,
                #  'PROPERTIES': None},
                #  'COMMANDQUEUE': {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': ...
                
                
                code2ack  = code2ack + '10B-' 
            except:
                message2print = "{(Step 10Z/70) failed" + "; eventinfo2dragon-failed-by-create2image4dragon: " + str(eventinfo2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '10Z-'
        
            task4imagecreate2dragon ['EVENT'] = eventinfo2dragon             
            
            # ---------------------------------------------------------------------------------------------
            # (11) FLAGS - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            flags2dragon = 0
           
            message2print = "{(Step 11A/70) trying" + "; flags2dragon-trying-by-create2image4dragon: " + str(flags2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                flags2dragon = 0                                                              
                message2print = "{(Step 11B/70) set" + "; flags2dragon-set-by-create2image4dragon to: " + str(flags2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '11B-' 
            except:
                message2print = "{(Step 11Z/70) failed" + "; flags2dragon-failed-by-create2image4dragon: " + str(flags2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '11Z-'
        
            task4imagecreate2dragon ['FLAGS'] = flags2dragon             
            
            # ---------------------------------------------------------------------------------------------
            # (14) WIDTH - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            width2dragon = 0

            quantity2core = 1                                                                      # ORIGIN 
            quantity2skeleton = 1                                                                  # SKELETON WITH DISTANCE 1
            quantity2layers = 33                                                                   # LAYERS 10, 100 ... 900, 1000 ... 3300
            quantity2frame = 1                                                                     # FRAME
            quantity2extra = 4                                                                     # CONTROLS FOR GUI ETC...
            width2layer = 10                                                                       # SCALED TO 10 EACH LAYER            
            
            message2print = "{(Step 14A/70) trying" + "; width2dragon-trying-by-create2image4dragon: " + str(width2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                width2dragon = int(width2layer * (quantity2core + quantity2skeleton + quantity2layers + quantity2frame + quantity2extra))
                width2dragon = 2 * width2dragon                                                          # POS and NEG X  2*400 = 800 PIXELS
                                                                                                                                         
                message2print = "{(Step 14B/70) set" + "; width2dragon-set-by-create2image4dragon to: " + str(width2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 800 
                code2ack  = code2ack + '14B-' 
            except:
                message2print = "{(Step 14Z/70) failed" + "; width2dragon-failed-by-create2image4dragon: " + str(width2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '14Z-'
        
            task4imagecreate2dragon ['WIDTH'] = width2dragon
            
            
            # ---------------------------------------------------------------------------------------------
            # (15) HEIGHT - REQUIRED                               
            # ---------------------------------------------------------------------------------------------             
            height2dragon = 0
            quantity2core = 1                                                                      # ORIGIN 
            quantity2skeleton = 1                                                                  # SKELETON WITH DISTANCE 1
            quantity2layers = 33                                                                   # LAYERS 10, 100 ... 900, 1000 ... 3300
            quantity2frame = 1                                                                     # FRAME
            quantity2extra = 4                                                                     # CONTROLS FOR GUI
            
            quantity2total = quantity2core + quantity2skeleton + quantity2layers + quantity2frame + quantity2extra
            height2layer = 10                                                                      # SCALED TO 10 
                       
            message2print = "{(Step 15A/70) trying" + "; height2dragon-trying-by-create2image4dragon: " + str(height2dragon)  + "; time: " + time4xi() + "}"  
            print4xi(class2print, method2print, message2print) 
                        
            try:
                height2dragon = height2layer * quantity2total
                
                height2dragon = 2 * height2dragon                                                        # POS and NEG Y   2*400 = 800 PIXELS
                                                                                                                                         
                message2print = "{(Step 15B2/70) set" + "; height2dragon-set-by-create2image4dragon to: " + str(height2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 800 
                code2ack  = code2ack + '15B-' 
            except:
                message2print = "{(Step 15Z/70) failed" + "; height2dragon-failed-by-create2image4dragon: " + str(height2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '15Z-'
             
                        
            task4imagecreate2dragon ['HEIGHT'] = height2dragon 
            
            # ---------------------------------------------------------------------------------------------
            # (16) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.


            rowpitch2dragon = 0
            size2element = 1                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity2layer = 1                                                                                 # MAX NUMBER OF VOXELS IN EACH 2D LAYER 
            quantity2direction = 1                                                                                  # ONE DIRECTIONS IN 2D
            
            message2print = "{(Step 16A/70) trying" + "; rowpitch2dragon-trying-by-create2image4dragon: " + str(rowpitch2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch2dragon = int(width2dragon * size2element * voxelquantity2layer / quantity2direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 16B/70) set" + "; rowpitch2dragon-set-by-create2image4dragon to: " + str(rowpitch2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 800                                                                                             
                 
                code2ack  = code2ack + '16B-' 
            except:
                message2print = "{(Step 16Z/70) failed" + "; rowpitch2dragon-failed-by-create2image4dragon: " + str(rowpitch2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '16Z-'
        
            task4imagecreate2dragon ['ROWPITCH'] = rowpitch2dragon     
            
            # ---------------------------------------------------------------------------------------------
            # (17) POINTER - REQUIRED                           
            # ---------------------------------------------------------------------------------------------             
            pointer2dragon = None
            size2dragon = int(width2dragon * height2dragon)                                                                  # CALCULATED MEMORY
            if size2dragon == 0:
                size2dragon = 4400                                                                                     # MINIMUM MEMORY
            
            message2print = "{(Step 17A/70) trying" + "; pointer2dragon-trying-by-create2image4dragon: " + str(pointer2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:
                arena2dragon = Arena4Xi(size2dragon)                                                                      # SYSTEM RESERVER MEMORY FOR IMAGE                             
                                                                                                                                         
                message2print = "{(Step 17B1/70) got" + "; arena2dragon-got-by-create2image4dragon: " + str(arena2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # <arena4xi.Arena4Xi object at 0x00000000041C0D68>
                
                size2arena = arena2dragon.size                                                                         # 
                              
                message2print = "{(Step 17B2/70) got" + "; size2arena-got-by-create2buffer4dragon: " + str(size2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 640000 
                
                buffer2arena = arena2dragon.buffer
                
                message2print = "{(Step 17B3/70) got" + "; buffer2arena-got-by-create2buffer4dragon: " + str(buffer2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # <mmap.mmap object at 0x0000000003AAF298> 
                
                code2ack  = code2ack + '17B-' 
                
                if size2arena < size2dragon:
                    size2dragon = size2arena                                                                           # AVAILABLE MEMORY FOR IMAGE
                    message2print = "{(Step 17C/70) set" + "; size2dragon-set-by-create2buffer4dragon to: " + str(size2dragon)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    
                    code2ack  = code2ack + '17C-'
                else:
                    pass
                    
                pointer2dragon = buffer2arena
                message2print = "{(Step 17D/70) set" + "; pointer2dragon-set-by-create2buffer4dragon to: " + str(pointer2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # <mmap.mmap object at 0x0000000003AAF298>
                
                code2ack  = code2ack + '17D-'       
            except:
                message2print = "{(Step 17Z/70) failed" + "; pointer2dragon-failed-by-create2image4dragon: " + str(pointer2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '17Z-'
        
            task4imagecreate2dragon ['POINTER'] = pointer2dragon                            
            task4imagecreate2dragon ['SIZE']    = size2dragon                                                             # EXTRA
            
            
            # ---------------------------------------------------------------------------------------------
            # (18) FORMAT - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            format2dragon = {}
           
            message2print = "{(Step 18A/70) trying" + "; format2dragon-trying-by-create2image4dragon: " + str(format2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                format2dragon['ORDER'] = 'CL_RGBA'                                                              
                message2print = "{(Step 18B/70) set" + "; format2dragon-set-by-create2image4dragon to: " + str(format2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'ORDER': 'CL_RGBA'}
                 
                code2ack  = code2ack + '18B-'
                
                format2dragon['TYPE'] = 'CL_UNORM_INT8'                                                              
                message2print = "{(Step 18C/70) set" + "; format2dragon-set-by-create2image4dragon to: " + str(format2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'}
                
                code2ack  = code2ack + '18C-'                
                 
            except:
                message2print = "{(Step 18Z/70) failed" + "; format2dragon-failed-by-create2image4dragon: " + str(format2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '18Z-'
        
            task4imagecreate2dragon ['FORMAT'] = format2dragon                  
            
            # ---------------------------------------------------------------------------------------------
            # (19) DEPTH - MUST BE 0                               
            # ---------------------------------------------------------------------------------------------             
            depth2dragon = 0
            message2print = "{(Step 19A/70) set" + "; depth2dragon-set-by-create2image4dragon to: " + str(depth2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            #  
            code2ack  = code2ack + '19A-' 
            task4imagecreate2dragon ['DEPTH'] = depth2dragon
                                                                                                                                       
        else:
            pass                                                                                                    # PARAMETER DATA RECEIVED                       
    except: 
        message2print = "{(Step 1Z1/70) yielded" + "; task4imagecreate2dragon-yielded-by-create2image4dragon: " + str(task4imagecreate2dragon) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '1Z1-' 

    # ---------------------------------------------------------------------------------------------
    # TASK EITHER GENERATED BY THIS PROGRAM (NONE GIVEN) OR GIVEN BY USER    
    # ---------------------------------------------------------------------------------------------
    keys2task = task4imagecreate2dragon.keys()        
    message2print = "{(Step 19A/70) using" + "; keys2task-using-by-create2image4dragon: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
    # ['BUFFER',
    #  'CODE2ERROR',
    #  'COMMANDQUEUE',
    #  'CONTEXT',
    #  'DATA2CALLBACK',
    #  'DEVICE',
    #  'EVENT',
    #  'FLAGS',
    #  'FORMAT',
    #  'FUNCTION2CALLBACK',
    #  'HEIGHT',
    #  'PIPE',
    #  'PLATFORM',
    #  'POINTER',
    #  'QUEUE',
    #  'ROWPITCH',
    #  'SIZE',
    #  'WIDTH']; time: 00:53:55 10/05/11 Pacific Daylight Time}
  
    
    # ---------------------------------------------------------------------------------------------
    # (Step 20A/70) - CREATE 2D IMAGE FOR DRAGON! 
    # ---------------------------------------------------------------------------------------------    
    dragon2image4xi = {}                                                                                                                                                                               
    message2print = "{(Step 20A/70) initialized" + "; dragon2image4xi-initialized-by-create2image4dragon to: " + str(dragon2image4xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # {} 
    
    message2print = "{(Step 21A0/70) try" + "; data2callback-try-by-create2image4dragon: {}" + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)      
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 21A/70) - DATA2CALLBACK - EXTRA 
        # -----------------------------------------------------------------------------------------               
        dragon2image4xi['DATA2CALLBACK'] = task4imagecreate2dragon['DATA2CALLBACK']
        
        message2print = "{(Step 21A1/70) initialized" + "; data2callback-created-by-create2image4dragon to: " + str(dragon2image4xi['DATA2CALLBACK'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'MESSAGE': '{(Step 1Z1/33) failed; create2image4dragon-failed ; time: 05:56:13 09/06/11 Pacific Daylight Time}',
        #  'CLASS': 'create2image4dragon',
        #  'METHOD': 'create2image4dragon'}; time: 05:56:13 09/06/11 Pacific Daylight Time}

        
        code2ack  = code2ack + '21A1-'
    except: 
        message2print = "{(Step 21Z1/70) yielded" + "; data2callback-dragon2image4xi-yielded-by-create2image4dragon: " + str(dragon2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '21Z1-'
    
    message2print = "{(Step 22B0/70) try" + "; function2callback-try-by-create2image4dragon: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 22B/70) - FUNCTION2CALLBACK - EXTRA
        # -----------------------------------------------------------------------------------------               
        dragon2image4xi['FUNCTION2CALLBACK'] = task4imagecreate2dragon['FUNCTION2CALLBACK']
        
        message2print = "{(Step 22B1/70) initialized" + "; function2callback-created-by-create2image4dragon to: " + str(dragon2image4xi['FUNCTION2CALLBACK'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # <function trace2image4dragon at 0x0000000003B04348>  
        code2ack  = code2ack + '22B1-'
    except: 
        message2print = "{(Step 22Z2/70) yielded" + "; function2callback-dragon2image4xi-yielded-by-create2image4dragon: " + str(dragon2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '22Z2-' 
    
    message2print = "{(Step 22C0/70) try" + "; platform-try-by-create2image4dragon: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 22C/70) - PLATFORM - EXTRA
        # -----------------------------------------------------------------------------------------               
        dragon2image4xi['PLATFORM'] = task4imagecreate2dragon['PLATFORM']
        
        message2print = "{(Step 22C1/70) initialized" + "; platform-created-by-create2image4dragon to: " + str(dragon2image4xi['PLATFORM'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}; time: 05:56:13 09/06/11 Pacific Daylight Time}
  
        code2ack  = code2ack + '22C1-'
    except: 
        message2print = "{(Step 22Z3/70) yielded" + "; platform-dragon2image4xi-yielded-by-create2image4dragon: " + str(dragon2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '22Z3-' 
        
        
      
    message2print = "{(Step 23A/70) try" + "; context2dragon-try-by-create2image4dragon: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)       
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 23/70) - CONTEXT - REQUIRED
        # -----------------------------------------------------------------------------------------               
        dragon2image4xi['CONTEXT'] = task4imagecreate2dragon['CONTEXT']
        
        message2print = "{(Step 23B/70) initialized" + "; context-created-by-create2image4dragon to: " + str(dragon2image4xi['CONTEXT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'COUNT2REFERENCE': 0,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4dragon-failed ; time: 05:56:12 09/06/11 Pacific Daylight Time}', 
        #  'CLASS': 'create2context4dragon',
        #  'METHOD': 'create2context4dragon'},
        #  'FUNCTION2CALLBACK': <function trace2context4dragon at 0x00000000036B0148>,
        #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-',
        #  'INFO2PRIVATE': None,
        #  'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
        #  'NACK2RETURN': ' ',
        #  'SIZE2PRIVATE': 0,
        #  'PROPERTIES': None}; time: 05:56:13 09/06/11 Pacific Daylight Time}

        
        code2ack  = code2ack + '23B-'
    except: 
        message2print = "{(Step 23Z1/70) yielded" + "; context-dragon2image4xi-yielded-by-create2image4dragon: " + str(dragon2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
    
        code2nack  = code2nack + '23Z1-' 
        
    message2print = "{(Step 24A/70) try" + "; device-try-by-create2image4dragon: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 24/70) - DEVICE - EXTRA
        # -----------------------------------------------------------------------------------------               
        dragon2image4xi['DEVICE'] = task4imagecreate2dragon['DEVICE']
        
        message2print = "{(Step 24B/70) initialized" + "; device-created-by-create2image4dragon to: " + str(dragon2image4xi['DEVICE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'RESOLUTION2TIMER4PROFILING': 318,
        #  'ID2PLATFORM': '0000000001509568',
        #  'PROFILING2QUEUE': 'ACK',
        #  'EXECUTE2KERNELS4OPENCL': 'ACK',
        #  'NATIVE2WIDTHLONG4VECTOR': 2,
        #  'MAX2UNITS4COMPUTE': 4,
        #  'SIZE2MEMORY4GLOBAL': 3221225472,
        #  'NATIVE2WIDTHFLOAT4VECTOR': 4,
        #  'MAX2SIZE4WORKGROUP': 1024,
        #  'PREFERRED2WIDTHCHAR4VECTOR': 16,
        #  'SIZE2CACHE': 65536,
        #  'IEEE754-2008FUSED2MULTIPLY-ADD': 'NACK',
        #  'TYPE4CACHE': 'Read/Write',
        #  'PREFERRED2WIDTHINT4VECTOR': 4,
        #  'SIZE2PREFERREDWORKGROUPMULTIPLE4KERNEL': 1,
        #  'NATIVE2WIDTHINT4VECTOR': 4,
        #  'SIZE2BUFFER4CONSTANT': 65536,
        #  'EXECUTE2FUNCTION4NATIVE': 'ACK',
        #  'PREFERRED2WIDTHDOUBLE4VECTOR': 0,
        #  'MAX2NUMBER4CONSTANTARGS': 8,
        #  'VERSION2DRIVER': '2.0', 'EXTENSIONS': 'cl_amd_fp64 ...
        
        
        code2ack  = code2ack + '24B-'
    except: 
        message2print = "{(Step 24Z1/70) yielded" + "; device-dragon2image4xi-yielded-by-create2image4dragon: " + str(dragon2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '24Z1-'
                 
    message2print = "{(Step 25A/70) try" + "; code2error-try-by-create2image4dragon: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print) 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 25/70) - ERROR CODE - REQUIRED
        # -----------------------------------------------------------------------------------------               
        dragon2image4xi['CODE2ERROR'] = task4imagecreate2dragon['CODE2ERROR']
        
        message2print = "{(Step 25B/70) initialized" + "; code2error-created-by-create2image4dragon to: " + str(dragon2image4xi['CODE2ERROR'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0 
        code2ack  = code2ack + '25B-'
    except: 
        message2print = "{(Step 25Z1/70) yielded" + "; code2error-dragon2image4xi-yielded-by-create2image4dragon: " + str(dragon2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '25Z1-'                
                
          
    message2print = "{(Step 26A/70) try" + "; queue-try-by-create2image4dragon: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 26/70) - QUEUE - EXTRA
        # -----------------------------------------------------------------------------------------               
        dragon2image4xi['QUEUE'] = task4imagecreate2dragon['QUEUE']
        
        message2print = "{(Step 26B/70) initialized" + "; queue-created-by-create2image4dragon to: " + str(dragon2image4xi['QUEUE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # <queue4xi.Queue4Xi object at 0x0000000003ED99E8>  
        code2ack  = code2ack + '26B-'
    except: 
        message2print = "{(Step 26Z1/70) yielded" + "; queue-dragon2image4xi-yielded-by-create2image4dragon: " + str(dragon2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '26Z1-' 
                                
    message2print = "{(Step 27A/70) try" + "; pipe-try-by-create2image4dragon: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                     
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 27/70) - PIPE - EXTRA
        # -----------------------------------------------------------------------------------------               
        dragon2image4xi['PIPE'] = task4imagecreate2dragon['PIPE']
        
        message2print = "{(Step 27B/70) initialized" + "; pipe-created-by-create2image4dragon to: " + str(dragon2image4xi['PIPE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # (<read-write PipeConnection, handle 376>, <read-write PipeConnection, handle 380>)  
        
        code2ack  = code2ack + '27B-'
    except: 
        message2print = "{(Step 27Z1/70) yielded" + "; pipe-dragon2image4xi-yielded-by-create2image4dragon: " + str(dragon2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '27Z1-'   

    message2print = "{(Step 28A/70) try" + "; commandqueue-try-by-create2image4dragon: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (28) COMMAND QUEUE - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        dragon2image4xi['COMMANDQUEUE']  = task4imagecreate2dragon['COMMANDQUEUE']  
 
        message2print = "{(Step 28B/70) initialized" + "; commandqueue-created-by-create2image4dragon to: " + str(dragon2image4xi['COMMANDQUEUE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0,
        #  'COUNT2REFERENCE': 0,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4dragon-failed ; time: 07:13:59 09/06/11 Pacific Daylight Time}',
        #  'CLASS': 'create2commandqueue4dragon',
        #  'METHOD': 'create2commandqueue4dragon'},
        #  'FUNCTION2CALLBACK': <function trace2commandqueue4dragon at 0x00000000037BC848>,
        #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-8A-9B-10B-9B-10B-11B-12B-13B-14B-15B-16B-17B-18B-',
        #  'INFO2PRIVATE': None,
        #  'QUEUE': <queue4xi.Queue4Xi object at 0x000000000432B3C8>,
        #  'NACK2RETURN': ' ',
        #  'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': ...
        
        code2ack  = code2ack + '28B-'
    except: 
        message2print = "{(Step 28Z1/70) yielded" + "; commandqueue-yielded-by-create2image4dragon: " + str(dragon2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
              
        code2nack  = code2nack + '28Z1-'    

    message2print = "{(Step 29A/70) try" + "; bufferinfo-try-by-create2image4dragon: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (29) BUFFER INFO - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        dragon2image4xi['BUFFER']  = task4imagecreate2dragon['BUFFER']  
 
        message2print = "{(Step 29B/70) initialized" + "; bufferinfo-created-by-create2image4dragon to: " + str(dragon2image4xi['BUFFER'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0,
        #  'COUNT2REFERENCE': 0,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4dragon-failed ; time: 05:56:12 09/06/11 Pacific Daylight Time}', 
        #  'CLASS': 'create2buffer4dragon',
        #  'METHOD': 'create2buffer4dragon'},
        #  'FUNCTION2CALLBACK': <function trace2buffer4dragon at 0x0000000003AFC348>,
        #  'POINTER2BUFFER': <mmap.mmap object at 0x0000000003AAF2F0>,
        #  'FLAGS': 0,
        #  'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4dragon-failed ...
        
        code2ack  = code2ack + '29B-'
    except: 
        message2print = "{(Step 29Z1/70) yielded" + "; bufferinfo-yielded-by-create2image4dragon: " + str(dragon2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '29Z1-'            

    message2print = "{(Step 30A/70) try" + "; eventinfo-try-by-create2image4dragon: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (30) EVENT INFO - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        dragon2image4xi['EVENT']  = task4imagecreate2dragon['EVENT']  
 
        message2print = "{(Step 30B/70) initialized" + "; eventinfo-created-by-create2image4dragon to: " + str(dragon2image4xi['EVENT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2event4dragon-failed ; time: 05:56:12 09/06/11 Pacific Daylight Time}',
        #                     'CLASS': 'create2event4dragon',
        #                     'METHOD': 'create2event4dragon'},
        #  'FUNCTION2CALLBACK': <function trace2event4dragon at 0x0000000003707D48>,
        #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-6B-7B-8B-9B-10B-11B-12B-14B-15B-16B-',
        #  'QUEUE': <queue4xi.Queue4Xi object at 0x00000000039C3860>,
        #  'NACK2RETURN': ' ',
        #  'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': ...
        
        code2ack  = code2ack + '30B-'
    except: 
        message2print = "{(Step 30Z1/70) yielded" + "; eventinfo-yielded-by-create2image4dragon: " + str(dragon2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '30Z1-'            
                

    message2print = "{(Step 31A/70) try" + "; flags-try-by-create2image4dragon: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (31) FLAGS - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        dragon2image4xi['FLAGS']  = task4imagecreate2dragon['FLAGS']  
 
        message2print = "{(Step 31B/70) initialized" + "; flags-created-by-create2image4dragon to: " + str(dragon2image4xi['FLAGS'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 0
        code2ack  = code2ack + '31B-'
    except: 
        message2print = "{(Step 31Z1/70) yielded" + "; flags-yielded-by-create2image4dragon: " + str(dragon2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '31Z1-'     

    message2print = "{(Step 34A/70) try" + "; width-try-by-create2image4dragon: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (34) WIDTH - REQUIRED                              
        # ---------------------------------------------------------------------------------------------      
        dragon2image4xi['WIDTH']  = task4imagecreate2dragon['WIDTH']  
 
        message2print = "{(Step 34H/70) initialized" + "; width-created-by-create2image4dragon to: " + str(dragon2image4xi['WIDTH'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 800 
        code2ack  = code2ack + '34H-'
    except: 
        message2print = "{(Step 34Z1/70) yielded" + "; width-yielded-by-create2image4dragon: " + str(dragon2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '34Z1-'   


    message2print = "{(Step 35A/70) try" + "; height-try-by-create2image4dragon: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (35) HEIGHT - REQUIRED                              
        # ---------------------------------------------------------------------------------------------      
        dragon2image4xi['HEIGHT']  = task4imagecreate2dragon['HEIGHT']  
 
        message2print = "{(Step 35H/70) initialized" + "; height-created-by-create2image4dragon to: " + str(dragon2image4xi['HEIGHT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 800 
        code2ack  = code2ack + '35H-'
    except: 
        message2print = "{(Step 35Z1/70) yielded" + "; height-yielded-by-create2image4dragon: " + str(dragon2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '35Z1-'   
 

    message2print = "{(Step 36A/70) try" + "; rowpitch-try-by-create2image4dragon: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (36) ROWPITCH - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        dragon2image4xi['ROWPITCH']  = task4imagecreate2dragon['ROWPITCH']  
 
        message2print = "{(Step 36B/70) initialized" + "; rowpitch-created-by-create2image4dragon to: " + str(dragon2image4xi['ROWPITCH'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 800
        code2ack  = code2ack + '36B-'
    except: 
        message2print = "{(Step 36Z1/70) yielded" + "; rowpitch-yielded-by-create2image4dragon: " + str(dragon2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '36Z1-'

    message2print = "{(Step 37A/70) try" + "; pointer-try-by-create2image4dragon: None"  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (37) POINTER - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        dragon2image4xi['POINTER']  = task4imagecreate2dragon['POINTER']  
 
        message2print = "{(Step 37B/70) initialized" + "; pointer-created-by-create2image4dragon to: " + str(dragon2image4xi['POINTER'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # <mmap.mmap object at 0x0000000003C050E0>
         
        code2ack  = code2ack + '37B-'
    except: 
        message2print = "{(Step 37Z1/70) yielded" + "; pointer-yielded-by-create2image4dragon: " + str(dragon2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '37Z1-'

    message2print = "{(Step 38A/70) try" + "; format-try-by-create2image4dragon: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (38) FORMAT - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        dragon2image4xi['FORMAT']  = task4imagecreate2dragon['FORMAT']  
 
        message2print = "{(Step 38B/70) initialized" + "; format-created-by-create2image4dragon to: " + str(dragon2image4xi['FORMAT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'} 
        code2ack  = code2ack + '38B-'
    except: 
        message2print = "{(Step 38Z1/70) yielded" + "; format-yielded-by-create2image4dragon: " + str(dragon2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '38Z1-'

    message2print = "{(Step 39A/70) try" + "; depth-try-by-create2image4dragon: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (39) DEPTH - MUST BE 0                              
        # ---------------------------------------------------------------------------------------------             
                        
        dragon2image4xi['DEPTH']  = task4imagecreate2dragon['DEPTH']  
 
        message2print = "{(Step 39B/70) initialized" + "; depth-created-by-create2image4dragon to: " + str(dragon2image4xi['DEPTH'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        #  
        code2ack  = code2ack + '39B-'
    except: 
        message2print = "{(Step 39Z1/70) yielded" + "; depth-yielded-by-create2image4dragon: " + str(dragon2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '39Z1-'
                                                                                                               
    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    dragon2image4xi ['ACK2RETURN']  = code2ack
    dragon2image4xi ['NACK2RETURN'] = code2nack  
    
    message2print = "{(Step 70/70) returning" + "; dragon2image4xi-returning-by-create2image4dragon: " + str(dragon2image4xi) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    # 'DEPTH'
    # 'WIDTH'
    # 'HEIGHT'
    return dragon2image4xi
            
#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "set2image4dragon" usage = "
#    
#   print('(Step 4B/33) OF DRAGON2IMAGE4XI5TEST - TRYING TO GET PLATFORMIDS4XI', time4xi())
#   platform2xi = {}
#   try:
#       platform2xi = get2platformids4xi()
#       print('(Step 4B/33) of CONTEXT4DRAGON5TEST - GOT PLATFORMIDS4XI', platform2xi) 
#   except:
#       print('(Step 4Z/33) of CONTEXT4DRAGON5TEST *** FAILED TO GET PLATFORMIDS4XI', time4xi())     
#   
#   print('(Step 2A/33) OF DEVICE4DRAGON5TEST - TRYING TO INITIALIZE DEVICE4DRAGON ', time4xi())   
#   print('=' *33)        
#   try:
#       device2dragon = init2dragon4xi()                                                                                
#       print('(Step 2B/33) OF DEVICE4DRAGON5TEST - INITIALIZED DEVICE4DRAGON TO: ', device2dragon)        
#   except:
#       print('(Step 2Z/33) OF DEVICE4DRAGON5TEST *** FAILED TO INITIALIZE DEVICE4DRAGON', time4xi()) 
#   print('(Step 3A/33) OF DRAGON2IMAGE4XI5TEST - TRYING TO CREATE CONTEXT4DRAGON ', time4xi())    
#   print('=' *33)        
#   try:
#       context2dragon = create2context4dragon()                                                                               
#       print('(Step 3B/33) OF DRAGON2IMAGE4XI5TEST - CREATED CONTEXT4DRAGON: ', context2dragon)                                  
#   except:
#       print('(Step 3Z/33) OF DRAGON2IMAGE4XI5TEST *** FAILED TO CREATE CONTEXT4DRAGON', time4xi()) 
#          
#   task4set2image = {}
#   task4set2image  ['CONTEXT'] = context2dragon  
# 
#   ... other data for set operation ...
#    
#        
#   response4set2Dimage = {} 
#   print('(Step 11A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO SET 2D IMAGE4DRAGON ', time4xi())   
#   try:                               
#       response4set2Dimage = set2image4dragon(task4set2image)                                                                                                         
#                                                                                     
#       print('(Step 11B/700) OF DRAGON2IMAGE4XI5TEST - SET 2D IMAGE4DRAGON ', time4xi())
#       keys2response = response4set2Dimage.keys()
#       for key2response in sorted(keys2response):
#           print(key2response, ' : ', response4set2Dimage[key2response])
#           # 
#   except:
#       print('(Step 11Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO SET 2D IMAGE4DRAGON *** ', time4xi()) 
#
#">

# -------------------------------------------------------------------------------------------------   
def set2image4dragon(task4imageset2dragon=None):

    '''
    sets 2D images for DRAGON
    '''
    global dragon2image4xi  
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "set2image4dragon"
    method2print = "set2image4dragon"

    message2print = "{(Step 1A/33) started" + "; set2image4dragon-started: " + str(set2image4dragon)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function set2image4dragon at 0x0000000003A62848> 
    
    message2print = "{(Step 1B/33) passed" + "; task4imageset2dragon-to-set2image4dragon: " + str(task4imageset2dragon) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # {'CODE2ERROR': 0,
    #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003D4A710>,
    #  'CONTEXT': {'COUNT2REFERENCE': 0,
    #              'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4dragon-failed ; time: 07:13:59 09/06/11 Pacific Daylight Time}',
    #              'CLASS': 'create2context4dragon',
    #              'METHOD': 'create2context4dragon'},
    #              'FUNCTION2CALLBACK': <function trace2context4dragon at 0x0000000003770148>,
    #              'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-',
    #              'INFO2PRIVATE': None,
    #              'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
    #              'NACK2RETURN': ' ',
    #              'SIZE2PRIVATE': 0,
    #              'PROPERTIES': None},
    #  'COMMANDQUEUE': {'CODE2ERROR': 0,
    #              'COUNT2REFERENCE': 0,
    #              'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4dragon-failed ; time: 07:13:59 09/06/11 Pacific Daylight Time}',
    #              'CLASS': 'create2commandqueue4dragon',
    #              'METHOD': 'create2commandqueue4dragon'},
    #  'FUNCTION2CALLBACK': <function trace2commandqueue4dragon at 0x00000000037BC848>,
    #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-8A-9B-10B-9B-10B-11B-12B-13B-14B-15B-16B-17B-18B-',
    #  'INFO2PRIVATE': None,
    #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003EDA780>,
    #  'NACK2RETURN': ' ',
    #  'CONTEXT': {'COUNT2REFERENCE': 0,
    #              'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4dragon-failed ; time: 07:13:59 09/06/11 Pacific Daylight Time}',
    #              'CLASS': 'create2context4dragon',
    #              'METHOD': 'create2context4dragon'},
    #              'FUNCTION2CALLBACK': <function trace2context4dragon at 0x0000000003770148>,
    #              'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-',
    #              'INFO2PRIVATE': None,
    #              'PLATFORM': {'list2platforms': ['DragonXi'],
    #                           'ack2return': ' 2C-2E-2F-2G-',
    #                           'nack2return': ' ', 'available2platform': 1, 'max2entries': 1},
    #              'NACK2RETURN': ' ',
    #              'SIZE2PRIVATE': 0,
    #              'PROPERTIES': None},
    #              'DEVICE': {'RESOLUTION2TIMER4PROFILING': 318,
    #                          'ID2PLATFORM': '0000000001509568',
    #  'PROPERTIES': None,
    #  'PIPE': (<read-write PipeConnection, handle 552>, <read-write PipeConnection, handle 556>)},
    #  'DEVICE': {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568', 'PROFILING2QUEUE': 'ACK', 'EXECUTE2KERNELS4OPENCL': 'ACK',
    #              'NATIVE2WIDTHLONG4VECTOR': 2, 'MAX2UNITS4COMPUTE': 4, 'SIZE2MEMORY4GLOBAL': 3221225472, 'NATIVE2WIDTHFLOAT4VECTOR': 4, 'MAX2SIZE4WORKGROUP': 1024, 'PREFERRED2WIDTHCHAR4VECTOR': 16, 'SIZE2CACHE': 65536, 
    #              'IEEE754-2008FUSED2MULTIPLY-ADD': 'NACK', 'TYPE4CACHE': 'Read/Write', 'PREFERRED2WIDTHINT4VECTOR': 4, 'SIZE2PREFERREDWORKGROUPMULTIPLE4KERNEL': 1, 'NATIVE2WIDTHINT4VECTOR': 4, 
    #              'SIZE2BUFFER4CONSTANT': 65536, 'EXECUTE2FUNCTION4NATIVE': 'ACK', 'PREFERRED2WIDTHDOUBLE4VECTOR': 0, 'MAX2NUMBER4CONSTANTARGS': 8, 'VERSION2DRIVER': '2.0', 
            
    code2ack  = ' '
    code2nack = ' '
    # -----------------------------------------------------------------------------------------
    # CREATE IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if dragon2image4xi is None:
            dragon2image4xi = create2image4dragon()                                                                              # INITIALIZE IMAGE FOR DRAGON 
            message2print = "{(Step 1C/70) set" + "; dragon2image4xi-set-by-set2image4dragon to: " + str(dragon2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create2image4dragon-skipped-by-set2image4dragon: " + str(create2image4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # <function create2image4dragon at 0x0000000003A627C8>;                                  
    except:
        try:
            dragon2image4xi = create2image4dragon()
            message2print = "{(Step 1E/70) set" + "; dragon2image4xi-set-by-set2image4dragon to: " + str(dragon2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            dragon2image4xi = {}
            message2print = "{(Step 1Z/70) yielded" + "; dragon2image4xi-yielded-by-set2image4dragon: " + str(dragon2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z-' 
                
    # ---------------------------------------------------------------------------------------------
    # SET            
    # ---------------------------------------------------------------------------------------------     
    message2print = "{(Step 22A0/70) try" + "; data2callback-try-by-set2image4dragon: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    #     
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 21A/70) - DATA2CALLBACK - EXTRA
        # -----------------------------------------------------------------------------------------               
        dragon2image4xi['DATA2CALLBACK'] = task4imageset2dragon['DATA2CALLBACK']
                                        
        message2print = "{(Step 21A1/70) set" + "; data2callback-set-by-set2image4dragon to: " + str(dragon2image4xi['DATA2CALLBACK'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        
        code2ack  = code2ack + '21A1-'
    except: 
        message2print = "{(Step 21Z1/70) yielded" + "; data2callback-dragon2image4xi-yielded-by-set2image4dragon: " + str(dragon2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded
        code2nack  = code2nack + '21Z1-'
    
    message2print = "{(Step 22B0/70) try" + "; function2callback-try-by-set2image4dragon: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 22B/70) - FUNCTION2CALLBACK - EXTRA
        # -----------------------------------------------------------------------------------------               
        dragon2image4xi['FUNCTION2CALLBACK'] = task4imageset2dragon['FUNCTION2CALLBACK']
        
        message2print = "{(Step 22B1/70) set" + "; function2callback-set-by-set2image4dragon to: " + str(dragon2image4xi['FUNCTION2CALLBACK'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #  
        code2ack  = code2ack + '22B1-'
    except: 
        message2print = "{(Step 22Z2/70) yielded" + "; function2callback-dragon2image4xi-yielded-by-set2image4dragon: " + str(dragon2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded
        code2nack  = code2nack + '22Z2-' 
    
    message2print = "{(Step 22C0/70) try" + "; platform2callback-try-by-set2image4dragon: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 22C/70) - PLATFORM - EXTRA
        # -----------------------------------------------------------------------------------------               
        dragon2image4xi['PLATFORM'] = task4imageset2dragon['PLATFORM']
        
        message2print = "{(Step 2C1/70) set" + "; platform-set-by-set2image4dragon to: " + str(dragon2image4xi['PLATFORM'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # yielded  
        code2ack  = code2ack + '2C1-'
    except: 
        message2print = "{(Step 22Z3/70) yielded" + "; platform-dragon2image4xi-yielded-by-set2image4dragon: " + str(dragon2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '22Z3-' 
        
        
      
    message2print = "{(Step 23A/70) try" + "; context2dragon-try-by-set2image4dragon: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)       
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 23/70) - CONTEXT - REQUIRED
        # -----------------------------------------------------------------------------------------               
        dragon2image4xi['CONTEXT'] = task4imageset2dragon['CONTEXT']
        
        message2print = "{(Step 23B/70) set" + "; context-set-by-set2image4dragon to: " + str(dragon2image4xi['CONTEXT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'COUNT2REFERENCE': 0,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4dragon-failed ; time: 07:13:59 09/06/11 Pacific Daylight Time}',
        #                    'CLASS': 'create2context4dragon',
        #                    'METHOD': 'create2context4dragon'},
        #  'FUNCTION2CALLBACK': <function trace2context4dragon at 0x0000000003770148>,
        #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-',
        #  'INFO2PRIVATE': None,
        #  'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 'NACK2RETURN': ' ',
        #  'SIZE2PRIVATE': 0,
        #  'PROPERTIES': None}; time: 07:13:59 09/06/11 Pacific Daylight Time} 

        
        code2ack  = code2ack + '23B-'
    except: 
        message2print = "{(Step 23Z1/70) yielded" + "; context-dragon2image4xi-yielded-by-set2image4dragon: " + str(dragon2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
    
        code2nack  = code2nack + '23Z1-' 
        
    message2print = "{(Step 24A/70) try" + "; device-try-by-set2image4dragon: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 24/70) - DEVICE - EXTRA
        # -----------------------------------------------------------------------------------------               
        dragon2image4xi['DEVICE'] = task4imageset2dragon['DEVICE']
        
        message2print = "{(Step 24B/70) set" + "; device-set-by-set2image4dragon to: " + str(dragon2image4xi['DEVICE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'RESOLUTION2TIMER4PROFILING': 318,
        #  'ID2PLATFORM': '0000000001509568',
        #  'PROFILING2QUEUE': 'ACK',
        #  'EXECUTE2KERNELS4OPENCL': 'ACK',
        #  'NATIVE2WIDTHLONG4VECTOR': 2,
        #  'MAX2UNITS4COMPUTE': 4,
        #  'SIZE2MEMORY4GLOBAL': 3221225472,
        #  'NATIVE2WIDTHFLOAT4VECTOR': 4,
        #  'MAX2SIZE4WORKGROUP': 1024,
        #  'PREFERRED2WIDTHCHAR4VECTOR': 16,
        #  'SIZE2CACHE': 65536,
        #  'IEEE754-2008FUSED2MULTIPLY-ADD': 'NACK',
        #  'TYPE4CACHE': 'Read/Write',
        #  'PREFERRED2WIDTHINT4VECTOR': 4,
        #  'SIZE2PREFERREDWORKGROUPMULTIPLE4KERNEL': 1,
        #  'NATIVE2WIDTHINT4VECTOR': 4,
        #  'SIZE2BUFFER4CONSTANT': 65536,
        #  'EXECUTE2FUNCTION4NATIVE': 'ACK',
        #  'PREFERRED2WIDTHDOUBLE4VECTOR': 0,
        #  'MAX2NUMBER4CONSTANTARGS': 8,
        #  'VERSION2DRIVER': '2.0',
        #  'EXTENSIONS': 'cl_amd_fp64 cl_khr_global_int32_base_atomics ...
        #  'AVAILABLE2COMPILER': 'ACK',
        #  'MAX2DIMENSIONS4WORKITEMS': 3,
        #  'SUPPORT2CORRECTION4ERROR': 0,
        #  'BITS2ADDRESS': 64,
        #  'AVAILABLE': 'ACK',
        #  'SIZE2MEMORY4LOCAL': 32768,
        #  'VENDOR': 'AuthenticAMD',
        #  'SUPPORT2IMAGE': 'NACK',
        #  'MAX2SIZE4KERNELARGUMENT': 4096,
        #  'ACK2RETURN': ' 1C-1D-1E-1F-1D-2A-3A-4A-5A-6A-7A-8A-9A-10A-11A-...
        #  'OUT-OF-ORDER2QUEUE': 'NACK',
        #  'ALIGNMENT2ADDRESS4BASE': 1024,
        #  'PROFILE': 'FULL_PROFILE',
        #  'TYPE2MEMORY4LOCAL': 'Global',
        #  'PREFERRED2WIDTHFLOAT4VECTOR': 4,
        #  'PREFERRED2WIDTHLONG4VECTOR': 2,
        #  'MAX2FREQUENCY4CLOCK': 3210,
        #  'PREFERRED2WIDTHSHORT4VECTOR': 8,
        #  'ID': 4033,
        #  'MAX2WORKITEM2': 1024,
        #  'NATIVE2WIDTHDOUBLE4VECTOR': 0,
        #  'ROUND2ZERO': 'ACK',
        #  'NAME': 'AMD64 Family 16 Model 4 Stepping 2, AuthenticAMD',
        #  'MAX2WORKITEM1': 1024,
        #  'MAX2WORKITEM0': 1024,
        #  'SIZE2LINE4CACHE': 64,
        #  'MAX2ALLOCATION4MEMORY': 1073741824,
        #  'NATIVE2WIDTHCHAR4VECTOR': 16,
        #  'QUIET2NANS': 'ACK',
        #  'NATIVE2WIDTHSHORT4VECTOR': 8,
        #  'ROUND2NEAREST4EVEN': 'ACK',
        #  'VERSION': 'OpenCL 1.1 ATI-Stream-v2.3 (451)',
        #  'NACK2RETURN': ' ',
        #  'MEMORY2UNIFIED4HOSTANDDEVICE': 1,
        #  'ENDIANESS2DEVICE': 'Little',
        #  'MINIMUM2ALIGNMENT4DATATYPE': 128,
        #  'ROUND2+VE3INFINITY': 'ACK',
        #  'TYPE': 'CL_DEVICE_TYPE_DRAGON',
        #  'DENORMS2SINGLEPRECISION4FLOATINGPOINTCAPABILITY': 'ACK'}; time: 07:13:59 09/06/11 Pacific Daylight Time}

        
        code2ack  = code2ack + '24B-'
    except: 
        message2print = "{(Step 24Z1/70) yielded" + "; device-dragon2image4xi-yielded-by-set2image4dragon: " + str(dragon2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '24Z1-'
                 
    message2print = "{(Step 25A/70) try" + "; code2error-try-by-set2image4dragon: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print) 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 25/70) - ERROR CODE - REQUIRED
        # -----------------------------------------------------------------------------------------               
        dragon2image4xi['CODE2ERROR'] = task4imageset2dragon['CODE2ERROR']
        
        message2print = "{(Step 25B/70) set" + "; code2error-set-by-set2image4dragon to: " + str(dragon2image4xi['CODE2ERROR'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0 
        code2ack  = code2ack + '25B-'
    except: 
        message2print = "{(Step 25Z1/70) yielded" + "; code2error-dragon2image4xi-yielded-by-set2image4dragon: " + str(dragon2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '25Z1-'                
                
          
    message2print = "{(Step 26A/70) try" + "; queue-try-by-set2image4dragon: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 26/70) - QUEUE - EXTRA
        # -----------------------------------------------------------------------------------------               
        dragon2image4xi['QUEUE'] = task4imageset2dragon['QUEUE']
        
        message2print = "{(Step 26B/70) set" + "; queue-set-by-set2image4dragon to: " + str(dragon2image4xi['QUEUE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # <queue4xi.Queue4Xi object at 0x0000000003D4A710> 
        code2ack  = code2ack + '26B-'
    except: 
        message2print = "{(Step 26Z1/70) yielded" + "; queue-dragon2image4xi-yielded-by-set2image4dragon: " + str(dragon2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '26Z1-' 
                                
    message2print = "{(Step 27A/70) try" + "; pipe-try-by-set2image4dragon: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                     
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 27/70) - PIPE - EXTRA
        # -----------------------------------------------------------------------------------------               
        dragon2image4xi['PIPE'] = task4imageset2dragon['PIPE']
        
        message2print = "{(Step 27B/70) set" + "; pipe-set-by-set2image4dragon to: " + str(dragon2image4xi['PIPE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # (<read-write PipeConnection, handle 520>, <read-write PipeConnection, handle 524>)  
        
        code2ack  = code2ack + '27B-'
    except: 
        message2print = "{(Step 27Z1/70) yielded" + "; pipe-dragon2image4xi-yielded-by-set2image4dragon: " + str(dragon2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '27Z1-'   

    message2print = "{(Step 28A/70) try" + "; commandqueue-try-by-set2image4dragon: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (28) COMMAND QUEUE - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        dragon2image4xi['COMMANDQUEUE']  = task4imageset2dragon['COMMANDQUEUE']  
 
        message2print = "{(Step 28B/70) set" + "; commandqueue-set-by-set2image4dragon to: " + str(dragon2image4xi['COMMANDQUEUE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': ...
        
        code2ack  = code2ack + '28B-'
    except: 
        message2print = "{(Step 28Z1/70) yielded" + "; commandqueue-yielded-by-set2image4dragon: " + str(dragon2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '28Z1-'    

    message2print = "{(Step 29A/70) try" + "; bufferinfo-try-by-set2image4dragon: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (29) BUFFER INFO - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        dragon2image4xi['BUFFER']  = task4imageset2dragon['BUFFER']  
 
        message2print = "{(Step 29B/70) set" + "; bufferinfo-set-by-set2image4dragon to: " + str(dragon2image4xi['BUFFER'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        #  
        code2ack  = code2ack + '29B-'
    except: 
        message2print = "{(Step 29Z1/70) yielded" + "; bufferinfo-yielded-by-set2image4dragon: " + str(dragon2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded
        code2nack  = code2nack + '29Z1-'            

    message2print = "{(Step 30A/70) try" + "; eventinfo-try-by-set2image4dragon: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (30) EVENT INFO - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        dragon2image4xi['EVENT']  = task4imageset2dragon['EVENT']  
 
        message2print = "{(Step 30B/70) set" + "; eventinfo-set-by-set2image4dragon to: " + str(dragon2image4xi['EVENT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '30B-'
    except: 
        message2print = "{(Step 30Z1/70) yielded" + "; eventinfo-yielded-by-set2image4dragon: " + str(dragon2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded
        
        code2nack  = code2nack + '30Z1-'            
                

    message2print = "{(Step 31A/70) try" + "; flags-try-by-set2image4dragon: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (31) FLAGS - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        dragon2image4xi['FLAGS']  = task4imageset2dragon['FLAGS']  
 
        message2print = "{(Step 31B/70) set" + "; flags-set-by-set2image4dragon to: " + str(dragon2image4xi['FLAGS'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '31B-'
    except: 
        message2print = "{(Step 31Z1/70) yielded" + "; flags-yielded-by-set2image4dragon: " + str(dragon2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded
        
        code2nack  = code2nack + '31Z1-'     

    message2print = "{(Step 34A/70) try" + "; width-try-by-set2image4dragon: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (34) WIDTH - REQUIRED                              
        # ---------------------------------------------------------------------------------------------      
        dragon2image4xi['WIDTH']  = task4imageset2dragon['WIDTH']  
 
        message2print = "{(Step 34H/70) set" + "; width-set-by-set2image4dragon to: " + str(dragon2image4xi['WIDTH'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '34H-'
    except: 
        message2print = "{(Step 34Z1/70) yielded" + "; width-yielded-by-set2image4dragon: " + str(dragon2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '34Z1-'   


    message2print = "{(Step 35A/70) try" + "; height-try-by-set2image4dragon: None"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (35) HEIGHT - REQUIRED                              
        # ---------------------------------------------------------------------------------------------      
        dragon2image4xi['HEIGHT']  = task4imageset2dragon['HEIGHT']  
 
        message2print = "{(Step 35H/70) set" + "; height-set-by-set2image4dragon to: " + str(dragon2image4xi['HEIGHT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        #  
        code2ack  = code2ack + '35H-'
    except: 
        message2print = "{(Step 35Z1/70) yielded" + "; height-yielded-by-set2image4dragon: " + str(dragon2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded 
        code2nack  = code2nack + '35Z1-'   
 

    message2print = "{(Step 36A/70) try" + "; rowpitch-try-by-set2image4dragon: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (36) ROWPITCH - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        dragon2image4xi['ROWPITCH']  = task4imageset2dragon['ROWPITCH']  
 
        message2print = "{(Step 36B/70) set" + "; rowpitch-set-by-set2image4dragon to: " + str(dragon2image4xi['ROWPITCH'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '36B-'
    except: 
        message2print = "{(Step 36Z1/70) yielded" + "; rowpitch-yielded-by-set2image4dragon: " + str(dragon2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded 
        code2nack  = code2nack + '36Z1-'

    message2print = "{(Step 37A/70) try" + "; pointer-try-by-set2image4dragon: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (37) POINTER - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        dragon2image4xi['POINTER']  = task4imageset2dragon['POINTER']  
 
        message2print = "{(Step 37B/70) set" + "; pointer-set-by-set2image4dragon to: " + str(dragon2image4xi['POINTER'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '37B-'
    except: 
        message2print = "{(Step 37Z1/70) yielded" + "; pointer-yielded-by-set2image4dragon: " + str(dragon2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded 
        code2nack  = code2nack + '37Z1-'

    message2print = "{(Step 38A/70) try" + "; format-try-by-set2image4dragon: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (38) FORMAT - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        dragon2image4xi['FORMAT']  = task4imageset2dragon['FORMAT']  
 
        message2print = "{(Step 38B/70) set" + "; format-set-by-set2image4dragon to: " + str(dragon2image4xi['FORMAT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '38B-'
    except: 
        message2print = "{(Step 38Z1/70) yielded" + "; format-yielded-by-set2image4dragon: " + str(dragon2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded 
        code2nack  = code2nack + '38Z1-'
                        
    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    dragon2image4xi ['ACK2RETURN']  = code2ack
    dragon2image4xi ['NACK2RETURN'] = code2nack 
    
    message2print = "{(Step 70/70) returning" + "; dragon2image4xi-returning-by-set2image4dragon: " + str(dragon2image4xi) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'CODE2ERROR': 0, 'ACK2RETURN': ' 23B-24B-25B-26B-27B-28B-', 'ROWPITCH': 800, 'DATA2CALLBACK': ...
    
    
    return dragon2image4xi

           
#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "copy2image4dragon" usage = "
#                                                                                                              
#   # -----------------------------------------------------------------------------------------------
#   # (16) ESTABLISH CONTROL & COMMAND WITH COPY2IMAGE4DRAGON                      
#   # -----------------------------------------------------------------------------------------------    
#   print('(Step 16A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL & COMMAND WITH COPY2IMAGE4DRAGON ', time4xi())    
#   response4copy2image = {} 
#   task4copy2image = None
#   try:                               
#       response4copy2image = copy2image4dragon(task4copy2image)                                                                                                          
#       print('(Step 16B/700) OF DRAGON2IMAGE4XI5TEST - ESTABLISHED CONTROL & COMMAND WITH COPY2IMAGE4DRAGON : ', response4copy2image)
#       queue2copy4dragon      = response4copy2image['QUEUE']
#       channel2copypublish4dragon = response4copy2image['PUBLISHERCHANNEL']
#       channel2copyconsume4dragon = response4copy2image['CONSUMERCHANNEL']
#   except:
#       print('(Step 16Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL & COMMAND WITH COPY2IMAGE4DRAGON ', time4xi()) 
#   # -----------------------------------------------------------------------------------------------
#   # (17) SEND CONTROL & COMMAND COPY VIA PIPE FOR MANOEUVRE2IMAGE4DRAGON TO RECEIVE                 
#   # -----------------------------------------------------------------------------------------------      
#   print('(Step 17A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGE4DRAGON ', time4xi())     
#   try:                               
#       message2copyimagesend4dragon = "<copy4xi>copy2copy4dragon</copy4xi>"
#       channel2copypublish4dragon = response4copy2image['PUBLISHERCHANNEL']
#       channel2copypublish4dragon.send(message2copyimagesend4dragon)
#       print("(Step 17B/700) OF DRAGON2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4DRAGON: ", channel2copypublish4dragon, " MESSAGE2SEND4DRAGON: " , message2copyimagesend4dragon)                                                                                                
#   except:
#       print('(Step 17Z1/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGE4DRAGON ', time4xi())                                                
#   # -----------------------------------------------------------------------------------------------
#   # (18) LET MANOEUVRE2IMAGE4DRAGON TO RECEIVE COPY AND AND SEND ROGER VIA PIPE                
#   # -----------------------------------------------------------------------------------------------       
#   print("(Step 18C/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4DRAGON ", time4xi()) 
#   try:        
#       response2manoeuvre4dragon = manoeuvre2copyimage4dragon(queue2copy4dragon, channel2copyconsume4dragon, channel2copypublish4dragon)
#       print("(Step 18D/700) OF DRAGON2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4DRAGON ", manoeuvre2copyimage4dragon)
#   except:
#       print("(Step 18Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4DRAGON ", time4xi())     
#   else:
#       pass # trust activated thread    
#   # -----------------------------------------------------------------------------------------------
#   # (19) RECEIVE CONTROL & COMMAND ROGER VIA PIPE                   
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 19A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH COPY2IMAGE4DRAGON ', time4xi())     
#   try:
#       channel2copyconsume4dragon = response4copy2image['CONSUMERCHANNEL']                              
#       message2copyimagereceive4dragon = channel2copyconsume4dragon.recv()
#       print("(Step 19B/700) OF DRAGON2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4DRAGON: ", channel2copyconsume4dragon, " CONTROL & COMMAND: ", message2copyimagereceive4dragon)                                                                                                  
#       # <roger4xi>roger2copy4dragon</roger4xi>
#   except:
#       print('(Step 19Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH COPY2IMAGE4DRAGON ', time4xi())         
#   # -----------------------------------------------------------------------------------------------
#   # (20) SEND CONTROL & COMMAND READY VIA PIPE FOR MANOEUVRE2IMAGE4DRAGON TO RECEIVE                
#   # -----------------------------------------------------------------------------------------------      
#   print('(Step 20A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGE4DRAGON ', time4xi())     
#   try:                               
#       message2copyimagesend4dragon = "<copy4xi>copy2copy4dragon</copy4xi>"
#       channel2copypublish4dragon = response4copy2image['PUBLISHERCHANNEL']
#       channel2copypublish4dragon.send(message2copyimagesend4dragon)
#       print("(Step 20B/700) OF DRAGON2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4DRAGON: ", channel2copypublish4dragon, " MESSAGE2SEND4DRAGON: " , message2copyimagesend4dragon)                                                                                                
#       # <copy4xi>copy2copy4dragon</copy4xi> 
#   except:
#        print('(Step 20Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGE4DRAGON ', time4xi())                                                
#   # -----------------------------------------------------------------------------------------------
#   # (21) LET MANOEUVRE2IMAGE4DRAGON TO RECEIVE READY AND TO SEND READY                
#   # -----------------------------------------------------------------------------------------------       
#   print("(Step 21A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4DRAGON ", time4xi()) 
#   try:        
#       response2manoeuvre4dragon = manoeuvre2copyimage4dragon(queue2copy4dragon, channel2copyconsume4dragon, channel2copypublish4dragon)
#       print("(Step 21B/700) OF DRAGON2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4DRAGON ", manoeuvre2copyimage4dragon)
#   except:
#       print("(Step 21Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4DRAGON ", time4xi())     
#   # -----------------------------------------------------------------------------------------------
#   # (22) RECEIVE CONTROL & COMMAND READY VIA PIPE                      
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 22A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH COPY2IMAGE4DRAGON ', time4xi())     
#   try:
#       channel2copyconsume4dragon = response4copy2image['CONSUMERCHANNEL']                              
#       message2copyimagereceive4dragon = channel2copyconsume4dragon.recv()
#       print("(Step 22B/700) OF DRAGON2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4DRAGON: ", channel2copyconsume4dragon, " CONTROL & COMMAND: ", message2copyimagereceive4dragon)                                                                                                  
#       # <copy4xi>copy2copy4dragon</copy4xi> 
#   except:
#       print('(Step 22Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH COPY2IMAGE4DRAGON ', time4xi())          
#   # ----------------------------------------------------------------------------------------------
#   # (24) PUT IMAGE PATH TO QUEUE
#   # ----------------------------------------------------------------------------------------------                          
#   print("(Step 24A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO PUT IMAGE PATH TO QUEUE ", time4xi())      
#   try:    
#       queue2copy4dragon.put(path2dragon)             
#       print("(Step 24B/700) OF DRAGON2IMAGE4XI5TEST - PUT TO QUEUE IMAGE PATH: ", path2dragon) 
#       # ..//..//..//..//..//web//images//png//dragonxi.png  
#   except:
#       print("(Step 24Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO PUT IMAGE PATH TO QUEUE ", time4xi()) 
#   # -----------------------------------------------------------------------------------------------
#   # (25) LET MANOEUVRE2IMAGE4DRAGON TO GET FROM QUEUE AND TO SEND EITHER ACK OR NACK VIA PIPE               
#   # -----------------------------------------------------------------------------------------------       
#   print("(Step 25A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4DRAGON ", time4xi()) 
#   try:        
#       response2manoeuvre4dragon = manoeuvre2copyimage4dragon(queue2copy4dragon, channel2copyconsume4dragon, channel2copypublish4dragon)
#       print("(Step 25B/700) OF DRAGON2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4DRAGON ", manoeuvre2copyimage4dragon)
#   except:
#       print("(Step 25Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4DRAGON ", time4xi())     
#   # -----------------------------------------------------------------------------------------------
#   # (26) RECEIVE ACK OR NACK ABOUT IMAGE OPERATION VIA PIPE                
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 26A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())      
#   try: 
#       message2copyimagereceive4dragon = channel2copyconsume4dragon.recv()
#       print("(Step 26D/700) OF DRAGON2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4DRAGON: ", channel2copyconsume4dragon, " CONTROL & COMMAND: ", message2copyimagereceive4dragon)                                                                                                   
#       # <ack4xi>ack2imageinfo4dragon</ack4xi>
#   except:
#       print('(Step 26Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())               
#   # -----------------------------------------------------------------------------------------------
#   (27) LET MANOEUVRE2IMAGE4DRAGON GO TO STATE 0               
#   # -----------------------------------------------------------------------------------------------       
#   print("(Step 27A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4DRAGON ", time4xi()) 
#   try:        
#       response2manoeuvre4dragon = manoeuvre2copyimage4dragon(queue2copy4dragon, channel2copyconsume4dragon, channel2copypublish4dragon)
#       print("(Step 27B/700) OF DRAGON2IMAGE4XI5TEST - GOT RESPONSE2MANOEUVRE4DRAGON ", response2manoeuvre4dragon)
#   except:
#       print("(Step 27Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4DRAGON ", time4xi())   

#">

# -------------------------------------------------------------------------------------------------   
def copy2image4dragon(task4imagecopy2dragon=None):

    '''
    copies 2D images for DRAGON
    '''
    global dragon2image4xi                                                                                             # 2D-IMAGES FOR DRAGON  
    global control2copyimage4dragon                                                                                    # COMMAND AND CONTROL FOR DRAGON IMAGES 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "copy2image4dragon"
    method2print = "copy2image4dragon"

    message2print = "{(Step 0A/33) started" + "; copy2image4dragon-started: " + str(copy2image4dragon)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                               
    # <function copy2image4dragon at 0x00000000038538C8>
    
    message2print = "{(Step 0B/33) passed" + "; task4imagecopy2dragon-to-copy2image4dragon: " + str(task4imagecopy2dragon) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None
            
    code2ack  = ' '
    code2nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # CREATE DRAGON2IMAGE4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if dragon2image4xi is None:
            dragon2image4xi = create2image4dragon()                                                                       # INITIALIZE IMAGE FOR DRAGON 
            message2print = "{(Step 0C/70) created" + "; dragon2image4xi-created-by-copy2image4dragon to: " + str(dragon2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        else:
            message2print = "{(Step 0D/70) skipped" + "; create2image4dragon-skipped-by-copy2image4dragon: " + str(copy2image4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # <function copy2image4dragon at 0x00000000038538C8>                                 
    except:
        try:
            dragon2image4xi = create2image4dragon()
            message2print = "{(Step 0E/70) set" + "; dragon2image4xi-set-by-copy2image4dragon to: " + str(dragon2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            dragon2image4xi = {}
            message2print = "{(Step 0Z/70) yielded" + "; dragon2image4xi-yielded-by-copy2image4dragon: " + str(dragon2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '0Z-' 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    try:             
        if task4imagecopy2dragon is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER FOR COPY - BUILD TASK! 
            # -----------------------------------------------------------------------------------------    
            
            task4imagecopy2dragon = {}                                                                                                                                                                                       
            
            message2print = "{(Step 1A/70) initialized" + "; task4imagecopy2dragon-created-by-copy2image4dragon to: " + str(task4imagecopy2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {} 
            
            # ---------------------------------------------------------------------------------------------
            # (1B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4dragon = trace2image4dragon                                                                 # CALLBACK FUNCTION FOR DRAGON
            
            message2print = "{(Step 1B1/70) initialized" + "; function2callback4dragon-created-by-copy2image4dragon to: " + str(function2callback4dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # <function trace2image4dragon at 0x000000000384E9C8> 
            
            code2ack  = code2ack + '1B1-'
           
            args2callback = {}                                                                                      # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "copy2image4dragon"                                                            # MODULE OF DRAGONIZED OPENCL MODULE
            args2callback['METHOD']  = "copy2image4dragon"                                                            # FUNCTION OF DRAGONIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; copy2image4dragon-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR DRAGONIZED OPENCL TO RETURN    

            message2print = "{(Step 1B2/70) initialized" + "; args2callback-created-by-copy2image4dragon to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {'MESSAGE': '{(Step 1Z1/33) failed; copy2image4dragon-failed ; time: 07:01:44 10/04/11 Pacific Daylight Time}',
            #  'CLASS': 'copy2image4dragon',
            #  'METHOD': 'copy2image4dragon'}; time: 07:01:44 10/04/11 Pacific Daylight Time}


            code2ack  = code2ack + '1B2-'
            
            # ---------------------------------------------------------------------------------------------
            # (1C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imagecopy2dragon['DATA2CALLBACK'] = args2callback                                                    # USER DATA PASSED TO DRAGONIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagecopy2dragon['FUNCTION2CALLBACK'] = function2callback4dragon                                        # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '1C-'
            message2print = "{(Step 1C/70) initialized" + "; task4imagecopy2dragon-created-by-copy2image4dragon to: " + str(task4imagecopy2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # {'FUNCTION2CALLBACK': <function trace2image4dragon at 0x000000000384E9C8>,
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; copy2image4dragon-failed ; time: 07:01:44 10/04/11 Pacific Daylight Time}',
            #  'CLASS': 'copy2image4dragon',
            #  'METHOD': 'copy2image4dragon'}}; time: 07:01:44 10/04/11 Pacific Daylight Time}

            
            # ---------------------------------------------------------------------------------------------
            # (2) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-copy2image4dragon to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # {}
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-copy2image4dragon: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}; time: 07:01:44 10/04/11 Pacific Daylight Time}
                 
                code2ack  = code2ack + '2B-'
            except:
                message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-copy2image4dragon: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '2Z-'
                 
            task4imagecopy2dragon['PLATFORM'] = platform2xi                                                           # PLATFORM OF SYSTEM 
              
            
            # ---------------------------------------------------------------------------------------------
            # (3) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2dragon = {}
            message2print = "{(Step 3A/70) trying" + "; context2dragon-trying-by-copy2image4dragon: " + str(context2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2dragon = get2context4dragon()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 3B/70) got" + "; context2dragon-got-by-copy2image4dragon: " + str(context2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) ...
                
                code2ack  = code2ack + '3B-'         
            except:
                message2print = "{(Step 3Z/70) failed" + "; context2dragon-failed-by-copy2image4dragon: " + str(context2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '3Z-'
                
            task4imagecopy2dragon ['CONTEXT'] = context2dragon           
            
            # ---------------------------------------------------------------------------------------------
            # (4) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2dragon = {}
            
            message2print = "{(Step 4A/70) trying" + "; device2dragon-trying-by-copy2image4dragon: " + str(device2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2dragon = get2dragon4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 4B/70) got" + "; device2dragon-got-by-copy2image4dragon: " + str(device2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568', 'PROFILING2QUEUE': 'ACK', ...
                
                code2ack  = code2ack + '4B-'  
            except:
                message2print = "{(Step 4Z/70) failed" + "; device2dragon-failed-by-copy2image4dragon: " + str(device2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '4Z-'
            
            task4imagecopy2dragon ['DEVICE'] = device2dragon
            
                   
            # ---------------------------------------------------------------------------------------------
            # (5) IMAGE SOURCE OBJECT - REQUIRED                         
            # ---------------------------------------------------------------------------------------------         
            imagesourceinfo2dragon = {}
             
            message2print = "{(Step 5A/70) trying" + "; imagesourceinfo2dragon-trying-by-copy2image4dragon: " + str(imagesourceinfo2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imagesourceinfo2dragon = get2imageinfo4dragon()                                                                         
                
                message2print = "{(Step 5B/70) got" + "; imagesourceinfo2dragon-got-by-copy2image4dragon: " + str(imagesourceinfo2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'SIZE2QUEUE': 2147483646, 
                #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
                #  'ORDER': 'CL_RGBA'}, 
                #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004472BE0>, 
                #  'WIDTH': 800, 
                #  'CONTEXT': {'COUNT2REFERENCE': 0, ...
                
                code2ack  = code2ack + '5B-'  
            except:
                message2print = "{(Step 5Z/70) failed" + "; imagesourceinfo2dragon-failed-by-copy2image4dragon: " + str(imagesourceinfo2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '5Z-'          
                      
            task4imagecopy2dragon ['IMAGESOURCE'] = imagesourceinfo2dragon         
                   
            # ---------------------------------------------------------------------------------------------
            # (6) IMAGE DESTINATION OBJECT - REQUIRED                        
            # ---------------------------------------------------------------------------------------------                         
            imagedestinationinfo2dragon = {}
             
            message2print = "{(Step 6A/70) trying" + "; imagedestinationinfo2dragon-trying-by-copy2image4dragon: " + str(imagedestinationinfo2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imagedestinationinfo2dragon = get2imageinfo4dragon()                                                                         
                
                message2print = "{(Step 6B/70) got" + "; imagedestinationinfo2dragon-got-by-copy2image4dragon: " + str(imagedestinationinfo2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'SIZE2QUEUE': 2147483646, 
                #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
                #  'ORDER': 'CL_RGBA'}, 
                #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004472BE0>, 
                #  'WIDTH': 800, 
                #  'CONTEXT': {'COUNT2REFERENCE': 0, ...
                # note: source and destination will be the same !             
                code2ack  = code2ack + '6B-'  
            except:
                message2print = "{(Step 6Z/70) failed" + "; imagedestinationinfo2dragon-failed-by-copy2image4dragon: " + str(imagedestinationinfo2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '6Z-'          
                      
            task4imagecopy2dragon ['IMAGEDESTINATION'] = imagedestinationinfo2dragon     
            
            # ---------------------------------------------------------------------------------------------
            # (7) QUEUE - REQUIRED BY DRAGONXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2dragon = None
            
            message2print = "{(Step 7A/70) trying" + "; queue2dragon-trying-by-copy2image4dragon: " + str(queue2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2dragon = Queue4Xi()                                                                              # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 7B/70) got" + "; queue2dragon-got-by-copy2image4dragon: " + str(queue2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # <queue4xi.Queue4Xi object at 0x000000000453EB38>
                
                code2ack  = code2ack + '7B-'  
            except:
                message2print = "{(Step 7Z/70) failed" + "; queue2dragon-failed-by-copy2image4dragon: " + str(queue2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '7Z-'
                
            task4imagecopy2dragon ['QUEUE'] = queue2dragon

            
            # ---------------------------------------------------------------------------------------------
            # (8) PIPE - REQUIRED BY DRAGONXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2dragon = None
           
            message2print = "{(Step 8A/70) trying" + "; pipe2dragon-trying-by-copy2image4dragon: " + str(pipe2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2dragon = Pipe4Xi()                                                                                # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 8B/70) got" + "; pipe2dragon-got-by-copy2image4dragon: " + str(pipe2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # (<read-write PipeConnection, handle 612>, <read-write PipeConnection, handle 616>)
                
                code2ack  = code2ack + '8B-'  
            except:
                message2print = "{(Step 8Z/70) failed" + "; pipe2dragon-failed-by-copy2image4dragon: " + str(pipe2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '8Z-'
                
            task4imagecopy2dragon ['PIPE'] = pipe2dragon 
            
            # ---------------------------------------------------------------------------------------------
            # (9) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2dragon = {}
           
            message2print = "{(Step 9A/70) trying" + "; commandqueueinfo2dragon-trying-by-copy2image4dragon: " + str(commandqueueinfo2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2dragon = get2commandqueueinfo4dragon()                                                   # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 9B/70) got" + "; commandqueueinfo2dragon-got-by-copy2image4dragon: " + str(commandqueueinfo2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 
                #  'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4dragon-failed ; time: 07:01:44 10/04/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2commandqueue4dragon', 
                #  'METHOD': 'create2commandqueue4dragon'}, 
                #  'FUNCTION2CALLBACK': <function trace2commandqueue4dragon at 0x00000000037EDA48> ...
                
                code2ack  = code2ack + '9B-'  
            except:
                message2print = "{(Step 9Z/70) failed" + "; commandqueueinfo2dragon-failed-by-copy2image4dragon: " + str(commandqueueinfo2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '9Z-'
        
            task4imagecopy2dragon ['COMMANDQUEUE'] = commandqueueinfo2dragon
            
            
            
            # ---------------------------------------------------------------------------------------------
            # (10) BUFFER - AUTOMATIC BY DRAGONXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2dragon = {}
           
            message2print = "{(Step 10A/70) trying" + "; bufferinfo2dragon-trying-by-copy2image4dragon: " + str(bufferinfo2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2dragon = get2bufferinfo4dragon()                                                               # SYSTEM CREATES BUFFER FOR DRAGON, IF NECESSARY
                message2print = "{(Step 10B/70) got" + "; bufferinfo2dragon-got-by-copy2image4dragon: " + str(bufferinfo2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE' ...
                
                code2ack  = code2ack + '10B-' 
            except:
                message2print = "{(Step 10Z/70) failed" + "; bufferinfo2dragon-failed-by-copy2image4dragon: " + str(bufferinfo2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '10Z-'
        
            task4imagecopy2dragon ['BUFFER'] = bufferinfo2dragon 
            
            # ---------------------------------------------------------------------------------------------
            # (11) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2dragon = {}
           
            message2print = "{(Step 11A/70) trying" + "; eventinfo2dragon-trying-by-copy2image4dragon: " + str(eventinfo2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2dragon = get2eventinfo4dragon()                                                                    
                message2print = "{(Step 11B/70) got" + "; eventinfo2dragon-got-by-copy2image4dragon: " + str(eventinfo2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0,
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2event4dragon-failed ; time: 07:01:44 10/04/11 Pacific Daylight Time}',
                #  'CLASS': 'create2event4dragon',
                #  'METHOD': 'create2event4dragon'},
                #  'FUNCTION2CALLBACK': <function trace2event4dragon at 0x0000000003803F48>,
                #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-6B-7B-8B-9B-10B-11B-12B-14B-15B-16B-',
                #  'QUEUE': <queue4xi.Queue4Xi object at 0x000000000423B668>,
                #  'NACK2RETURN': ' ',
                #  'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': ...
                
                code2ack  = code2ack + '11B-' 
            except:
                message2print = "{(Step 11Z/70) failed" + "; eventinfo2dragon-failed-by-copy2image4dragon: " + str(eventinfo2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '11Z-'
        
            task4imagecopy2dragon ['EVENT'] = eventinfo2dragon 
            
            
            # ---------------------------------------------------------------------------------------------
            # (12) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2dragon = {}
           
            message2print = "{(Step 12A/70) trying" + "; eventwaitlist2dragon-trying-by-copy2image4dragon: " + str(eventwaitlist2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2dragon = {}                                                                   
                message2print = "{(Step 12B/70) set" + "; eventwaitlist2dragon-set-by-copy2image4dragon: " + str(eventwaitlist2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {} 
                
                code2ack  = code2ack + '12B-' 
            except:
                message2print = "{(Step 12Z/70) failed" + "; eventwaitlist2dragon-failed-by-copy2image4dragon: " + str(eventwaitlist2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '12Z-'
        
            task4imagecopy2dragon ['EVENTWAITLIST'] = eventwaitlist2dragon                     

            
            # ---------------------------------------------------------------------------------------------
            # (13) EVENTWAITLISTQUANTITY - REQUIRED                               
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2dragon = 0
            
            message2print = "{(Step 13A/70) trying" + "; eventwaitlistquantity2dragon-trying-by-copy2image4dragon: " + str(eventwaitlistquantity2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2dragon.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2dragon = eventwaitlistquantity2dragon + 1    
                
                message2print = "{(Step 13B/70) set" + "; eventwaitlistquantity2dragon-set-by-copy2image4dragon: " + str(eventwaitlistquantity2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0  
                
                code2ack  = code2ack + '13B-' 
            except:
                message2print = "{(Step 13Z/70) failed" + "; eventwaitlistquantity2dragon-failed-by-copy2image4dragon: " + str(eventwaitlistquantity2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '13Z-'
        
            task4imagecopy2dragon ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2dragon       
                                                 
            
            # ---------------------------------------------------------------------------------------------
            # (14) ORIGIN FOR SOURCE - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin4source2dragon = {}
            
            message2print = "{(Step 14A/70) trying" + "; origin4source2dragon-trying-by-copy2image4dragon: " + str(origin4source2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin4source2dragon['X'] = 0 
                origin4source2dragon['Y'] = 0  
                origin4source2dragon['Z'] = 0                                                                                                                          
                message2print = "{(Step 14B/70) set" + "; origin4source2dragon-set-by-copy2image4dragon to: " + str(origin4source2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '14B-' 
            except:
                message2print = "{(Step 14Z/70) failed" + "; origin4source2dragon-failed-by-copy2image4dragon: " + str(origin4source2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '14Z-'
        
            task4imagecopy2dragon ['SOURCEORIGIN'] = origin4source2dragon
            
            # ---------------------------------------------------------------------------------------------
            # (15) ORIGIN FOR DESTINATION - REQUIRED                             
            # ---------------------------------------------------------------------------------------------              
            origin4destination2dragon = {}
            
            message2print = "{(Step 15A/70) trying" + "; origin4destination2dragon-trying-by-copy2image4dragon: " + str(origin4destination2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin4destination2dragon['X'] = 100 
                origin4destination2dragon['Y'] = 100  
                origin4destination2dragon['Z'] = 100                                                                                                                          
                message2print = "{(Step 15B/70) set" + "; origin4destination2dragon-set-by-copy2image4dragon to: " + str(origin4destination2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '15B-' 
            except:
                message2print = "{(Step 15Z/70) failed" + "; origin4destination2dragon-failed-by-copy2image4dragon: " + str(origin4destination2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '15Z-'
        
            task4imagecopy2dragon ['DESTINATIONORIGIN'] = origin4destination2dragon
                        
            
            
            # --------------------------------------------------------------------------------------------------------
            # (16) REGION - REGION OF A IMAGE TO BE COPIED                         
            # --------------------------------------------------------------------------------------------------------             
            region4copy2dragon = {}                                                                                                         
           
            message2print = "{(Step 16A/70) trying" + "; region4copy2dragon-trying-by-copy2image4dragon: " + str(region4copy2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                region4copy2dragon['X'] = 10 
                region4copy2dragon['Y'] = 10  
                region4copy2dragon['Z'] = 10                                                                           
                message2print = "{(Step 16B/70) set" + "; region4copy2dragon-set-by-copy2image4dragon to: " + str(region4copy2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 

                code2ack  = code2ack + '16B-'                
                 
            except:
                message2print = "{(Step 16Z/70) failed" + "; region4copy2dragon-failed-by-copy2image4dragon: " + str(region4copy2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '16Z-'
            
            task4imagecopy2dragon ['REGION'] = region4copy2dragon              
            
            # ---------------------------------------------------------------------------------------------
            # EOF PARAMETER PROVIDING                
            # ---------------------------------------------------------------------------------------------                                                                                                                               
        else:
            pass
            
        # ---------------------------------------------------------------------------------------------
        # CHECK PARAMETERS - WHETHER AUTOMATICALLY GENERATED OR USER GIVEN 
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imagecopy2dragon.keys()
        message2print = "{(Step 20/70) using" + "; keys2task-using-by-copy2image4dragon: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # ['BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DESTINATIONORIGIN',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGEDESTINATION',
        #  'IMAGESOURCE',
        #  'PIPE',
        #  'PLATFORM',
        #  'QUEUE',
        #  'REGION',
        #  'SOURCEORIGIN']; time: 01:21:58 10/05/11 Pacific Daylight Time}
 
        if (('DATA2CALLBACK' in keys2task) and ('FUNCTION2CALLBACK' in keys2task)):
            pass
        else:                
            # ---------------------------------------------------------------------------------------------
            # (21B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4dragon = trace2image4dragon                                                            # CALLBACK FUNCTION FOR DRAGON
            
            message2print = "{(Step 21B1/70) initialized" + "; function2callback4dragon-created-by-copy2image4dragon to: " + str(function2callback4dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code2ack  = code2ack + '21B1-'
           
            args2callback = {}                                                                                 # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "copy2image4dragon"                                                        # MODULE OF DRAGONIZED OPENCL MODULE
            args2callback['METHOD']  = "copy2image4dragon"                                                        # FUNCTION OF DRAGONIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; copy2image4dragon-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR DRAGONIZED OPENCL TO RETURN    
            
            message2print = "{(Step 21B2/70) initialized" + "; args2callback-created-by-copy2image4dragon to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 
    
            code2ack  = code2ack + '21B2-'
            
            # ---------------------------------------------------------------------------------------------
            # (21C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imagecopy2dragon['DATA2CALLBACK'] = args2callback                                                # USER DATA PASSED TO DRAGONIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagecopy2dragon['FUNCTION2CALLBACK'] = function2callback4dragon                                    # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
                
            code2ack  = code2ack + '21C-'
            message2print = "{(Step 21C/70) initialized" + "; task4imagecopy2dragon-created-by-copy2image4dragon to: " + str(task4imagecopy2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #
                 
        if 'PLATFORM' in keys2task:
            pass
        else:  
            # ---------------------------------------------------------------------------------------------
            # (22) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 22A/70) initialized" + "; platform2xi-created-by-copy2image4dragon to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
                
            try:
                platform2xi = get2platformids4xi()                                                              # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 22B/70) got" + "; platform2xi-got-by-copy2image4dragon: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                 
                code2ack  = code2ack + '22B-'
            except:
                message2print = "{(Step 22Z/70) failed" + "; platform2xi-failed-by-copy2image4dragon: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '22Z-'
                 
            task4imagecopy2dragon['PLATFORM'] = platform2xi                                                       # PLATFORM OF SYSTEM 
          
        if 'CONTEXT' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (23) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2dragon = {}
            message2print = "{(Step 23A/70) trying" + "; context2dragon-trying-by-copy2image4dragon: " + str(context2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2dragon = get2context4dragon()                                                                 # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 23B/70) got" + "; context2dragon-got-by-copy2image4dragon: " + str(context2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '23B-'         
            except:
                message2print = "{(Step 23Z/70) failed" + "; context2dragon-failed-by-copy2image4dragon: " + str(context2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '23Z-'
                
            task4imagecopy2dragon ['CONTEXT'] = context2dragon           
        
        if 'DEVICE' in keys2task:
            pass
        else:                   
            # ---------------------------------------------------------------------------------------------
            # (24) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2dragon = {}
            
            message2print = "{(Step 24A/70) trying" + "; device2dragon-trying-by-copy2image4dragon: " + str(device2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2dragon = get2dragon4xi()                                                                       # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 24B/70) got" + "; device2dragon-got-by-copy2image4dragon: " + str(device2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '24B-'  
            except:
                message2print = "{(Step 24Z/70) failed" + "; device2dragon-failed-by-copy2image4dragon: " + str(device2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '24Z-'
            
            task4imagecopy2dragon ['DEVICE'] = device2dragon
        
        if 'IMAGESOURCE' in keys2task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (25) SOURCE IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            sourceimageinfo2dragon = {}
             
            message2print = "{(Step 25A/70) trying" + "; sourceimageinfo2dragon-trying-by-copy2image4dragon: " + str(sourceimageinfo2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                sourceimageinfo2dragon = get2imageinfo4dragon()                                                                         
                
                message2print = "{(Step 25B/70) got" + "; sourceimageinfo2dragon-got-by-copy2image4dragon: " + str(sourceimageinfo2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code2ack  = code2ack + '25B-'  
            except:
                message2print = "{(Step 25Z/70) failed" + "; sourceimageinfo2dragon-failed-by-copy2image4dragon: " + str(sourceimageinfo2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '25Z-'          
                      
            task4imagecopy2dragon ['IMAGESOURCE'] = sourceimageinfo2dragon         
        
        if 'IMAGEDESTINATION' in keys2task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (26) DESTINATION IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            destinationimageinfo2dragon = {}
             
            message2print = "{(Step 26A/70) trying" + "; destinationimageinfo2dragon-trying-by-copy2image4dragon: " + str(destinationimageinfo2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                destinationimageinfo2dragon = get2imageinfo4dragon()                                                                         
                
                message2print = "{(Step 26B/70) got" + "; destinationimageinfo2dragon-got-by-copy2image4dragon: " + str(destinationimageinfo2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code2ack  = code2ack + '26B-'  
            except:
                message2print = "{(Step 26Z/70) failed" + "; destinationimageinfo2dragon-failed-by-copy2image4dragon: " + str(destinationimageinfo2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '26Z-'          
                      
            task4imagecopy2dragon ['IMAGEDESTINATION'] = destinationimageinfo2dragon    
                                    
        if 'QUEUE' in keys2task:
            pass
        else:   
            # ---------------------------------------------------------------------------------------------
            # (27) QUEUE - REQUIRED BY DRAGONXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2dragon = None
            
            message2print = "{(Step 27A/70) trying" + "; queue2dragon-trying-by-copy2image4dragon: " + str(queue2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2dragon = Queue4Xi()                                                                          # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 27B/70) got" + "; queue2dragon-got-by-copy2image4dragon: " + str(queue2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '27B-'  
            except:
                message2print = "{(Step 27Z/70) failed" + "; queue2dragon-failed-by-copy2image4dragon: " + str(queue2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '27Z-'
                
            task4imagecopy2dragon ['QUEUE'] = queue2dragon

        if 'PIPE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) PIPE - REQUIRED BY DRAGONXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2dragon = None
           
            message2print = "{(Step 28A/70) trying" + "; pipe2dragon-trying-by-copy2image4dragon: " + str(pipe2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2dragon = Pipe4Xi()                                                                            # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 28B/70) got" + "; pipe2dragon-got-by-copy2image4dragon: " + str(pipe2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '28B-'  
            except:
                message2print = "{(Step 28Z/70) failed" + "; pipe2dragon-failed-by-copy2image4dragon: " + str(pipe2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '28Z-'
                
            task4imagecopy2dragon ['PIPE'] = pipe2dragon 
            
        if 'COMMANDQUEUE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2dragon = {}
           
            message2print = "{(Step 29A/70) trying" + "; commandqueueinfo2dragon-trying-by-copy2image4dragon: " + str(commandqueueinfo2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2dragon = get2commandqueueinfo4dragon()                                               # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 29B/70) got" + "; commandqueueinfo2dragon-got-by-copy2image4dragon: " + str(commandqueueinfo2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '29B-'  
            except:
                message2print = "{(Step 29Z/70) failed" + "; commandqueueinfo2dragon-failed-by-copy2image4dragon: " + str(commandqueueinfo2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '29Z-'
        
            task4imagecopy2dragon ['COMMANDQUEUE'] = commandqueueinfo2dragon
        
        
        if 'BUFFER' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (30) BUFFER - AUTOMATIC BY DRAGONXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2dragon = {}
           
            message2print = "{(Step 30A/70) trying" + "; bufferinfo2dragon-trying-by-copy2image4dragon: " + str(bufferinfo2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2dragon = get2bufferinfo4dragon()                                                           # SYSTEM CREATES BUFFER FOR DRAGON, IF NECESSARY
                message2print = "{(Step 30B/70) got" + "; bufferinfo2dragon-got-by-copy2image4dragon: " + str(bufferinfo2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '30B-' 
            except:
                message2print = "{(Step 30Z/70) failed" + "; bufferinfo2dragon-failed-by-copy2image4dragon: " + str(bufferinfo2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '30Z-'
        
            task4imagecopy2dragon ['BUFFER'] = bufferinfo2dragon
             
        if 'EVENT' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (31) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2dragon = {}
           
            message2print = "{(Step 31A/70) trying" + "; eventinfo2dragon-trying-by-copy2image4dragon: " + str(eventinfo2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2dragon = get2eventinfo4dragon()                                                                    
                message2print = "{(Step 31B/70) got" + "; eventinfo2dragon-got-by-copy2image4dragon: " + str(eventinfo2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '31B-' 
            except:
                message2print = "{(Step 31Z/70) failed" + "; eventinfo2dragon-failed-by-copy2image4dragon: " + str(eventinfo2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '31Z-'
        
            task4imagecopy2dragon ['EVENT'] = eventinfo2dragon 
        
             
        if 'EVENTWAITLIST' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (32) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2dragon = {}
           
            message2print = "{(Step 32A/70) trying" + "; eventwaitlist2dragon-trying-by-copy2image4dragon: " + str(eventwaitlist2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2dragon = {}                                                                   
                message2print = "{(Step 32B/70) set" + "; eventwaitlist2dragon-set-by-copy2image4dragon: " + str(eventwaitlist2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '32B-' 
            except:
                message2print = "{(Step 32Z/70) failed" + "; eventwaitlist2dragon-failed-by-copy2image4dragon: " + str(eventwaitlist2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '32Z-'
        
            task4imagecopy2dragon ['EVENTWAITLIST'] = eventwaitlist2dragon                     
              
        if 'EVENTWAITLISTQUANTITY' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (33) EVENTWAITLISTQUANTITY - REQUIRED                              
            # ---------------------------------------------------------------------------------------------
            eventwaitlistquantity2dragon = 0
            
            message2print = "{(Step 33A/70) trying" + "; eventwaitlistquantity2dragon-trying-by-copy2image4dragon: " + str(eventwaitlistquantity2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2dragon.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2dragon = eventwaitlistquantity2dragon + 1    
                
                message2print = "{(Step 33B/70) set" + "; eventwaitlistquantity2dragon-set-by-copy2image4dragon: " + str(eventwaitlistquantity2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 
                
                code2ack  = code2ack + '33B-' 
            except:
                message2print = "{(Step 33Z/70) failed" + "; eventwaitlistquantity2dragon-failed-by-copy2image4dragon: " + str(eventwaitlistquantity2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '33Z-'
                    
            task4imagecopy2dragon ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2dragon       
                                             
                    
        if 'SOURCEORIGIN' in keys2task:
            pass
        else:                    
            # ---------------------------------------------------------------------------------------------
            # (34) SOURCEORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            source4origin2dragon = {}
            
            message2print = "{(Step 34A/70) trying" + "; source4origin2dragon-trying-by-copy2image4dragon: " + str(source4origin2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                source4origin2dragon['X'] = 0 
                source4origin2dragon['Y'] = 0  
                source4origin2dragon['Z'] = 0                                                                                                                          
                message2print = "{(Step 34B/70) set" + "; source4origin2dragon-set-by-copy2image4dragon to: " + str(source4origin2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '34B-' 
            except:
                message2print = "{(Step 34Z/70) failed" + "; source4origin2dragon-failed-by-copy2image4dragon: " + str(source4origin2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '34Z-'
        
            task4imagecopy2dragon ['SOURCEORIGIN'] = source4origin2dragon
             
        if 'DESTINATIONORIGIN' in keys2task:
            pass
        else:                    
            # ---------------------------------------------------------------------------------------------
            # (35) DESTINATIONORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            destination4origin2dragon = {}
            
            message2print = "{(Step 35A/70) trying" + "; destination4origin2dragon-trying-by-copy2image4dragon: " + str(destination4origin2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                destination4origin2dragon['X'] = 100 
                destination4origin2dragon['Y'] = 100  
                destination4origin2dragon['Z'] = 100                                                                                                                          
                message2print = "{(Step 35B/70) set" + "; destination4origin2dragon-set-by-copy2image4dragon to: " + str(destination4origin2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '35B-' 
            except:
                message2print = "{(Step 35Z/70) failed" + "; destination4origin2dragon-failed-by-copy2image4dragon: " + str(destination4origin2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '35Z-'
        
            task4imagecopy2dragon ['DESTINATIONSOURCEORIGIN'] = destination4origin2dragon                
             
        if 'REGION' in keys2task:
            pass
        else:            
            # ---------------------------------------------------------------------------------------------
            # (36) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2dragon = {}
            
            message2print = "{(Step 36A/70) trying" + "; region2dragon-trying-by-copy2image4dragon: " + str(region2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                region2dragon['X'] = 160                                                                               # WIDTH  IN PIXELS TO WRITE 
                region2dragon['Y'] = 90                                                                                # HEIGHT IN PIXELS TO WRITE
                region2dragon['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO WRITE                                                                                                                          
                message2print = "{(Step 36B/70) set" + "; region2dragon-set-by-copy2image4dragon to: " + str(region2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '36B-' 
            except:
                message2print = "{(Step 36Z/70) failed" + "; region2dragon-failed-by-copy2image4dragon: " + str(region2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '36Z-'
        
            task4imagecopy2dragon ['REGION'] = region2dragon
        
        # ---------------------------------------------------------------------------------------------
        #  EOF PARAMETER CHECKING AND PROVIDING FOR MISSING ONES
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imagecopy2dragon.keys()
        message2print = "{(Step 39/70) set" + "; keys2task-set-by-copy2image4dragon to: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DESTINATIONORIGIN',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGEDESTINATION',
        #  'IMAGESOURCE',
        #  'PIPE',
        #  'PLATFORM',
        #  'QUEUE',
        #  'REGION',
        #  'SOURCEORIGIN']; time: 01:28:44 10/05/11 Pacific Daylight Time}
    except: 
        message2print = "{(Step 39Z/70) yielded" + "; task4imagecopy2dragon-yielded-by-copy2image4dragon: " + str(task4imagecopy2dragon) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '39Z-' 

    # ---------------------------------------------------------------------------------------------
    # CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    control2copyimage4dragon = {}       
                                
    # ---------------------------------------------------------------------------------------------
    # CONSUMER AND PUBLISHER CHANNEL FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4channels2dragon = False
    channels2dragon = ()
    message2print = "{(Step 40A/70) trying" + "channels2dragon-trying-by-copy2image4dragon: " + str(channels2dragon)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        # 
        consumerchannel2dragon, publisherchannel2dragon = task4imagecopy2dragon ['PIPE']

        message2print = "{(Step 40B/70) got" + "; consumerchannel2dragon-got-by-copy2image4dragon: " + str(consumerchannel2dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 612> 

        message2print = "{(Step 40C/70) got" + "; publisherchannel2dragon-got-by-copy2image4dragon: " + str(publisherchannel2dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 616>
                              
        ack4channels2dragon = True
        
        control2copyimage4dragon['CONSUMERCHANNEL']  = consumerchannel2dragon
        control2copyimage4dragon['PUBLISHERCHANNEL'] = publisherchannel2dragon
        
        code2ack  = code2ack + '40D-' 
        
    except:
        ack4channels2dragon = False
        code2nack  = code2nack + '40Z-'      
                        
    # ---------------------------------------------------------------------------------------------
    # QUEUE FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4queue2dragon = False
    queue2dragon = None
    message2print = "{(Step 41A/70) trying" + "queue2dragon-trying-by-copy2image4dragon: " + str(queue2dragon)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        queue2dragon = task4imagecopy2dragon ['QUEUE']

        message2print = "{(Step 41B/70) got" + "; queue2dragon-got-by-copy2image4dragon: " + str(queue2dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <queue4xi.Queue4Xi object at 0x0000000004721CF8>
              
        ack4queue2dragon = True
        control2copyimage4dragon['QUEUE']  = queue2dragon        
        code2ack  = code2ack + '41C-'
    except:
        ack4queue2dragon = False  
        code2nack  = code2nack + '41Z-'
    
    
    # -----------------------------------------------------------------------------------------    
    # DUMMY PROCESS OBJECT = THREAD CREATION
    # -----------------------------------------------------------------------------------------           
    # dummy2image4dragon = None
    # message2print = "{(Step 69A/70) trying" + "dummy2image4dragon-trying-by-copy2image4dragon: " + str(dummy2image4dragon)  + "; time: " + time4xi() + "}"   
    # print4xi(class2print, method2print, message2print)     
    # try:                            
        # dummy2image4dragon = DummyProcess4Xi(target=manoeuvre2copyimage4dragon,                                               # in this module to handle queue and pipe channels
        #                                   args =(queue2dragon,                                                         # caller puts and gets - does NOT call any functions
        #                                          consumerchannel2dragon, publisherchannel2dragon))                        # caller sends and receives - does NOT call any functions
        # message2print = "{(Step 69B/70) created" + "dummy2image4dragon-created-by-copy2image4dragon: " + str(dummy2image4dragon)  + "; time: " + time4xi() + "}" 
        # print4xi(class2print, method2print, message2print)  
        # code2ack  = code2ack + '69B-'
        
    # except:
        # code2nack  = code2nack + '69Z-'
        
    # ---------------------------------------------------------------------------------------------
    # RETURN CONTROL AND COMMAND DICTIONARY
    # ---------------------------------------------------------------------------------------------          
    message2print = "{(Step 70A/70) set" + "; code2ack-set-by-copy2image4dragon: " + str(code2ack) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                             
    # 1B1-1B2-1C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-12B-13B-14B-15B-16B-40D-41C- 
    message2print = "{(Step 70B/70) set" + "; code2nack-set-by-copy2image4dragon: " + str(code2nack) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #  
        
    message2print = "{(Step 70C/70) returning" + "; control2copyimage4dragon-returning-by-copy2image4dragon: " + str(control2copyimage4dragon) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004721CF8>,
    #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 616>,
    #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 612>}; time: 01:28:44 10/05/11 Pacific Daylight Time}
   
    return control2copyimage4dragon
                    
#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "copy2imagetobuffer4dragon" usage = "
#                                                                                                              
#   # ---------------------------------------------------------------------------------------------
#   # (60) ESTABLISH CONTROL & COMMAND WITH COPY2IMAGETOBUFFER4DRAGON                      
#   # ---------------------------------------------------------------------------------------------    
#   print('(Step 60A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL & COMMAND WITH COPY2IMAGETOBUFFER4DRAGON ', time4xi())    
#   response4copy2imagetobuffer = {} 
#   task4copy2imagetobuffer = None
#   try:                               
#       response4copy2imagetobuffer = copy2imagetobuffer4dragon(task4copy2imagetobuffer)                                                                                                          
#       print('(Step 60B/700) OF DRAGON2IMAGE4XI5TEST - ESTABLISHED CONTROL & COMMAND WITH COPY2IMAGETOBUFFER4DRAGON : ', response4copy2imagetobuffer)
#       queue2copyimagetobuffer4dragon      = response4copy2imagetobuffer['QUEUE']
#       channel2copyimagetobufferpublish4dragon = response4copy2imagetobuffer['PUBLISHERCHANNEL']
#       channel2copyimagetobufferconsume4dragon = response4copy2imagetobuffer['CONSUMERCHANNEL']
#   except:
#       print('(Step 60Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL & COMMAND WITH COPY2IMAGETOBUFFER4DRAGON ', time4xi()) 
#   # --------------------------------------------------------------------------------------------------
#   # (61) SEND CONTROL & COMMAND COPY VIA PIPE FOR MANOEUVRE2COPYIMAGETOBUFFER4DRAGON TO RECEIVE                 
#   # --------------------------------------------------------------------------------------------------      
#   print('(Step 61A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGETOBUFFER4DRAGON ', time4xi())     
#   try:                               
#       message2copyimagetobuffersend4dragon = "<copy4xi>copy2copyimagetobuffer4dragon</copy4xi>"
#       channel2copyimagetobufferpublish4dragon = response4copy2imagetobuffer['PUBLISHERCHANNEL']
#       channel2copyimagetobufferpublish4dragon.send(message2copyimagetobuffersend4dragon)
#       print("(Step 61B/700) OF DRAGON2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4DRAGON: ", channel2copyimagetobufferpublish4dragon, " MESSAGE2SEND4DRAGON: " , message2copyimagetobuffersend4dragon)                                                                                                
#   except:
#       print('(Step 61Z1/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGETOBUFFER4DRAGON ', time4xi())                                                
#   # ------------------------------------------------------------------------------------------
#   # (62) LET MANOEUVRE2COPYIMAGETOBUFFER4DRAGON TO RECEIVE COPY AND AND SEND ROGER VIA PIPE                
#   # ------------------------------------------------------------------------------------------       
#   print("(Step 62A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4DRAGON ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4dragon = manoeuvre2copyimagetobuffer4dragon(queue2copyimagetobuffer4dragon, channel2copyimagetobufferconsume4dragon, channel2copyimagetobufferpublish4dragon)
#       print("(Step 62B/700) OF DRAGON2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4DRAGON ", manoeuvre2copyimagetobuffer4dragon)
#   except:
#       print("(Step 62Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4DRAGON ", time4xi())     
#   else:
#       pass # trust activated thread    
#   # ---------------------------------------------------------------------------------------------
#   # (63) RECEIVE CONTROL & COMMAND ROGER VIA PIPE                   
#   # ---------------------------------------------------------------------------------------------          
#   print('(Step 63A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH COPY2IMAGETOBUFFER4DRAGON ', time4xi())     
#   try:
#       channel2copyimagetobufferconsume4dragon = response4copy2imagetobuffer['CONSUMERCHANNEL']                              
#       message2copyimagereceive4dragon = channel2copyimagetobufferconsume4dragon.recv()
#       print("(Step 63B/700) OF DRAGON2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4DRAGON: ", channel2copyimagetobufferconsume4dragon, " CONTROL & COMMAND: ", message2copyimagereceive4dragon)                                                                                                  
#       # 
#   except:
#       print('(Step 63Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH COPY2IMAGETOBUFFER4DRAGON ', time4xi())         
#   # ---------------------------------------------------------------------------------------------------
#   # (64) SEND CONTROL & COMMAND READY VIA PIPE FOR MANOEUVRE2COPYIMAGETOBUFFER4DRAGON TO RECEIVE                
#   # ---------------------------------------------------------------------------------------------------      
#   print('(Step 64A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGETOBUFFER4DRAGON ', time4xi())     
#   try:                               
#       message2copyimagetobuffersend4dragon = "<copy4xi>copy2copyimagetobuffer4dragon</copy4xi>"
#       channel2copyimagetobufferpublish4dragon = response4copy2imagetobuffer['PUBLISHERCHANNEL']
#       channel2copyimagetobufferpublish4dragon.send(message2copyimagetobuffersend4dragon)
#       print("(Step 64B/700) OF DRAGON2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4DRAGON: ", channel2copyimagetobufferpublish4dragon, " MESSAGE2SEND4DRAGON: " , message2copyimagetobuffersend4dragon)                                                                                                
#       #  
#   except:
#       print('(Step 64Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGETOBUFFER4DRAGON ', time4xi())                                                
#   # -----------------------------------------------------------------------------------
#   # (65) LET MANOEUVRE2COPYIMAGETOBUFFER4DRAGON TO RECEIVE READY AND TO SEND READY                
#   # -----------------------------------------------------------------------------------       
#   print("(Step 65A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4DRAGON ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4dragon = manoeuvre2copyimagetobuffer4dragon(queue2copyimagetobuffer4dragon, channel2copyimagetobufferconsume4dragon, channel2copyimagetobufferpublish4dragon)
#       print("(Step 65B/700) OF DRAGON2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4DRAGON ", manoeuvre2copyimagetobuffer4dragon)
#   except:
#       print("(Step 65Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4DRAGON ", time4xi())     
#   # ---------------------------------------------------------------------------------------
#   # (66) RECEIVE CONTROL & COMMAND READY VIA PIPE                      
#   # ---------------------------------------------------------------------------------------          
#   print('(Step 66A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH COPY2IMAGETOBUFFER4DRAGON ', time4xi())     
#   try:
#       channel2copyimagetobufferconsume4dragon = response4copy2imagetobuffer['CONSUMERCHANNEL']                              
#       message2copyimagereceive4dragon = channel2copyimagetobufferconsume4dragon.recv()
#       print("(Step 66B/700) OF DRAGON2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4DRAGON: ", channel2copyimagetobufferconsume4dragon, " CONTROL & COMMAND: ", message2copyimagereceive4dragon)                                                                                                  
#       # 
#   except:
#       print('(Step 66Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH COPY2IMAGETOBUFFER4DRAGON ', time4xi())          
#   # -----------------------------------------------------------------------------------------
#   # (67) PUT IMAGE PATH TO QUEUE
#   # -----------------------------------------------------------------------------------------                          
#   print("(Step 67A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO PUT IMAGE PATH TO QUEUE ", time4xi())      
#   try:    
#       queue2copyimagetobuffer4dragon.put(path2dragon)             
#       print("(Step 67B/700) OF DRAGON2IMAGE4XI5TEST - PUT TO QUEUE IMAGE PATH: ", path2dragon) 
#       # ..//..//..//..//..//web//images//png//dragonxi.png  
#   except:
#       print("(Step 67Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO PUT IMAGE PATH TO QUEUE ", time4xi()) 
#   # ------------------------------------------------------------------------------------------------------
#   # (68) LET MANOEUVRE2COPYIMAGETOBUFFER4DRAGON TO GET FROM QUEUE AND TO SEND EITHER ACK OR NACK VIA PIPE               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 68A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4DRAGON ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4dragon = manoeuvre2copyimagetobuffer4dragon(queue2copyimagetobuffer4dragon, channel2copyimagetobufferconsume4dragon, channel2copyimagetobufferpublish4dragon)
#       print("(Step 68B/700) OF DRAGON2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4DRAGON ", manoeuvre2copyimagetobuffer4dragon)
#   except:
#       print("(Step 68Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4DRAGON ", time4xi())     
#   # -----------------------------------------------------------------------------------------------
#   # (69) RECEIVE ACK OR NACK ABOUT IMAGE OPERATION VIA PIPE                
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 69A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())      
#   try: 
#       message2copyimagereceive4dragon = channel2copyimagetobufferconsume4dragon.recv()
#       print("(Step 69D/700) OF DRAGON2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4DRAGON: ", channel2copyimagetobufferconsume4dragon, " CONTROL & COMMAND: ", message2copyimagereceive4dragon)                                                                                                   
#   except:
#       print('(Step 69Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())               
#   # -------------------------------------------------------------------------------------------------------
#   # (70) LET MANOEUVRE2COPYIMAGETOBUFFER4DRAGON GO TO STATE 0               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 70A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4DRAGON ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4dragon = manoeuvre2copyimagetobuffer4dragon(queue2copyimagetobuffer4dragon, channel2copyimagetobufferconsume4dragon, channel2copyimagetobufferpublish4dragon)
#       print("(Step 70B/700) OF DRAGON2IMAGE4XI5TEST - GOT RESPONSE2MANOEUVRE4DRAGON ", response2manoeuvreimagecopytobuffer4dragon)
#   except:
#       print("(Step 70Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4DRAGON ", time4xi())   

#">

# -------------------------------------------------------------------------------------------------   
def copy2imagetobuffer4dragon(task4copyimagetobuffer2dragon=None):

    '''
    copies 2D images to buffer for DRAGON
    '''
    global dragon2image4xi                                                                                             # 2D-IMAGES FOR DRAGON  
    global dragon2buffer4xi                                                                                            # BUFFER FOR DRAGON
    
    global control2copyimagetobuffer4dragon                                                                            # COMMAND AND CONTROL FOR DRAGON IMAGES 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "copy2imagetobuffer4dragon"
    method2print = "copy2imagetobuffer4dragon"

    message2print = "{(Step 1A/33) started" + "; copy2imagetobuffer4dragon-started: " + str(copy2imagetobuffer4dragon)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                 
    # <function copy2imagetobuffer4dragon at 0x000000000390B948>
    
    message2print = "{(Step 1B/33) passed" + "; task4copyimagetobuffer2dragon-to-copy2imagetobuffer4dragon: " + str(task4copyimagetobuffer2dragon) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None
            
    code2ack  = ' '
    code2nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # CREATE DRAGON2IMAGE4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if dragon2image4xi is None:
            dragon2image4xi = create2image4dragon()                                                                              # INITIALIZE IMAGE FOR DRAGON 
            message2print = "{(Step 1C/70) created" + "; dragon2image4xi-created-by-copy2imagetobuffer4dragon to: " + str(dragon2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create2image4dragon-skipped-by-copy2imagetobuffer4dragon: " + str(copy2imagetobuffer4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # create2image4dragon-skipped-by-copy2imagetobuffer4dragon                                 
    except:
        try:
            dragon2image4xi = create2image4dragon()
            message2print = "{(Step 1E/70) set" + "; dragon2image4xi-set-by-copy2imagetobuffer4dragon to: " + str(dragon2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            dragon2image4xi = {}
            message2print = "{(Step 1Z1/70) yielded" + "; dragon2image4xi-yielded-by-copy2imagetobuffer4dragon: " + str(dragon2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z1-' 
    
    # -----------------------------------------------------------------------------------------
    # CREATE DRAGON2BUFFER4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if dragon2buffer4xi is None:
            dragon2buffer4xi = create2buffer4dragon()                                                                     # INITIALIZE BUFFER FOR DRAGON 
            message2print = "{(Step 1F/70) created" + "; dragon2buffer4xi-created-by-copy2imagetobuffer4dragon to: " + str(dragon2buffer4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1G/70) skipped" + "; create2buffer4dragon-skipped-by-copy2imagetobuffer4dragon: " + str(copy2imagetobuffer4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #                                  
    except:
        try:
            dragon2buffer4xi = create2buffer4dragon()
            message2print = "{(Step 1H/70) set" + "; dragon2buffer4xi-set-by-copy2imagetobuffer4dragon to: " + str(dragon2buffer4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'CODE2ERROR': 0,
            #  'COUNT2REFERENCE': 0,
            #   'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4dragon-failed ; time: 03:40:47 10/04/11 Pacific Daylight Time}',
            #   'CLASS': 'create2buffer4dragon',
            #   'METHOD': 'create2buffer4dragon'},
            #   'FUNCTION2CALLBACK': <function trace2buffer4dragon at 0x0000000004782B48>,
            #   'POINTER2BUFFER': <mmap.mmap object at 0x000000000423BD98>,
            #   'FLAGS': 0,
            #   'CONTEXT': {'COUNT2REFERENCE': 0,
            #   'DATA2CALLBACK': ...
            
        except:
            dragon2buffer4xi = {}
            message2print = "{(Step 1Z2/70) yielded" + "; dragon2buffer4xi-yielded-by-copy2imagetobuffer4dragon: " + str(dragon2buffer4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z2-' 
                            
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - ANALYZE TASK          
    # -----------------------------------------------------------------------------------------          
              
    if task4copyimagetobuffer2dragon is None:
        # -----------------------------------------------------------------------------------------
        # NO PARAMETERS GIVEN BY CALLER FOR COPY - BUILD TASK! 
        # -----------------------------------------------------------------------------------------    
            
        task4copyimagetobuffer2dragon = {}                                                                                                                                                                                       
            
        message2print = "{(Step 1A/70) initialized" + "; task4copyimagetobuffer2dragon-created-by-copy2imagetobuffer4dragon to: " + str(task4copyimagetobuffer2dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)               
        # 
            
        # ---------------------------------------------------------------------------------------------
        # (1B) TRACE - AUTOMATIC                      
        # ---------------------------------------------------------------------------------------------              
        function2callback4dragon = trace2image4dragon                                                                 # CALLBACK FUNCTION FOR DRAGON
            
        message2print = "{(Step 1B1/70) initialized" + "; function2callback4dragon-created-by-copy2imagetobuffer4dragon to: " + str(function2callback4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <function trace2image4dragon at 0x0000000004797548> 
            
        code2ack  = code2ack + '1B1-'
           
        args2callback = {}                                                                                      # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
        args2callback['CLASS']   = "copy2imagetobuffer4dragon"                                                            # MODULE OF DRAGONIZED OPENCL MODULE
        args2callback['METHOD']  = "copy2imagetobuffer4dragon"                                                            # FUNCTION OF DRAGONIZED OPENCL MODULE
        args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; copy2imagetobuffer4dragon-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR DRAGONIZED OPENCL TO RETURN    

        message2print = "{(Step 1B2/70) initialized" + "; args2callback-created-by-copy2imagetobuffer4dragon to: " + str(args2callback)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)               
        # {'MESSAGE': '{(Step 1Z1/33) failed; copy2imagetobuffer4dragon-failed; time: 03:40:47 10/04/11 Pacific Daylight Time}', 
        #  'CLASS': 'copy2imagetobuffer4dragon', 
        #  'METHOD': 'copy2imagetobuffer4dragon'}; time: 03:40:47 10/04/11 Pacific Daylight Time}


        code2ack  = code2ack + '1B2-'
            
        # ---------------------------------------------------------------------------------------------
        # (1C) CALLBACK - AUTOMATIC                      
        # ---------------------------------------------------------------------------------------------              
        task4copyimagetobuffer2dragon['DATA2CALLBACK'] = args2callback                                                    # USER DATA PASSED TO DRAGONIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
        task4copyimagetobuffer2dragon['FUNCTION2CALLBACK'] = function2callback4dragon                                        # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
        
        code2ack  = code2ack + '1C-'
        message2print = "{(Step 1C/70) initialized" + "; task4copyimagetobuffer2dragon-created-by-copy2imagetobuffer4dragon to: " + str(task4copyimagetobuffer2dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # {'FUNCTION2CALLBACK': <function trace2image4dragon at 0x0000000004797548>,
        #   'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; copy2imagetobuffer4dragon-failed ; time: 03:40:47 10/04/11 Pacific Daylight Time}', 
        #  'CLASS': 'copy2imagetobuffer4dragon', 
        #  'METHOD': 'copy2imagetobuffer4dragon'}}; time: 03:40:47 10/04/11 Pacific Daylight Time}

            
        # ---------------------------------------------------------------------------------------------
        # (2) PLATFORM - AUTOMATIC                   
        # ---------------------------------------------------------------------------------------------             
        platform2xi = {}
        message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-copy2imagetobuffer4dragon to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # {}
            
        try:
            platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
            message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-copy2imagetobuffer4dragon: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}; time: 03:40:47 10/04/11 Pacific Daylight Time}
                 
            code2ack  = code2ack + '2B-'
        except:
            message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-copy2imagetobuffer4dragon: " + str(failed)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                                
            code2nack  = code2nack + '2Z-'
                 
        task4copyimagetobuffer2dragon['PLATFORM'] = platform2xi                                                           # PLATFORM OF SYSTEM 
              
            
        # ---------------------------------------------------------------------------------------------
        # (3) CONTEXT - AUTOMATIC                 
        # ---------------------------------------------------------------------------------------------    
        context2dragon = {}
        message2print = "{(Step 3A/70) trying" + "; context2dragon-trying-by-copy2imagetobuffer4dragon: " + str(context2dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            context2dragon = get2context4dragon()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
            message2print = "{(Step 3B/70) got" + "; context2dragon-got-by-copy2imagetobuffer4dragon: " + str(context2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'COUNT2REFERENCE': 0, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4dragon-failed ; time: 03:40:47 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'create2context4dragon', 
            #  'METHOD': 'create2context4dragon'}, 
            #  'FUNCTION2CALLBACK': <function trace2context4dragon at 0x000000000394F348>, 
            #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-', 
            #  'INFO2PRIVATE': None, 
            #  'PLATFORM': {'list2platforms': ['DragonXi'],'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
            #  'NACK2RETURN': ' ', 
            #  'SIZE2PRIVATE': 0, 
            #  'PROPERTIES': None}; time: 03:40:47 10/04/11 Pacific Daylight Time}
                
            code2ack  = code2ack + '3B-'         
        except:
            message2print = "{(Step 3Z/70) failed" + "; context2dragon-failed-by-copy2imagetobuffer4dragon: " + str(context2dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z-'
            
        task4copyimagetobuffer2dragon ['CONTEXT'] = context2dragon           
            
        # ---------------------------------------------------------------------------------------------
        # (4) DEVICE - AUTOMATIC                       
        # ---------------------------------------------------------------------------------------------    
        device2dragon = {}
            
        message2print = "{(Step 4A/70) trying" + "; device2dragon-trying-by-copy2imagetobuffer4dragon: " + str(device2dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            device2dragon = get2dragon4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
            
            message2print = "{(Step 4B/70) got" + "; device2dragon-got-by-copy2imagetobuffer4dragon: " + str(device2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'RESOLUTION2TIMER4PROFILING': 318, 
            #  'ID2PLATFORM': '0000000001509568', 
            #  'PROFILING2QUEUE': 'ACK', 
            #  'EXECUTE2KERNELS4OPENCL': 'ACK', 
            #  'NATIVE2WIDTHLONG4VECTOR': 2, 
            #  'MAX2UNITS4COMPUTE': 4, 
            #  'SIZE2MEMORY4GLOBAL': 3221225472, 
            #  'NATIVE2WIDTHFLOAT4VECTOR': 4, 
            #  'MAX2SIZE4WORKGROUP': 1024, 
            #  'PREFERRED2WIDTHCHAR4VECTOR': 16, 
            #  'SIZE2CACHE': 65536, 
            #  'IEEE754-2008FUSED2MULTIPLY-ADD': 'NACK', 
            #  'TYPE4CACHE': 'Read/Write', 
            #  'PREFERRED2WIDTHINT4VECTOR': 4, 
            #  'SIZE2PREFERREDWORKGROUPMULTIPLE4KERNEL': 1, 
            #  'NATIVE2WIDTHINT4VECTOR': 4, 
            #  'SIZE2BUFFER4CONSTANT': 65536, 
            #  'EXECUTE2FUNCTION4NATIVE': 'ACK', 
            #  'PREFERRED2WIDTHDOUBLE4VECTOR': 0, 
            #  'MAX2NUMBER4CONSTANTARGS': 8, 
            #  'VERSION2DRIVER': '2.0', 
            #  'EXTENSIONS': 'cl_amd_fp64 ...
            #  'AVAILABLE2COMPILER': 'ACK', 
            #  'MAX2DIMENSIONS4WORKITEMS': 3, 
            #  'SUPPORT2CORRECTION4ERROR': 0, 
            #  'BITS2ADDRESS': 64, 
            #  'AVAILABLE': 'ACK', 
            #  'SIZE2MEMORY4LOCAL': 32768, 
            #  'VENDOR': 'AuthenticAMD', 
            #  'SUPPORT2IMAGE': 'NACK', 
            #  'MAX2SIZE4KERNELARGUMENT': 4096, 
            #  'ACK2RETURN': ' 1C-1D-1E-1F-1D-2A-...
            #  'OUT-OF-ORDER2QUEUE': 'NACK', 
            #  'ALIGNMENT2ADDRESS4BASE': 1024, 
            #  'PROFILE': 'FULL_PROFILE', 
            #  'TYPE2MEMORY4LOCAL': 'Global', 
            #  'PREFERRED2WIDTHFLOAT4VECTOR': 4, 
            #  'PREFERRED2WIDTHLONG4VECTOR': 2, 
            #  'MAX2FREQUENCY4CLOCK': 3210, 
            #  'PREFERRED2WIDTHSHORT4VECTOR': 8, 
            #  'ID': 4033, 
            #  'MAX2WORKITEM2': 1024, 
            #  'NATIVE2WIDTHDOUBLE4VECTOR': 0, 
            #  'ROUND2ZERO': 'ACK', 
            #  'NAME': 'AMD64 Family 16 Model 4 Stepping 2, 
            #   AuthenticAMD', 
            #  'MAX2WORKITEM1': 1024, 
            #  'MAX2WORKITEM0': 1024, 
            #  'SIZE2LINE4CACHE': 64, 
            #  'MAX2ALLOCATION4MEMORY': 1073741824, 
            #  'NATIVE2WIDTHCHAR4VECTOR': 16, 
            #  'QUIET2NANS': 'ACK', 
            #  'NATIVE2WIDTHSHORT4VECTOR': 8, 
            #  'ROUND2NEAREST4EVEN': 'ACK', 
            #  'VERSION': 'OpenCL 1.1 ATI-Stream-v2.3 (451)', 
            #  'NACK2RETURN': ' ', 
            #  'MEMORY2UNIFIED4HOSTANDDEVICE': 1, 
            #  'ENDIANESS2DEVICE': 'Little', 
            #  'MINIMUM2ALIGNMENT4DATATYPE': 128, 
            #  'ROUND2+VE3INFINITY': 'ACK', 
            #  'TYPE': 'CL_DEVICE_TYPE_DRAGON', 
            #  'DENORMS2SINGLEPRECISION4FLOATINGPOINTCAPABILITY': 'ACK'}; time: 03:40:47 10/04/11 Pacific Daylight Time}
            
            code2ack  = code2ack + '4B-'  
        except:
            message2print = "{(Step 4Z/70) failed" + "; device2dragon-failed-by-copy2imagetobuffer4dragon: " + str(device2dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z-'
        
        task4copyimagetobuffer2dragon ['DEVICE'] = device2dragon
        
               
        # ---------------------------------------------------------------------------------------------
        # (5) IMAGE SOURCE OBJECT - REQUIRED                         
        # ---------------------------------------------------------------------------------------------         
        imagesourceinfo2dragon = {}
         
        message2print = "{(Step 5A/70) trying" + "; imagesourceinfo2dragon-trying-by-copy2imagetobuffer4dragon: " + str(imagesourceinfo2dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            imagesourceinfo2dragon = get2imageinfo4dragon()                                                                         
            
            message2print = "{(Step 5B/70) got" + "; imagesourceinfo2dragon-got-by-copy2imagetobuffer4dragon: " + str(imagesourceinfo2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'SIZE2QUEUE': 2147483646, 
            #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
            #  'ORDER': 'CL_RGBA'}, 
            #  'QUEUE': <queue4xi.Queue4Xi object at 0x00000000042514E0>, 
            #  'WIDTH': 800, 
            #  'CONTEXT': ...

            code2ack  = code2ack + '5B-'  
        except:
            message2print = "{(Step 5Z/70) failed" + "; imagesourceinfo2dragon-failed-by-copy2imagetobuffer4dragon: " + str(imagesourceinfo2dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '5Z-'          
                  
        task4copyimagetobuffer2dragon ['IMAGESOURCE'] = imagesourceinfo2dragon         
               
        # ---------------------------------------------------------------------------------------------
        # (6) BUFFER DESTINATION OBJECT - REQUIRED                        
        # ---------------------------------------------------------------------------------------------                         
        bufferdestinationinfo2dragon = {}
         
        message2print = "{(Step 6A/70) trying" + "; bufferdestinationinfo2dragon-trying-by-copy2imagetobuffer4dragon: " + str(bufferdestinationinfo2dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            bufferdestinationinfo2dragon = get2bufferinfo4dragon()                                                                         
            
            message2print = "{(Step 6B/70) got" + "; bufferdestinationinfo2dragon-got-by-copy2imagetobuffer4dragon: " + str(bufferdestinationinfo2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'CODE2ERROR': 0, 
            #  'COUNT2REFERENCE': 0, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4dragon-failed ; time: 03:40:47 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'create2buffer4dragon', 
            #  'METHOD': 'create2buffer4dragon'}, 
            #  'FUNCTION2CALLBACK': <function trace2buffer4dragon at 0x0000000004782B48>, 
            #  'POINTER2BUFFER': <mmap.mmap object at 0x000000000423BD98>, 
            #  'FLAGS': 0, 
            #  'CONTEXT': {'COUNT2REFERENCE': 0, ...
            code2ack  = code2ack + '6B-'  
        except:
            message2print = "{(Step 6Z/70) failed" + "; bufferdestinationinfo2dragon-failed-by-copy2imagetobuffer4dragon: " + str(bufferdestinationinfo2dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '6Z-'          
                  
        task4copyimagetobuffer2dragon ['BUFFERDESTINATION'] = bufferdestinationinfo2dragon     
        
        # ---------------------------------------------------------------------------------------------
        # (7) QUEUE - REQUIRED BY DRAGONXI                          
        # ---------------------------------------------------------------------------------------------         
        queue2copyimagetobuffer4dragon = None
        
        message2print = "{(Step 7A/70) trying" + "; queue2copyimagetobuffer4dragon-trying-by-copy2imagetobuffer4dragon: " + str(queue2copyimagetobuffer4dragon)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        try:
            queue2copyimagetobuffer4dragon = Queue4Xi()                                                                              # THE SYSTEM CREATES QUEUE    
            
            message2print = "{(Step 7B/70) got" + "; queue2copyimagetobuffer4dragon-got-by-copy2imagetobuffer4dragon: " + str(queue2copyimagetobuffer4dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # <queue4xi.Queue4Xi object at 0x00000000042B2208> 
            
            code2ack  = code2ack + '7B-'  
        except:
            message2print = "{(Step 7Z/70) failed" + "; queue2copyimagetobuffer4dragon-failed-by-copy2imagetobuffer4dragon: " + str(queue2copyimagetobuffer4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '7Z-'
            
        task4copyimagetobuffer2dragon ['QUEUE'] = queue2copyimagetobuffer4dragon
         
        # ---------------------------------------------------------------------------------------------
        # (8) PIPE - REQUIRED BY DRAGONXI                              
        # ---------------------------------------------------------------------------------------------         
        pipe2dragon = None
       
        message2print = "{(Step 8A/70) trying" + "; pipe2dragon-trying-by-copy2imagetobuffer4dragon: " + str(pipe2dragon)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        try:
            pipe2dragon = Pipe4Xi()                                                                                # THE SYSTEM CREATES PIPE    
            
            message2print = "{(Step 8B/70) got" + "; pipe2dragon-got-by-copy2imagetobuffer4dragon: " + str(pipe2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # (<read-write PipeConnection, handle 660>, <read-write PipeConnection, handle 664>)
            
            code2ack  = code2ack + '8B-'  
        except:
            message2print = "{(Step 8Z/70) failed" + "; pipe2dragon-failed-by-copy2imagetobuffer4dragon: " + str(pipe2dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '8Z-'
            
        task4copyimagetobuffer2dragon ['PIPE'] = pipe2dragon 
        
        # ---------------------------------------------------------------------------------------------
        # (9) COMMAND QUEUE - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        commandqueueinfo2dragon = {}
       
        message2print = "{(Step 9A/70) trying" + "; commandqueueinfo2dragon-trying-by-copy2imagetobuffer4dragon: " + str(commandqueueinfo2dragon)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)  
                    
        try:
            commandqueueinfo2dragon = get2commandqueueinfo4dragon()                                                   # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
            message2print = "{(Step 9B/70) got" + "; commandqueueinfo2dragon-got-by-copy2imagetobuffer4dragon: " + str(commandqueueinfo2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CODE2ERROR': 0, 
            #  'COUNT2REFERENCE': 0, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4dragon-failed ; time: 03:40:47 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'create2commandqueue4dragon', 
            #  'METHOD': 'create2commandqueue4dragon'}, 
            #  'FUNCTION2CALLBACK': <function trace2commandqueue4dragon at 0x000000000395EA48>, 
            #  'ACK2RETURN': ...
            
            code2ack  = code2ack + '9B-'  
        except:
            message2print = "{(Step 9Z/70) failed" + "; commandqueueinfo2dragon-failed-by-copy2imagetobuffer4dragon: " + str(commandqueueinfo2dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '9Z-'
    
        task4copyimagetobuffer2dragon ['COMMANDQUEUE'] = commandqueueinfo2dragon
        
        
        
        # ---------------------------------------------------------------------------------------------
        # (11) EVENT - REQUIRED                            
        # ---------------------------------------------------------------------------------------------             
        eventinfo2dragon = {}
       
        message2print = "{(Step 11A/70) trying" + "; eventinfo2dragon-trying-by-copy2imagetobuffer4dragon: " + str(eventinfo2dragon)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            eventinfo2dragon = get2eventinfo4dragon()                                                                    
            message2print = "{(Step 11B/70) got" + "; eventinfo2dragon-got-by-copy2imagetobuffer4dragon: " + str(eventinfo2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CODE2ERROR': 0, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2event4dragon-failed ; time: 03:40:47 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'create2event4dragon', 
            #  'METHOD': 'create2event4dragon'}, 
            #  'FUNCTION2CALLBACK': <function trace2event4dragon at 0x0000000003971F48>, 
            #  'ACK2RETURN': ' 1D-1E-...
            code2ack  = code2ack + '11B-' 
        except:
            message2print = "{(Step 11Z/70) failed" + "; eventinfo2dragon-failed-by-copy2imagetobuffer4dragon: " + str(eventinfo2dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '11Z-'
    
        task4copyimagetobuffer2dragon ['EVENT'] = eventinfo2dragon 
        
        
        # ---------------------------------------------------------------------------------------------
        # (12) EVENTWAITLIST - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        eventwaitlist2dragon = {}
       
        message2print = "{(Step 12A/70) trying" + "; eventwaitlist2dragon-trying-by-copy2imagetobuffer4dragon: " + str(eventwaitlist2dragon)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            eventwaitlist2dragon = {}                                                                   
            message2print = "{(Step 12B/70) set" + "; eventwaitlist2dragon-set-by-copy2imagetobuffer4dragon: " + str(eventwaitlist2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {}
            
            code2ack  = code2ack + '12B-' 
        except:
            message2print = "{(Step 12Z/70) failed" + "; eventwaitlist2dragon-failed-by-copy2imagetobuffer4dragon: " + str(eventwaitlist2dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '12Z-'
    
        task4copyimagetobuffer2dragon ['EVENTWAITLIST'] = eventwaitlist2dragon                     
         
        # ---------------------------------------------------------------------------------------------
        # (13) EVENTWAITLISTQUANTITY - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        eventwaitlistquantity2dragon = 0
       
        message2print = "{(Step 13A/70) trying" + "; eventwaitlistquantity2dragon-trying-by-copy2imagetobuffer4dragon: " + str(eventwaitlistquantity2dragon)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            keys2event = eventwaitlist2dragon.keys()
            for key2event in keys2event:
                eventwaitlistquantity2dragon = eventwaitlistquantity2dragon + 1                                                                      
            message2print = "{(Step 13B/70) set" + "; eventwaitlistquantity2dragon-set-by-copy2imagetobuffer4dragon: " + str(eventwaitlistquantity2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 0 
            
            code2ack  = code2ack + '13B-' 
        except:
            message2print = "{(Step 13Z/70) failed" + "; eventwaitlistquantity2dragon-failed-by-copy2imagetobuffer4dragon: " + str(eventwaitlistquantity2dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '13Z-'
    
        task4copyimagetobuffer2dragon ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2dragon       
                                             
        
        # ---------------------------------------------------------------------------------------------
        # (14) ORIGIN FOR SOURCE - REQUIRED                             
        # ---------------------------------------------------------------------------------------------             
        origin4source2dragon = {}
        
        message2print = "{(Step 14A/70) trying" + "; origin4source2dragon-trying-by-copy2imagetobuffer4dragon: " + str(origin4source2dragon)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:                                                                                    
            origin4source2dragon['X'] = 0 
            origin4source2dragon['Y'] = 0  
            origin4source2dragon['Z'] = 0                                                                                                                          
            message2print = "{(Step 14B/70) set" + "; origin4source2dragon-set-by-copy2imagetobuffer4dragon to: " + str(origin4source2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'Y': 0, 'X': 0, 'Z': 0} 
            code2ack  = code2ack + '14B-' 
        except:
            message2print = "{(Step 14Z/70) failed" + "; origin4source2dragon-failed-by-copy2imagetobuffer4dragon: " + str(origin4source2dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '14Z-'
    
        task4copyimagetobuffer2dragon ['SOURCEORIGIN'] = origin4source2dragon
        
       
        
        # --------------------------------------------------------------------------------------------------------
        # (16) REGION - REGION OF A IMAGE TO BE COPIED                         
        # --------------------------------------------------------------------------------------------------------             
        region4copy2dragon = {}                                                                                                         
       
        message2print = "{(Step 16A/70) trying" + "; region4copy2dragon-trying-by-copy2imagetobuffer4dragon: " + str(region4copy2dragon)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            region4copy2dragon['X'] = 10 
            region4copy2dragon['Y'] = 10  
            region4copy2dragon['Z'] = 10                                                                           
            message2print = "{(Step 16B/70) set" + "; region4copy2dragon-set-by-copy2imagetobuffer4dragon to: " + str(region4copy2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'Y': 10, 'X': 10, 'Z': 10} 
            code2ack  = code2ack + '16B-'                
             
        except:
            message2print = "{(Step 16Z/70) failed" + "; region4copy2dragon-failed-by-copy2imagetobuffer4dragon: " + str(region4copy2dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '16Z-'
        
        task4copyimagetobuffer2dragon ['REGION'] = region4copy2dragon              
        
        # --------------------------------------------------------------------------------------------------------
        # (17) OFFSET - WHERE TO BEGIN COPYING DATA INTO DESTINATION BUFFER                     
        # --------------------------------------------------------------------------------------------------------             
        offset4copy2dragon = 0                                                                                                         
        message2print = "{(Step 17A/70) set" + "; offset4copy2dragon-set-by-copy2imagetobuffer4dragon to: " + str(offset4copy2dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 0
        code2ack  = code2ack + '17A-'                
        task4copyimagetobuffer2dragon ['OFFSET'] = offset4copy2dragon            
                    
        # ---------------------------------------------------------------------------------------------
        # EOF PARAMETER PROVIDING                
        # ---------------------------------------------------------------------------------------------                                                                                                                               
    else:
        pass
                    
    # ---------------------------------------------------------------------------------------------
    # ANALYZE TASK - WHETHER AUTOMATICALLY GENERATED OR USER GIVEN 
    # ---------------------------------------------------------------------------------------------
    keys2task = task4copyimagetobuffer2dragon.keys()
    message2print = "{(Step 20/70) got" + "; keys2task-got-by-copy2imagetobuffer4dragon: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)            
    # ['BUFFERDESTINATION',
    #  'COMMANDQUEUE',
    #  'CONTEXT',
    #  'DATA2CALLBACK',
    #  'DEVICE',
    #  'EVENT',
    #  'EVENTWAITLIST',
    #  'EVENTWAITLISTQUANTITY',
    #  'FUNCTION2CALLBACK',
    #  'IMAGESOURCE',
    #  'OFFSET',
    #  'PIPE',
    #  'PLATFORM',
    #  'QUEUE',
    #  'REGION',
    #  'SOURCEORIGIN']; time: 01:28:44 10/05/11 Pacific Daylight Time}


    if (('DATA2CALLBACK' in keys2task) and ('FUNCTION2CALLBACK' in keys2task)):
        pass
    else:                
        # ---------------------------------------------------------------------------------------------
        # (21B) TRACE - AUTOMATIC                      
        # ---------------------------------------------------------------------------------------------              
        function2callback4dragon = trace2image4dragon                                                             # CALLBACK FUNCTION FOR DRAGON
        
        message2print = "{(Step 21B1/70) initialized" + "; function2callback4dragon-created-by-copy2imagetobuffer4dragon to: " + str(function2callback4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 
            
        code2ack  = code2ack + '21B1-'
           
        args2callback = {}                                                                                  # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
        args2callback['CLASS']   = "copy2imagetobuffer4dragon"                                                        # MODULE OF DRAGONIZED OPENCL MODULE
        args2callback['METHOD']  = "copy2imagetobuffer4dragon"                                                        # FUNCTION OF DRAGONIZED OPENCL MODULE
        args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; copy2imagetobuffer4dragon-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR DRAGONIZED OPENCL TO RETURN    

        message2print = "{(Step 21B2/70) initialized" + "; args2callback-created-by-copy2imagetobuffer4dragon to: " + str(args2callback)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)               
        # 

        code2ack  = code2ack + '21B2-'
        
        # ---------------------------------------------------------------------------------------------
        # (21C) CALLBACK - AUTOMATIC                      
        # ---------------------------------------------------------------------------------------------              
        task4copyimagetobuffer2dragon['DATA2CALLBACK'] = args2callback                                                # USER DATA PASSED TO DRAGONIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
        task4copyimagetobuffer2dragon['FUNCTION2CALLBACK'] = function2callback4dragon                                    # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
        
        code2ack  = code2ack + '21C-'
        message2print = "{(Step 21C/70) initialized" + "; task4copyimagetobuffer2dragon-created-by-copy2imagetobuffer4dragon to: " + str(task4copyimagetobuffer2dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #
             
    if 'PLATFORM' in keys2task:
        pass
    else:  
        # ---------------------------------------------------------------------------------------------
        # (22) PLATFORM - AUTOMATIC                   
        # ---------------------------------------------------------------------------------------------             
        platform2xi = {}
        message2print = "{(Step 22A/70) initialized" + "; platform2xi-created-by-copy2imagetobuffer4dragon to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
            
        try:
            platform2xi = get2platformids4xi()                                                              # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
            message2print = "{(Step 22B/70) got" + "; platform2xi-got-by-copy2imagetobuffer4dragon: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
                 
            code2ack  = code2ack + '22B-'
        except:
            message2print = "{(Step 22Z/70) failed" + "; platform2xi-failed-by-copy2imagetobuffer4dragon: " + str(failed)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                                
            code2nack  = code2nack + '22Z-'
                 
        task4copyimagetobuffer2dragon['PLATFORM'] = platform2xi                                                       # PLATFORM OF SYSTEM 
          
    if 'CONTEXT' in keys2task:
        pass
    else:              
        # ---------------------------------------------------------------------------------------------
        # (23) CONTEXT - AUTOMATIC                 
        # ---------------------------------------------------------------------------------------------    
        context2dragon = {}
        message2print = "{(Step 23A/70) trying" + "; context2dragon-trying-by-copy2imagetobuffer4dragon: " + str(context2dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            context2dragon = get2context4dragon()                                                                 # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
            
            message2print = "{(Step 23B/70) got" + "; context2dragon-got-by-copy2imagetobuffer4dragon: " + str(context2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # 
            
            code2ack  = code2ack + '23B-'         
        except:
            message2print = "{(Step 23Z/70) failed" + "; context2dragon-failed-by-copy2imagetobuffer4dragon: " + str(context2dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '23Z-'
            
        task4copyimagetobuffer2dragon ['CONTEXT'] = context2dragon           
    
    if 'DEVICE' in keys2task:
        pass
    else:                   
        # ---------------------------------------------------------------------------------------------
        # (24) DEVICE - AUTOMATIC                       
        # ---------------------------------------------------------------------------------------------    
        device2dragon = {}
        
        message2print = "{(Step 24A/70) trying" + "; device2dragon-trying-by-copy2imagetobuffer4dragon: " + str(device2dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            device2dragon = get2dragon4xi()                                                                       # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
            
            message2print = "{(Step 24B/70) got" + "; device2dragon-got-by-copy2imagetobuffer4dragon: " + str(device2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # 
                
            code2ack  = code2ack + '24B-'  
        except:
            message2print = "{(Step 24Z/70) failed" + "; device2dragon-failed-by-copy2imagetobuffer4dragon: " + str(device2dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '24Z-'
        
        task4copyimagetobuffer2dragon ['DEVICE'] = device2dragon
    
    if 'IMAGESOURCE' in keys2task:
        pass
    else:                      
        # ---------------------------------------------------------------------------------------------
        # (25) SOURCE IMAGE - REQUIRED                            
        # ---------------------------------------------------------------------------------------------         
        sourceimageinfo2dragon = {}
         
        message2print = "{(Step 25A/70) trying" + "; sourceimageinfo2dragon-trying-by-copy2imagetobuffer4dragon: " + str(sourceimageinfo2dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            sourceimageinfo2dragon = get2imageinfo4dragon()                                                                         
            
            message2print = "{(Step 25B/70) got" + "; sourceimageinfo2dragon-got-by-copy2imagetobuffer4dragon: " + str(sourceimageinfo2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            #                 
            code2ack  = code2ack + '25B-'  
        except:
            message2print = "{(Step 25Z/70) failed" + "; sourceimageinfo2dragon-failed-by-copy2imagetobuffer4dragon: " + str(sourceimageinfo2dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '25Z-'          
                  
        task4copyimagetobuffer2dragon ['IMAGESOURCE'] = sourceimageinfo2dragon         
    
    if 'BUFFERDESTINATION' in keys2task:
        pass
    else:                      
        # ---------------------------------------------------------------------------------------------
        # (26) DESTINATION BUFFER - REQUIRED                            
        # ---------------------------------------------------------------------------------------------         
        destinationbufferinfo2dragon = {}
         
        message2print = "{(Step 26A/70) trying" + "; destinationbufferinfo2dragon-trying-by-copy2imagetobuffer4dragon: " + str(destinationbufferinfo2dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            destinationbufferinfo2dragon = get2bufferinfo4dragon()                                                                         
            
            message2print = "{(Step 26B/70) got" + "; destinationbufferinfo2dragon-got-by-copy2imagetobuffer4dragon: " + str(destinationbufferinfo2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            #                 
            code2ack  = code2ack + '26B-'  
        except:
            message2print = "{(Step 26Z/70) failed" + "; destinationbufferinfo2dragon-failed-by-copy2imagetobuffer4dragon: " + str(destinationbufferinfo2dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '26Z-'          
                  
        task4copyimagetobuffer2dragon ['BUFFERDESTINATION'] = destinationbufferinfo2dragon    
                                
    if 'QUEUE' in keys2task:
        pass
    else:   
        # ---------------------------------------------------------------------------------------------
        # (27) QUEUE - REQUIRED BY DRAGONXI                          
        # ---------------------------------------------------------------------------------------------         
        queue2copyimagetobuffer4dragon = None
        
        message2print = "{(Step 27A/70) trying" + "; queue2copyimagetobuffer4dragon-trying-by-copy2imagetobuffer4dragon: " + str(queue2copyimagetobuffer4dragon)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        try:
            queue2copyimagetobuffer4dragon = Queue4Xi()                                                                          # THE SYSTEM CREATES QUEUE    
            
            message2print = "{(Step 27B/70) got" + "; queue2copyimagetobuffer4dragon-got-by-copy2imagetobuffer4dragon: " + str(queue2copyimagetobuffer4dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # 
            
            code2ack  = code2ack + '27B-'  
        except:
            message2print = "{(Step 27Z/70) failed" + "; queue2copyimagetobuffer4dragon-failed-by-copy2imagetobuffer4dragon: " + str(queue2copyimagetobuffer4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '27Z-'
            
        task4copyimagetobuffer2dragon ['QUEUE'] = queue2copyimagetobuffer4dragon
        
    if 'PIPE' in keys2task:
        pass
    else:              
        # ---------------------------------------------------------------------------------------------
        # (28) PIPE - REQUIRED BY DRAGONXI                              
        # ---------------------------------------------------------------------------------------------         
        pipe2dragon = None
       
        message2print = "{(Step 28A/70) trying" + "; pipe2dragon-trying-by-copy2imagetobuffer4dragon: " + str(pipe2dragon)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        try:
            pipe2dragon = Pipe4Xi()                                                                            # THE SYSTEM CREATES PIPE    
            
            message2print = "{(Step 28B/70) got" + "; pipe2dragon-got-by-copy2imagetobuffer4dragon: " + str(pipe2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # 
            
            code2ack  = code2ack + '28B-'  
        except:
            message2print = "{(Step 28Z/70) failed" + "; pipe2dragon-failed-by-copy2imagetobuffer4dragon: " + str(pipe2dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '28Z-'
            
        task4copyimagetobuffer2dragon ['PIPE'] = pipe2dragon 
        
    if 'COMMANDQUEUE' in keys2task:
        pass
    else:              
        # ---------------------------------------------------------------------------------------------
        # (28) COMMAND QUEUE - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        commandqueueinfo2dragon = {}
       
        message2print = "{(Step 29A/70) trying" + "; commandqueueinfo2dragon-trying-by-copy2imagetobuffer4dragon: " + str(commandqueueinfo2dragon)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)  
                    
        try:
            commandqueueinfo2dragon = get2commandqueueinfo4dragon()                                               # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
            message2print = "{(Step 29B/70) got" + "; commandqueueinfo2dragon-got-by-copy2imagetobuffer4dragon: " + str(commandqueueinfo2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            
            code2ack  = code2ack + '29B-'  
        except:
            message2print = "{(Step 29Z/70) failed" + "; commandqueueinfo2dragon-failed-by-copy2imagetobuffer4dragon: " + str(commandqueueinfo2dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '29Z-'
    
        task4copyimagetobuffer2dragon ['COMMANDQUEUE'] = commandqueueinfo2dragon
    
         
    if 'EVENT' in keys2task:
        pass
    else:               
        # ---------------------------------------------------------------------------------------------
        # (31) EVENT - REQUIRED                            
        # ---------------------------------------------------------------------------------------------             
        eventinfo2dragon = {}
       
        message2print = "{(Step 31A/70) trying" + "; eventinfo2dragon-trying-by-copy2imagetobuffer4dragon: " + str(eventinfo2dragon)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            eventinfo2dragon = get2eventinfo4dragon()                                                                    
            message2print = "{(Step 31B/70) got" + "; eventinfo2dragon-got-by-copy2imagetobuffer4dragon: " + str(eventinfo2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            
            code2ack  = code2ack + '31B-' 
        except:
            message2print = "{(Step 31Z/70) failed" + "; eventinfo2dragon-failed-by-copy2imagetobuffer4dragon: " + str(eventinfo2dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '31Z-'
    
        task4copyimagetobuffer2dragon ['EVENT'] = eventinfo2dragon 
    
         
    if 'EVENTWAITLIST' in keys2task:
        pass
    else:               
        # ---------------------------------------------------------------------------------------------
        # (32) EVENTWAITLIST - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        eventwaitlist2dragon = {}
       
        message2print = "{(Step 32A/70) trying" + "; eventwaitlist2dragon-trying-by-copy2imagetobuffer4dragon: " + str(eventwaitlist2dragon)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            eventwaitlist2dragon = {}                                                                   
            message2print = "{(Step 32B/70) set" + "; eventwaitlist2dragon-set-by-copy2imagetobuffer4dragon: " + str(eventwaitlist2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            
            code2ack  = code2ack + '32B-' 
        except:
            message2print = "{(Step 32Z/70) failed" + "; eventwaitlist2dragon-failed-by-copy2imagetobuffer4dragon: " + str(eventwaitlist2dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '32Z-'
    
        task4copyimagetobuffer2dragon ['EVENTWAITLIST'] = eventwaitlist2dragon                     
          
    if 'EVENTWAITLISTQUANTITY' in keys2task:
        pass
    else:               
        # ---------------------------------------------------------------------------------------------
        # (33) EVENTWAITLISTQUANTITY - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        eventwaitlistquantity2dragon = 0
       
        message2print = "{(Step 33A/70) trying" + "; eventwaitlistquantity2dragon-trying-by-copy2imagetobuffer4dragon: " + str(eventwaitlistquantity2dragon)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            keys2event = eventwaitlist2dragon.keys()
            for key2event in keys2event:
                eventwaitlistquantity2dragon = eventwaitlistquantity2dragon + 1                                                                   
            message2print = "{(Step 33B/70) set" + "; eventwaitlistquantity2dragon-set-by-copy2imagetobuffer4dragon: " + str(eventwaitlistquantity2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 0 
            
            code2ack  = code2ack + '33B-' 
        except:
            message2print = "{(Step 33Z/70) failed" + "; eventwaitlistquantity2dragon-failed-by-copy2imagetobuffer4dragon: " + str(eventwaitlistquantity2dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '33Z-'
    
        task4copyimagetobuffer2dragon ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2dragon       
                                         
                
    if 'SOURCEORIGIN' in keys2task:
        pass
    else:                    
        # ---------------------------------------------------------------------------------------------
        # (34) SOURCEORIGIN - REQUIRED                             
        # ---------------------------------------------------------------------------------------------             
        source4origin2dragon = {}
        
        message2print = "{(Step 34A/70) trying" + "; source4origin2dragon-trying-by-copy2imagetobuffer4dragon: " + str(source4origin2dragon)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:                                                                                    
            source4origin2dragon['X'] = 0 
            source4origin2dragon['Y'] = 0  
            source4origin2dragon['Z'] = 0                                                                                                                          
            message2print = "{(Step 34B/70) set" + "; source4origin2dragon-set-by-copy2imagetobuffer4dragon to: " + str(source4origin2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            code2ack  = code2ack + '34B-' 
        except:
            message2print = "{(Step 34Z/70) failed" + "; source4origin2dragon-failed-by-copy2imagetobuffer4dragon: " + str(source4origin2dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '34Z-'
    
        task4copyimagetobuffer2dragon ['SOURCEORIGIN'] = source4origin2dragon
             
   
             
    if 'REGION' in keys2task:
        pass
    else:            
        # ---------------------------------------------------------------------------------------------
        # (36) REGION - REQUIRED                         
        # ---------------------------------------------------------------------------------------------             
        region2dragon = {}
        
        message2print = "{(Step 36A/70) trying" + "; region2dragon-trying-by-copy2imagetobuffer4dragon: " + str(region2dragon)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:                                                                                    
            region2dragon['X'] = 160                                                                               # WIDTH  IN PIXELS TO WRITE 
            region2dragon['Y'] = 90                                                                                # HEIGHT IN PIXELS TO WRITE
            region2dragon['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO WRITE                                                                                                                          
            message2print = "{(Step 36B/70) set" + "; region2dragon-set-by-copy2imagetobuffer4dragon to: " + str(region2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            code2ack  = code2ack + '36B-' 
        except:
            message2print = "{(Step 36Z/70) failed" + "; region2dragon-failed-by-copy2imagetobuffer4dragon: " + str(region2dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '36Z-'
    
        task4copyimagetobuffer2dragon ['REGION'] = region2dragon
    
         
    if 'OFFSET' in keys2task:
        pass
    else:            
        # ---------------------------------------------------------------------------------------------
        # (37) OFFSET - WHERE TO BEGIN COPYING DATA INTO DESTINATION BUFFER - REQUIRED                         
        # ---------------------------------------------------------------------------------------------             
        offset2dragon = 0
        message2print = "{(Step 37A/70) set" + "; offset2dragon-set-by-copy2imagetobuffer4dragon to: " + str(offset2dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        task4copyimagetobuffer2dragon ['OFFSET'] = offset2dragon        
        code2ack  = code2ack + '37A-' 
        
    # ---------------------------------------------------------------------------------------------
    # EOF TASK
    # ---------------------------------------------------------------------------------------------
    keys2task = task4copyimagetobuffer2dragon.keys()
    message2print = "{(Step 39/70) set" + "; keys2task-set-by-copy2imagetobuffer4dragon to: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)            
    # ['BUFFERDESTINATION',
    #  'COMMANDQUEUE',
    #  'CONTEXT',
    #  'DATA2CALLBACK',
    #  'DEVICE',
    #  'EVENT',
    #  'EVENTWAITLIST',
    #  'EVENTWAITLISTQUANTITY',
    #  'FUNCTION2CALLBACK',
    #  'IMAGESOURCE',
    #  'OFFSET',
    #  'PIPE',
    #  'PLATFORM',
    #  'QUEUE',
    #  'REGION',
    #  'SOURCEORIGIN']; time: 01:28:44 10/05/11 Pacific Daylight Time}
               


    # ---------------------------------------------------------------------------------------------
    # CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    control2copyimagetobuffer4dragon = {}       
                                
    # ---------------------------------------------------------------------------------------------
    # CONSUMER AND PUBLISHER CHANNEL FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4channels2dragon = False
    channels2dragon = ()
    message2print = "{(Step 40A/70) trying" + "channels2dragon-trying-by-copy2imagetobuffer4dragon: " + str(channels2dragon)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        # 
        consumerchannel2copyimagetobuffer4dragon, publisherchannel2copyimagetobuffer4dragon = task4copyimagetobuffer2dragon ['PIPE']

        message2print = "{(Step 40B/70) got" + "; consumerchannel2copyimagetobuffer4dragon-got-by-copy2imagetobuffer4dragon: " + str(consumerchannel2copyimagetobuffer4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 660>

        message2print = "{(Step 40C/70) got" + "; publisherchannel2copyimagetobuffer4dragon-got-by-copy2imagetobuffer4dragon: " + str(publisherchannel2copyimagetobuffer4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 664>                     
        ack4channels2dragon = True
        
        control2copyimagetobuffer4dragon['CONSUMERCHANNEL']  = consumerchannel2copyimagetobuffer4dragon
        
        
        control2copyimagetobuffer4dragon['PUBLISHERCHANNEL'] = publisherchannel2copyimagetobuffer4dragon
        
        code2ack  = code2ack + '40D-' 
        
    except:
        ack4channels2dragon = False
        code2nack  = code2nack + '40Z-'      
                        
    # ---------------------------------------------------------------------------------------------
    # QUEUE FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4queue2copyimagetobuffer4dragon = False
    queue2copyimagetobuffer4dragon = None
    message2print = "{(Step 41A/70) trying" + "queue2copyimagetobuffer4dragon-trying-by-copy2imagetobuffer4dragon: " + str(queue2copyimagetobuffer4dragon)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        queue2copyimagetobuffer4dragon = task4copyimagetobuffer2dragon ['QUEUE']

        message2print = "{(Step 41B/70) got" + "; queue2copyimagetobuffer4dragon-got-by-copy2imagetobuffer4dragon: " + str(queue2copyimagetobuffer4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <queue4xi.Queue4Xi object at 0x0000000004465E48>
              
        ack4queue2copyimagetobuffer4dragon = True
        control2copyimagetobuffer4dragon['QUEUE']  = queue2copyimagetobuffer4dragon        
        code2ack  = code2ack + '41C-'
    except:
        ack4queue2copyimagetobuffer4dragon = False  
        code2nack  = code2nack + '41Z-'
    
    
    # -----------------------------------------------------------------------------------------    
    # DUMMY PROCESS OBJECT = THREAD CREATION
    # -----------------------------------------------------------------------------------------           
    # dummy2image4dragon = None
    # message2print = "{(Step 69A/70) trying" + "dummy2image4dragon-trying-by-copy2imagetobuffer4dragon: " + str(dummy2image4dragon)  + "; time: " + time4xi() + "}"   
    # print4xi(class2print, method2print, message2print)     
    # try:                            
        # dummy2image4dragon = DummyProcess4Xi(target=manoeuvre2copyimagetobuffer4dragon,                                               # in this module to handle queue and pipe channels
        #                                   args =(queue2copyimagetobuffer4dragon,                                                         # caller puts and gets - does NOT call any functions
        #                                          consumerchannel2copyimagetobuffer4dragon, publisherchannel2copyimagetobuffer4dragon))                        # caller sends and receives - does NOT call any functions
        # message2print = "{(Step 69B/70) created" + "dummy2image4dragon-created-by-copy2imagetobuffer4dragon: " + str(dummy2image4dragon)  + "; time: " + time4xi() + "}" 
        # print4xi(class2print, method2print, message2print)  
        # code2ack  = code2ack + '69B-'
        
    # except:
        # code2nack  = code2nack + '69Z-'
        
    # ---------------------------------------------------------------------------------------------
    # RETURN CONTROL AND COMMAND DICTIONARY
    # ---------------------------------------------------------------------------------------------          
    message2print = "{(Step 70A/70) set" + "; code2ack-set-by-copy2imagetobuffer4dragon: " + str(code2ack) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                             
    # 1B1-1B2-1C-2B-3B-4B-5B-6B-7B-8B-9B-11B-12B-13B-14B-16B-17A-40D-41C- 
    message2print = "{(Step 70B/70) set" + "; code2nack-set-by-copy2imagetobuffer4dragon: " + str(code2nack) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #  
        
    message2print = "{(Step 70C/70) returning" + "; control2copyimagetobuffer4dragon-returning-by-copy2imagetobuffer4dragon: " + str(control2copyimagetobuffer4dragon) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004465E48>,
    #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 664>,
    #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 660>}; time: 01:28:44 10/05/11 Pacific Daylight Time}

        
    return control2copyimagetobuffer4dragon
                    
#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "copy2buffertoimage4dragon" usage = "
#                                                                                                              
#   # ---------------------------------------------------------------------------------------------
#   # (80) ESTABLISH CONTROL & COMMAND WITH COPYBUFFERTOIMAGE4DRAGON                      
#   # ---------------------------------------------------------------------------------------------    
#   print('(Step 80A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL & COMMAND WITH COPYBUFFERTOIMAGE4DRAGON ', time4xi())    
#   response4copy2buffertoimage = {} 
#   task4copy2buffertoimage = None
#   try:                               
#       response4copy2buffertoimage = copy2buffertoimage4dragon(task4copy2buffertoimage)                                                                                                          
#       print('(Step 80B/700) OF DRAGON2IMAGE4XI5TEST - ESTABLISHED CONTROL & COMMAND WITH COPYBUFFERTOIMAGE4DRAGON : ', response4copy2buffertoimage)
#       queue2copybuffertoimage4dragon      = response4copy2buffertoimage['QUEUE']
#       channel2copybuffertoimagepublish4dragon = response4copy2buffertoimage['PUBLISHERCHANNEL']
#       channel2copybuffertoimageconsume4dragon = response4copy2buffertoimage['CONSUMERCHANNEL']
#   except:
#       print('(Step 80Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL & COMMAND WITH COPYBUFFERTOIMAGE4DRAGON ', time4xi()) 
#   # --------------------------------------------------------------------------------------------------
#   # (81) SEND CONTROL & COMMAND COPY VIA PIPE FOR MANOEUVRE2COPYIMAGETOBUFFER4DRAGON TO RECEIVE                 
#   # --------------------------------------------------------------------------------------------------      
#   print('(Step 81A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH COPYBUFFERTOIMAGE4DRAGON ', time4xi())     
#   try:                               
#       message2copybuffertoimagesend4dragon = "<copy4xi>copy2copybuffertoimage4dragon</copy4xi>"
#       channel2copybuffertoimagepublish4dragon = response4copy2buffertoimage['PUBLISHERCHANNEL']
#       channel2copybuffertoimagepublish4dragon.send(message2copybuffertoimagesend4dragon)
#       print("(Step 81B/700) OF DRAGON2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4DRAGON: ", channel2copybuffertoimagepublish4dragon, " MESSAGE2SEND4DRAGON: " , message2copybuffertoimagesend4dragon)                                                                                                
#   except:
#       print('(Step 81Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH COPYBUFFERTOIMAGE4DRAGON ', time4xi())                                                
#   # ------------------------------------------------------------------------------------------
#   # (82) LET MANOEUVRE2COPYIMAGETOBUFFER4DRAGON TO RECEIVE COPY AND AND SEND ROGER VIA PIPE                
#   # ------------------------------------------------------------------------------------------       
#   print("(Step 82C/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4DRAGON ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4dragon = manoeuvre2copybuffertoimage4dragon(queue2copybuffertoimage4dragon, channel2copybuffertoimageconsume4dragon, channel2copybuffertoimagepublish4dragon)
#       print("(Step 82D/700) OF DRAGON2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4DRAGON ", manoeuvre2copybuffertoimage4dragon)
#   except:
#       print("(Step 82Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4DRAGON ", time4xi())     
#   else:
#       pass # trust activated thread    
#   # ---------------------------------------------------------------------------------------------
#   # (83) RECEIVE CONTROL & COMMAND ROGER VIA PIPE                   
#   # ---------------------------------------------------------------------------------------------          
#   print('(Step 83A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH COPYBUFFERTOIMAGE4DRAGON ', time4xi())     
#   try:
#       channel2copybuffertoimageconsume4dragon = response4copy2buffertoimage['CONSUMERCHANNEL']                              
#       message2copyimagereceive4dragon = channel2copybuffertoimageconsume4dragon.recv()
#       print("(Step 83B/700) OF DRAGON2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4DRAGON: ", channel2copybuffertoimageconsume4dragon, " CONTROL & COMMAND: ", message2copyimagereceive4dragon)                                                                                                  
#       # 
#   except:
#       print('(Step 83Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH COPYBUFFERTOIMAGE4DRAGON ', time4xi())         
#   # ---------------------------------------------------------------------------------------------------
#   # (84) SEND CONTROL & COMMAND READY VIA PIPE FOR MANOEUVRE2COPYIMAGETOBUFFER4DRAGON TO RECEIVE                
#   # ---------------------------------------------------------------------------------------------------      
#   print('(Step 84A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH COPYBUFFERTOIMAGE4DRAGON ', time4xi())     
#   try:                               
#       message2copybuffertoimagesend4dragon = "<copy4xi>copy2copybuffertoimage4dragon</copy4xi>"
#       channel2copybuffertoimagepublish4dragon = response4copy2buffertoimage['PUBLISHERCHANNEL']
#       channel2copybuffertoimagepublish4dragon.send(message2copybuffertoimagesend4dragon)
#       print("(Step 84B/700) OF DRAGON2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4DRAGON: ", channel2copybuffertoimagepublish4dragon, " MESSAGE2SEND4DRAGON: " , message2copybuffertoimagesend4dragon)                                                                                                
#       #  
#   except:
#        print('(Step 84Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH COPYBUFFERTOIMAGE4DRAGON ', time4xi())                                                
#   # -----------------------------------------------------------------------------------
#   # (85) LET MANOEUVRE2COPYIMAGETOBUFFER4DRAGON TO RECEIVE READY AND TO SEND READY                
#   # -----------------------------------------------------------------------------------       
#   print("(Step 85A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4DRAGON ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4dragon = manoeuvre2copybuffertoimage4dragon(queue2copybuffertoimage4dragon, channel2copybuffertoimageconsume4dragon, channel2copybuffertoimagepublish4dragon)
#       print("(Step 85B/700) OF DRAGON2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4DRAGON ", manoeuvre2copybuffertoimage4dragon)
#   except:
#       print("(Step 85Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4DRAGON ", time4xi())     
#   # ---------------------------------------------------------------------------------------
#   # (86) RECEIVE CONTROL & COMMAND READY VIA PIPE                      
#   # ---------------------------------------------------------------------------------------          
#   print('(Step 86A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH COPYBUFFERTOIMAGE4DRAGON ', time4xi())     
#   try:
#       channel2copybuffertoimageconsume4dragon = response4copy2buffertoimage['CONSUMERCHANNEL']                              
#       message2copyimagereceive4dragon = channel2copybuffertoimageconsume4dragon.recv()
#       print("(Step 86B/700) OF DRAGON2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4DRAGON: ", channel2copybuffertoimageconsume4dragon, " CONTROL & COMMAND: ", message2copyimagereceive4dragon)                                                                                                  
#       # 
#   except:
#       print('(Step 86Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH COPYBUFFERTOIMAGE4DRAGON ', time4xi())          
#   # -----------------------------------------------------------------------------------------
#   # (87) PUT IMAGE PATH TO QUEUE
#   # -----------------------------------------------------------------------------------------                          
#   print("(Step 87A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO PUT IMAGE PATH TO QUEUE ", time4xi())      
#   try:    
#       queue2copybuffertoimage4dragon.put(path2dragon)             
#       print("(Step 87B/700) OF DRAGON2IMAGE4XI5TEST - PUT TO QUEUE IMAGE PATH: ", path2dragon) 
#       # ..//..//..//..//..//web//images//png//dragonxi.png  
#   except:
#       print("(Step 87Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO PUT IMAGE PATH TO QUEUE ", time4xi()) 
#   # ------------------------------------------------------------------------------------------------------
#   # (88) LET MANOEUVRE2COPYIMAGETOBUFFER4DRAGON TO GET FROM QUEUE AND TO SEND EITHER ACK OR NACK VIA PIPE               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 88A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4DRAGON ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4dragon = manoeuvre2copybuffertoimage4dragon(queue2copybuffertoimage4dragon, channel2copybuffertoimageconsume4dragon, channel2copybuffertoimagepublish4dragon)
#       print("(Step 88B/700) OF DRAGON2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4DRAGON ", manoeuvre2copybuffertoimage4dragon)
#   except:
#       print("(Step 88Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4DRAGON ", time4xi())     
#   # -----------------------------------------------------------------------------------------------
#   # (89) RECEIVE ACK OR NACK ABOUT IMAGE OPERATION VIA PIPE                
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 89A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())      
#   try: 
#       message2copyimagereceive4dragon = channel2copybuffertoimageconsume4dragon.recv()
#       print("(Step 89D/700) OF DRAGON2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4DRAGON: ", channel2copybuffertoimageconsume4dragon, " CONTROL & COMMAND: ", message2copyimagereceive4dragon)                                                                                                   
#   except:
#       print('(Step 89Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())               
#   # -------------------------------------------------------------------------------------------------------
#   # (90) LET MANOEUVRE2COPYIMAGETOBUFFER4DRAGON GO TO STATE 0               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 90A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4DRAGON ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4dragon = manoeuvre2copybuffertoimage4dragon(queue2copybuffertoimage4dragon, channel2copybuffertoimageconsume4dragon, channel2copybuffertoimagepublish4dragon)
#       print("(Step 90B/700) OF DRAGON2IMAGE4XI5TEST - GOT RESPONSE2MANOEUVRE4DRAGON ", response2manoeuvreimagecopytobuffer4dragon)
#   except:
#       print("(Step 90Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4DRAGON ", time4xi())   

#">

# -------------------------------------------------------------------------------------------------   
def copy2buffertoimage4dragon(task4copybuffertoimage2dragon=None):

    '''
    copies buffer to image for DRAGON
    '''
    global dragon2image4xi                                                                                             # 2D-IMAGES FOR DRAGON  
    global dragon2buffer4xi                                                                                            # BUFFER FOR DRAGON
    
    global control2copybuffertoimage4dragon                                                                            # COMMAND AND CONTROL FOR DRAGON IMAGES 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "copy2buffertoimage4dragon"
    method2print = "copy2buffertoimage4dragon"

    message2print = "{(Step 1A/33) started" + "; copy2buffertoimage4dragon-started: " + str(copy2buffertoimage4dragon)  + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                                
    # <function copy2buffertoimage4dragon at 0x0000000003869548>
    
    message2print = "{(Step 1B/33) passed" + "; task4copybuffertoimage2dragon-to-copy2buffertoimage4dragon: " + str(task4copybuffertoimage2dragon) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None
            
    code2ack  = ' '
    code2nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # CREATE DRAGON2IMAGE4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if dragon2image4xi is None:
            dragon2image4xi = create2image4dragon()                                                                              # INITIALIZE IMAGE FOR DRAGON 
            message2print = "{(Step 1C/70) created" + "; dragon2image4xi-created-by-copy2buffertoimage4dragon to: " + str(dragon2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create2image4dragon-skipped-by-copy2buffertoimage4dragon: " + str(copy2buffertoimage4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # skipped                                  
    except:
        try:
            dragon2image4xi = create2image4dragon()
            message2print = "{(Step 1E/70) set" + "; dragon2image4xi-set-by-copy2buffertoimage4dragon to: " + str(dragon2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            dragon2image4xi = {}
            message2print = "{(Step 1Z1/70) yielded" + "; dragon2image4xi-yielded-by-copy2buffertoimage4dragon: " + str(dragon2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z1-' 
    
    # -----------------------------------------------------------------------------------------
    # CREATE DRAGON2BUFFER4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if dragon2buffer4xi is None:
            dragon2buffer4xi = create2buffer4dragon()                                                                     # INITIALIZE BUFFER FOR DRAGON 
            message2print = "{(Step 1F/70) created" + "; dragon2buffer4xi-created-by-copy2buffertoimage4dragon to: " + str(dragon2buffer4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1G/70) skipped" + "; create2buffer4dragon-skipped-by-copy2buffertoimage4dragon: " + str(copy2buffertoimage4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # skipped                                  
    except:
        try:
            dragon2buffer4xi = create2buffer4dragon()
            message2print = "{(Step 1H/70) set" + "; dragon2buffer4xi-set-by-copy2buffertoimage4dragon to: " + str(dragon2buffer4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            dragon2buffer4xi = {}
            message2print = "{(Step 1Z2/70) yielded" + "; dragon2buffer4xi-yielded-by-copy2buffertoimage4dragon: " + str(dragon2buffer4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z2-' 
                            
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - ANALYZE TASKS           
    # -----------------------------------------------------------------------------------------          
    if task4copybuffertoimage2dragon is None:
        # -----------------------------------------------------------------------------------------
        # NO PARAMETERS GIVEN BY CALLER FOR COPY - BUILD TASK! 
        # -----------------------------------------------------------------------------------------    
        
        task4copybuffertoimage2dragon = {}                                                                                                                                                                                       
        
        message2print = "{(Step 1A/70) initialized" + "; task4copybuffertoimage2dragon-created-by-copy2buffertoimage4dragon to: " + str(task4copybuffertoimage2dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)               
        # 
        
        # ---------------------------------------------------------------------------------------------
        # (1B) TRACE - AUTOMATIC                      
        # ---------------------------------------------------------------------------------------------              
        function2callback4dragon = trace2image4dragon                                                                 # CALLBACK FUNCTION FOR DRAGON
        
        message2print = "{(Step 1B1/70) initialized" + "; function2callback4dragon-created-by-copy2buffertoimage4dragon to: " + str(function2callback4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <function trace2image4dragon at 0x0000000004645548>
        
        code2ack  = code2ack + '1B1-'
       
        args2callback = {}                                                                                      # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
        args2callback['CLASS']   = "copy2buffertoimage4dragon"                                                     # MODULE OF DRAGONIZED OPENCL MODULE
        args2callback['METHOD']  = "copy2buffertoimage4dragon"                                                     # FUNCTION OF DRAGONIZED OPENCL MODULE
        args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; copy2buffertoimage4dragon-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR DRAGONIZED OPENCL TO RETURN    

        message2print = "{(Step 1B2/70) initialized" + "; args2callback-created-by-copy2buffertoimage4dragon to: " + str(args2callback)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)               
        # {'MESSAGE': '{(Step 1Z1/33) failed; copy2buffertoimage4dragon-failed ; time: 04:41:15 10/04/11 Pacific Daylight Time}',
        #  'CLASS': 'copy2buffertoimage4dragon', 
        #  'METHOD': 'copy2buffertoimage4dragon'}; time: 04:41:15 10/04/11 Pacific Daylight Time}

        code2ack  = code2ack + '1B2-'
            
        # ---------------------------------------------------------------------------------------------
        # (1C) CALLBACK - AUTOMATIC                      
        # ---------------------------------------------------------------------------------------------              
        task4copybuffertoimage2dragon['DATA2CALLBACK'] = args2callback                                             # USER DATA PASSED TO DRAGONIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
        task4copybuffertoimage2dragon['FUNCTION2CALLBACK'] = function2callback4dragon                                 # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
        code2ack  = code2ack + '1C-'
        message2print = "{(Step 1C/70) initialized" + "; task4copybuffertoimage2dragon-created-by-copy2buffertoimage4dragon to: " + str(task4copybuffertoimage2dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # {'FUNCTION2CALLBACK': <function trace2image4dragon at 0x0000000004645548>, 
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; copy2buffertoimage4dragon-failed ; time: 04:41:15 10/04/11 Pacific Daylight Time}', 
        #  'CLASS': 'copy2buffertoimage4dragon', 
        #  'METHOD': 'copy2buffertoimage4dragon'}}; time: 04:41:15 10/04/11 Pacific Daylight Time}

            
        # ---------------------------------------------------------------------------------------------
        # (2) PLATFORM - AUTOMATIC                   
        # ---------------------------------------------------------------------------------------------             
        platform2xi = {}
        message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-copy2buffertoimage4dragon to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # {}
            
        try:
            platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
            message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-copy2buffertoimage4dragon: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'list2platforms': ['DragonXi'], 
            #  'ack2return': ' 2C-2E-2F-2G-', 
            #  'nack2return': ' ', 
            #  'available2platform': 1, 
            #  'max2entries': 1}; time: 04:41:15 10/04/11 Pacific Daylight Time}
                 
            code2ack  = code2ack + '2B-'
        except:
            message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-copy2buffertoimage4dragon: " + str(failed)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                                
            code2nack  = code2nack + '2Z-'
                 
        task4copybuffertoimage2dragon['PLATFORM'] = platform2xi                                                    # PLATFORM OF SYSTEM 
              
            
        # ---------------------------------------------------------------------------------------------
        # (3) CONTEXT - AUTOMATIC                 
        # ---------------------------------------------------------------------------------------------    
        context2dragon = {}
        message2print = "{(Step 3A/70) trying" + "; context2dragon-trying-by-copy2buffertoimage4dragon: " + str(context2dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            context2dragon = get2context4dragon()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
            message2print = "{(Step 3B/70) got" + "; context2dragon-got-by-copy2buffertoimage4dragon: " + str(context2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': ...
                
            code2ack  = code2ack + '3B-'         
        except:
            message2print = "{(Step 3Z/70) failed" + "; context2dragon-failed-by-copy2buffertoimage4dragon: " + str(context2dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z-'
                
        task4copybuffertoimage2dragon ['CONTEXT'] = context2dragon           
            
        # ---------------------------------------------------------------------------------------------
        # (4) DEVICE - AUTOMATIC                       
        # ---------------------------------------------------------------------------------------------    
        device2dragon = {}
            
        message2print = "{(Step 4A/70) trying" + "; device2dragon-trying-by-copy2buffertoimage4dragon: " + str(device2dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            device2dragon = get2dragon4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
            
            message2print = "{(Step 4B/70) got" + "; device2dragon-got-by-copy2buffertoimage4dragon: " + str(device2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'RESOLUTION2TIMER4PROFILING': 318, 
            #  'ID2PLATFORM': '0000000001509568', 
            #  'PROFILING2QUEUE': 'ACK', 
            #  'EXECUTE2KERNELS4OPENCL': 'ACK', 
            #  'NATIVE2WIDTHLONG4VECTOR': 2, 
            #  'MAX2UNITS4COMPUTE': 4, 
            #  'SIZE2MEMORY4GLOBAL': 3221225472, ...
               
            code2ack  = code2ack + '4B-'  
        except:
            message2print = "{(Step 4Z/70) failed" + "; device2dragon-failed-by-copy2buffertoimage4dragon: " + str(device2dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z-'
            
        task4copybuffertoimage2dragon ['DEVICE'] = device2dragon
            
                   
        # ---------------------------------------------------------------------------------------------
        # (5) IMAGE DESTINATION OBJECT - REQUIRED                         
        # ---------------------------------------------------------------------------------------------         
        imagedestinationinfo2dragon = {}
             
        message2print = "{(Step 5A/70) trying" + "; imagedestinationinfo2dragon-trying-by-copy2buffertoimage4dragon: " + str(imagedestinationinfo2dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            imagedestinationinfo2dragon = get2imageinfo4dragon()                                                                         
            
            message2print = "{(Step 5B/70) got" + "; imagedestinationinfo2dragon-got-by-copy2buffertoimage4dragon: " + str(imagedestinationinfo2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'SIZE2QUEUE': 2147483646, 
            #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
            #  'ORDER': 'CL_RGBA'}, 
            #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004105358>, 
            #  'WIDTH': 800, 
            #  'CONTEXT': {'COUNT2REFERENCE': 0, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4dragon-failed ; time: 04:41:15 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'create2context4dragon', 
            #  'METHOD': 'create2context4dragon'}, 
            #  'FUNCTION2CALLBACK': <function trace2context4dragon at 0x00000000037FF348>, 
            #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-', 
            #  'INFO2PRIVATE': None, 
            #  'PLATFORM': {'list2platforms': ['DragonXi'],'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
            #  'NACK2RETURN': ' ', 
            #  'SIZE2PRIVATE': 0, 
            #  'PROPERTIES': None}, 
            #  'PIPE': (<read-write PipeConnection, handle 524>, <read-write PipeConnection, handle 528>), 
            #  'COUNT2REFERENCE': 1, 
            #  'FUNCTION2CALLBACK': <function trace2image4dragon at 0x0000000004645548>, 
            #  'COMMANDQUEUE': ...
            
            code2ack  = code2ack + '5B-'  
        except:
            message2print = "{(Step 5Z/70) failed" + "; imagedestinationinfo2dragon-failed-by-copy2buffertoimage4dragon: " + str(imagedestinationinfo2dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '5Z-'          
                  
        task4copybuffertoimage2dragon ['IMAGEDESTINATION'] = imagedestinationinfo2dragon         
               
        # ---------------------------------------------------------------------------------------------
        # (6) BUFFER SOURCE OBJECT - REQUIRED                        
        # ---------------------------------------------------------------------------------------------                         
        buffersourceinfo2dragon = {}
         
        message2print = "{(Step 6A/70) trying" + "; buffersourceinfo2dragon-trying-by-copy2buffertoimage4dragon: " + str(buffersourceinfo2dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            buffersourceinfo2dragon = get2bufferinfo4dragon()                                                                         
            
            message2print = "{(Step 6B/70) got" + "; buffersourceinfo2dragon-got-by-copy2buffertoimage4dragon: " + str(buffersourceinfo2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'CODE2ERROR': 0, 
            #  'COUNT2REFERENCE': 0, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4dragon-failed ; time: 04:41:15 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'create2buffer4dragon', 
            #  'METHOD': 'create2buffer4dragon'}, 
            #  'FUNCTION2CALLBACK': <function trace2buffer4dragon at 0x0000000004631B48>, 
            #  'POINTER2BUFFER': <mmap.mmap object at 0x00000000040EBD98>, 
            #  'FLAGS': 0, 
            #  'CONTEXT': ...
            
            code2ack  = code2ack + '6B-'  
        except:
            message2print = "{(Step 6Z/70) failed" + "; buffersourceinfo2dragon-failed-by-copy2buffertoimage4dragon: " + str(buffersourceinfo2dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '6Z-'          
                  
        task4copybuffertoimage2dragon ['BUFFERSOURCE'] = buffersourceinfo2dragon     
        
        # ---------------------------------------------------------------------------------------------
        # (7) QUEUE - REQUIRED BY DRAGONXI                          
        # ---------------------------------------------------------------------------------------------         
        queue2copybuffertoimage4dragon = None
        
        message2print = "{(Step 7A/70) trying" + "; queue2copybuffertoimage4dragon-trying-by-copy2buffertoimage4dragon: " + str(queue2copybuffertoimage4dragon)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        try:
            queue2copybuffertoimage4dragon = Queue4Xi()                                                                              # THE SYSTEM CREATES QUEUE    
            
            message2print = "{(Step 7B/70) got" + "; queue2copybuffertoimage4dragon-got-by-copy2buffertoimage4dragon: " + str(queue2copybuffertoimage4dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # <queue4xi.Queue4Xi object at 0x000000000492B208> 
            
            code2ack  = code2ack + '7B-'  
        except:
            message2print = "{(Step 7Z/70) failed" + "; queue2copybuffertoimage4dragon-failed-by-copy2buffertoimage4dragon: " + str(queue2copybuffertoimage4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '7Z-'
            
        task4copybuffertoimage2dragon ['QUEUE'] = queue2copybuffertoimage4dragon
         
        # ---------------------------------------------------------------------------------------------
        # (8) PIPE - REQUIRED BY DRAGONXI                              
        # ---------------------------------------------------------------------------------------------         
        pipe2dragon = None
       
        message2print = "{(Step 8A/70) trying" + "; pipe2dragon-trying-by-copy2buffertoimage4dragon: " + str(pipe2dragon)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        try:
            pipe2dragon = Pipe4Xi()                                                                                # THE SYSTEM CREATES PIPE    
            
            message2print = "{(Step 8B/70) got" + "; pipe2dragon-got-by-copy2buffertoimage4dragon: " + str(pipe2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # (<read-write PipeConnection, handle 708>, <read-write PipeConnection, handle 712>) 
            
            code2ack  = code2ack + '8B-'  
        except:
            message2print = "{(Step 8Z/70) failed" + "; pipe2dragon-failed-by-copy2buffertoimage4dragon: " + str(pipe2dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '8Z-'
            
        task4copybuffertoimage2dragon ['PIPE'] = pipe2dragon 
        
        # ---------------------------------------------------------------------------------------------
        # (9) COMMAND QUEUE - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        commandqueueinfo2dragon = {}
       
        message2print = "{(Step 9A/70) trying" + "; commandqueueinfo2dragon-trying-by-copy2buffertoimage4dragon: " + str(commandqueueinfo2dragon)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)  
                    
        try:
            commandqueueinfo2dragon = get2commandqueueinfo4dragon()                                                   # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
            message2print = "{(Step 9B/70) got" + "; commandqueueinfo2dragon-got-by-copy2buffertoimage4dragon: " + str(commandqueueinfo2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': ...
            
            code2ack  = code2ack + '9B-'  
        except:
            message2print = "{(Step 9Z/70) failed" + "; commandqueueinfo2dragon-failed-by-copy2buffertoimage4dragon: " + str(commandqueueinfo2dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '9Z-'
    
        task4copybuffertoimage2dragon ['COMMANDQUEUE'] = commandqueueinfo2dragon
        
        
        
        # ---------------------------------------------------------------------------------------------
        # (11) EVENT - REQUIRED                            
        # ---------------------------------------------------------------------------------------------             
        eventinfo2dragon = {}
       
        message2print = "{(Step 11A/70) trying" + "; eventinfo2dragon-trying-by-copy2buffertoimage4dragon: " + str(eventinfo2dragon)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            eventinfo2dragon = get2eventinfo4dragon()                                                                    
            message2print = "{(Step 11B/70) got" + "; eventinfo2dragon-got-by-copy2buffertoimage4dragon: " + str(eventinfo2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CODE2ERROR': 0, 'DATA2CALLBACK': ...
            
            code2ack  = code2ack + '11B-' 
        except:
            message2print = "{(Step 11Z/70) failed" + "; eventinfo2dragon-failed-by-copy2buffertoimage4dragon: " + str(eventinfo2dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '11Z-'
    
        task4copybuffertoimage2dragon ['EVENT'] = eventinfo2dragon 
        
        
        # ---------------------------------------------------------------------------------------------
        # (12) EVENTWAITLIST - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        eventwaitlist2dragon = {}
       
        message2print = "{(Step 12A/70) trying" + "; eventwaitlist2dragon-trying-by-copy2buffertoimage4dragon: " + str(eventwaitlist2dragon)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            eventwaitlist2dragon = {}                                                                   
            message2print = "{(Step 12B/70) set" + "; eventwaitlist2dragon-set-by-copy2buffertoimage4dragon: " + str(eventwaitlist2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {}
            
            code2ack  = code2ack + '12B-' 
        except:
            message2print = "{(Step 12Z/70) failed" + "; eventwaitlist2dragon-failed-by-copy2buffertoimage4dragon: " + str(eventwaitlist2dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '12Z-'
    
        task4copybuffertoimage2dragon ['EVENTWAITLIST'] = eventwaitlist2dragon                     
         
        # ---------------------------------------------------------------------------------------------
        # (13) EVENTWAITLISTQUANTITY - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        eventwaitlistquantity2dragon = 0
       
        message2print = "{(Step 13A/70) trying" + "; eventwaitlistquantity2dragon-trying-by-copy2buffertoimage4dragon: " + str(eventwaitlistquantity2dragon)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            keys2event = eventwaitlist2dragon.keys()
            for key2event in keys2event:
                eventwaitlistquantity2dragon = eventwaitlistquantity2dragon + 1                                                                     
            message2print = "{(Step 13B/70) set" + "; eventwaitlistquantity2dragon-set-by-copy2buffertoimage4dragon: " + str(eventwaitlistquantity2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 0 
            
            code2ack  = code2ack + '13B-' 
        except:
            message2print = "{(Step 13Z/70) failed" + "; eventwaitlistquantity2dragon-failed-by-copy2buffertoimage4dragon: " + str(eventwaitlistquantity2dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '13Z-'
    
        task4copybuffertoimage2dragon ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2dragon       
                                             
        
        # ---------------------------------------------------------------------------------------------
        # (15) ORIGIN FOR DESTINATION - REQUIRED                             
        # ---------------------------------------------------------------------------------------------              
        origin4destination2dragon = {}
        
        message2print = "{(Step 15A/70) trying" + "; origin4destination2dragon-trying-by-copy2buffertoimage4dragon: " + str(origin4destination2dragon)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:                                                                                    
            origin4destination2dragon['X'] = 100 
            origin4destination2dragon['Y'] = 100  
            origin4destination2dragon['Z'] = 100                                                                                                                          
            message2print = "{(Step 15B/70) set" + "; origin4destination2dragon-set-by-copy2buffertoimage4dragon to: " + str(origin4destination2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'Y': 100, 'X': 100, 'Z': 100}
            code2ack  = code2ack + '15B-' 
        except:
            message2print = "{(Step 15Z/70) failed" + "; origin4destination2dragon-failed-by-copy2buffertoimage4dragon: " + str(origin4destination2dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '15Z-'
    
        task4copybuffertoimage2dragon ['DESTINATIONORIGIN'] = origin4destination2dragon
                    
        
        
        # --------------------------------------------------------------------------------------------------------
        # (16) REGION - REGION OF A IMAGE TO BE COPIED                         
        # --------------------------------------------------------------------------------------------------------             
        region4copy2dragon = {}                                                                                                         
       
        message2print = "{(Step 16A/70) trying" + "; region4copy2dragon-trying-by-copy2buffertoimage4dragon: " + str(region4copy2dragon)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            region4copy2dragon['X'] = 10 
            region4copy2dragon['Y'] = 10  
            region4copy2dragon['Z'] = 10                                                                           
            message2print = "{(Step 16B/70) set" + "; region4copy2dragon-set-by-copy2buffertoimage4dragon to: " + str(region4copy2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'Y': 10, 'X': 10, 'Z': 10}
            code2ack  = code2ack + '16B-'                
             
        except:
            message2print = "{(Step 16Z/70) failed" + "; region4copy2dragon-failed-by-copy2buffertoimage4dragon: " + str(region4copy2dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '16Z-'
        
        task4copybuffertoimage2dragon ['REGION'] = region4copy2dragon              
        
        # --------------------------------------------------------------------------------------------------------
        # (17) OFFSET - WHERE TO BEGIN COPYING DATA FROM SOURCE BUFFER                   
        # --------------------------------------------------------------------------------------------------------             
        offset4copy2dragon = 0                                                                                                         
        message2print = "{(Step 17A/70) set" + "; offset4copy2dragon-set-by-copy2buffertoimage4dragon to: " + str(offset4copy2dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 0
        
        task4copybuffertoimage2dragon ['OFFSET'] = offset4copy2dragon            
                                
        # ---------------------------------------------------------------------------------------------
        # EOF PARAMETER PROVIDING                
        # ---------------------------------------------------------------------------------------------                                                                                                                               
        
        # ---------------------------------------------------------------------------------------------
        # ANALYZE TASKS - WHETHER AUTOMATICALLY GENERATED OR USER GIVEN 
        # ---------------------------------------------------------------------------------------------
        keys2task = task4copybuffertoimage2dragon.keys()
        message2print = "{(Step 20/70) got" + "; keys2task-got-by-copy2buffertoimage4dragon: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BUFFERSOURCE',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DESTINATIONORIGIN',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGEDESTINATION',
        #  'OFFSET',
        #  'PIPE',
        #  'PLATFORM',
        #  'QUEUE',
        #  'REGION']; time: 04:41:15 10/04/11 Pacific Daylight Time}
 
        if (('DATA2CALLBACK' in keys2task) and ('FUNCTION2CALLBACK' in keys2task)):
            pass
        else:                
            # ---------------------------------------------------------------------------------------------
            # (21B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4dragon = trace2image4dragon                                                             # CALLBACK FUNCTION FOR DRAGON
            
            message2print = "{(Step 21B1/70) initialized" + "; function2callback4dragon-created-by-copy2buffertoimage4dragon to: " + str(function2callback4dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code2ack  = code2ack + '21B1-'
           
            args2callback = {}                                                                                  # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "copy2buffertoimage4dragon"                                                        # MODULE OF DRAGONIZED OPENCL MODULE
            args2callback['METHOD']  = "copy2buffertoimage4dragon"                                                        # FUNCTION OF DRAGONIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; copy2buffertoimage4dragon-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR DRAGONIZED OPENCL TO RETURN    

            message2print = "{(Step 21B2/70) initialized" + "; args2callback-created-by-copy2buffertoimage4dragon to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 

            code2ack  = code2ack + '21B2-'
        
            # ---------------------------------------------------------------------------------------------
            # (21C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4copybuffertoimage2dragon['DATA2CALLBACK'] = args2callback                                                # USER DATA PASSED TO DRAGONIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4copybuffertoimage2dragon['FUNCTION2CALLBACK'] = function2callback4dragon                                    # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '21C-'
            message2print = "{(Step 21C/70) initialized" + "; task4copybuffertoimage2dragon-created-by-copy2buffertoimage4dragon to: " + str(task4copybuffertoimage2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #
             
        if 'PLATFORM' in keys2task:
            pass
        else:  
            # ---------------------------------------------------------------------------------------------
            # (22) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 22A/70) initialized" + "; platform2xi-created-by-copy2buffertoimage4dragon to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            
            try:
                platform2xi = get2platformids4xi()                                                              # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 22B/70) got" + "; platform2xi-got-by-copy2buffertoimage4dragon: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                 
                code2ack  = code2ack + '22B-'
            except:
                message2print = "{(Step 22Z/70) failed" + "; platform2xi-failed-by-copy2buffertoimage4dragon: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '22Z-'
                 
            task4copybuffertoimage2dragon['PLATFORM'] = platform2xi                                                       # PLATFORM OF SYSTEM 
          
        if 'CONTEXT' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (23) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2dragon = {}
            message2print = "{(Step 23A/70) trying" + "; context2dragon-trying-by-copy2buffertoimage4dragon: " + str(context2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2dragon = get2context4dragon()                                                                 # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 23B/70) got" + "; context2dragon-got-by-copy2buffertoimage4dragon: " + str(context2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '23B-'         
            except:
                message2print = "{(Step 23Z/70) failed" + "; context2dragon-failed-by-copy2buffertoimage4dragon: " + str(context2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '23Z-'
                
            task4copybuffertoimage2dragon ['CONTEXT'] = context2dragon           
        
        if 'DEVICE' in keys2task:
            pass
        else:                   
            # ---------------------------------------------------------------------------------------------
            # (24) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2dragon = {}
            
            message2print = "{(Step 24A/70) trying" + "; device2dragon-trying-by-copy2buffertoimage4dragon: " + str(device2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2dragon = get2dragon4xi()                                                                       # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 24B/70) got" + "; device2dragon-got-by-copy2buffertoimage4dragon: " + str(device2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '24B-'  
            except:
                message2print = "{(Step 24Z/70) failed" + "; device2dragon-failed-by-copy2buffertoimage4dragon: " + str(device2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '24Z-'
            
            task4copybuffertoimage2dragon ['DEVICE'] = device2dragon
        
        if 'IMAGEDESTINATION' in keys2task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (25) DESTINATION IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            destinationimageinfo2dragon = {}
             
            message2print = "{(Step 25A/70) trying" + "; destinationimageinfo2dragon-trying-by-copy2buffertoimage4dragon: " + str(destinationimageinfo2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                destinationimageinfo2dragon = get2imageinfo4dragon()                                                                         
                
                message2print = "{(Step 25B/70) got" + "; destinationimageinfo2dragon-got-by-copy2buffertoimage4dragon: " + str(destinationimageinfo2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code2ack  = code2ack + '25B-'  
            except:
                message2print = "{(Step 25Z/70) failed" + "; destinationimageinfo2dragon-failed-by-copy2buffertoimage4dragon: " + str(destinationimageinfo2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '25Z-'          
                      
            task4copybuffertoimage2dragon ['IMAGEDESTINATION'] = destinationimageinfo2dragon         
        
        if 'BUFFERSOURCE' in keys2task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (26) SOURCE BUFFER - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            sourcebufferinfo2dragon = {}
             
            message2print = "{(Step 26A/70) trying" + "; sourcebufferinfo2dragon-trying-by-copy2buffertoimage4dragon: " + str(sourcebufferinfo2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                sourcebufferinfo2dragon = get2bufferinfo4dragon()                                                                         
                
                message2print = "{(Step 26B/70) got" + "; sourcebufferinfo2dragon-got-by-copy2buffertoimage4dragon: " + str(sourcebufferinfo2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code2ack  = code2ack + '26B-'  
            except:
                message2print = "{(Step 26Z/70) failed" + "; sourcebufferinfo2dragon-failed-by-copy2buffertoimage4dragon: " + str(sourcebufferinfo2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '26Z-'          
                      
            task4copybuffertoimage2dragon ['BUFFERSOURCE'] = sourcebufferinfo2dragon    
                                    
        if 'QUEUE' in keys2task:
            pass
        else:   
            # ---------------------------------------------------------------------------------------------
            # (27) QUEUE - REQUIRED BY DRAGONXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2copybuffertoimage4dragon = None
            
            message2print = "{(Step 27A/70) trying" + "; queue2copybuffertoimage4dragon-trying-by-copy2buffertoimage4dragon: " + str(queue2copybuffertoimage4dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2copybuffertoimage4dragon = Queue4Xi()                                                                          # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 27B/70) got" + "; queue2copybuffertoimage4dragon-got-by-copy2buffertoimage4dragon: " + str(queue2copybuffertoimage4dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '27B-'  
            except:
                message2print = "{(Step 27Z/70) failed" + "; queue2copybuffertoimage4dragon-failed-by-copy2buffertoimage4dragon: " + str(queue2copybuffertoimage4dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '27Z-'
                
            task4copybuffertoimage2dragon ['QUEUE'] = queue2copybuffertoimage4dragon
            
        if 'PIPE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) PIPE - REQUIRED BY DRAGONXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2dragon = None
           
            message2print = "{(Step 28A/70) trying" + "; pipe2dragon-trying-by-copy2buffertoimage4dragon: " + str(pipe2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2dragon = Pipe4Xi()                                                                            # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 28B/70) got" + "; pipe2dragon-got-by-copy2buffertoimage4dragon: " + str(pipe2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '28B-'  
            except:
                message2print = "{(Step 28Z/70) failed" + "; pipe2dragon-failed-by-copy2buffertoimage4dragon: " + str(pipe2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '28Z-'
                
            task4copybuffertoimage2dragon ['PIPE'] = pipe2dragon 
            
        if 'COMMANDQUEUE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2dragon = {}
           
            message2print = "{(Step 29A/70) trying" + "; commandqueueinfo2dragon-trying-by-copy2buffertoimage4dragon: " + str(commandqueueinfo2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2dragon = get2commandqueueinfo4dragon()                                               # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 29B/70) got" + "; commandqueueinfo2dragon-got-by-copy2buffertoimage4dragon: " + str(commandqueueinfo2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '29B-'  
            except:
                message2print = "{(Step 29Z/70) failed" + "; commandqueueinfo2dragon-failed-by-copy2buffertoimage4dragon: " + str(commandqueueinfo2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '29Z-'
        
            task4copybuffertoimage2dragon ['COMMANDQUEUE'] = commandqueueinfo2dragon
        
             
        if 'EVENT' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (31) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2dragon = {}
           
            message2print = "{(Step 31A/70) trying" + "; eventinfo2dragon-trying-by-copy2buffertoimage4dragon: " + str(eventinfo2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2dragon = get2eventinfo4dragon()                                                                    
                message2print = "{(Step 31B/70) got" + "; eventinfo2dragon-got-by-copy2buffertoimage4dragon: " + str(eventinfo2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '31B-' 
            except:
                message2print = "{(Step 31Z/70) failed" + "; eventinfo2dragon-failed-by-copy2buffertoimage4dragon: " + str(eventinfo2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '31Z-'
        
            task4copybuffertoimage2dragon ['EVENT'] = eventinfo2dragon 
        
             
        if 'EVENTWAITLIST' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (32) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2dragon = {}
           
            message2print = "{(Step 32A/70) trying" + "; eventwaitlist2dragon-trying-by-copy2buffertoimage4dragon: " + str(eventwaitlist2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2dragon = {}                                                                   
                message2print = "{(Step 32B/70) set" + "; eventwaitlist2dragon-set-by-copy2buffertoimage4dragon: " + str(eventwaitlist2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '32B-' 
            except:
                message2print = "{(Step 32Z/70) failed" + "; eventwaitlist2dragon-failed-by-copy2buffertoimage4dragon: " + str(eventwaitlist2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '32Z-'
        
            task4copybuffertoimage2dragon ['EVENTWAITLIST'] = eventwaitlist2dragon                     
              
        if 'EVENTWAITLISTQUANTITY' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (33) EVENTWAITLISTQUANTITY - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2dragon = 0
           
            message2print = "{(Step 33A/70) trying" + "; eventwaitlistquantity2dragon-trying-by-copy2buffertoimage4dragon: " + str(eventwaitlistquantity2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2dragon.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2dragon = eventwaitlistquantity2dragon + 1                                                                     
                message2print = "{(Step 33B/70) set" + "; eventwaitlistquantity2dragon-set-by-copy2buffertoimage4dragon: " + str(eventwaitlistquantity2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 
                
                code2ack  = code2ack + '33B-' 
            except:
                message2print = "{(Step 33Z/70) failed" + "; eventwaitlistquantity2dragon-failed-by-copy2buffertoimage4dragon: " + str(eventwaitlistquantity2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '33Z-'
        
            task4copybuffertoimage2dragon ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2dragon       
                                             
  
             
        if 'DESTINATIONORIGIN' in keys2task:
            pass
        else:                    
            # ---------------------------------------------------------------------------------------------
            # (34) DESTINATIONORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            destination4origin2dragon = {}
            
            message2print = "{(Step 34A/70) trying" + "; destination4origin2dragon-trying-by-copy2buffertoimage4dragon: " + str(destination4origin2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                destination4origin2dragon['X'] = 0 
                destination4origin2dragon['Y'] = 0  
                destination4origin2dragon['Z'] = 0                                                                                                                          
                message2print = "{(Step 34B/70) set" + "; destination4origin2dragon-set-by-copy2buffertoimage4dragon to: " + str(destination4origin2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '34B-' 
            except:
                message2print = "{(Step 34Z/70) failed" + "; destination4origin2dragon-failed-by-copy2buffertoimage4dragon: " + str(destination4origin2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '34Z-'
        
            task4copybuffertoimage2dragon ['DESTINATIONORIGIN'] = destination4origin2dragon
             
     
             
        if 'REGION' in keys2task:
            pass
        else:            
            # ---------------------------------------------------------------------------------------------
            # (36) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2dragon = {}
            
            message2print = "{(Step 36A/70) trying" + "; region2dragon-trying-by-copy2buffertoimage4dragon: " + str(region2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                region2dragon['X'] = 160                                                                               # WIDTH  IN PIXELS TO WRITE 
                region2dragon['Y'] = 90                                                                                # HEIGHT IN PIXELS TO WRITE
                region2dragon['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO WRITE                                                                                                                          
                message2print = "{(Step 36B/70) set" + "; region2dragon-set-by-copy2buffertoimage4dragon to: " + str(region2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '36B-' 
            except:
                message2print = "{(Step 36Z/70) failed" + "; region2dragon-failed-by-copy2buffertoimage4dragon: " + str(region2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '36Z-'
        
            task4copybuffertoimage2dragon ['REGION'] = region2dragon
        
             
        if 'OFFSET' in keys2task:
            pass
        else:            
            # ---------------------------------------------------------------------------------------------
            # (37) OFFSET - WHERE TO BEGIN COPYING DATA FROM SOURCE BUFFER - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            offset2dragon = 0
            message2print = "{(Step 37A/70) set" + "; offset2dragon-set-by-copy2buffertoimage4dragon to: " + str(offset2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            task4copybuffertoimage2dragon ['OFFSET'] = offset2dragon        
            code2ack  = code2ack + '37A-' 
                    
        # ---------------------------------------------------------------------------------------------
        # EOF TASK ANALYSIS
        # ---------------------------------------------------------------------------------------------
        keys2task = task4copybuffertoimage2dragon.keys()
        message2print = "{(Step 39/70) set" + "; keys2task-set-by-copy2buffertoimage4dragon to: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BUFFERSOURCE',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DESTINATIONORIGIN',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGEDESTINATION',
        #  'OFFSET',
        #  'PIPE',
        #  'PLATFORM',
        #  'QUEUE',
        #  'REGION']; time: 04:52:11 10/04/11 Pacific Daylight Time}
                


    # ---------------------------------------------------------------------------------------------
    # CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    control2copybuffertoimage4dragon = {}       
                                
    # ---------------------------------------------------------------------------------------------
    # CONSUMER AND PUBLISHER CHANNEL FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4channels2dragon = False
    channels2dragon = ()
    message2print = "{(Step 40A/70) trying" + "channels2dragon-trying-by-copy2buffertoimage4dragon: " + str(channels2dragon)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        # 
        consumerchannel2copybuffertoimage4dragon, publisherchannel2copybuffertoimage4dragon = task4copybuffertoimage2dragon ['PIPE']

        message2print = "{(Step 40B/70) got" + "; consumerchannel2copybuffertoimage4dragon-got-by-copy2buffertoimage4dragon: " + str(consumerchannel2copybuffertoimage4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 704>

        message2print = "{(Step 40C/70) got" + "; publisherchannel2copybuffertoimage4dragon-got-by-copy2buffertoimage4dragon: " + str(publisherchannel2copybuffertoimage4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 708>                      
        ack4channels2dragon = True
        
        control2copybuffertoimage4dragon['CONSUMERCHANNEL']  = consumerchannel2copybuffertoimage4dragon
        
        
        control2copybuffertoimage4dragon['PUBLISHERCHANNEL'] = publisherchannel2copybuffertoimage4dragon
        
        code2ack  = code2ack + '40D-' 
        
    except:
        ack4channels2dragon = False
        code2nack  = code2nack + '40Z-'      
                        
    # ---------------------------------------------------------------------------------------------
    # QUEUE FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4queue2copybuffertoimage4dragon = False
    queue2copybuffertoimage4dragon = None
    message2print = "{(Step 41A/70) trying" + "queue2copybuffertoimage4dragon-trying-by-copy2buffertoimage4dragon: " + str(queue2copybuffertoimage4dragon)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        queue2copybuffertoimage4dragon = task4copybuffertoimage2dragon ['QUEUE']

        message2print = "{(Step 41B/70) got" + "; queue2copybuffertoimage4dragon-got-by-copy2buffertoimage4dragon: " + str(queue2copybuffertoimage4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <queue4xi.Queue4Xi object at 0x0000000003DF2198>
              
        ack4queue2copybuffertoimage4dragon = True
        control2copybuffertoimage4dragon['QUEUE']  = queue2copybuffertoimage4dragon        
        code2ack  = code2ack + '41C-'
    except:
        ack4queue2copybuffertoimage4dragon = False  
        code2nack  = code2nack + '41Z-'
    
    
    # -----------------------------------------------------------------------------------------    
    # DUMMY PROCESS OBJECT = THREAD CREATION
    # -----------------------------------------------------------------------------------------           
    # dummy2image4dragon = None
    # message2print = "{(Step 69A/70) trying" + "dummy2image4dragon-trying-by-copy2buffertoimage4dragon: " + str(dummy2image4dragon)  + "; time: " + time4xi() + "}"   
    # print4xi(class2print, method2print, message2print)     
    # try:                            
        # dummy2image4dragon = DummyProcess4Xi(target=manoeuvre2copybuffertoimage4dragon,                                               # in this module to handle queue and pipe channels
        #                                   args =(queue2copybuffertoimage4dragon,                                                         # caller puts and gets - does NOT call any functions
        #                                          consumerchannel2copybuffertoimage4dragon, publisherchannel2copybuffertoimage4dragon))                        # caller sends and receives - does NOT call any functions
        # message2print = "{(Step 69B/70) created" + "dummy2image4dragon-created-by-copy2buffertoimage4dragon: " + str(dummy2image4dragon)  + "; time: " + time4xi() + "}" 
        # print4xi(class2print, method2print, message2print)  
        # code2ack  = code2ack + '69B-'
        
    # except:
        # code2nack  = code2nack + '69Z-'
        
    # ---------------------------------------------------------------------------------------------
    # RETURN CONTROL AND COMMAND DICTIONARY
    # ---------------------------------------------------------------------------------------------          
    message2print = "{(Step 70A/70) set" + "; code2ack-set-by-copy2buffertoimage4dragon: " + str(code2ack) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                             
    # 1B1-1B2-1C-2B-3B-4B-5B-6B-7B-8B-9B-11B-12B-13B-15B-16B-40D-41C- 
    message2print = "{(Step 70B/70) set" + "; code2nack-set-by-copy2buffertoimage4dragon: " + str(code2nack) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # 
        
    message2print = "{(Step 70C/70) returning" + "; control2copybuffertoimage4dragon-returning-by-copy2buffertoimage4dragon: " + str(control2copybuffertoimage4dragon) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003DF2198>,
    #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 708>,
    #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 704>}; time: 04:52:11 10/04/11 Pacific Daylight Time}

        
    return control2copybuffertoimage4dragon
                    
#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "read2image4dragon" usage = "
#                                                                                                              
#   # ---------------------------------------------------------------------------------------------
#   # (16) ESTABLISH CONTROL & COMMAND WITH READ2IMAGE4DRAGON                      
#   # ---------------------------------------------------------------------------------------------    
#   print('(Step 16A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL & COMMAND WITH READ2IMAGE4DRAGON ', time4xi())    
#   respon4read2image = {} 
#   task4read2image = None
#   try:                               
#       response4read2image = read2image4dragon(task4read2image)                                                                                                          
#       print('(Step 16B/700) OF DRAGON2IMAGE4XI5TEST - ESTABLISHED CONTROL & COMMAND WITH READ2IMAGE4DRAGON : ', response4read2image)
#       queue2read4dragon      = response4read2image['QUEUE']
#       channel2readpublish4dragon = response4read2image['PUBLISHERCHANNEL']
#       channel2readconsume4dragon = response4read2image['CONSUMERCHANNEL']
#   except:
#       print('(Step 16Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL & COMMAND WITH READ2IMAGE4DRAGON ', time4xi()) 
#   # --------------------------------------------------------------------------------------------------
#   # (17) SEND CONTROL & COMMAND COPY VIA PIPE FOR MANOEUVRE2IMAGE4DRAGON TO RECEIVE                 
#   # --------------------------------------------------------------------------------------------------      
#   print('(Step 17A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4DRAGON ', time4xi())     
#   try:                               
#       message2readsend4dragon = "<copy4xi>copy2read4dragon</copy4xi>"
#       channel2readpublish4dragon = response4read2image['PUBLISHERCHANNEL']
#       channel2readpublish4dragon.send(message2readsend4dragon)
#       print("(Step 17B/700) OF DRAGON2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4DRAGON: ", channel2readpublish4dragon, " MESSAGE2SEND4DRAGON: " , message2readsend4dragon)                                                                                                
#   except:
#       print('(Step 17Z1/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4DRAGON ', time4xi())                                                
#   # ------------------------------------------------------------------------------------------
#   # (18) LET MANOEUVRE2IMAGE4DRAGON TO RECEIVE COPY AND AND SEND ROGER VIA PIPE                
#   # ------------------------------------------------------------------------------------------       
#   print("(Step 18C/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4DRAGON ", time4xi()) 
#   try:        
#       response2manoeuvre4dragon = manoeuvre2readimage4dragon(queue2read4dragon, channel2readconsume4dragon, channel2readpublish4dragon)
#       print("(Step 18D/700) OF DRAGON2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4DRAGON ", manoeuvre2readimage4dragon)
#   except:
#       print("(Step 18Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4DRAGON ", time4xi())     
#   else:
#       pass # trust activated thread    
#   # ---------------------------------------------------------------------------------------------
#   # (19) RECEIVE CONTROL & COMMAND ROGER VIA PIPE                   
#   # ---------------------------------------------------------------------------------------------          
#   print('(Step 19A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4DRAGON ', time4xi())     
#   try:
#       channel2readconsume4dragon = response4read2image['CONSUMERCHANNEL']                              
#       message2readreceive4dragon = channel2readconsume4dragon.recv()
#       print("(Step 19B/700) OF DRAGON2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4DRAGON: ", channel2readconsume4dragon, " CONTROL & COMMAND: ", message2readreceive4dragon)                                                                                                  
#       # <roger4xi>roger2read4dragon</roger4xi>
#   except:
#       print('(Step 19Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH READ2IMAGE4DRAGON ', time4xi())         
#   # ---------------------------------------------------------------------------------------------------
#   # (20) SEND CONTROL & COMMAND READY VIA PIPE FOR MANOEUVRE2IMAGE4DRAGON TO RECEIVE                
#   # ---------------------------------------------------------------------------------------------------      
#   print('(Step 20A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4DRAGON ', time4xi())     
#   try:                               
#       message2readsend4dragon = "<ready4xi>ready2read4dragon</ready4xi>"
#       channel2readpublish4dragon = response4read2image['PUBLISHERCHANNEL']
#       channel2readpublish4dragon.send(message2readsend4dragon)
#       print("(Step 20B/700) OF DRAGON2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4DRAGON: ", channel2readpublish4dragon, " MESSAGE2SEND4DRAGON: " , message2readsend4dragon)                                                                                                
#       # <ready4xi>ready2read4dragon</ready4xi> 
#   except:
#        print('(Step 20Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4DRAGON ', time4xi())                                                
#   # -----------------------------------------------------------------------------------
#   # (21) LET MANOEUVRE2IMAGE4DRAGON TO RECEIVE READY AND TO SEND READY                
#   # -----------------------------------------------------------------------------------       
#   print("(Step 21A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4DRAGON ", time4xi()) 
#   try:        
#       response2manoeuvre4dragon = manoeuvre2readimage4dragon(queue2read4dragon, channel2readconsume4dragon, channel2readpublish4dragon)
#       print("(Step 21B/700) OF DRAGON2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4DRAGON ", manoeuvre2readimage4dragon)
#   except:
#       print("(Step 21Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4DRAGON ", time4xi())     
#   # ---------------------------------------------------------------------------------------
#   # (22) RECEIVE CONTROL & COMMAND READY VIA PIPE                      
#   # ---------------------------------------------------------------------------------------          
#   print('(Step 22A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4DRAGON ', time4xi())     
#   try:
#       channel2readconsume4dragon = response4read2image['CONSUMERCHANNEL']                              
#       message2readreceive4dragon = channel2readconsume4dragon.recv()
#       print("(Step 22B/700) OF DRAGON2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4DRAGON: ", channel2readconsume4dragon, " CONTROL & COMMAND: ", message2readreceive4dragon)                                                                                                  
#       # <ready4xi>ready2read4dragon</ready4xi> 
#   except:
#       print('(Step 22Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH READ2IMAGE4DRAGON ', time4xi())          
#   # -----------------------------------------------------------------------------------------
#   # (24) PUT IMAGE PATH TO QUEUE
#   # -----------------------------------------------------------------------------------------                          
#   print("(Step 24A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO PUT IMAGE PATH TO QUEUE ", time4xi())      
#   try:    
#       queue2read4dragon.put(path2dragon)             
#       print("(Step 24B/700) OF DRAGON2IMAGE4XI5TEST - PUT TO QUEUE IMAGE PATH: ", path2dragon) 
#       # ..//..//..//..//..//web//images//png//dragonxi.png  
#   except:
#       print("(Step 24Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO PUT IMAGE PATH TO QUEUE ", time4xi()) 
#   # ------------------------------------------------------------------------------------------------------
#   # (25) LET MANOEUVRE2IMAGE4DRAGON TO GET FROM QUEUE AND TO SEND EITHER ACK OR NACK VIA PIPE               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 25A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4DRAGON ", time4xi()) 
#   try:        
#       response2manoeuvre4dragon = manoeuvre2readimage4dragon(queue2read4dragon, channel2readconsume4dragon, channel2readpublish4dragon)
#       print("(Step 25B/700) OF DRAGON2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4DRAGON ", manoeuvre2readimage4dragon)
#   except:
#       print("(Step 25Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4DRAGON ", time4xi())     
#   # -----------------------------------------------------------------------------------------------
#   # (26) RECEIVE ACK OR NACK ABOUT IMAGE OPERATION VIA PIPE                
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 26A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())      
#   try: 
#       message2readreceive4dragon = channel2readconsume4dragon.recv()
#       print("(Step 26D/700) OF DRAGON2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4DRAGON: ", channel2readconsume4dragon, " CONTROL & COMMAND: ", message2readreceive4dragon)                                                                                                   
#       # <ack4xi>ack2imageinfo4dragon</ack4xi>
#   except:
#       print('(Step 26Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())               
#   # -------------------------------------------------------------------------------------------------------
#   # (27) LET MANOEUVRE2IMAGE4DRAGON GO TO STATE 0               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 27A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4DRAGON ", time4xi()) 
#   try:        
#       response2manoeuvre4dragon = manoeuvre2readimage4dragon(queue2read4dragon, channel2readconsume4dragon, channel2readpublish4dragon)
#       print("(Step 27B/700) OF DRAGON2IMAGE4XI5TEST - GOT RESPONSE2MANOEUVRE4DRAGON ", response2manoeuvre4dragon)
#   except:
#       print("(Step 27Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4DRAGON ", time4xi())   
#">

# -------------------------------------------------------------------------------------------------   
def read2image4dragon(task4imageread2dragon=None):

    '''
    reads 2D images for DRAGON
    '''
    global dragon2image4xi                                                     # 2D-IMAGES FOR DRAGON  
    global control2readimage4dragon                                              # COMMAND AND CONTROL FOR DRAGON IMAGES 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "read2image4dragon"
    method2print = "read2image4dragon"

    message2print = "{(Step 1A/33) started" + "; read2image4dragon-started: " + str(read2image4dragon)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function read2image4dragon at 0x00000000039595C8> 
    
    message2print = "{(Step 1B/33) passed" + "; task4imageread2dragon-to-read2image4dragon: " + str(task4imageread2dragon) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None
            
    code2ack  = ' '
    code2nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # CREATE DRAGON2IMAGE4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if dragon2image4xi is None:
            dragon2image4xi = create2image4dragon()                                                                              # INITIALIZE IMAGE FOR DRAGON 
            message2print = "{(Step 1C/70) created" + "; dragon2image4xi-created-by-read2image4dragon to: " + str(dragon2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create2image4dragon-skipped-by-read2image4dragon: " + str(read2image4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # skipped                                 
    except:
        try:
            dragon2image4xi = create2image4dragon()
            message2print = "{(Step 1E/70) set" + "; dragon2image4xi-set-by-read2image4dragon to: " + str(dragon2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            dragon2image4xi = {}
            message2print = "{(Step 1Z/70) yielded" + "; dragon2image4xi-yielded-by-read2image4dragon: " + str(dragon2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z-' 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    try:             
        if task4imageread2dragon is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER FOR READ - BUILD TASK! 
            # -----------------------------------------------------------------------------------------    
            
            task4imageread2dragon = {}                                                                                                                                                                                       
            
            message2print = "{(Step 1A/70) initialized" + "; task4imageread2dragon-created-by-read2image4dragon to: " + str(task4imageread2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 
            
            # ---------------------------------------------------------------------------------------------
            # (1B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4dragon = trace2image4dragon                                                              # CALLBACK FUNCTION FOR DRAGON
            
            message2print = "{(Step 1B1/70) initialized" + "; function2callback4dragon-created-by-read2image4dragon to: " + str(function2callback4dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # <function trace2image4dragon at 0x0000000004737548>;
            
            code2ack  = code2ack + '1B1-'
           
            args2callback = {}                                                                                   # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "read2image4dragon"                                                          # MODULE OF DRAGONIZED OPENCL MODULE
            args2callback['METHOD']  = "read2image4dragon"                                                          # FUNCTION OF DRAGONIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; read2image4dragon-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR DRAGONIZED OPENCL TO RETURN    

            message2print = "{(Step 1B2/70) initialized" + "; args2callback-created-by-read2image4dragon to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {'MESSAGE': '{(Step 1Z1/33) failed; read2image4dragon-failed ; time: 05:06:38 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'read2image4dragon', 
            #  'METHOD': 'read2image4dragon'}; time: 05:06:38 10/04/11 Pacific Daylight Time} 

            code2ack  = code2ack + '1B2-'
            
            # ---------------------------------------------------------------------------------------------
            # (1C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imageread2dragon['DATA2CALLBACK'] = args2callback                                                   # USER DATA PASSED TO DRAGONIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imageread2dragon['FUNCTION2CALLBACK'] = function2callback4dragon                                       # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '1C-'
            message2print = "{(Step 1C/70) initialized" + "; task4imageread2dragon-created-by-read2image4dragon to: " + str(task4imageread2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # {'FUNCTION2CALLBACK': <function trace2image4dragon at 0x0000000004737548>, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; read2image4dragon-failed ; time: 05:06:38 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'read2image4dragon', 
            #  'METHOD': 'read2image4dragon'}}; time: 05:06:38 10/04/11 Pacific Daylight Time}
             
            
            # ---------------------------------------------------------------------------------------------
            # (2) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-read2image4dragon to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # {}
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-read2image4dragon: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}
                 
                code2ack  = code2ack + '2B-'
            except:
                message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-read2image4dragon: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '2Z-'
                 
            task4imageread2dragon['PLATFORM'] = platform2xi                                                          # PLATFORM OF SYSTEM 
              
            
            # ---------------------------------------------------------------------------------------------
            # (3) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2dragon = {}
            message2print = "{(Step 3A/70) trying" + "; context2dragon-trying-by-read2image4dragon: " + str(context2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2dragon = get2context4dragon()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 3B/70) got" + "; context2dragon-got-by-read2image4dragon: " + str(context2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': ...
    
                code2ack  = code2ack + '3B-'         
            except:
                message2print = "{(Step 3Z/70) failed" + "; context2dragon-failed-by-read2image4dragon: " + str(context2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '3Z-'
                
            task4imageread2dragon ['CONTEXT'] = context2dragon           
            
            # ---------------------------------------------------------------------------------------------
            # (4) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2dragon = {}
            
            message2print = "{(Step 4A/70) trying" + "; device2dragon-trying-by-read2image4dragon: " + str(device2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2dragon = get2dragon4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 4B/70) got" + "; device2dragon-got-by-read2image4dragon: " + str(device2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'RESOLUTION2TIMER4PROFILING': 318, 
                #  'ID2PLATFORM': '0000000001509568', 
                #  'PROFILING2QUEUE': 'ACK', 
                #  'EXECUTE2KERNELS4OPENCL': 'ACK', 
                #  'NATIVE2WIDTHLONG4VECTOR': 2 ...
                
                code2ack  = code2ack + '4B-'  
            except:
                message2print = "{(Step 4Z/70) failed" + "; device2dragon-failed-by-read2image4dragon: " + str(device2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '4Z-'
            
            task4imageread2dragon ['DEVICE'] = device2dragon
            
                   
            # ---------------------------------------------------------------------------------------------
            # (5) IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            imageinfo2dragon = {}
             
            message2print = "{(Step 5A/70) trying" + "; imageinfo2dragon-trying-by-read2image4dragon: " + str(imageinfo2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imageinfo2dragon = get2imageinfo4dragon()                                                                         
                
                message2print = "{(Step 5B/70) got" + "; imageinfo2dragon-got-by-read2image4dragon: " + str(imageinfo2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'SIZE2QUEUE': 2147483646, 
                #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
                #  'ORDER': 'CL_RGBA'}, 
                #  'QUEUE': <queue4xi.Queue4Xi object at 0x00000000041F9320>, 
                #  'WIDTH': 800, 
                #  'CONTEXT': {'COUNT2REFERENCE': 0, ...
                
                code2ack  = code2ack + '5B-'  
            except:
                message2print = "{(Step 5Z/70) failed" + "; imageinfo2dragon-failed-by-read2image4dragon: " + str(imageinfo2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '5Z-'          
                      
            task4imageread2dragon ['IMAGE'] = imageinfo2dragon         
                        

            # ---------------------------------------------------------------------------------------------
            # (6) QUEUE - REQUIRED BY DRAGONXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2dragon = None
            
            message2print = "{(Step 6A/70) trying" + "; queue2dragon-trying-by-read2image4dragon: " + str(queue2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2dragon = Queue4Xi()                                                                                  # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 6B/70) got" + "; queue2dragon-got-by-read2image4dragon: " + str(queue2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # <queue4xi.Queue4Xi object at 0x0000000004607160> 
                
                code2ack  = code2ack + '6B-'  
            except:
                message2print = "{(Step 6Z/70) failed" + "; queue2dragon-failed-by-read2image4dragon: " + str(queue2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '6Z-'
                
            task4imageread2dragon ['QUEUE'] = queue2dragon

            
            # ---------------------------------------------------------------------------------------------
            # (7) PIPE - REQUIRED BY DRAGONXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2dragon = None
           
            message2print = "{(Step 7A/70) trying" + "; pipe2dragon-trying-by-read2image4dragon: " + str(pipe2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2dragon = Pipe4Xi()                                                                                    # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 7B/70) got" + "; pipe2dragon-got-by-read2image4dragon: " + str(pipe2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # (<read-write PipeConnection, handle 408>, <read-write PipeConnection, handle 412>) 
                
                code2ack  = code2ack + '7B-'  
            except:
                message2print = "{(Step 7Z/70) failed" + "; pipe2dragon-failed-by-read2image4dragon: " + str(pipe2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '7Z-'
                
            task4imageread2dragon ['PIPE'] = pipe2dragon 
            
            # ---------------------------------------------------------------------------------------------
            # (8) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2dragon = {}
           
            message2print = "{(Step 8A/70) trying" + "; commandqueueinfo2dragon-trying-by-read2image4dragon: " + str(commandqueueinfo2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2dragon = get2commandqueueinfo4dragon()                                                    # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 8B/70) got" + "; commandqueueinfo2dragon-got-by-read2image4dragon: " + str(commandqueueinfo2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 
                #  'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': ...
                
                code2ack  = code2ack + '8B-'  
            except:
                message2print = "{(Step 8Z/70) failed" + "; commandqueueinfo2dragon-failed-by-read2image4dragon: " + str(commandqueueinfo2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '8Z-'
        
            task4imageread2dragon ['COMMANDQUEUE'] = commandqueueinfo2dragon
            
            
            
            # ---------------------------------------------------------------------------------------------
            # (9) BUFFER - AUTOMATIC BY DRAGONXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2dragon = {}
           
            message2print = "{(Step 9A/70) trying" + "; bufferinfo2dragon-trying-by-read2image4dragon: " + str(bufferinfo2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2dragon = get2bufferinfo4dragon()                                                                   # SYSTEM CREATES BUFFER FOR DRAGON, IF NECESSARY
                message2print = "{(Step 9B/70) got" + "; bufferinfo2dragon-got-by-read2image4dragon: " + str(bufferinfo2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': ...
                
                code2ack  = code2ack + '9B-' 
            except:
                message2print = "{(Step 9Z/70) failed" + "; bufferinfo2dragon-failed-by-read2image4dragon: " + str(bufferinfo2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '9Z-'
        
            task4imageread2dragon ['BUFFER'] = bufferinfo2dragon 
            
            # ---------------------------------------------------------------------------------------------
            # (10) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2dragon = {}
           
            message2print = "{(Step 10A/70) trying" + "; eventinfo2dragon-trying-by-read2image4dragon: " + str(eventinfo2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2dragon = get2eventinfo4dragon()                                                                    
                message2print = "{(Step 10B/70) got" + "; eventinfo2dragon-got-by-read2image4dragon: " + str(eventinfo2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 'DATA2CALLBACK': {'MESSAGE': ...
                
                code2ack  = code2ack + '10B-' 
            except:
                message2print = "{(Step 10Z/70) failed" + "; eventinfo2dragon-failed-by-read2image4dragon: " + str(eventinfo2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '10Z-'
        
            task4imageread2dragon ['EVENT'] = eventinfo2dragon 
            
            
            # ---------------------------------------------------------------------------------------------
            # (11) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2dragon = {}
           
            message2print = "{(Step 11A/70) trying" + "; eventwaitlist2dragon-trying-by-read2image4dragon: " + str(eventwaitlist2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2dragon = {}                                                                   
                message2print = "{(Step 11B/70) set" + "; eventwaitlist2dragon-set-by-read2image4dragon: " + str(eventwaitlist2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {}
                
                code2ack  = code2ack + '11B-' 
            except:
                message2print = "{(Step 11Z/70) failed" + "; eventwaitlist2dragon-failed-by-read2image4dragon: " + str(eventwaitlist2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '11Z-'
        
            task4imageread2dragon ['EVENTWAITLIST'] = eventwaitlist2dragon                     

            
            # ---------------------------------------------------------------------------------------------
            # (12) EVENTWAITLISTQUANTITY - REQUIRED                               
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2dragon = 0
           
            message2print = "{(Step 12A/70) trying" + "; eventwaitlistquantity2dragon-trying-by-read2image4dragon: " + str(eventwaitlistquantity2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2dragon.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2dragon = eventwaitlistquantity2dragon + 1                                                                   
                message2print = "{(Step 12B/70) set" + "; eventwaitlistquantity2dragon-set-by-read2image4dragon: " + str(eventwaitlistquantity2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 
                
                code2ack  = code2ack + '12B-' 
            except:
                message2print = "{(Step 12Z/70) failed" + "; eventwaitlistquantity2dragon-failed-by-read2image4dragon: " + str(eventwaitlistquantity2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '12Z-'
        
            task4imageread2dragon ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2dragon       
                                                 
            
            # ---------------------------------------------------------------------------------------------
            # (13) BLOCKING - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            blocking2dragon = ' '                                                                   
     
            message2print = "{(Step 13A/70) trying" + "; blocking2dragon-trying-by-read2image4dragon: " + str(blocking2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                blocking2dragon = 'CL_FALSE'                                                          # the read command is non-blocking                                                              
                message2print = "{(Step 13B/70) set" + "; blocking2dragon-set-by-read2image4dragon to: " + str(blocking2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # CL_FALSE
                code2ack  = code2ack + '13B-' 
            except:
                message2print = "{(Step 13Z/70) failed" + "; blocking2dragon-failed-by-read2image4dragon: " + str(blocking2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '13Z-'
        
            task4imageread2dragon ['BLOCKING'] = blocking2dragon             
            
            # ---------------------------------------------------------------------------------------------
            # (14) ORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin2dragon = {}
            
            message2print = "{(Step 14A/70) trying" + "; origin2dragon-trying-by-read2image4dragon: " + str(origin2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin2dragon['X'] = 0 
                origin2dragon['Y'] = 0  
                origin2dragon['Z'] = 0                                                                                                                          
                message2print = "{(Step 14B/70) set" + "; origin2dragon-set-by-read2image4dragon to: " + str(origin2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'Y': 0, 'X': 0, 'Z': 0} 
                code2ack  = code2ack + '14B-' 
            except:
                message2print = "{(Step 14Z/70) failed" + "; origin2dragon-failed-by-read2image4dragon: " + str(origin2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '14Z-'
        
            task4imageread2dragon ['ORIGIN'] = origin2dragon
            
            
            # ---------------------------------------------------------------------------------------------
            # (15) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2dragon = {}
            
            message2print = "{(Step 15A/70) trying" + "; region2dragon-trying-by-read2image4dragon: " + str(region2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                region2dragon['X'] = 160                                                                               # WIDTH  IN PIXELS TO READ 
                region2dragon['Y'] = 90                                                                                # HEIGHT IN PIXELS TO READ 
                region2dragon['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO READ                                                                                                                          
                message2print = "{(Step 15B/70) set" + "; region2dragon-set-by-read2image4dragon to: " + str(region2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'Y': 90, 'X': 160, 'Z': 0}
                code2ack  = code2ack + '15B-' 
            except:
                message2print = "{(Step 15Z/70) failed" + "; region2dragon-failed-by-read2image4dragon: " + str(region2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '15Z-'
        
            task4imageread2dragon ['REGION'] = region2dragon
            
            
            # ---------------------------------------------------------------------------------------------
            # (16) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.


            rowpitch2dragon = 0
            size2element = 8                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity2layer = 10                                                                                # MAX NUMBER OF VOXELS IN EACH 2D LAYER 
            quantity2direction = 8                                                                                  # ONE DIRECTIONS IN 2D
            width2dragon = 800
            
            message2print = "{(Step 16A/70) trying" + "; rowpitch2dragon-trying-by-read2image4dragon: " + str(rowpitch2dragon)  + "; time: " + time4xi() + "}"  
            print4xi(class2print, method2print, message2print) 
            # 
                     
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch2dragon = int(width2dragon * size2element * voxelquantity2layer / quantity2direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 16B/70) set" + "; rowpitch2dragon-set-by-read2image4dragon to: " + str(rowpitch2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #  8000                                                                                          
                 
                code2ack  = code2ack + '16B-' 
            except:
                message2print = "{(Step 16Z/70) failed" + "; rowpitch2dragon-failed-by-read2image4dragon: " + str(rowpitch2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '16Z-'
        
            task4imageread2dragon ['ROWPITCH'] = rowpitch2dragon     
            
            # ----------------------------------------------------------------------------------------------------
            # (17) POINTER - REQUIRED - THE POINTER TO A BUFFER IN HOST MEMORY WHERE IMAGE DATA IS TO BE READ FROM                          
            # ----------------------------------------------------------------------------------------------------             
            pointer2dragon = None
            height2dragon = 800
            size2dragon = int(width2dragon * height2dragon)                                                                  # CALCULATED MEMORY
            if size2dragon == 0:
                size2dragon = 4400                                                                                     # MINIMUM MEMORY
            
            message2print = "{(Step 17A/70) trying" + "; pointer2dragon-trying-by-read2image4dragon: " + str(pointer2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:
                arena2dragon = Arena4Xi(size2dragon)                                                                      # SYSTEM RESERVER MEMORY FOR IMAGE                             
                                                                                                                                         
                message2print = "{(Step 17B/70) got" + "; arena2dragon-got-by-read2image4dragon: " + str(arena2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # <arena4xi.Arena4Xi object at 0x0000000004449EF0>
                
                size2arena = arena2dragon.size                                                                         # 
                              
                message2print = "{(Step 17C/70) got" + "; size2arena-got-by-read2buffer4dragon: " + str(size2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 640000
                
                buffer2arena = arena2dragon.buffer
                
                message2print = "{(Step 17D/70) got" + "; buffer2arena-got-by-read2buffer4dragon: " + str(buffer2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # <mmap.mmap object at 0x00000000041A0C90>
                
                code2ack  = code2ack + '17D-' 
                
                if size2arena < size2dragon:
                    size2dragon = size2arena                                                                           # AVAILABLE MEMORY FOR IMAGE
                    message2print = "{(Step 17E/70) set" + "; size2dragon-set-by-read2buffer4dragon to: " + str(size2dragon)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    
                    code2ack  = code2ack + '17E-'
                else:
                    pass
                    
                pointer2dragon = buffer2arena
                message2print = "{(Step 17F/70) set" + "; pointer2dragon-set-by-read2buffer4dragon to: " + str(pointer2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # <mmap.mmap object at 0x00000000041A0C90>
                
                code2ack  = code2ack + '17F-'       
            except:
                message2print = "{(Step 17Z/70) failed" + "; pointer2dragon-failed-by-read2image4dragon: " + str(pointer2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '17Z-'
        
            task4imageread2dragon ['POINTER'] = pointer2dragon                            
            task4imageread2dragon ['SIZE']    = size2dragon                                                               # AUTOMATIC BY DRAGONXI
            
            
            # --------------------------------------------------------------------------------------------------------
            # (18) SLICEPITCH - REQUIRED - THE SIZE IN BYTES OF THE 2D SLICE OF THE 3D REGION OF A 3D IMAGE BEING READ                         
            # --------------------------------------------------------------------------------------------------------             
            slicepitch2dragon = None                                                                                   # SLICEPITCH IS ZERO FOR 2D IMAGES                           
           
            message2print = "{(Step 18A/70) trying" + "; slicepitch2dragon-trying-by-read2image4dragon: " + str(slicepitch2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                slicepitch2dragon = 0                                                          
                message2print = "{(Step 18B/70) set" + "; slicepitch2dragon-set-by-read2image4dragon to: " + str(slicepitch2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 

                code2ack  = code2ack + '18B-'                
                 
            except:
                message2print = "{(Step 18Z/70) failed" + "; slicepitch2dragon-failed-by-read2image4dragon: " + str(slicepitch2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '18Z-'
            
            task4imageread2dragon ['SLICEPITCH'] = slicepitch2dragon                  
            
            # ---------------------------------------------------------------------------------------------
            # EOF PARAMETER PROVIDING                
            # ---------------------------------------------------------------------------------------------                                                                                                                               
        else:
            pass
        # ---------------------------------------------------------------------------------------------
        # ANALYSE TASKS - WHETHER AUTOMATICALLY GENERATED OR USER GIVEN 
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imageread2dragon.keys()
        message2print = "{(Step 20/70) got" + "; keys2task-got-by-read2image4dragon: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BLOCKING',
        #  'BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGE',
        #  'ORIGIN',
        #  'PIPE',
        #  'PLATFORM',
        #  'POINTER',
        #  'QUEUE',
        #  'REGION',
        #  'ROWPITCH',
        #  'SIZE',
        #  'SLICEPITCH']; time: 05:25:42 10/04/11 Pacific Daylight Time}

        if (('DATA2CALLBACK' in keys2task) and ('FUNCTION2CALLBACK' in keys2task)):
            pass
        else:                
            # ---------------------------------------------------------------------------------------------
            # (21B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4dragon = trace2image4dragon                                                              # CALLBACK FUNCTION FOR DRAGON
            
            message2print = "{(Step 21B1/70) initialized" + "; function2callback4dragon-created-by-read2image4dragon to: " + str(function2callback4dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code2ack  = code2ack + '21B1-'
           
            args2callback = {}                                                                                   # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "read2image4dragon"                                                          # MODULE OF DRAGONIZED OPENCL MODULE
            args2callback['METHOD']  = "read2image4dragon"                                                          # FUNCTION OF DRAGONIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; read2image4dragon-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR DRAGONIZED OPENCL TO RETURN    

            message2print = "{(Step 21B2/70) initialized" + "; args2callback-created-by-read2image4dragon to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 

            code2ack  = code2ack + '21B2-'
        
            # ---------------------------------------------------------------------------------------------
            # (21C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imageread2dragon['DATA2CALLBACK'] = args2callback                                                   # USER DATA PASSED TO DRAGONIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imageread2dragon['FUNCTION2CALLBACK'] = function2callback4dragon                                       # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '21C-'
            message2print = "{(Step 21C/70) initialized" + "; task4imageread2dragon-created-by-read2image4dragon to: " + str(task4imageread2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #
             
        if 'PLATFORM' in keys2task:
            pass
        else:  
            # ---------------------------------------------------------------------------------------------
            # (22) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 22A/70) initialized" + "; platform2xi-created-by-read2image4dragon to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 22B/70) got" + "; platform2xi-got-by-read2image4dragon: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                 
                code2ack  = code2ack + '22B-'
            except:
                message2print = "{(Step 22Z/70) failed" + "; platform2xi-failed-by-read2image4dragon: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '22Z-'
                 
            task4imageread2dragon['PLATFORM'] = platform2xi                                                          # PLATFORM OF SYSTEM 
          
        if 'CONTEXT' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (23) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2dragon = {}
            message2print = "{(Step 23A/70) trying" + "; context2dragon-trying-by-read2image4dragon: " + str(context2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2dragon = get2context4dragon()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 23B/70) got" + "; context2dragon-got-by-read2image4dragon: " + str(context2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '23B-'         
            except:
                message2print = "{(Step 23Z/70) failed" + "; context2dragon-failed-by-read2image4dragon: " + str(context2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '23Z-'
                
            task4imageread2dragon ['CONTEXT'] = context2dragon           
        
        if 'DEVICE' in keys2task:
            pass
        else:                   
            # ---------------------------------------------------------------------------------------------
            # (24) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2dragon = {}
            
            message2print = "{(Step 24A/70) trying" + "; device2dragon-trying-by-read2image4dragon: " + str(device2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2dragon = get2dragon4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 24B/70) got" + "; device2dragon-got-by-read2image4dragon: " + str(device2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '24B-'  
            except:
                message2print = "{(Step 24Z/70) failed" + "; device2dragon-failed-by-read2image4dragon: " + str(device2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '24Z-'
            
            task4imageread2dragon ['DEVICE'] = device2dragon
        
        if 'IMAGE' in keys2task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (25) IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            imageinfo2dragon = {}
             
            message2print = "{(Step 25A/70) trying" + "; imageinfo2dragon-trying-by-read2image4dragon: " + str(imageinfo2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imageinfo2dragon = get2imageinfo4dragon()                                                                         
                
                message2print = "{(Step 25B/70) got" + "; imageinfo2dragon-got-by-read2image4dragon: " + str(imageinfo2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code2ack  = code2ack + '25B-'  
            except:
                message2print = "{(Step 25Z/70) failed" + "; imageinfo2dragon-failed-by-read2image4dragon: " + str(imageinfo2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '25Z-'          
                      
            task4imageread2dragon ['IMAGE'] = imageinfo2dragon         
                    
        if 'QUEUE' in keys2task:
            pass
        else:   
            # ---------------------------------------------------------------------------------------------
            # (26) QUEUE - REQUIRED BY DRAGONXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2dragon = None
            
            message2print = "{(Step 26A/70) trying" + "; queue2dragon-trying-by-read2image4dragon: " + str(queue2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2dragon = Queue4Xi()                                                                                  # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 26B/70) got" + "; queue2dragon-got-by-read2image4dragon: " + str(queue2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '26B-'  
            except:
                message2print = "{(Step 26Z/70) failed" + "; queue2dragon-failed-by-read2image4dragon: " + str(queue2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '26Z-'
                
            task4imageread2dragon ['QUEUE'] = queue2dragon
        
        if 'PIPE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (27) PIPE - REQUIRED BY DRAGONXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2dragon = None
           
            message2print = "{(Step 27A/70) trying" + "; pipe2dragon-trying-by-read2image4dragon: " + str(pipe2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2dragon = Pipe4Xi()                                                                                    # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 27B/70) got" + "; pipe2dragon-got-by-read2image4dragon: " + str(pipe2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '27B-'  
            except:
                message2print = "{(Step 27Z/70) failed" + "; pipe2dragon-failed-by-read2image4dragon: " + str(pipe2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '27Z-'
                
            task4imageread2dragon ['PIPE'] = pipe2dragon 
            
        if 'COMMANDQUEUE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2dragon = {}
           
            message2print = "{(Step 28A/70) trying" + "; commandqueueinfo2dragon-trying-by-read2image4dragon: " + str(commandqueueinfo2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2dragon = get2commandqueueinfo4dragon()                                                    # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 28B/70) got" + "; commandqueueinfo2dragon-got-by-read2image4dragon: " + str(commandqueueinfo2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '28B-'  
            except:
                message2print = "{(Step 28Z/70) failed" + "; commandqueueinfo2dragon-failed-by-read2image4dragon: " + str(commandqueueinfo2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '28Z-'
        
            task4imageread2dragon ['COMMANDQUEUE'] = commandqueueinfo2dragon
        
        
        if 'BUFFER' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (29) BUFFER - AUTOMATIC BY DRAGONXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2dragon = {}
           
            message2print = "{(Step 29A/70) trying" + "; bufferinfo2dragon-trying-by-read2image4dragon: " + str(bufferinfo2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2dragon = get2bufferinfo4dragon()                                                                   # SYSTEM CREATES BUFFER FOR DRAGON, IF NECESSARY
                message2print = "{(Step 29B/70) got" + "; bufferinfo2dragon-got-by-read2image4dragon: " + str(bufferinfo2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '29B-' 
            except:
                message2print = "{(Step 29Z/70) failed" + "; bufferinfo2dragon-failed-by-read2image4dragon: " + str(bufferinfo2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '29Z-'
        
            task4imageread2dragon ['BUFFER'] = bufferinfo2dragon
             
        if 'EVENT' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (30) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2dragon = {}
           
            message2print = "{(Step 30A/70) trying" + "; eventinfo2dragon-trying-by-read2image4dragon: " + str(eventinfo2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2dragon = get2eventinfo4dragon()                                                                    
                message2print = "{(Step 30B/70) got" + "; eventinfo2dragon-got-by-read2image4dragon: " + str(eventinfo2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '30B-' 
            except:
                message2print = "{(Step 30Z/70) failed" + "; eventinfo2dragon-failed-by-read2image4dragon: " + str(eventinfo2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '30Z-'
        
            task4imageread2dragon ['EVENT'] = eventinfo2dragon 
        
             
        if 'EVENTWAITLIST' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (31) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2dragon = {}
           
            message2print = "{(Step 31A/70) trying" + "; eventwaitlist2dragon-trying-by-read2image4dragon: " + str(eventwaitlist2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2dragon = {}                                                                   
                message2print = "{(Step 31B/70) set" + "; eventwaitlist2dragon-set-by-read2image4dragon: " + str(eventwaitlist2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '31B-' 
            except:
                message2print = "{(Step 31Z/70) failed" + "; eventwaitlist2dragon-failed-by-read2image4dragon: " + str(eventwaitlist2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '31Z-'
        
            task4imageread2dragon ['EVENTWAITLIST'] = eventwaitlist2dragon                     
              
        if 'EVENTWAITLISTQUANTITY' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (32) EVENTWAITLISTQUANTITY - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2dragon = 0
           
            message2print = "{(Step 32A/70) trying" + "; eventwaitlistquantity2dragon-trying-by-read2image4dragon: " + str(eventwaitlistquantity2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2dragon.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2dragon = eventwaitlistquantity2dragon + 1                                                                      
                message2print = "{(Step 32B/70) set" + "; eventwaitlistquantity2dragon-set-by-read2image4dragon: " + str(eventwaitlistquantity2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0
                
                code2ack  = code2ack + '32B-' 
            except:
                message2print = "{(Step 32Z/70) failed" + "; eventwaitlistquantity2dragon-failed-by-read2image4dragon: " + str(eventwaitlistquantity2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '32Z-'
        
            task4imageread2dragon ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2dragon       
                                                 
                 
        if 'BLOCKING' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (33) BLOCKING - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            blocking2dragon = ' '                                                                   
            message2print = "{(Step 33A/70) trying" + "; blocking2dragon-trying-by-read2image4dragon: " + str(blocking2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                blocking2dragon = 'CL_FALSE'                                                          # the read command is non-blocking                                                              
                message2print = "{(Step 33B/70) set" + "; blocking2dragon-set-by-read2image4dragon to: " + str(blocking2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '33B-' 
            except:
                message2print = "{(Step 33Z/70) failed" + "; blocking2dragon-failed-by-read2image4dragon: " + str(blocking2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '33Z-'
        
            task4imageread2dragon ['BLOCKING'] = blocking2dragon             
             
        if 'ORIGIN' in keys2task:
            pass
        else:                    
            # ---------------------------------------------------------------------------------------------
            # (34) ORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin2dragon = {}
            
            message2print = "{(Step 34A/70) trying" + "; origin2dragon-trying-by-read2image4dragon: " + str(origin2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin2dragon['X'] = 0 
                origin2dragon['Y'] = 0  
                origin2dragon['Z'] = 0                                                                                                                          
                message2print = "{(Step 34B/70) set" + "; origin2dragon-set-by-read2image4dragon to: " + str(origin2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '34B-' 
            except:
                message2print = "{(Step 34Z/70) failed" + "; origin2dragon-failed-by-read2image4dragon: " + str(origin2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '34Z-'
        
            task4imageread2dragon ['ORIGIN'] = origin2dragon
            
             
        if 'REGION' in keys2task:
            pass
        else:            
            # ---------------------------------------------------------------------------------------------
            # (35) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2dragon = {}
            
            message2print = "{(Step 35A/70) trying" + "; region2dragon-trying-by-read2image4dragon: " + str(region2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                            
            try:                                                                                    
                region2dragon['X'] = 160                                                                               # WIDTH  IN PIXELS TO READ 
                region2dragon['Y'] = 90                                                                                # HEIGHT IN PIXELS TO READ 
                region2dragon['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO READ                                                                                                                          
                message2print = "{(Step 35B/70) set" + "; region2dragon-set-by-read2image4dragon to: " + str(region2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '35B-' 
            except:
                message2print = "{(Step 35Z/70) failed" + "; region2dragon-failed-by-read2image4dragon: " + str(region2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '35Z-'
        
            task4imageread2dragon ['REGION'] = region2dragon
        
            
             
        if 'ROWPITCH' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (36) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.
     
            rowpitch2dragon = 0
            size2element = 1                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity2layer = 1                                                                                 # MAX NUMBER OF VOXELS IN EACH 2D LAYER 
            quantity2direction = 1                                                                                  # ONE DIRECTIONS IN 2D
            
            message2print = "{(Step 36A/70) trying" + "; rowpitch2dragon-trying-by-read2image4dragon: " + str(rowpitch2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch2dragon = int(width2dragon * size2element * voxelquantity2layer / quantity2direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 36B/70) set" + "; rowpitch2dragon-set-by-read2image4dragon to: " + str(rowpitch2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #                                                                                             
                 
                code2ack  = code2ack + '36B-' 
            except:
                message2print = "{(Step 36Z/70) failed" + "; rowpitch2dragon-failed-by-read2image4dragon: " + str(rowpitch2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '36Z-'
        
            task4imageread2dragon ['ROWPITCH'] = rowpitch2dragon     
             
        if 'POINTER' in keys2task:
            pass
        else:                
            # ----------------------------------------------------------------------------------------------------
            # (37) POINTER - REQUIRED - THE POINTER TO A BUFFER IN HOST MEMORY WHERE IMAGE DATA IS TO BE READ FROM                          
            # ----------------------------------------------------------------------------------------------------             
            pointer2dragon = None
            size2dragon = int(width2dragon * height2dragon)                                                                  # CALCULATED MEMORY
            if size2dragon == 0:
                size2dragon = 4400                                                                                     # MINIMUM MEMORY
            
            message2print = "{(Step 37A/70) trying" + "; pointer2dragon-trying-by-read2image4dragon: " + str(pointer2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:
                arena2dragon = Arena4Xi(size2dragon)                                                                      # SYSTEM RESERVER MEMORY FOR IMAGE                             
                                                                                                                                         
                message2print = "{(Step 37B/70) got" + "; arena2dragon-got-by-read2image4dragon: " + str(arena2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                size2arena = arena2dragon.size                                                                         # 
                              
                message2print = "{(Step 37C/70) got" + "; size2arena-got-by-read2buffer4dragon: " + str(size2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                
                buffer2arena = arena2dragon.buffer
                
                message2print = "{(Step 37D/70) got" + "; buffer2arena-got-by-read2buffer4dragon: " + str(buffer2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # 
                
                code2ack  = code2ack + '37D-' 
                
                if size2arena < size2dragon:
                    size2dragon = size2arena                                                                           # AVAILABLE MEMORY FOR IMAGE
                    message2print = "{(Step 37E/70) set" + "; size2dragon-set-by-read2buffer4dragon to: " + str(size2dragon)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    
                    code2ack  = code2ack + '37E-'
                else:
                    pass
                    
                pointer2dragon = buffer2arena
                message2print = "{(Step 37F/70) set" + "; pointer2dragon-set-by-read2buffer4dragon to: " + str(pointer2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                
                code2ack  = code2ack + '37F-'       
            except:
                message2print = "{(Step 37Z/70) failed" + "; pointer2dragon-failed-by-read2image4dragon: " + str(pointer2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '37Z-'
        
            task4imageread2dragon ['POINTER'] = pointer2dragon                            
            task4imageread2dragon ['SIZE']    = size2dragon                                                               # AUTOMATIC BY DRAGONXI
        
             
        if 'SLICEPITCH' in keys2task:
            pass
        else:               
            # --------------------------------------------------------------------------------------------------------
            # (38) SLICEPITCH - REQUIRED - THE SIZE IN BYTES OF THE 2D SLICE OF THE 3D REGION OF A 3D IMAGE BEING READ                         
            # --------------------------------------------------------------------------------------------------------             
            slicepitch2dragon = None                                                                                   # SLICEPITCH IS ZERO FOR 2D IMAGES                           
           
            message2print = "{(Step 38A/70) trying" + "; slicepitch2dragon-trying-by-read2image4dragon: " + str(slicepitch2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                slicepitch2dragon = 0                                                          
                message2print = "{(Step 38B/70) set" + "; slicepitch2dragon-set-by-read2image4dragon to: " + str(slicepitch2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '38B-'                
                 
            except:
                message2print = "{(Step 38Z/70) failed" + "; slicepitch2dragon-failed-by-read2image4dragon: " + str(slicepitch2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '38Z-'
            
            task4imageread2dragon ['SLICEPITCH'] = slicepitch2dragon
                              
        # ---------------------------------------------------------------------------------------------
        # EOF TASK ANALYSIS
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imageread2dragon.keys()
        message2print = "{(Step 39/70) set" + "; keys2task-set-by-read2image4dragon to: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BLOCKING',
        #  'BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGE',
        #  'ORIGIN',
        #  'PIPE',
        #  'PLATFORM',
        #  'POINTER',
        #  'QUEUE',
        #  'REGION',
        #  'ROWPITCH',
        #  'SIZE',
        #  'SLICEPITCH']; time: 05:25:42 10/04/11 Pacific Daylight Time}                
    except: 
        message2print = "{(Step 39Z/70) yielded" + "; task4imageread2dragon-yielded-by-read2image4dragon: " + str(task4imageread2dragon) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '39Z-' 

    # ---------------------------------------------------------------------------------------------
    # CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    control2readimage4dragon = {}       
                                
    # ---------------------------------------------------------------------------------------------
    # CONSUMER AND PUBLISHER CHANNEL FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4channels2dragon = False
    channels2dragon = ()
    message2print = "{(Step 40A/70) trying" + "channels2dragon-trying-by-read2image4dragon: " + str(channels2dragon)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        # (<read-write PipeConnection, handle 520>, <read-write PipeConnection, handle 524>) 
        consumerchannel2dragon, publisherchannel2dragon = task4imageread2dragon ['PIPE']

        message2print = "{(Step 40B/70) got" + "; consumerchannel2dragon-got-by-read2image4dragon: " + str(consumerchannel2dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 

        message2print = "{(Step 40C/70) got" + "; publisherchannel2dragon-got-by-read2image4dragon: " + str(publisherchannel2dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        #                      
        ack4channels2dragon = True
        
        control2readimage4dragon['CONSUMERCHANNEL']  = consumerchannel2dragon
        
        
        control2readimage4dragon['PUBLISHERCHANNEL'] = publisherchannel2dragon
        
        code2ack  = code2ack + '40D-' 
        
    except:
        ack4channels2dragon = False
        code2nack  = code2nack + '40Z-'      
                        
    # ---------------------------------------------------------------------------------------------
    # QUEUE FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4queue2dragon = False
    queue2dragon = None
    message2print = "{(Step 41A/70) trying" + "; queue2dragon-trying-by-read2image4dragon: " + str(queue2dragon)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        queue2dragon = task4imageread2dragon ['QUEUE']

        message2print = "{(Step 41B/70) got" + "; queue2dragon-got-by-read2image4dragon: " + str(queue2dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 
              
        ack4queue2dragon = True
        control2readimage4dragon['QUEUE']  = queue2dragon        
        code2ack  = code2ack + '41C-'
    except:
        ack4queue2dragon = False  
        code2nack  = code2nack + '41Z-'
    
    
    # -----------------------------------------------------------------------------------------    
    # DUMMY PROCESS OBJECT = THREAD CREATION
    # -----------------------------------------------------------------------------------------           
    # dummy2image4dragon = None
    # message2print = "{(Step 69A/70) trying" + "dummy2image4dragon-trying-by-read2image4dragon: " + str(dummy2image4dragon)  + "; time: " + time4xi() + "}"   
    # print4xi(class2print, method2print, message2print)     
    # try:                            
        # dummy2image4dragon = DummyProcess4Xi(target=manoeuvre2readimage4dragon,                                               # in this module to handle queue and pipe channels
        #                                   args =(queue2dragon,                                                         # caller puts and gets - does NOT call any functions
        #                                          consumerchannel2dragon, publisherchannel2dragon))                        # caller sends and receives - does NOT call any functions
        # message2print = "{(Step 69B/70) created" + "dummy2image4dragon-created-by-read2image4dragon: " + str(dummy2image4dragon)  + "; time: " + time4xi() + "}" 
        # print4xi(class2print, method2print, message2print)  
        # code2ack  = code2ack + '69B-'

    # except:
        # code2nack  = code2nack + '69Z-'
        
    # ---------------------------------------------------------------------------------------------
    # RETURN CONTROL AND COMMAND DICTIONARY
    # ---------------------------------------------------------------------------------------------          
    message2print = "{(Step 70A/70) set" + "; code2ack-set-by-read2image4dragon: " + str(code2ack) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                             
    # 1B1-1B2-1C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-12B-13B-14B-15B-16B-17D-17F-18B-40D-41C-
     
    message2print = "{(Step 70B/70) set" + "; code2nack-set-by-read2image4dragon: " + str(code2nack) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #   
        
    message2print = "{(Step 70C/70) returning" + "; control2readimage4dragon-returning-by-read2image4dragon: " + str(control2readimage4dragon) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003FE3B70>,
    #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 416>, 
    #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 412>}
        
    return control2readimage4dragon
                    
#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "map2image4dragon" usage = "
#                                                                                                              
#   # ---------------------------------------------------------------------------------------------
#   # ESTABLISH CONTROL & COMMAND                 
#   # ---------------------------------------------------------------------------------------------    
#   print('(Step 92A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL & COMMAND WITH MAP2IMAGE4DRAGON ', time4xi())           
#                    
#   response4map2image = {} 
#   task4map2image = None                                                                                  # WE ONLY WANT TO ESTABLISH CONTROL AND CONTROL FIRST
#
#   try:                               
#       response4map2image = map2image4dragon(task4map2image)                                                                                                          
#       print('(Step 92B/700) OF DRAGON2IMAGE4XI5TEST - ESTABLISHED CONTROL & COMMAND WITH MAP2IMAGE4DRAGON : ', response4map2image)
#    
#       queue4map2image4dragon      = response4map2image['QUEUE']
#       channel4map2imagepublish4dragon = response4map2image['PUBLISHERCHANNEL']
#       channel4map2imageconsume4dragon = response4map2image['CONSUMERCHANNEL']
#   except:
#       print('(Step 92Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL & COMMAND WITH MAP2IMAGE4DRAGON ', time4xi()) 
#        
#   # -----------------------------------------------------------------------------------------------------------
#   # SEND CONTROL & COMMAND COPY VIA PIPE FOR MANOEUVRE2COPYIMAGE4DRAGON TO RECEIVE                 
#   # -----------------------------------------------------------------------------------------------------------      
#   print('(Step 93A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH MAP2IMAGE4DRAGON ', time4xi())      
#   try:                               
#       message2send4dragon = "<copy4xi>copy2mapimage4dragon</copy4xi>"
#       channel4map2imagepublish4dragon.send(message2send4dragon)
#       print("(Step 93B/700) OF DRAGON2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4DRAGON: ", channel4map2imagepublish4dragon, " MESSAGE2SEND4DRAGON: " , message2send4dragon)                                                                                                
#   except:
#       print('(Step 93Z1/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH MAP2IMAGE4DRAGON ', time4xi())   
# 
#   # ---------------------------------------------------------------------------------------------------------------------
#   # SIMULATE MANOEUVRE2COPYIMAGE4DRAGON  - LET IT TO RECEIVE COPY AND TO SEND ROGER VIA PIPE                
#   # ---------------------------------------------------------------------------------------------------------------------       
#   if ack2simulate4dragon is True:
#       print("(Step 94A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4DRAGON ", time4xi())   
#       try:        
#           response2manoeuvre4copy = manoeuvre2mapimage4dragon(queue4map2image4dragon, channel4map2imageconsume4dragon, channel4map2imagepublish4dragon)
#           print("(Step 94B/700) OF DRAGON2IMAGE4XI5TEST - PERFORMED MANOEUVRE2COPYIMAGE4DRAGON, RESPONSE2MANOEUVRE4COPY: ", response2manoeuvre4copy)
#        except:
#           print("(Step 94Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4DRAGON ", time4xi())     
#   else:
#       pass # trust activated thread    
#
#   # ---------------------------------------------------------------------------------------------
#   # RECEIVE CONTROL & COMMAND ROGER VIA PIPE                  
#   # ---------------------------------------------------------------------------------------------          
#   print('(Step 95A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH MAP2IMAGE4DRAGON ', time4xi())      
#   try:
#       message2receive4dragon = channel4map2imageconsume4dragon.recv()
#       print("(Step 95B/700) OF DRAGON2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4DRAGON: ", channel4map2imageconsume4dragon, " CONTROL & COMMAND: ", message2receive4dragon)                                                                                                  
#   except:
#       print('(Step 95Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH MAP2IMAGE4DRAGON ', time4xi())          
#   
#   # ------------------------------------------------------------------------------------------------------------
#   # SEND CONTROL & COMMAND READY VIA PIPE FOR MANOEUVRE2COPYIMAGE4DRAGON TO RECEIVE                
#   # ------------------------------------------------------------------------------------------------------------      
#   print('(Step 96A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH MAP2IMAGE4DRAGON ', time4xi())     
#   try:                               
#       message2send4dragon = "<ready4xi>ready2mapimage4dragon</ready4xi>"
#       channel4map2imagepublish4dragon.send(message2send4dragon)
#       print("(Step 96B/700) OF DRAGON2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4DRAGON: ", channel4map2imagepublish4dragon, " MESSAGE2SEND4DRAGON: " , message2send4dragon)                                                                                                
#   except:
#       print('(Step 84Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH MAP2IMAGE4DRAGON ', time4xi())                                                
# 
#   # -------------------------------------------------------------------------------------------------------
#   # SIMULATE MANOEUVRE2COPYIMAGE4DRAGON  - LET IT RECEIVE READY AND SEND READY                
#   # -------------------------------------------------------------------------------------------------------       
#   if ack2simulate4dragon is True:
#       print("(Step 97A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4DRAGON ", time4xi())  
#       try:        
#           response2manoeuvre4copy = manoeuvre2mapimage4dragon(queue4map2image4dragon, channel4map2imageconsume4dragon, channel4map2imagepublish4dragon)
#           print("(Step 97B/700) OF DRAGON2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGE4DRAGON ", manoeuvre2copyimage4dragon)
#       except:
#           print("(Step 97Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4DRAGON ", time4xi())     
#   else:
#       pass # trust activated thread    
#            
#    
#   # ---------------------------------------------------------------------------------------
#   # RECEIVE CONTROL & COMMAND READY VIA PIPE                
#   # ---------------------------------------------------------------------------------------          
#   print('(Step 98A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH MAP2IMAGE4DRAGON ', time4xi())     
#   try:
#       message2receive4dragon = channel4map2imageconsume4dragon.recv()
#       print("(Step 98B/700) OF DRAGON2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4DRAGON: ", channel4map2imageconsume4dragon, " CONTROL & COMMAND: ", message2receive4dragon)                                                                                                  
#   except:
#       print('(Step 86Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH MAP2IMAGE4DRAGON ', time4xi())          
#                         
#   # -----------------------------------------------------------------------------------------
#   # IMAGE DRAGONXI COPY OPERATIONS 
#   # -----------------------------------------------------------------------------------------         
#   print('(Step 99A/700) OF DRAGON2IMAGE4XI5TEST - TRYING FILE OPERATIONS WITH IMAGE DRAGONXI ', time4xi())       
#   try:        
#       path2dragon = '..//..//..//..//..//web//images//png//dragonxi.png'
#       file2dragon = open(path2dragon)
#       print("(Step 99B/700) OF DRAGON2IMAGE4XI5TEST - FILENO: ", file2dragon.fileno( ))  
#       print("(Step 99C/700) OF DRAGON2IMAGE4XI5TEST - MODE: ", file2dragon.mode)
#       print("(Step 99D/700) OF DRAGON2IMAGE4XI5TEST - NAME: ", file2dragon.name)
#       offset2dragon = 10
#       file2dragon.seek(offset2dragon)
#       print("(Step 99E/700) OF DRAGON2IMAGE4XI5TEST - PERFORMED SEEK WITH OFFSET: ", offset2dragon)
#       file2dragon.close()
#       print("(Step 99F/700) OF DRAGON2IMAGE4XI5TEST - CLOSED IMAGE FILE: ", path2dragon)  
#   except:
#       print("(Step 87Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO PERFORM FILE OPERATIONS WITH IMAGE: ", path2dragon)        
# 
#   # -----------------------------------------------------------------------------------------
#   # PUT IMAGE PATH TO QUEUE
#   # -----------------------------------------------------------------------------------------                          
#   print("(Step 100A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO PUT IMAGE PATH TO QUEUE ", time4xi())      
#   try:    
#       queue4map2image4dragon.put(path2dragon)             
#       print("(Step 100B/700) OF DRAGON2IMAGE4XI5TEST - PUT TO QUEUE IMAGE PATH: ", path2dragon) 
#   except:
#       print("(Step 88Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO PUT IMAGE PATH TO QUEUE ", time4xi()) 
#   
#   # -------------------------------------------------------------------------------------------------------------------------------------
#   # SIMULATE MANOEUVRE2COPYIMAGE4DRAGON  - STATE III - GETS FROM QUEUE AND SENDS EITHER ACK OR NACK VIA PIPE               
#   # -------------------------------------------------------------------------------------------------------------------------------------       
#   if ack2simulate4dragon is True:
#       print("(Step 101A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4DRAGON ", time4xi()) 
#       try:        
#           response2manoeuvre4copy = manoeuvre2mapimage4dragon(queue4map2image4dragon, channel4map2imageconsume4dragon, channel4map2imagepublish4dragon)
#           print("(Step 101B/700) OF DRAGON2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGE4DRAGON ", manoeuvre2copyimage4dragon)
#       except:
#           print("(Step 101Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4DRAGON ", time4xi())     
#   else:
#       pass # trust activated thread                     
#                 
#                                  
#   # ----------------------------------------------------------------------------------------------------
#   # RECEIVE ACK OR NACK ABOUT IMAGE OPERATION CONTROL & COMMAND VIA PIPE                
#   # ----------------------------------------------------------------------------------------------------          
#   print('(Step 102A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())      
#   try: 
#       message2receive4dragon = channel4map2imageconsume4dragon.recv()
#       print("(Step 102B/700) OF DRAGON2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4DRAGON: ", channel4map2imageconsume4dragon, " CONTROL & COMMAND: ", message2receive4dragon)                                                                                                   
#   except:
#       print('(Step 90Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())               
#
#   
#   # ----------------------------------------------------------------------------------------------------------------
#   # SIMULATE MANOEUVRE2COPYIMAGE4DRAGON  - LET MANOEUVRE2COPYIMAGE4DRAGON TO GO TO STATE 0               
#   # ----------------------------------------------------------------------------------------------------------------       
#   if ack2simulate4dragon is True:
#       print("(Step 103A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4DRAGON ", time4xi())  
#       try:        
#           response2manoeuvre4copy = manoeuvre2mapimage4dragon(queue4map2image4dragon, channel4map2imageconsume4dragon, channel4map2imagepublish4dragon)
#           print("(Step 103B/700) OF DRAGON2IMAGE4XI5TEST - GOT RESPONSE2MANOEUVRE4DRAGON ", response2manoeuvre4copy)
#       except:
#           print("(Step 103Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4DRAGON ", time4xi())     
#   else:
#       pass # trust activated thread  
#                                           
#">

# -------------------------------------------------------------------------------------------------   
def map2image4dragon(task4imagemap2dragon=None):

    '''
    maps 2D images for DRAGON
    '''
    global dragon2image4xi                                                                                             # 2D-IMAGES FOR DRAGON  
    global control2mapimage4dragon                                                                                     # COMMAND AND CONTROL FOR DRAGON IMAGES 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "map2image4dragon"
    method2print = "map2image4dragon"

    message2print = "{(Step 1A/33) started" + "; map2image4dragon-started: " + str(map2image4dragon)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # 
    
    message2print = "{(Step 1B/33) passed" + "; task4imagemap2dragon-to-map2image4dragon: " + str(task4imagemap2dragon) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
            
    code2ack  = ' '
    code2nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # CREATE DRAGON2IMAGE4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if dragon2image4xi is None:
            dragon2image4xi = create2image4dragon()                                                                              # INITIALIZE IMAGE FOR DRAGON 
            message2print = "{(Step 1C/70) created" + "; dragon2image4xi-created-by-map2image4dragon to: " + str(dragon2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create2image4dragon-skipped-by-map2image4dragon: " + str(map2image4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #                                
    except:
        try:
            dragon2image4xi = create2image4dragon()
            message2print = "{(Step 1E/70) set" + "; dragon2image4xi-set-by-map2image4dragon to: " + str(dragon2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            dragon2image4xi = {}
            message2print = "{(Step 1Z/70) yielded" + "; dragon2image4xi-yielded-by-map2image4dragon: " + str(dragon2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z-' 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    try:             
        if task4imagemap2dragon is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER FOR MAP - BUILD TASK! 
            # -----------------------------------------------------------------------------------------    
            
            task4imagemap2dragon = {}                                                                                                                                                                                       
            
            message2print = "{(Step 1A/70) initialized" + "; task4imagemap2dragon-created-by-map2image4dragon to: " + str(task4imagemap2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 
            
            # ---------------------------------------------------------------------------------------------
            # (1B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4dragon = trace2image4dragon                                                              # CALLBACK FUNCTION FOR DRAGON
            
            message2print = "{(Step 1B1/70) initialized" + "; function2callback4dragon-created-by-map2image4dragon to: " + str(function2callback4dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code2ack  = code2ack + '1B1-'
           
            args2callback = {}                                                                                   # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "map2image4dragon"                                                          # MODULE OF DRAGONIZED OPENCL MODULE
            args2callback['METHOD']  = "map2image4dragon"                                                          # FUNCTION OF DRAGONIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; map2image4dragon-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR DRAGONIZED OPENCL TO RETURN    

            message2print = "{(Step 1B2/70) initialized" + "; args2callback-created-by-map2image4dragon to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 
            code2ack  = code2ack + '1B2-'
            
            # ---------------------------------------------------------------------------------------------
            # (1C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imagemap2dragon['DATA2CALLBACK'] = args2callback                                                   # USER DATA PASSED TO DRAGONIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagemap2dragon['FUNCTION2CALLBACK'] = function2callback4dragon                                       # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '1C-'
            message2print = "{(Step 1C/70) initialized" + "; task4imagemap2dragon-created-by-map2image4dragon to: " + str(task4imagemap2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
            
            # ---------------------------------------------------------------------------------------------
            # (2) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-map2image4dragon to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-map2image4dragon: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                code2ack  = code2ack + '2B-'
            except:
                message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-map2image4dragon: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '2Z-'
                 
            task4imagemap2dragon['PLATFORM'] = platform2xi                                                          # PLATFORM OF SYSTEM 
              
            
            # ---------------------------------------------------------------------------------------------
            # (3) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2dragon = {}
            message2print = "{(Step 3A/70) trying" + "; context2dragon-trying-by-map2image4dragon: " + str(context2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2dragon = get2context4dragon()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 3B/70) got" + "; context2dragon-got-by-map2image4dragon: " + str(context2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '3B-'         
            except:
                message2print = "{(Step 3Z/70) failed" + "; context2dragon-failed-by-map2image4dragon: " + str(context2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '3Z-'
                
            task4imagemap2dragon ['CONTEXT'] = context2dragon           
            
            # ---------------------------------------------------------------------------------------------
            # (4) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2dragon = {}
            
            message2print = "{(Step 4A/70) trying" + "; device2dragon-trying-by-map2image4dragon: " + str(device2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2dragon = get2dragon4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 4B/70) got" + "; device2dragon-got-by-map2image4dragon: " + str(device2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                code2ack  = code2ack + '4B-'  
            except:
                message2print = "{(Step 4Z/70) failed" + "; device2dragon-failed-by-map2image4dragon: " + str(device2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '4Z-'
            
            task4imagemap2dragon ['DEVICE'] = device2dragon
            
                   
            # ---------------------------------------------------------------------------------------------
            # (5) IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            imageinfo2dragon = {}
             
            message2print = "{(Step 5A/70) trying" + "; imageinfo2dragon-trying-by-map2image4dragon: " + str(imageinfo2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imageinfo2dragon = get2imageinfo4dragon()                                                                         
                
                message2print = "{(Step 5B/70) got" + "; imageinfo2dragon-got-by-map2image4dragon: " + str(imageinfo2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                code2ack  = code2ack + '5B-'  
            except:
                message2print = "{(Step 5Z/70) failed" + "; imageinfo2dragon-failed-by-map2image4dragon: " + str(imageinfo2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '5Z-'          
                      
            task4imagemap2dragon ['IMAGE'] = imageinfo2dragon         
                        

            # ---------------------------------------------------------------------------------------------
            # (6) QUEUE - REQUIRED BY DRAGONXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2dragon = None
            
            message2print = "{(Step 6A/70) trying" + "; queue2dragon-trying-by-map2image4dragon: " + str(queue2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2dragon = Queue4Xi()                                                                                  # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 6B/70) got" + "; queue2dragon-got-by-map2image4dragon: " + str(queue2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '6B-'  
            except:
                message2print = "{(Step 6Z/70) failed" + "; queue2dragon-failed-by-map2image4dragon: " + str(queue2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '6Z-'
                
            task4imagemap2dragon ['QUEUE'] = queue2dragon

            
            # ---------------------------------------------------------------------------------------------
            # (7) PIPE - REQUIRED BY DRAGONXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2dragon = None
           
            message2print = "{(Step 7A/70) trying" + "; pipe2dragon-trying-by-map2image4dragon: " + str(pipe2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2dragon = Pipe4Xi()                                                                                    # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 7B/70) got" + "; pipe2dragon-got-by-map2image4dragon: " + str(pipe2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '7B-'  
            except:
                message2print = "{(Step 7Z/70) failed" + "; pipe2dragon-failed-by-map2image4dragon: " + str(pipe2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '7Z-'
                
            task4imagemap2dragon ['PIPE'] = pipe2dragon 
            
            # ---------------------------------------------------------------------------------------------
            # (8) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2dragon = {}
           
            message2print = "{(Step 8A/70) trying" + "; commandqueueinfo2dragon-trying-by-map2image4dragon: " + str(commandqueueinfo2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2dragon = get2commandqueueinfo4dragon()                                                    # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 8B/70) got" + "; commandqueueinfo2dragon-got-by-map2image4dragon: " + str(commandqueueinfo2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '8B-'  
            except:
                message2print = "{(Step 8Z/70) failed" + "; commandqueueinfo2dragon-failed-by-map2image4dragon: " + str(commandqueueinfo2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '8Z-'
        
            task4imagemap2dragon ['COMMANDQUEUE'] = commandqueueinfo2dragon
            
            
            
            # ---------------------------------------------------------------------------------------------
            # (9) BUFFER - AUTOMATIC BY DRAGONXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2dragon = {}
           
            message2print = "{(Step 9A/70) trying" + "; bufferinfo2dragon-trying-by-map2image4dragon: " + str(bufferinfo2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2dragon = get2bufferinfo4dragon()                                                                   # SYSTEM CREATES BUFFER FOR DRAGON, IF NECESSARY
                message2print = "{(Step 9B/70) got" + "; bufferinfo2dragon-got-by-map2image4dragon: " + str(bufferinfo2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '9B-' 
            except:
                message2print = "{(Step 9Z/70) failed" + "; bufferinfo2dragon-failed-by-map2image4dragon: " + str(bufferinfo2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '9Z-'
        
            task4imagemap2dragon ['BUFFER'] = bufferinfo2dragon 
            
            # ---------------------------------------------------------------------------------------------
            # (10) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2dragon = {}
           
            message2print = "{(Step 10A/70) trying" + "; eventinfo2dragon-trying-by-map2image4dragon: " + str(eventinfo2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2dragon = get2eventinfo4dragon()                                                                    
                message2print = "{(Step 10B/70) got" + "; eventinfo2dragon-got-by-map2image4dragon: " + str(eventinfo2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '10B-' 
            except:
                message2print = "{(Step 10Z/70) failed" + "; eventinfo2dragon-failed-by-map2image4dragon: " + str(eventinfo2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '10Z-'
        
            task4imagemap2dragon ['EVENT'] = eventinfo2dragon 
            
            
            # ---------------------------------------------------------------------------------------------
            # (11) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2dragon = {}
           
            message2print = "{(Step 11A/70) trying" + "; eventwaitlist2dragon-trying-by-map2image4dragon: " + str(eventwaitlist2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2dragon = {}                                                                   
                message2print = "{(Step 11B/70) set" + "; eventwaitlist2dragon-set-by-map2image4dragon: " + str(eventwaitlist2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '11B-' 
            except:
                message2print = "{(Step 11Z/70) failed" + "; eventwaitlist2dragon-failed-by-map2image4dragon: " + str(eventwaitlist2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '11Z-'
        
            task4imagemap2dragon ['EVENTWAITLIST'] = eventwaitlist2dragon                     

            
            # ---------------------------------------------------------------------------------------------
            # (12) EVENTWAITLISTQUANTITY - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2dragon = 0
           
            message2print = "{(Step 12A/70) trying" + "; eventwaitlistquantity2dragon-trying-by-map2image4dragon: " + str(eventwaitlistquantity2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2dragon.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2dragon = eventwaitlistquantity2dragon + 1                                                                      
                message2print = "{(Step 12B/70) set" + "; eventwaitlistquantity2dragon-set-by-map2image4dragon: " + str(eventwaitlistquantity2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 
                
                code2ack  = code2ack + '12B-' 
            except:
                message2print = "{(Step 12Z/70) failed" + "; eventwaitlistquantity2dragon-failed-by-map2image4dragon: " + str(eventwaitlistquantity2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '12Z-'
        
            task4imagemap2dragon ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2dragon       
                                                 
            
            # ---------------------------------------------------------------------------------------------
            # (13) BLOCKING - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            blocking2dragon = ' '                                                                   
     
            message2print = "{(Step 13A/70) trying" + "; blocking2dragon-trying-by-map2image4dragon: " + str(blocking2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                blocking2dragon = 'CL_FALSE'                                                          # the map command is non-blocking                                                              
                message2print = "{(Step 13B/70) set" + "; blocking2dragon-set-by-map2image4dragon to: " + str(blocking2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '13B-' 
            except:
                message2print = "{(Step 13Z/70) failed" + "; blocking2dragon-failed-by-map2image4dragon: " + str(blocking2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '13Z-'
        
            task4imagemap2dragon ['BLOCKING'] = blocking2dragon             
            
            # ---------------------------------------------------------------------------------------------
            # (14) ORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin2dragon = {}
            
            message2print = "{(Step 14A/70) trying" + "; origin2dragon-trying-by-map2image4dragon: " + str(origin2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin2dragon['X'] = 0 
                origin2dragon['Y'] = 0  
                origin2dragon['Z'] = 0                                                                                                                          
                message2print = "{(Step 14B/70) set" + "; origin2dragon-set-by-map2image4dragon to: " + str(origin2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                 
                code2ack  = code2ack + '14B-' 
            except:
                message2print = "{(Step 14Z/70) failed" + "; origin2dragon-failed-by-map2image4dragon: " + str(origin2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '14Z-'
        
            task4imagemap2dragon ['ORIGIN'] = origin2dragon
            
            
            # ---------------------------------------------------------------------------------------------
            # (15) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2dragon = {}
            
            message2print = "{(Step 15A/70) trying" + "; region2dragon-trying-by-map2image4dragon: " + str(region2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                region2dragon['X'] = 160                                                                               # WIDTH  IN PIXELS TO MAP 
                region2dragon['Y'] = 90                                                                                # HEIGHT IN PIXELS TO MAP 
                region2dragon['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO MAP                                                                                                                          
                message2print = "{(Step 15B/70) set" + "; region2dragon-set-by-map2image4dragon to: " + str(region2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '15B-' 
            except:
                message2print = "{(Step 15Z/70) failed" + "; region2dragon-failed-by-map2image4dragon: " + str(region2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '15Z-'
        
            task4imagemap2dragon ['REGION'] = region2dragon
            
            
            # ---------------------------------------------------------------------------------------------
            # (16) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.


            rowpitch2dragon = 0
            size2element = 8                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity2layer = 10                                                                                # MAX NUMBER OF VOXELS IN EACH 2D LAYER 
            quantity2direction = 8                                                                                  # ONE DIRECTIONS IN 2D
            width2dragon = 800
            
            message2print = "{(Step 16A/70) trying" + "; rowpitch2dragon-trying-by-map2image4dragon: " + str(rowpitch2dragon)  + "; time: " + time4xi() + "}"  
            print4xi(class2print, method2print, message2print) 
            # 
                     
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch2dragon = int(width2dragon * size2element * voxelquantity2layer / quantity2direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 16B/70) set" + "; rowpitch2dragon-set-by-map2image4dragon to: " + str(rowpitch2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #  8000                                                                                          
                 
                code2ack  = code2ack + '16B-' 
            except:
                message2print = "{(Step 16Z/70) failed" + "; rowpitch2dragon-failed-by-map2image4dragon: " + str(rowpitch2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '16Z-'
        
            task4imagemap2dragon ['ROWPITCH'] = rowpitch2dragon     
            
            # --------------------------------------------------------------------------------------------------------
            # (18) SLICEPITCH - REQUIRED - THE SIZE IN BYTES OF THE 2D SLICE OF THE 3D REGION OF A 3D IMAGE BEING MAP                         
            # --------------------------------------------------------------------------------------------------------             
            slicepitch2dragon = None                                                                                   # SLICEPITCH IS ZERO FOR 2D IMAGES                           
           
            message2print = "{(Step 18A/70) trying" + "; slicepitch2dragon-trying-by-map2image4dragon: " + str(slicepitch2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                slicepitch2dragon = 0                                                          
                message2print = "{(Step 18B/70) set" + "; slicepitch2dragon-set-by-map2image4dragon to: " + str(slicepitch2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 

                code2ack  = code2ack + '18B-'                
                 
            except:
                message2print = "{(Step 18Z/70) failed" + "; slicepitch2dragon-failed-by-map2image4dragon: " + str(slicepitch2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '18Z-'
            
            task4imagemap2dragon ['SLICEPITCH'] = slicepitch2dragon                  
            
            # ---------------------------------------------------------------------------------------------
            # (19) FLAGS - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            flags2dragon = 'CL_MAP_READ'                                                                
     
            message2print = "{(Step 19A/70) trying" + "; flags2dragon-trying-by-map2image4dragon: " + str(flags2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                flags2dragon = 'CL_MAP_WRITE'  # THE REGION SPECIFIED BY (ORIGIN, REGION) IN THE IMAGE OBJECT IS BEING MAPPED FOR WRITING                                                                                                                     
                message2print = "{(Step 19B/70) set" + "; flags2dragon-set-by-map2image4dragon to: " + str(flags2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '19B-' 
            except:
                message2print = "{(Step 19Z/70) failed" + "; flags2dragon-failed-by-map2image4dragon: " + str(flags2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '19Z-'
        
            task4imagemap2dragon ['FLAGS'] = flags2dragon 
                                  
            # ---------------------------------------------------------------------------------------------
            # EOF PARAMETER PROVIDING                
            # ---------------------------------------------------------------------------------------------                                                                                                                               
        else:
            pass
            
        # ---------------------------------------------------------------------------------------------
        # ANALYSE TASKS - WHETHER AUTOMATICALLY GENERATED OR USER GIVEN 
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imagemap2dragon.keys()
        message2print = "{(Step 20/70) got" + "; keys2task-got-by-map2image4dragon: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BLOCKING',
        #  'BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FLAGS',
        #  'FUNCTION2CALLBACK',
        #  'IMAGE',
        #  'ORIGIN',
        #  'PIPE',
        #  'PLATFORM',
        #  'QUEUE',
        #  'REGION',
        #  'ROWPITCH',
        #  'SLICEPITCH']; time: 01:28:44 10/05/11 Pacific Daylight Time}
 

        if (('DATA2CALLBACK' in keys2task) and ('FUNCTION2CALLBACK' in keys2task)):
            pass
        else:                
            # ---------------------------------------------------------------------------------------------
            # (21B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4dragon = trace2image4dragon                                                              # CALLBACK FUNCTION FOR DRAGON
            
            message2print = "{(Step 21B1/70) initialized" + "; function2callback4dragon-created-by-map2image4dragon to: " + str(function2callback4dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code2ack  = code2ack + '21B1-'
           
            args2callback = {}                                                                                   # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "map2image4dragon"                                                          # MODULE OF DRAGONIZED OPENCL MODULE
            args2callback['METHOD']  = "map2image4dragon"                                                          # FUNCTION OF DRAGONIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; map2image4dragon-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR DRAGONIZED OPENCL TO RETURN    

            message2print = "{(Step 21B2/70) initialized" + "; args2callback-created-by-map2image4dragon to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 

            code2ack  = code2ack + '21B2-'
        
            # ---------------------------------------------------------------------------------------------
            # (21C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imagemap2dragon['DATA2CALLBACK'] = args2callback                                                   # USER DATA PASSED TO DRAGONIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagemap2dragon['FUNCTION2CALLBACK'] = function2callback4dragon                                       # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '21C-'
            message2print = "{(Step 21C/70) initialized" + "; task4imagemap2dragon-created-by-map2image4dragon to: " + str(task4imagemap2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #
             
        if 'PLATFORM' in keys2task:
            pass
        else:  
            # ---------------------------------------------------------------------------------------------
            # (22) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 22A/70) initialized" + "; platform2xi-created-by-map2image4dragon to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 22B/70) got" + "; platform2xi-got-by-map2image4dragon: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                 
                code2ack  = code2ack + '22B-'
            except:
                message2print = "{(Step 22Z/70) failed" + "; platform2xi-failed-by-map2image4dragon: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '22Z-'
                 
            task4imagemap2dragon['PLATFORM'] = platform2xi                                                          # PLATFORM OF SYSTEM 
          
        if 'CONTEXT' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (23) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2dragon = {}
            message2print = "{(Step 23A/70) trying" + "; context2dragon-trying-by-map2image4dragon: " + str(context2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2dragon = get2context4dragon()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 23B/70) got" + "; context2dragon-got-by-map2image4dragon: " + str(context2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '23B-'         
            except:
                message2print = "{(Step 23Z/70) failed" + "; context2dragon-failed-by-map2image4dragon: " + str(context2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '23Z-'
                
            task4imagemap2dragon ['CONTEXT'] = context2dragon           
        
        if 'DEVICE' in keys2task:
            pass
        else:                   
            # ---------------------------------------------------------------------------------------------
            # (24) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2dragon = {}
            
            message2print = "{(Step 24A/70) trying" + "; device2dragon-trying-by-map2image4dragon: " + str(device2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2dragon = get2dragon4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 24B/70) got" + "; device2dragon-got-by-map2image4dragon: " + str(device2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '24B-'  
            except:
                message2print = "{(Step 24Z/70) failed" + "; device2dragon-failed-by-map2image4dragon: " + str(device2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '24Z-'
            
            task4imagemap2dragon ['DEVICE'] = device2dragon
        
        if 'IMAGE' in keys2task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (25) IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            imageinfo2dragon = {}
             
            message2print = "{(Step 25A/70) trying" + "; imageinfo2dragon-trying-by-map2image4dragon: " + str(imageinfo2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imageinfo2dragon = get2imageinfo4dragon()                                                                         
                
                message2print = "{(Step 25B/70) got" + "; imageinfo2dragon-got-by-map2image4dragon: " + str(imageinfo2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code2ack  = code2ack + '25B-'  
            except:
                message2print = "{(Step 25Z/70) failed" + "; imageinfo2dragon-failed-by-map2image4dragon: " + str(imageinfo2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '25Z-'          
                      
            task4imagemap2dragon ['IMAGE'] = imageinfo2dragon         
                    
        if 'QUEUE' in keys2task:
            pass
        else:   
            # ---------------------------------------------------------------------------------------------
            # (26) QUEUE - REQUIRED BY DRAGONXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2dragon = None
            
            message2print = "{(Step 26A/70) trying" + "; queue2dragon-trying-by-map2image4dragon: " + str(queue2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2dragon = Queue4Xi()                                                                                  # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 26B/70) got" + "; queue2dragon-got-by-map2image4dragon: " + str(queue2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '26B-'  
            except:
                message2print = "{(Step 26Z/70) failed" + "; queue2dragon-failed-by-map2image4dragon: " + str(queue2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '26Z-'
                
            task4imagemap2dragon ['QUEUE'] = queue2dragon
        
        if 'PIPE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (27) PIPE - REQUIRED BY DRAGONXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2dragon = None
           
            message2print = "{(Step 27A/70) trying" + "; pipe2dragon-trying-by-map2image4dragon: " + str(pipe2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2dragon = Pipe4Xi()                                                                                    # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 27B/70) got" + "; pipe2dragon-got-by-map2image4dragon: " + str(pipe2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '27B-'  
            except:
                message2print = "{(Step 27Z/70) failed" + "; pipe2dragon-failed-by-map2image4dragon: " + str(pipe2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '27Z-'
                
            task4imagemap2dragon ['PIPE'] = pipe2dragon 
            
        if 'COMMANDQUEUE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2dragon = {}
           
            message2print = "{(Step 28A/70) trying" + "; commandqueueinfo2dragon-trying-by-map2image4dragon: " + str(commandqueueinfo2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2dragon = get2commandqueueinfo4dragon()                                                    # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 28B/70) got" + "; commandqueueinfo2dragon-got-by-map2image4dragon: " + str(commandqueueinfo2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '28B-'  
            except:
                message2print = "{(Step 28Z/70) failed" + "; commandqueueinfo2dragon-failed-by-map2image4dragon: " + str(commandqueueinfo2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '28Z-'
        
            task4imagemap2dragon ['COMMANDQUEUE'] = commandqueueinfo2dragon
        
        
        if 'BUFFER' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (29) BUFFER - AUTOMATIC BY DRAGONXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2dragon = {}
           
            message2print = "{(Step 29A/70) trying" + "; bufferinfo2dragon-trying-by-map2image4dragon: " + str(bufferinfo2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2dragon = get2bufferinfo4dragon()                                                                   # SYSTEM CREATES BUFFER FOR DRAGON, IF NECESSARY
                message2print = "{(Step 29B/70) got" + "; bufferinfo2dragon-got-by-map2image4dragon: " + str(bufferinfo2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '29B-' 
            except:
                message2print = "{(Step 29Z/70) failed" + "; bufferinfo2dragon-failed-by-map2image4dragon: " + str(bufferinfo2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '29Z-'
        
            task4imagemap2dragon ['BUFFER'] = bufferinfo2dragon
             
        if 'EVENT' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (30) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2dragon = {}
           
            message2print = "{(Step 30A/70) trying" + "; eventinfo2dragon-trying-by-map2image4dragon: " + str(eventinfo2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2dragon = get2eventinfo4dragon()                                                                    
                message2print = "{(Step 30B/70) got" + "; eventinfo2dragon-got-by-map2image4dragon: " + str(eventinfo2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '30B-' 
            except:
                message2print = "{(Step 30Z/70) failed" + "; eventinfo2dragon-failed-by-map2image4dragon: " + str(eventinfo2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '30Z-'
        
            task4imagemap2dragon ['EVENT'] = eventinfo2dragon 
        
             
        if 'EVENTWAITLIST' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (31) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2dragon = {}
           
            message2print = "{(Step 31A/70) trying" + "; eventwaitlist2dragon-trying-by-map2image4dragon: " + str(eventwaitlist2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2dragon = {}                                                                   
                message2print = "{(Step 31B/70) set" + "; eventwaitlist2dragon-set-by-map2image4dragon: " + str(eventwaitlist2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '31B-' 
            except:
                message2print = "{(Step 31Z/70) failed" + "; eventwaitlist2dragon-failed-by-map2image4dragon: " + str(eventwaitlist2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '31Z-'
        
            task4imagemap2dragon ['EVENTWAITLIST'] = eventwaitlist2dragon                     
              
        if 'EVENTWAITLISTQUANTITY' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (32) EVENTWAITLISTQUANTITY - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2dragon = 0
           
            message2print = "{(Step 32A/70) trying" + "; eventwaitlistquantity2dragon-trying-by-map2image4dragon: " + str(eventwaitlistquantity2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2dragon.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2dragon = eventwaitlistquantity2dragon + 1                                                                   
                message2print = "{(Step 32B/70) set" + "; eventwaitlistquantity2dragon-set-by-map2image4dragon: " + str(eventwaitlistquantity2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '32B-' 
            except:
                message2print = "{(Step 32Z/70) failed" + "; eventwaitlistquantity2dragon-failed-by-map2image4dragon: " + str(eventwaitlistquantity2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '32Z-'
        
            task4imagemap2dragon ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2dragon       
                                                 
                 
        if 'BLOCKING' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (33) BLOCKING - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            blocking2dragon = ' '                                                                   
            message2print = "{(Step 33A/70) trying" + "; blocking2dragon-trying-by-map2image4dragon: " + str(blocking2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                blocking2dragon = 'CL_FALSE'                                                          # the map command is non-blocking                                                              
                message2print = "{(Step 33B/70) set" + "; blocking2dragon-set-by-map2image4dragon to: " + str(blocking2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '33B-' 
            except:
                message2print = "{(Step 33Z/70) failed" + "; blocking2dragon-failed-by-map2image4dragon: " + str(blocking2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '33Z-'
        
            task4imagemap2dragon ['BLOCKING'] = blocking2dragon             
             
        if 'ORIGIN' in keys2task:
            pass
        else:                    
            # ---------------------------------------------------------------------------------------------
            # (34) ORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin2dragon = {}
            
            message2print = "{(Step 34A/70) trying" + "; origin2dragon-trying-by-map2image4dragon: " + str(origin2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin2dragon['X'] = 0 
                origin2dragon['Y'] = 0  
                origin2dragon['Z'] = 0                                                                                                                          
                message2print = "{(Step 34B/70) set" + "; origin2dragon-set-by-map2image4dragon to: " + str(origin2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '34B-' 
            except:
                message2print = "{(Step 34Z/70) failed" + "; origin2dragon-failed-by-map2image4dragon: " + str(origin2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '34Z-'
        
            task4imagemap2dragon ['ORIGIN'] = origin2dragon
            
             
        if 'REGION' in keys2task:
            pass
        else:            
            # ---------------------------------------------------------------------------------------------
            # (35) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2dragon = {}
            
            message2print = "{(Step 35A/70) trying" + "; region2dragon-trying-by-map2image4dragon: " + str(region2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                            
            try:                                                                                    
                region2dragon['X'] = 160                                                                               # WIDTH  IN PIXELS TO MAP 
                region2dragon['Y'] = 90                                                                                # HEIGHT IN PIXELS TO MAP 
                region2dragon['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO MAP                                                                                                                          
                message2print = "{(Step 35B/70) set" + "; region2dragon-set-by-map2image4dragon to: " + str(region2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '35B-' 
            except:
                message2print = "{(Step 35Z/70) failed" + "; region2dragon-failed-by-map2image4dragon: " + str(region2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '35Z-'
        
            task4imagemap2dragon ['REGION'] = region2dragon
        
            
             
        if 'ROWPITCH' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (36) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.
     
            rowpitch2dragon = 0
            size2element = 1                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity2layer = 1                                                                                 # MAX NUMBER OF VOXELS IN EACH 2D LAYER 
            quantity2direction = 1                                                                                  # ONE DIRECTIONS IN 2D
            
            message2print = "{(Step 36A/70) trying" + "; rowpitch2dragon-trying-by-map2image4dragon: " + str(rowpitch2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch2dragon = int(width2dragon * size2element * voxelquantity2layer / quantity2direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 36B/70) set" + "; rowpitch2dragon-set-by-map2image4dragon to: " + str(rowpitch2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #                                                                                             
                 
                code2ack  = code2ack + '36B-' 
            except:
                message2print = "{(Step 36Z/70) failed" + "; rowpitch2dragon-failed-by-map2image4dragon: " + str(rowpitch2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '36Z-'
        
            task4imagemap2dragon ['ROWPITCH'] = rowpitch2dragon     
   
        if 'SLICEPITCH' in keys2task:
            pass
        else:               
            # --------------------------------------------------------------------------------------------------------
            # (38) SLICEPITCH - REQUIRED - THE SIZE IN BYTES OF THE 2D SLICE OF THE 3D REGION OF A 3D IMAGE BEING MAP                         
            # --------------------------------------------------------------------------------------------------------             
            slicepitch2dragon = None                                                                                   # SLICEPITCH IS ZERO FOR 2D IMAGES                           
           
            message2print = "{(Step 38A/70) trying" + "; slicepitch2dragon-trying-by-map2image4dragon: " + str(slicepitch2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                slicepitch2dragon = 0                                                          
                message2print = "{(Step 38B/70) set" + "; slicepitch2dragon-set-by-map2image4dragon to: " + str(slicepitch2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '38B-'                
                 
            except:
                message2print = "{(Step 38Z/70) failed" + "; slicepitch2dragon-failed-by-map2image4dragon: " + str(slicepitch2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '38Z-'
            
            task4imagemap2dragon ['SLICEPITCH'] = slicepitch2dragon
   
        if 'FLAGS' in keys2task:
            pass
        else:               
            # --------------------------------------------------------------------------------------------------------
            # (39) FLAGS - REQUIRED                   
            # --------------------------------------------------------------------------------------------------------             
            flags2dragon = 'CL_MAP_READ'                                                                                                             
           
            message2print = "{(Step 39A/70) trying" + "; flags2dragon-trying-by-map2image4dragon: " + str(flags2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                flags2dragon = 'CL_MAP_WRITE'                                                          
                message2print = "{(Step 39B/70) set" + "; flags2dragon-set-by-map2image4dragon to: " + str(flags2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '39B-'                
                 
            except:
                message2print = "{(Step 39Z/70) failed" + "; flags2dragon-failed-by-map2image4dragon: " + str(flags2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '39Z-'
            
            task4imagemap2dragon ['FLAGS'] = flags2dragon
                                          
        # ---------------------------------------------------------------------------------------------
        # EOF TASK ANALYSIS
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imagemap2dragon.keys()
        message2print = "{(Step 40A/70) set" + "; keys2task-set-by-map2image4dragon to: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BLOCKING',
        #  'BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FLAGS',
        #  'FUNCTION2CALLBACK',
        #  'IMAGE',
        #  'ORIGIN',
        #  'PIPE',
        #  'PLATFORM',
        #  'QUEUE',
        #  'REGION',
        #  'ROWPITCH',
        #  'SLICEPITCH']; time: 01:28:44 10/05/11 Pacific Daylight Time}

                       
    except: 
        message2print = "{(Step 40Z/70) yielded" + "; task4imagemap2dragon-yielded-by-map2image4dragon: " + str(task4imagemap2dragon) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '40Z-' 

    # ---------------------------------------------------------------------------------------------
    # CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    control2mapimage4dragon = {}       
                                
    # ---------------------------------------------------------------------------------------------
    # CONSUMER AND PUBLISHER CHANNEL FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4channels2dragon = False
    channels2dragon = ()
    message2print = "{(Step 40A/70) trying" + "channels2dragon-trying-by-map2image4dragon: " + str(channels2dragon)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        # 
        consumerchannel2dragon, publisherchannel2dragon = task4imagemap2dragon ['PIPE']

        message2print = "{(Step 40B/70) got" + "; consumerchannel2dragon-got-by-map2image4dragon: " + str(consumerchannel2dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 748>

        message2print = "{(Step 40C/70) got" + "; publisherchannel2dragon-got-by-map2image4dragon: " + str(publisherchannel2dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 752>                     
        ack4channels2dragon = True
        
        control2mapimage4dragon['CONSUMERCHANNEL']  = consumerchannel2dragon
        
        
        control2mapimage4dragon['PUBLISHERCHANNEL'] = publisherchannel2dragon
        
        code2ack  = code2ack + '40D-' 
        
    except:
        ack4channels2dragon = False
        code2nack  = code2nack + '40Z-'      
                        
    # ---------------------------------------------------------------------------------------------
    # QUEUE FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4queue2dragon = False
    queue2dragon = None
    message2print = "{(Step 41A/70) trying" + "queue2dragon-trying-by-map2image4dragon: " + str(queue2dragon)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        queue2dragon = task4imagemap2dragon ['QUEUE']

        message2print = "{(Step 41B/70) got" + "; queue2dragon-got-by-map2image4dragon: " + str(queue2dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <queue4xi.Queue4Xi object at 0x0000000004B3FB70> 
              
        ack4queue2dragon = True
        control2mapimage4dragon['QUEUE']  = queue2dragon        
        code2ack  = code2ack + '41C-'
    except:
        ack4queue2dragon = False  
        code2nack  = code2nack + '41Z-'
    
    
    # -----------------------------------------------------------------------------------------    
    # DUMMY PROCESS OBJECT = THREAD CREATION
    # -----------------------------------------------------------------------------------------           
    # dummy2image4dragon = None
    # message2print = "{(Step 69A/70) trying" + "dummy2image4dragon-trying-by-map2image4dragon: " + str(dummy2image4dragon)  + "; time: " + time4xi() + "}"   
    # print4xi(class2print, method2print, message2print)     
    # try:                            
        # dummy2image4dragon = DummyProcess4Xi(target=manoeuvre2mapimage4dragon,                                            # in this module to handle queue and pipe channels
        #                                   args =(queue2dragon,                                                         # caller puts and gets - does NOT call any functions
        #                                          consumerchannel2dragon, publisherchannel2dragon))                        # caller sends and receives - does NOT call any functions
        # message2print = "{(Step 69B/70) created" + "dummy2image4dragon-created-by-map2image4dragon: " + str(dummy2image4dragon)  + "; time: " + time4xi() + "}" 
        # print4xi(class2print, method2print, message2print)  
        # code2ack  = code2ack + '69B-'
        
    # except:
        # code2nack  = code2nack + '69Z-'
        
    # ---------------------------------------------------------------------------------------------
    # RETURN CONTROL AND COMMAND DICTIONARY
    # ---------------------------------------------------------------------------------------------          
    message2print = "{(Step 70A/70) set" + "; code2ack-set-by-map2image4dragon: " + str(code2ack) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                             
    # 
    message2print = "{(Step 70B/70) set" + "; code2nack-set-by-map2image4dragon: " + str(code2nack) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # 1B1-1B2-1C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-12B-13B-14B-15B-16B-18B-19B-40D-41C- 
        
    message2print = "{(Step 70C/70) returning" + "; control2mapimage4dragon-returning-by-map2image4dragon: " + str(control2mapimage4dragon) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004B3FB70>,
    #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 752>,
    #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 748>}; time: 01:28:44 10/05/11 Pacific Daylight Time}

        
    return control2mapimage4dragon
                    
#</function4xi> 

# -----------------------------------------------------------------------------------------------------------------------
#<function4xi function = "manoeuvre2copyimage4dragon" usage = "
#   # ---------------------------------------------------------------------------------------------------------------------
#   # CALL MANOEUVRE2COPYIMAGE4DRAGON - LET IT TO RECEIVE COPY MESSAGE AND TO SEND ROGER MESSAGE VIA PIPE                
#   # ---------------------------------------------------------------------------------------------------------------------       
#   if ack2simulate4dragon is True:
#       print("(Step 42A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4DRAGON ", time4xi())   
#       try:        
#           response2manoeuvre4copy = manoeuvre2copyimage4dragon(queue2copy4dragon, channel2copyconsume4dragon, channel2copypublish4dragon)
#           print("(Step 42B/700) OF DRAGON2IMAGE4XI5TEST - PERFORMED MANOEUVRE2COPYIMAGE4DRAGON, RESPONSE2MANOEUVRE4COPY: ", response2manoeuvre4copy)
#           # {'MESSAGE2SEND4DRAGON': '<roger4xi>roger2copy4dragon</roger4xi>', 'ACK2INFO4DRAGON': False, 'MESSAGE2GET4DRAGON': '<default4xi>default2copy4dragon</default4xi>', 
#           #  'MESSAGE2RECEIVE4DRAGON': '<copy4xi>copy2copy4dragon</copy4xi>', 
#           'CODE2ACK4DRAGON': ' 0D-1C-', 'MESSAGE2PUT4DRAGON': '<default4xi>default2copy4dragon</default4xi>', 'CODE2NACK4DRAGON': ' ', 'STATE2MANOEUVRE4DRAGON': 1}
#        except:
#           print("(Step 42Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4DRAGON ", time4xi())     
#   else:
#       pass # trust activated thread
#">            

# ----------------------------------------------------------------------------------------------------------------------- 
def manoeuvre2copyimage4dragon(queue2copymanoeuvre4dragon, consumerchannel2copymanoeuvre4dragon, publisherchannel2copymanoeuvre4dragon):
    '''
    manoeuvres copy control and command
    '''
    global state2copyimage4dragon                                                                                           # STATE FOR COMMAND AND CONTROL
    global max2copyimagestate4dragon                                                                                        # MAX FOR STATE
    global ack2copyimageinfo4dragon                                                                                         # ACK FOR IMAGE INFO
    global message2copyimageget4dragon                                 
    global message2copyimageput4dragon
    global message2copyimagereceive4dragon
    global message2copyimagesend4dragon
    
    # -----------------------------------------------------------------------------------------
    # (Step 0/33) PARAMETERS            
    # -----------------------------------------------------------------------------------------          
    class2print  = "manoeuvre2copyimage4dragon" 
    method2print = "manoeuvre2copyimage4dragon" 

    code2ack  = ' '
    code2nack = ' '
    
    message2print = "{(Step 0A/33) started" + "; manoeuvre2copyimage4dragon-started: " + str(manoeuvre2copyimage4dragon)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function manoeuvre2copyimage4dragon at 0x0000000003DB8C48>  
    
    message2print = "{(Step 0B/33) passed" + "; queue2copymanoeuvre4dragon-to-manoeuvre2copyimage4dragon: " + str(queue2copymanoeuvre4dragon) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <queue4xi.Queue4Xi object at 0x0000000003AD9400>
    
    message2print = "{(Step 0C/33) passed" + "; consumerchannel2copymanoeuvre4dragon-to-manoeuvre2copyimage4dragon: " + str(consumerchannel2copymanoeuvre4dragon) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <copy-write PipeConnection, handle 408>
    
    message2print = "{(Step 0D/33) passed" + "; publisherchannel2copymanoeuvre4dragon-to-manoeuvre2copyimage4dragon: " + str(publisherchannel2copymanoeuvre4dragon) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <copy-write PipeConnection, handle 412>     
    
    code2ack  = code2ack + '0D-' 
    response2manouvre4copy = {}
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # SETTINGS
    # ------------------------------------------------------------------------------------------------------------------------    
    message2print = "{(Step 0E/70) try" + "; state2copyimage4dragon-try-by-manoeuvre2copyimage4dragon"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        if state2copyimage4dragon is None:
            state2copyimage4dragon = 0
            message2print = "{(Step 0F/33) initialized" + "; state2copyimage4dragon-initialized-by-manoeuvre2copyimage4dragon to: " + str(state2copyimage4dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        else:
            message2print = "{(Step 0G/33) used" + "; state2copyimage4dragon-used-by-manoeuvre2copyimage4dragon: " + str(state2copyimage4dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1; 2; 3;  
    except:
        state2copyimage4dragon = 0
        message2print = "{(Step 0Z1/33) initialized" + "; state2copyimage4dragon-initialized-by-manoeuvre2copyimage4dragon to: " + str(state2copyimage4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2copyimageinfo4dragon        = False
        message2print = "{(Step 0Z2/33) initialized" + "; ack2copyimageinfo4dragon-initialized-by-manoeuvre2copyimage4dragon to: " + str(ack2copyimageinfo4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2copyimageget4dragon     = '<default4xi>default2copy4dragon</default4xi>'
        message2print = "{(Step 0Z3/33) initialized" + "; message2copyimageget4dragon-initialized-by-manoeuvre2copyimage4dragon to: " + str(message2copyimageget4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2copy4dragon</default4xi>
        message2copyimageput4dragon     = '<default4xi>default2copy4dragon</default4xi>'
        message2print = "{(Step 0Z4/33) initialized" + "; message2copyimageput4dragon-initialized-by-manoeuvre2copyimage4dragon to: " + str(message2copyimageput4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2copy4dragon</default4xi>
        message2copyimagereceive4dragon = '<default4xi>default2copy4dragon</default4xi>'
        message2print = "{(Step 0Z5/33) initialized" + "; message2copyimagereceive4dragon-initialized-by-manoeuvre2copyimage4dragon to: " + str(message2copyimagereceive4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2copy4dragon</default4xi>
        message2copyimagesend4dragon    = '<default4xi>default2copy4dragon</default4xi>'
        message2print = "{(Step 0Z6/33) initialized" + "; message2copyimagesend4dragon-initialized-by-manoeuvre2copyimage4dragon to: " + str(message2copyimagesend4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2copy4dragon</default4xi>
        max2copyimagestate4dragon = 3
        message2print = "{(Step 0Z7/33) initialized" + ";  max2copyimagestate4dragon-initialized-by-manoeuvre2copyimage4dragon to: " + str( max2copyimagestate4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE UPDATE
    # ------------------------------------------------------------------------------------------------------------------------                      
    state2copyimage4dragon = state2copyimage4dragon + 1                                                                   
    message2print = "{(Step 0H/33) added" + "; state2copyimage4dragon-added-by-manoeuvre2copyimage4dragon: " + str(state2copyimage4dragon) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 1; 2; 3; 4  
    if state2copyimage4dragon > max2copyimagestate4dragon:
        state2copyimage4dragon = 0
        message2print = "{(Step 0I/33) set" + "; state2copyimage4dragon-set-by-manoeuvre2copyimage4dragon to: " + str(state2copyimage4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2copyimageinfo4dragon        = False
        message2print = "{(Step 0J/33) set" + "; ack2copyimageinfo4dragon-set-by-manoeuvre2copyimage4dragon to: " + str(ack2copyimageinfo4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2copyimageget4dragon     = '<default4xi>default2copy4dragon</default4xi>'
        message2print = "{(Step 0K/33) set" + "; message2copyimageget4dragon-set-by-manoeuvre2copyimage4dragon to: " + str(message2copyimageget4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2copy4dragon</default4xi>
        message2copyimageput4dragon     = '<default4xi>default2copy4dragon</default4xi>'
        message2print = "{(Step 0L/33) set" + "; message2copyimageput4dragon-set-by-manoeuvre2copyimage4dragon to: " + str(message2copyimageput4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2copy4dragon</default4xi>
        message2copyimagereceive4dragon = '<default4xi>default2copy4dragon</default4xi>'
        message2print = "{(Step 0M/33) set" + "; message2copyimagereceive4dragon-set-by-manoeuvre2copyimage4dragon to: " + str(message2copyimagereceive4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2copy4dragon</default4xi>
        message2copyimagesend4dragon    = '<default4xi>default2copy4dragon</default4xi>'
        message2print = "{(Step 0N/33) set" + "; message2copyimagesend4dragon-set-by-manoeuvre2copyimage4dragon to: " + str(message2copyimagesend4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2copy4dragon</default4xi>
        max2copyimagestate4dragon = 3
        message2print = "{(Step 0O/33) initialized" + ";  max2copyimagestate4dragon-set-by-manoeuvre2copyimage4dragon to: " + str( max2copyimagestate4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #     
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE I - RECEIVE CONTROL AND COMMAND COPY MESSAGE VIA PIPE AND SEND ROGER OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copyimage4dragon == 1:
        message2print = "{(Step 1A/70) try" + "; receive-try-by-manoeuvre2copyimage4dragon"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2copyimagereceive4dragon = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2copyimagereceive4dragon = consumerchannel2copymanoeuvre4dragon.recv()
            message2print = "{(Step 1B/33) received" + "; message2copyimagereceive4dragon-received-by-manoeuvre2copyimage4dragon: " + str(message2copyimagereceive4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # <copy4xi>copy2copy4dragon</copy4xi>
                         
            if message2copyimagereceive4dragon == '<copy4xi>copy2copy4dragon</copy4xi>':
                message2copyimagesend4dragon = '<roger4xi>roger2copy4dragon</roger4xi>'
            else:
                message2copyimagesend4dragon = '<negative4xi><negative2copy4dragon</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2copymanoeuvre4dragon.send(message2copyimagesend4dragon)
            
            message2print = "{(Step 1C/33) sent" + "; message2copyimagesend4dragon-sent-by-manoeuvre2copyimage4dragon: " + str(message2copyimagesend4dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <roger4xi>roger2copy4dragon</roger4xi>
            
            code2ack  = code2ack + '1C-'
        except:
            message2print = "{(Step 1Z/33) failed" + "; message2copyimagereceive4dragon-failed-by-manoeuvre2copyimage4dragon: " + str(message2copyimagereceive4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '1Z-' 
    else:
        pass

                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE II - RECEIVE CONTROL AND COMMAND READY MESSAGE VIA PIPE AND SEND READY OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copyimage4dragon == 2:
        message2print = "{(Step 2A/70) try" + "; receive-try-by-manoeuvre2copyimage4dragon"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2copyimagereceive4dragon = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2copyimagereceive4dragon = consumerchannel2copymanoeuvre4dragon.recv()
            message2print = "{(Step 2B/33) received" + "; message2copyimagereceive4dragon-received-by-manoeuvre2copyimage4dragon: " + str(message2copyimagereceive4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # <ready4xi>ready2copy4dragon</ready4xi>
                         
            if message2copyimagereceive4dragon == '<ready4xi>ready2copy4dragon</ready4xi>':
                message2copyimagesend4dragon = '<ready4xi>ready2copy4dragon</ready4xi>'
            else:
                message2copyimagesend4dragon = '<negative4xi><negative2copy4dragon</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND READY OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2copymanoeuvre4dragon.send(message2copyimagesend4dragon)
            
            message2print = "{(Step 2C/33) sent" + "; message2copyimagesend4dragon-sent-by-manoeuvre2copyimage4dragon: " + str(message2copyimagesend4dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code2ack  = code2ack + '2C-'
        except:
            message2print = "{(Step 2Z/33) failed" + "; message2copyimagereceive4dragon-failed-by-manoeuvre2copyimage4dragon: " + str(message2copyimagereceive4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '2Z-' 
    else:
        pass        
                
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE III - GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE AND SEND ACK OR NACK VIA PIPE
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copyimage4dragon == 3:    
        info2image4dragon = None
        message2print = "{(Step 3A/70) try" + "; info2image4dragon-try-by-manoeuvre2copyimage4dragon"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        try: 
            # ----------------------------------------------------------------------------------------------------------------
            # GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE
            # ----------------------------------------------------------------------------------------------------------------        
            info2image4dragon = queue2copymanoeuvre4dragon.get()
            
            message2print = "{(Step 3B/33) got" + "; info2image4dragon-got-by-manoeuvre2copyimage4dragon: " + str(info2image4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # ..//..//..//..//..//web//images//png//dragonxi.png
            if info2image4dragon is None:
                code2nack  = code2nack + '3Z1-'
                ack2copyimageinfo4dragon = False 
            else:    
                code2ack  = code2ack + '3C-'
                ack2copyimageinfo4dragon = True
        except:
            message2print = "{(Step 3Z1/33) failed" + "; queue2copymanoeuvre4dragon-failed-by-manoeuvre2copyimage4dragon: " + str(queue2copymanoeuvre4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z1-'
            
        message2copyimagesend4dragon = ' '
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # SEND CONTROL INFO ABOUT IMAGE OPERATION - EITHER ACK OR NACK  
            # ----------------------------------------------------------------------------------------------------------------         
            if ack2copyimageinfo4dragon is True:
                message2copyimagesend4dragon = '<ack4xi>ack2imageinfo4dragon</ack4xi>'
            else:
                message2copyimagesend4dragon = '<nack4xi>nack2imageinfo4dragon</nack4xi>'
            message2print = "{(Step 4B/33) set" + "; message2copyimagesend4dragon-set-by-manoeuvre2copyimage4dragon to: " + str(message2copyimagesend4dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # <ack4xi>ack2imageinfo4dragon</ack4xi>       
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2copyimagesend4dragon-failed-by-manoeuvre2copyimage4dragon: " + str(message2copyimagesend4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            code2nack  = code2nack + '4Z1-'    
        try:                
            message2print = "{(Step 4C/70) using" + "; publisherchannel2copymanoeuvre4dragon-calling-by-manoeuvre2copyimage4dragon: " + str(publisherchannel2copymanoeuvre4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # <copy-write PipeConnection, handle 412>
            message2print = "{{(Step 4D/70) passing" + "; message2copyimagesend4dragon-passing-to-send-by-manoeuvre2copyimage4dragon: " + str(message2copyimagesend4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2copyimagesend4dragon-failed-by-manoeuvre2copyimage4dragon: " + str(message2copyimagesend4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z2-'                
        try:    
            publisherchannel2copymanoeuvre4dragon.send(message2copyimagesend4dragon)
            
            message2print = "{(Step 4E/33) sent" + "; message2copyimagesend4dragon-sent-by-manoeuvre2copyimage4dragon: " + str(message2copyimagesend4dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <ack4xi>ack2imageinfo4dragon</ack4xi>            
            code2ack  = code2ack + '4E-'
        except:
            message2print = "{(Step 4Z3/33) failed" + "; message2copyimagesend4dragon-failed-by-manoeuvre2copyimage4dragon: " + str(message2copyimagesend4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z3-'                
    else:
        pass
        

            
    # ------------------------------------------------------------------------------------------------------------------------
    # RETURN 
    # ------------------------------------------------------------------------------------------------------------------------
    response2manouvre4copy['ACK2INFO4DRAGON']        = ack2copyimageinfo4dragon
    response2manouvre4copy['CODE2ACK4DRAGON']        = code2ack
    response2manouvre4copy['CODE2NACK4DRAGON']       = code2nack
    response2manouvre4copy['MESSAGE2GET4DRAGON']     = message2copyimageget4dragon
    response2manouvre4copy['MESSAGE2PUT4DRAGON']     = message2copyimageput4dragon
    response2manouvre4copy['MESSAGE2RECEIVE4DRAGON'] = message2copyimagereceive4dragon
    response2manouvre4copy['MESSAGE2SEND4DRAGON']    = message2copyimagesend4dragon
    response2manouvre4copy['STATE2MANOEUVRE4DRAGON'] = state2copyimage4dragon
                          
    message2print = "{(Step 33/33) returning" + "; response2manouvre4copy-returning-by-manoeuvre2copyimage4dragon: " + str(response2manouvre4copy) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #1 {'MESSAGE2SEND4DRAGON': '<roger4xi>roger2copy4dragon</roger4xi>',
    #   'ACK2INFO4DRAGON': False,
    #   'MESSAGE2GET4DRAGON': '<default4xi>default2copy4dragon</default4xi>',
    #   'MESSAGE2RECEIVE4DRAGON': '<copy4xi>copy2copy4dragon</copy4xi>',
    #   'CODE2ACK4DRAGON': ' 0D-1C-',
    #   'MESSAGE2PUT4DRAGON': '<default4xi>default2copy4dragon</default4xi>',
    #   'CODE2NACK4DRAGON': ' ',
    #   'STATE2MANOEUVRE4DRAGON': 1}; time: 05:25:15 10/03/11 Pacific Daylight Time}

 
    #2 {'MESSAGE2SEND4DRAGON': '<ready4xi>ready2copy4dragon</ready4xi>',
    #   'ACK2INFO4DRAGON': False,
    #   'MESSAGE2GET4DRAGON': '<default4xi>default2copy4dragon</default4xi>',
    #   'MESSAGE2RECEIVE4DRAGON': '<ready4xi>ready2copy4dragon</ready4xi>',
    #   'CODE2ACK4DRAGON': ' 0D-2C-',
    #   'MESSAGE2PUT4DRAGON': '<default4xi>default2copy4dragon</default4xi>',
    #   'CODE2NACK4DRAGON': ' ',
    #   'STATE2MANOEUVRE4DRAGON': 2}; time: 05:25:15 10/03/11 Pacific Daylight Time}

   
    #3 {'MESSAGE2SEND4DRAGON': '<ack4xi>ack2imageinfo4dragon</ack4xi>',
    #   'ACK2INFO4DRAGON': True,
    #   'MESSAGE2GET4DRAGON': '<default4xi>default2copy4dragon</default4xi>',
    #   'MESSAGE2RECEIVE4DRAGON': '<ready4xi>ready2copy4dragon</ready4xi>',
    #   'CODE2ACK4DRAGON': ' 0D-3C-4E-',
    #   'MESSAGE2PUT4DRAGON': '<default4xi>default2copy4dragon</default4xi>',
    #   'CODE2NACK4DRAGON': ' ', 'STATE2MANOEUVRE4DRAGON': 3}; time: 05:25:15 10/03/11 Pacific Daylight Time}

    
    #4 {'MESSAGE2SEND4DRAGON': '<default4xi>default2copy4dragon</default4xi>',
    #   'ACK2INFO4DRAGON': False,
    #   'MESSAGE2GET4DRAGON': '<default4xi>default2copy4dragon</default4xi>',
    #   'MESSAGE2RECEIVE4DRAGON': '<default4xi>default2copy4dragon</default4xi>',
    #   'CODE2ACK4DRAGON': ' 0D-',
    #   'MESSAGE2PUT4DRAGON': '<default4xi>default2copy4dragon</default4xi>',
    #   'CODE2NACK4DRAGON': ' ', 'STATE2MANOEUVRE4DRAGON': 0}; time: 05:25:15 10/03/11 Pacific Daylight Time}

    return response2manouvre4copy           
          
#</function4xi>


# -----------------------------------------------------------------------------------------------------------------------
#<function4xi function = "manoeuvre2copyimagetobuffer4dragon" usage = "
#   # ---------------------------------------------------------------------------------------------------------------------
#   # CALL MANOEUVRE2COPYIMAGE4DRAGON  - LET IT TO RECEIVE COPY AND TO SEND ROGER VIA PIPE                
#   # ---------------------------------------------------------------------------------------------------------------------       
#   if ack2simulate4dragon is True:
#       print("(Step 62A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4DRAGON ", time4xi())   
#       try:        
#           response2manoeuvre4copy = manoeuvre2copyimagetobuffer4dragon(queue4copy2imagetobuffer4dragon, channel4copy2imagetobufferconsume4dragon, channel4copy2imagetobufferpublish4dragon)
#           print("(Step 62B/700) OF DRAGON2IMAGE4XI5TEST - PERFORMED MANOEUVRE2COPYIMAGE4DRAGON, RESPONSE2MANOEUVRE4COPY: ", response2manoeuvre4copy)
#           # { 'MESSAGE2SEND4DRAGON': '<roger4xi>roger2copyimagetobuffer4dragon</roger4xi>',
#           #   'ACK2INFO4DRAGON': False,
#           #   'MESSAGE2GET4DRAGON': '<default4xi>default2copyimagetobuffer4dragon</default4xi>',
#           #   'MESSAGE2RECEIVE4DRAGON': '<copy4xi>copy2copyimagetobuffer4dragon</copy4xi>',
#           #   'CODE2ACK4DRAGON': ' 0D-1C-',
#           #   'MESSAGE2PUT4DRAGON': '<default4xi>default2copyimagetobuffer4dragon</default4xi>',
#           #   'CODE2NACK4DRAGON': ' ',
#           #   'STATE2MANOEUVRE4DRAGON': 1}
#        except:
#           print("(Step 62Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4DRAGON ", time4xi())     
#   else:
#       pass # trust activated thread    
#">

# ----------------------------------------------------------------------------------------------------------------------- 
def manoeuvre2copyimagetobuffer4dragon(queue2copyimagetobuffermanoeuvre4dragon, consumerchannel2copyimagetobuffermanoeuvre4dragon, publisherchannel2copyimagetobuffermanoeuvre4dragon):
    '''
    manoeuvres copy control and command for copy2imagetobuffer4dragon
    '''
    global state2copyimagetobuffer4dragon                                                                                           # STATE FOR COMMAND AND CONTROL
    global max2copyimagetobufferstate4dragon                                                                                        # MAX FOR STATE
    global ack2copyimagetobufferinfo4dragon                                                                                         # ACK FOR IMAGE INFO
    global message2copyimagetobufferget4dragon                                 
    global message2copyimagetobufferput4dragon
    global message2copyimagetobufferreceive4dragon
    global message2copyimagetobuffersend4dragon
    
    # -----------------------------------------------------------------------------------------
    # (Step 0/33) PARAMETERS            
    # -----------------------------------------------------------------------------------------          
    class2print  = "manoeuvre2copyimagetobuffer4dragon" 
    method2print = "manoeuvre2copyimagetobuffer4dragon" 

    code2ack  = ' '
    code2nack = ' '
    
    message2print = "{(Step 0A/33) started" + "; manoeuvre2copyimagetobuffer4dragon-started: " + str(manoeuvre2copyimagetobuffer4dragon)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # 
    
    message2print = "{(Step 0B/33) passed" + "; queue2copyimagetobuffermanoeuvre4dragon-to-manoeuvre2copyimagetobuffer4dragon: " + str(queue2copyimagetobuffermanoeuvre4dragon) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0C/33) passed" + "; consumerchannel2copyimagetobuffermanoeuvre4dragon-to-manoeuvre2copyimagetobuffer4dragon: " + str(consumerchannel2copyimagetobuffermanoeuvre4dragon) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0D/33) passed" + "; publisherchannel2copyimagetobuffermanoeuvre4dragon-to-manoeuvre2copyimagetobuffer4dragon: " + str(publisherchannel2copyimagetobuffermanoeuvre4dragon) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    code2ack  = code2ack + '0D-' 
    response2manouvre4copyimagetobuffer = {}
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # SETTINGS
    # ------------------------------------------------------------------------------------------------------------------------    
    message2print = "{(Step 0E/70) try" + "; state2copyimagetobuffer4dragon-try-by-manoeuvre2copyimagetobuffer4dragon"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        if state2copyimagetobuffer4dragon is None:
            state2copyimagetobuffer4dragon = 0
            message2print = "{(Step 0F/33) initialized" + "; state2copyimagetobuffer4dragon-initialized-by-manoeuvre2copyimagetobuffer4dragon to: " + str(state2copyimagetobuffer4dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        else:
            message2print = "{(Step 0G/33) used" + "; state2copyimagetobuffer4dragon-used-by-manoeuvre2copyimagetobuffer4dragon: " + str(state2copyimagetobuffer4dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1; 2; 3;  
    except:
        state2copyimagetobuffer4dragon = 0
        message2print = "{(Step 0Z1/33) initialized" + "; state2copyimagetobuffer4dragon-initialized-by-manoeuvre2copyimagetobuffer4dragon to: " + str(state2copyimagetobuffer4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2copyimagetobufferinfo4dragon        = False
        message2print = "{(Step 0Z2/33) initialized" + "; ack2copyimagetobufferinfo4dragon-initialized-by-manoeuvre2copyimagetobuffer4dragon to: " + str(ack2copyimagetobufferinfo4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2copyimagetobufferget4dragon     = '<default4xi>default2copyimagetobuffer4dragon</default4xi>'
        message2print = "{(Step 0Z3/33) initialized" + "; message2copyimagetobufferget4dragon-initialized-by-manoeuvre2copyimagetobuffer4dragon to: " + str(message2copyimagetobufferget4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # 
        message2copyimagetobufferput4dragon     = '<default4xi>default2copyimagetobuffer4dragon</default4xi>'
        message2print = "{(Step 0Z4/33) initialized" + "; message2copyimagetobufferput4dragon-initialized-by-manoeuvre2copyimagetobuffer4dragon to: " + str(message2copyimagetobufferput4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copyimagetobufferreceive4dragon = '<default4xi>default2copyimagetobuffer4dragon</default4xi>'
        message2print = "{(Step 0Z5/33) initialized" + "; message2copyimagetobufferreceive4dragon-initialized-by-manoeuvre2copyimagetobuffer4dragon to: " + str(message2copyimagetobufferreceive4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copyimagetobuffersend4dragon    = '<default4xi>default2copyimagetobuffer4dragon</default4xi>'
        message2print = "{(Step 0Z6/33) initialized" + "; message2copyimagetobuffersend4dragon-initialized-by-manoeuvre2copyimagetobuffer4dragon to: " + str(message2copyimagetobuffersend4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        max2copyimagetobufferstate4dragon = 3
        message2print = "{(Step 0Z7/33) initialized" + ";  max2copyimagetobufferstate4dragon-initialized-by-manoeuvre2copyimagetobuffer4dragon to: " + str( max2copyimagetobufferstate4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE UPDATE
    # ------------------------------------------------------------------------------------------------------------------------                      
    state2copyimagetobuffer4dragon = state2copyimagetobuffer4dragon + 1                                                                   
    message2print = "{(Step 0H/33) added" + "; state2copyimagetobuffer4dragon-added-by-manoeuvre2copyimagetobuffer4dragon: " + str(state2copyimagetobuffer4dragon) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 1; 2; 3; 4 
     
    if state2copyimagetobuffer4dragon > max2copyimagetobufferstate4dragon:
        state2copyimagetobuffer4dragon = 0
        message2print = "{(Step 0I/33) set" + "; state2copyimagetobuffer4dragon-set-by-manoeuvre2copyimagetobuffer4dragon to: " + str(state2copyimagetobuffer4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        ack2copyimagetobufferinfo4dragon        = False
        message2print = "{(Step 0J/33) set" + "; ack2copyimagetobufferinfo4dragon-set-by-manoeuvre2copyimagetobuffer4dragon to: " + str(ack2copyimagetobufferinfo4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # 
        message2copyimagetobufferget4dragon     = '<default4xi>default2copyimagetobuffer4dragon</default4xi>'
        message2print = "{(Step 0K/33) set" + "; message2copyimagetobufferget4dragon-set-by-manoeuvre2copyimagetobuffer4dragon to: " + str(message2copyimagetobufferget4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # 
        message2copyimagetobufferput4dragon     = '<default4xi>default2copyimagetobuffer4dragon</default4xi>'
        message2print = "{(Step 0L/33) set" + "; message2copyimagetobufferput4dragon-set-by-manoeuvre2copyimagetobuffer4dragon to: " + str(message2copyimagetobufferput4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copyimagetobufferreceive4dragon = '<default4xi>default2copyimagetobuffer4dragon</default4xi>'
        message2print = "{(Step 0M/33) set" + "; message2copyimagetobufferreceive4dragon-set-by-manoeuvre2copyimagetobuffer4dragon to: " + str(message2copyimagetobufferreceive4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copyimagetobuffersend4dragon    = '<default4xi>default2copyimagetobuffer4dragon</default4xi>'
        message2print = "{(Step 0N/33) set" + "; message2copyimagetobuffersend4dragon-set-by-manoeuvre2copyimagetobuffer4dragon to: " + str(message2copyimagetobuffersend4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        max2copyimagetobufferstate4dragon = 3
        message2print = "{(Step 0O/33) initialized" + ";  max2copyimagetobufferstate4dragon-set-by-manoeuvre2copyimagetobuffer4dragon to: " + str( max2copyimagetobufferstate4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #     
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE I - RECEIVE CONTROL AND COMMAND COPY MESSAGE VIA PIPE AND SEND ROGER OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copyimagetobuffer4dragon == 1:
        message2print = "{(Step 1A/70) try" + "; receive-try-by-manoeuvre2copyimagetobuffer4dragon"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2copyimagetobufferreceive4dragon = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2copyimagetobufferreceive4dragon = consumerchannel2copyimagetobuffermanoeuvre4dragon.recv()
            message2print = "{(Step 1B/33) received" + "; message2copyimagetobufferreceive4dragon-received-by-manoeuvre2copyimagetobuffer4dragon: " + str(message2copyimagetobufferreceive4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message2copyimagetobufferreceive4dragon == '<copy4xi>copy2copyimagetobuffer4dragon</copy4xi>': 
                message2copyimagetobuffersend4dragon = '<roger4xi>roger2copyimagetobuffer4dragon</roger4xi>'
            else:
                message2copyimagetobuffersend4dragon = '<negative4xi><negative2copyimagetobuffer4dragon</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2copyimagetobuffermanoeuvre4dragon.send(message2copyimagetobuffersend4dragon)
            
            message2print = "{(Step 1C/33) sent" + "; message2copyimagetobuffersend4dragon-sent-by-manoeuvre2copyimagetobuffer4dragon: " + str(message2copyimagetobuffersend4dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code2ack  = code2ack + '1C-'
        except:
            message2print = "{(Step 1Z/33) failed" + "; message2copyimagetobufferreceive4dragon-failed-by-manoeuvre2copyimagetobuffer4dragon: " + str(message2copyimagetobufferreceive4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '1Z-' 
    else:
        pass

                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE II - RECEIVE CONTROL AND COMMAND READY MESSAGE VIA PIPE AND SEND READY OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copyimagetobuffer4dragon == 2:
        message2print = "{(Step 2A/70) try" + "; receive-try-by-manoeuvre2copyimagetobuffer4dragon"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2copyimagetobufferreceive4dragon = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2copyimagetobufferreceive4dragon = consumerchannel2copyimagetobuffermanoeuvre4dragon.recv()
            message2print = "{(Step 2B/33) received" + "; message2copyimagetobufferreceive4dragon-received-by-manoeuvre2copyimagetobuffer4dragon: " + str(message2copyimagetobufferreceive4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message2copyimagetobufferreceive4dragon == '<ready4xi>ready2copyimagetobuffer4dragon</ready4xi>':
                message2copyimagetobuffersend4dragon = '<ready4xi>ready2copyimagetobuffer4dragon</ready4xi>'
            else:
                message2copyimagetobuffersend4dragon = '<negative4xi><negative2copyimagetobuffer4dragon</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND READY OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2copyimagetobuffermanoeuvre4dragon.send(message2copyimagetobuffersend4dragon)
            
            message2print = "{(Step 2C/33) sent" + "; message2copyimagetobuffersend4dragon-sent-by-manoeuvre2copyimagetobuffer4dragon: " + str(message2copyimagetobuffersend4dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code2ack  = code2ack + '2C-'
        except:
            message2print = "{(Step 2Z/33) failed" + "; message2copyimagetobufferreceive4dragon-failed-by-manoeuvre2copyimagetobuffer4dragon: " + str(message2copyimagetobufferreceive4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '2Z-' 
    else:
        pass        
                
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE III - GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE AND SEND ACK OR NACK VIA PIPE
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copyimagetobuffer4dragon == 3:    
        info2image4dragon = None
        message2print = "{(Step 3A/70) try" + "; info2image4dragon-try-by-manoeuvre2copyimagetobuffer4dragon"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        try: 
            # ----------------------------------------------------------------------------------------------------------------
            # GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE
            # ----------------------------------------------------------------------------------------------------------------        
            info2image4dragon = queue2copyimagetobuffermanoeuvre4dragon.get()
            
            message2print = "{(Step 3B/33) got" + "; info2image4dragon-got-by-manoeuvre2copyimagetobuffer4dragon: " + str(info2image4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # ..//..//..//..//..//web//images//png//dragonxi.png
            if info2image4dragon is None:
                code2nack  = code2nack + '3Z1-'
                ack2copyimagetobufferinfo4dragon = False 
            else:    
                code2ack  = code2ack + '3C-'
                ack2copyimagetobufferinfo4dragon = True
        except:
            message2print = "{(Step 3Z1/33) failed" + "; queue2copyimagetobuffermanoeuvre4dragon-failed-by-manoeuvre2copyimagetobuffer4dragon: " + str(queue2copyimagetobuffermanoeuvre4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z1-'
            
        message2copyimagetobuffersend4dragon = ' '
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # SEND CONTROL INFO ABOUT IMAGE OPERATION - EITHER ACK OR NACK  
            # ----------------------------------------------------------------------------------------------------------------         
            if ack2copyimagetobufferinfo4dragon is True:
                message2copyimagetobuffersend4dragon = '<ack4xi>ack2imageinfo4dragon</ack4xi>'
            else:
                message2copyimagetobuffersend4dragon = '<nack4xi>nack2imageinfo4dragon</nack4xi>'
            message2print = "{(Step 4B/33) set" + "; message2copyimagetobuffersend4dragon-set-by-manoeuvre2copyimagetobuffer4dragon to: " + str(message2copyimagetobuffersend4dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # <ack4xi>ack2imageinfo4dragon</ack4xi>       
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2copyimagetobuffersend4dragon-failed-by-manoeuvre2copyimagetobuffer4dragon: " + str(message2copyimagetobuffersend4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            code2nack  = code2nack + '4Z1-'    
        try:                
            message2print = "{(Step 4C/70) using" + "; publisherchannel2copyimagetobuffermanoeuvre4dragon-calling-by-manoeuvre2copyimagetobuffer4dragon: " + str(publisherchannel2copyimagetobuffermanoeuvre4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # <copy-write PipeConnection, handle 412>
            message2print = "{{(Step 4D/70) passing" + "; message2copyimagetobuffersend4dragon-passing-to-send-by-manoeuvre2copyimagetobuffer4dragon: " + str(message2copyimagetobuffersend4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2copyimagetobuffersend4dragon-failed-by-manoeuvre2copyimagetobuffer4dragon: " + str(message2copyimagetobuffersend4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z2-'                
        try:    
            publisherchannel2copyimagetobuffermanoeuvre4dragon.send(message2copyimagetobuffersend4dragon)
            
            message2print = "{(Step 4E/33) sent" + "; message2copyimagetobuffersend4dragon-sent-by-manoeuvre2copyimagetobuffer4dragon: " + str(message2copyimagetobuffersend4dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <ack4xi>ack2imageinfo4dragon</ack4xi>            
            code2ack  = code2ack + '4E-'
        except:
            message2print = "{(Step 4Z3/33) failed" + "; message2copyimagetobuffersend4dragon-failed-by-manoeuvre2copyimagetobuffer4dragon: " + str(message2copyimagetobuffersend4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z3-'                
    else:
        pass
        

            
    # ------------------------------------------------------------------------------------------------------------------------
    # RETURN 
    # ------------------------------------------------------------------------------------------------------------------------
    response2manouvre4copyimagetobuffer['ACK2INFO4DRAGON']        = ack2copyimagetobufferinfo4dragon
    response2manouvre4copyimagetobuffer['CODE2ACK4DRAGON']        = code2ack
    response2manouvre4copyimagetobuffer['CODE2NACK4DRAGON']       = code2nack
    response2manouvre4copyimagetobuffer['MESSAGE2GET4DRAGON']     = message2copyimagetobufferget4dragon
    response2manouvre4copyimagetobuffer['MESSAGE2PUT4DRAGON']     = message2copyimagetobufferput4dragon
    response2manouvre4copyimagetobuffer['MESSAGE2RECEIVE4DRAGON'] = message2copyimagetobufferreceive4dragon
    response2manouvre4copyimagetobuffer['MESSAGE2SEND4DRAGON']    = message2copyimagetobuffersend4dragon
    response2manouvre4copyimagetobuffer['STATE2MANOEUVRE4DRAGON'] = state2copyimagetobuffer4dragon
                          
    message2print = "{(Step 33/33) returning" + "; response2manouvre4copyimagetobuffer-returning-by-manoeuvre2copyimagetobuffer4dragon: " + str(response2manouvre4copyimagetobuffer) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #1 {'MESSAGE2SEND4DRAGON': '<roger4xi>roger2copyimagetobuffer4dragon</roger4xi>',
    #   'ACK2INFO4DRAGON': False,
    #   'MESSAGE2GET4DRAGON': '<default4xi>default2copyimagetobuffer4dragon</default4xi>',
    #   'MESSAGE2RECEIVE4DRAGON': '<copy4xi>copy2copyimagetobuffer4dragon</copy4xi>',
    #   'CODE2ACK4DRAGON': ' 0D-1C-',
    #   'MESSAGE2PUT4DRAGON': '<default4xi>default2copyimagetobuffer4dragon</default4xi>',
    #   'CODE2NACK4DRAGON': ' ',
    #   'STATE2MANOEUVRE4DRAGON': 1}; time: 02:00:20 10/04/11 Pacific Daylight Time}

    
    #2 {'MESSAGE2SEND4DRAGON': '<ready4xi>ready2copyimagetobuffer4dragon</ready4xi>',
    #   'ACK2INFO4DRAGON': False,
    #   'MESSAGE2GET4DRAGON': '<default4xi>default2copyimagetobuffer4dragon</default4xi>',
    #   'MESSAGE2RECEIVE4DRAGON': '<ready4xi>ready2copyimagetobuffer4dragon</ready4xi>',
    #   'CODE2ACK4DRAGON': ' 0D-2C-',
    #   'MESSAGE2PUT4DRAGON': '<default4xi>default2copyimagetobuffer4dragon</default4xi>',
    #   'CODE2NACK4DRAGON': ' ',
    #   'STATE2MANOEUVRE4DRAGON': 2}; time: 02:10:33 10/04/11 Pacific Daylight Time}


    #3 {'MESSAGE2SEND4DRAGON': '<ack4xi>ack2imageinfo4dragon</ack4xi>',
    #   'ACK2INFO4DRAGON': True,
    #   'MESSAGE2GET4DRAGON': '<default4xi>default2copyimagetobuffer4dragon</default4xi>',
    #   'MESSAGE2RECEIVE4DRAGON': '<ready4xi>ready2copyimagetobuffer4dragon</ready4xi>',
    #   'CODE2ACK4DRAGON': ' 0D-3C-4E-',
    #   'MESSAGE2PUT4DRAGON': '<default4xi>default2copyimagetobuffer4dragon</default4xi>',
    #   'CODE2NACK4DRAGON': ' ',
    #   'STATE2MANOEUVRE4DRAGON': 3}; time: 02:10:33 10/04/11 Pacific Daylight Time}


    #4 {'MESSAGE2SEND4DRAGON': '<default4xi>default2copyimagetobuffer4dragon</default4xi>',
    #   'ACK2INFO4DRAGON': False,
    #   'MESSAGE2GET4DRAGON': '<default4xi>default2copyimagetobuffer4dragon</default4xi>',
    #   'MESSAGE2RECEIVE4DRAGON': '<default4xi>default2copyimagetobuffer4dragon</default4xi>',
    #   'CODE2ACK4DRAGON': ' 0D-',
    #   'MESSAGE2PUT4DRAGON': '<default4xi>default2copyimagetobuffer4dragon</default4xi>',
    #   'CODE2NACK4DRAGON': ' ',
    #   'STATE2MANOEUVRE4DRAGON': 0}; time: 02:10:33 10/04/11 Pacific Daylight Time}


    return response2manouvre4copyimagetobuffer           
          
#</function4xi>

# -----------------------------------------------------------------------------------------------------------------------
#<function4xi function = "manoeuvre2copybuffertoimage4dragon" usage = "
#   # ---------------------------------------------------------------------------------------------------------------------
#   # SIMULATE MANOEUVRE2COPYIMAGE4DRAGON - LET IT TO RECEIVE COPY AND TO SEND ROGER VIA PIPE                
#   # ---------------------------------------------------------------------------------------------------------------------       
#   if ack2simulate4dragon is True:
#       print("(Step 82A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4DRAGON ", time4xi())   
#       try:        
#           response2manoeuvre4copy = manoeuvre2copybuffertoimage4dragon(queue4copybuffertoimage4dragon, channel4copybuffertoimageconsume4dragon, channel4copybuffertoimagepublish4dragon)
#           print("(Step 82B/700) OF DRAGON2IMAGE4XI5TEST - PERFORMED MANOEUVRE2COPYIMAGE4DRAGON, RESPONSE2MANOEUVRE4COPY: ", response2manoeuvre4copy)
#           # {'MESSAGE2SEND4DRAGON': '<roger4xi>roger2copybuffertoimage4dragon</roger4xi>',
#           #   'ACK2INFO4DRAGON': False,
#           #   'MESSAGE2GET4DRAGON': '<default4xi>default2copybuffertoimage4dragon</default4xi>',
#           #   'MESSAGE2RECEIVE4DRAGON': '<copy4xi>copy2copybuffertoimage4dragon</copy4xi>',
#           #   'CODE2ACK4DRAGON': ' 0D-1C-',
#           #   'MESSAGE2PUT4DRAGON': '<default4xi>default2copybuffertoimage4dragon</default4xi>',
#           #   'CODE2NACK4DRAGON': ' ',
#           #   'STATE2MANOEUVRE4DRAGON': 1}
#       except:
#           print("(Step 82Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4DRAGON ", time4xi())     
#   else:
#       pass # trust activated thread    
#">

# ----------------------------------------------------------------------------------------------------------------------- 
def manoeuvre2copybuffertoimage4dragon(queue2copybuffertoimagemanoeuvre4dragon, consumerchannel2copybuffertoimagemanoeuvre4dragon, publisherchannel2copybuffertoimagemanoeuvre4dragon):
    '''
    manoeuvres copy control and command for copy2buffertoimage4dragon
    '''
    global state2copybuffertoimage4dragon                                                                                           # STATE FOR COMMAND AND CONTROL
    global max2copybuffertoimagestate4dragon                                                                                        # MAX FOR STATE
    global ack2copybuffertoimageinfo4dragon                                                                                         # ACK FOR IMAGE INFO
    global message2copybuffertoimageget4dragon                                 
    global message2copybuffertoimageput4dragon
    global message2copybuffertoimagereceive4dragon
    global message2copybuffertoimagesend4dragon
    
    # -----------------------------------------------------------------------------------------
    # (Step 0/33) PARAMETERS            
    # -----------------------------------------------------------------------------------------          
    class2print  = "manoeuvre2copybuffertoimage4dragon" 
    method2print = "manoeuvre2copybuffertoimage4dragon" 

    code2ack  = ' '
    code2nack = ' '
    
    message2print = "{(Step 0A/33) started" + "; manoeuvre2copybuffertoimage4dragon-started: " + str(manoeuvre2copybuffertoimage4dragon)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # 
    
    message2print = "{(Step 0B/33) passed" + "; queue2copybuffertoimagemanoeuvre4dragon-to-manoeuvre2copybuffertoimage4dragon: " + str(queue2copybuffertoimagemanoeuvre4dragon) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0C/33) passed" + "; consumerchannel2copybuffertoimagemanoeuvre4dragon-to-manoeuvre2copybuffertoimage4dragon: " + str(consumerchannel2copybuffertoimagemanoeuvre4dragon) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0D/33) passed" + "; publisherchannel2copybuffertoimagemanoeuvre4dragon-to-manoeuvre2copybuffertoimage4dragon: " + str(publisherchannel2copybuffertoimagemanoeuvre4dragon) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    code2ack  = code2ack + '0D-' 
    response2manouvre4copybuffertoimage = {}
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # SETTINGS
    # ------------------------------------------------------------------------------------------------------------------------    
    message2print = "{(Step 0E/70) try" + "; state2copybuffertoimage4dragon-try-by-manoeuvre2copybuffertoimage4dragon"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        if state2copybuffertoimage4dragon is None:
            state2copybuffertoimage4dragon = 0
            message2print = "{(Step 0F/33) initialized" + "; state2copybuffertoimage4dragon-initialized-by-manoeuvre2copybuffertoimage4dragon to: " + str(state2copybuffertoimage4dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        else:
            message2print = "{(Step 0G/33) used" + "; state2copybuffertoimage4dragon-used-by-manoeuvre2copybuffertoimage4dragon: " + str(state2copybuffertoimage4dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1; 2; 3;  
    except:
        state2copybuffertoimage4dragon = 0
        message2print = "{(Step 0Z1/33) initialized" + "; state2copybuffertoimage4dragon-initialized-by-manoeuvre2copybuffertoimage4dragon to: " + str(state2copybuffertoimage4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2copybuffertoimageinfo4dragon        = False
        message2print = "{(Step 0Z2/33) initialized" + "; ack2copybuffertoimageinfo4dragon-initialized-by-manoeuvre2copybuffertoimage4dragon to: " + str(ack2copybuffertoimageinfo4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2copybuffertoimageget4dragon     = '<default4xi>default2copybuffertoimage4dragon</default4xi>'
        message2print = "{(Step 0Z3/33) initialized" + "; message2copybuffertoimageget4dragon-initialized-by-manoeuvre2copybuffertoimage4dragon to: " + str(message2copybuffertoimageget4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # 
        message2copybuffertoimageput4dragon     = '<default4xi>default2copybuffertoimage4dragon</default4xi>'
        message2print = "{(Step 0Z4/33) initialized" + "; message2copybuffertoimageput4dragon-initialized-by-manoeuvre2copybuffertoimage4dragon to: " + str(message2copybuffertoimageput4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copybuffertoimagereceive4dragon = '<default4xi>default2copybuffertoimage4dragon</default4xi>'
        message2print = "{(Step 0Z5/33) initialized" + "; message2copybuffertoimagereceive4dragon-initialized-by-manoeuvre2copybuffertoimage4dragon to: " + str(message2copybuffertoimagereceive4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copybuffertoimagesend4dragon    = '<default4xi>default2copybuffertoimage4dragon</default4xi>'
        message2print = "{(Step 0Z6/33) initialized" + "; message2copybuffertoimagesend4dragon-initialized-by-manoeuvre2copybuffertoimage4dragon to: " + str(message2copybuffertoimagesend4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        max2copybuffertoimagestate4dragon = 3
        message2print = "{(Step 0Z7/33) initialized" + ";  max2copybuffertoimagestate4dragon-initialized-by-manoeuvre2copybuffertoimage4dragon to: " + str( max2copybuffertoimagestate4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE UPDATE
    # ------------------------------------------------------------------------------------------------------------------------                      
    state2copybuffertoimage4dragon = state2copybuffertoimage4dragon + 1                                                                   
    message2print = "{(Step 0H/33) added" + "; state2copybuffertoimage4dragon-added-by-manoeuvre2copybuffertoimage4dragon: " + str(state2copybuffertoimage4dragon) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 1; 2; 3; 4 
     
    if state2copybuffertoimage4dragon > max2copybuffertoimagestate4dragon:
        state2copybuffertoimage4dragon = 0
        message2print = "{(Step 0I/33) set" + "; state2copybuffertoimage4dragon-set-by-manoeuvre2copybuffertoimage4dragon to: " + str(state2copybuffertoimage4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        ack2copybuffertoimageinfo4dragon        = False
        message2print = "{(Step 0J/33) set" + "; ack2copybuffertoimageinfo4dragon-set-by-manoeuvre2copybuffertoimage4dragon to: " + str(ack2copybuffertoimageinfo4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # 
        message2copybuffertoimageget4dragon     = '<default4xi>default2copybuffertoimage4dragon</default4xi>'
        message2print = "{(Step 0K/33) set" + "; message2copybuffertoimageget4dragon-set-by-manoeuvre2copybuffertoimage4dragon to: " + str(message2copybuffertoimageget4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # 
        message2copybuffertoimageput4dragon     = '<default4xi>default2copybuffertoimage4dragon</default4xi>'
        message2print = "{(Step 0L/33) set" + "; message2copybuffertoimageput4dragon-set-by-manoeuvre2copybuffertoimage4dragon to: " + str(message2copybuffertoimageput4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copybuffertoimagereceive4dragon = '<default4xi>default2copybuffertoimage4dragon</default4xi>'
        message2print = "{(Step 0M/33) set" + "; message2copybuffertoimagereceive4dragon-set-by-manoeuvre2copybuffertoimage4dragon to: " + str(message2copybuffertoimagereceive4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copybuffertoimagesend4dragon    = '<default4xi>default2copybuffertoimage4dragon</default4xi>'
        message2print = "{(Step 0N/33) set" + "; message2copybuffertoimagesend4dragon-set-by-manoeuvre2copybuffertoimage4dragon to: " + str(message2copybuffertoimagesend4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        max2copybuffertoimagestate4dragon = 3
        message2print = "{(Step 0O/33) initialized" + ";  max2copybuffertoimagestate4dragon-set-by-manoeuvre2copybuffertoimage4dragon to: " + str( max2copybuffertoimagestate4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #     
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE I - RECEIVE CONTROL AND COMMAND COPY MESSAGE VIA PIPE AND SEND ROGER OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copybuffertoimage4dragon == 1:
        message2print = "{(Step 1A/70) try" + "; receive-try-by-manoeuvre2copybuffertoimage4dragon"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2copybuffertoimagereceive4dragon = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2copybuffertoimagereceive4dragon = consumerchannel2copybuffertoimagemanoeuvre4dragon.recv()
            message2print = "{(Step 1B/33) received" + "; message2copybuffertoimagereceive4dragon-received-by-manoeuvre2copybuffertoimage4dragon: " + str(message2copybuffertoimagereceive4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message2copybuffertoimagereceive4dragon == '<copy4xi>copy2copybuffertoimage4dragon</copy4xi>':
                message2copybuffertoimagesend4dragon = '<roger4xi>roger2copybuffertoimage4dragon</roger4xi>'
            else:
                message2copybuffertoimagesend4dragon = '<negative4xi><negative2copybuffertoimage4dragon</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2copybuffertoimagemanoeuvre4dragon.send(message2copybuffertoimagesend4dragon)
            
            message2print = "{(Step 1C/33) sent" + "; message2copybuffertoimagesend4dragon-sent-by-manoeuvre2copybuffertoimage4dragon: " + str(message2copybuffertoimagesend4dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code2ack  = code2ack + '1C-'
        except:
            message2print = "{(Step 1Z/33) failed" + "; message2copybuffertoimagereceive4dragon-failed-by-manoeuvre2copybuffertoimage4dragon: " + str(message2copybuffertoimagereceive4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '1Z-' 
    else:
        pass

                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE II - RECEIVE CONTROL AND COMMAND READY MESSAGE VIA PIPE AND SEND READY OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copybuffertoimage4dragon == 2:
        message2print = "{(Step 2A/70) try" + "; receive-try-by-manoeuvre2copybuffertoimage4dragon"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2copybuffertoimagereceive4dragon = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2copybuffertoimagereceive4dragon = consumerchannel2copybuffertoimagemanoeuvre4dragon.recv()
            message2print = "{(Step 2B/33) received" + "; message2copybuffertoimagereceive4dragon-received-by-manoeuvre2copybuffertoimage4dragon: " + str(message2copybuffertoimagereceive4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message2copybuffertoimagereceive4dragon == '<ready4xi>ready2copybuffertoimage4dragon</ready4xi>':
                message2copybuffertoimagesend4dragon = '<ready4xi>ready2copybuffertoimage4dragon</ready4xi>'
            else:
                message2copybuffertoimagesend4dragon = '<negative4xi><negative2copybuffertoimage4dragon</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND READY OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2copybuffertoimagemanoeuvre4dragon.send(message2copybuffertoimagesend4dragon)
            
            message2print = "{(Step 2C/33) sent" + "; message2copybuffertoimagesend4dragon-sent-by-manoeuvre2copybuffertoimage4dragon: " + str(message2copybuffertoimagesend4dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code2ack  = code2ack + '2C-'
        except:
            message2print = "{(Step 2Z/33) failed" + "; message2copybuffertoimagereceive4dragon-failed-by-manoeuvre2copybuffertoimage4dragon: " + str(message2copybuffertoimagereceive4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '2Z-' 
    else:
        pass        
                
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE III - GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE AND SEND ACK OR NACK VIA PIPE
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copybuffertoimage4dragon == 3:    
        info2image4dragon = None
        message2print = "{(Step 3A/70) try" + "; info2image4dragon-try-by-manoeuvre2copybuffertoimage4dragon"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        try: 
            # ----------------------------------------------------------------------------------------------------------------
            # GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE
            # ----------------------------------------------------------------------------------------------------------------        
            info2image4dragon = queue2copybuffertoimagemanoeuvre4dragon.get()
            
            message2print = "{(Step 3B/33) got" + "; info2image4dragon-got-by-manoeuvre2copybuffertoimage4dragon: " + str(info2image4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # ..//..//..//..//..//web//images//png//dragonxi.png
            if info2image4dragon is None:
                code2nack  = code2nack + '3Z1-'
                ack2copybuffertoimageinfo4dragon = False 
            else:    
                code2ack  = code2ack + '3C-'
                ack2copybuffertoimageinfo4dragon = True
        except:
            message2print = "{(Step 3Z1/33) failed" + "; queue2copybuffertoimagemanoeuvre4dragon-failed-by-manoeuvre2copybuffertoimage4dragon: " + str(queue2copybuffertoimagemanoeuvre4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z1-'
            
        message2copybuffertoimagesend4dragon = ' '
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # SEND CONTROL INFO ABOUT IMAGE OPERATION - EITHER ACK OR NACK  
            # ----------------------------------------------------------------------------------------------------------------         
            if ack2copybuffertoimageinfo4dragon is True:
                message2copybuffertoimagesend4dragon = '<ack4xi>ack2imageinfo4dragon</ack4xi>'
            else:
                message2copybuffertoimagesend4dragon = '<nack4xi>nack2imageinfo4dragon</nack4xi>'
            message2print = "{(Step 4B/33) set" + "; message2copybuffertoimagesend4dragon-set-by-manoeuvre2copybuffertoimage4dragon to: " + str(message2copybuffertoimagesend4dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # <ack4xi>ack2imageinfo4dragon</ack4xi>       
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2copybuffertoimagesend4dragon-failed-by-manoeuvre2copybuffertoimage4dragon: " + str(message2copybuffertoimagesend4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            code2nack  = code2nack + '4Z1-'    
        try:                
            message2print = "{(Step 4C/70) using" + "; publisherchannel2copybuffertoimagemanoeuvre4dragon-calling-by-manoeuvre2copybuffertoimage4dragon: " + str(publisherchannel2copybuffertoimagemanoeuvre4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # <copy-write PipeConnection, handle 412>
            message2print = "{{(Step 4D/70) passing" + "; message2copybuffertoimagesend4dragon-passing-to-send-by-manoeuvre2copybuffertoimage4dragon: " + str(message2copybuffertoimagesend4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2copybuffertoimagesend4dragon-failed-by-manoeuvre2copybuffertoimage4dragon: " + str(message2copybuffertoimagesend4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z2-'                
        try:    
            publisherchannel2copybuffertoimagemanoeuvre4dragon.send(message2copybuffertoimagesend4dragon)
            
            message2print = "{(Step 4E/33) sent" + "; message2copybuffertoimagesend4dragon-sent-by-manoeuvre2copybuffertoimage4dragon: " + str(message2copybuffertoimagesend4dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <ack4xi>ack2imageinfo4dragon</ack4xi>            
            code2ack  = code2ack + '4E-'
        except:
            message2print = "{(Step 4Z3/33) failed" + "; message2copybuffertoimagesend4dragon-failed-by-manoeuvre2copybuffertoimage4dragon: " + str(message2copybuffertoimagesend4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z3-'                
    else:
        pass
        

            
    # ------------------------------------------------------------------------------------------------------------------------
    # RETURN 
    # ------------------------------------------------------------------------------------------------------------------------
    response2manouvre4copybuffertoimage['ACK2INFO4DRAGON']        = ack2copybuffertoimageinfo4dragon
    response2manouvre4copybuffertoimage['CODE2ACK4DRAGON']        = code2ack
    response2manouvre4copybuffertoimage['CODE2NACK4DRAGON']       = code2nack
    response2manouvre4copybuffertoimage['MESSAGE2GET4DRAGON']     = message2copybuffertoimageget4dragon
    response2manouvre4copybuffertoimage['MESSAGE2PUT4DRAGON']     = message2copybuffertoimageput4dragon
    response2manouvre4copybuffertoimage['MESSAGE2RECEIVE4DRAGON'] = message2copybuffertoimagereceive4dragon
    response2manouvre4copybuffertoimage['MESSAGE2SEND4DRAGON']    = message2copybuffertoimagesend4dragon
    response2manouvre4copybuffertoimage['STATE2MANOEUVRE4DRAGON'] = state2copybuffertoimage4dragon
                          
    message2print = "{(Step 33/33) returning" + "; response2manouvre4copybuffertoimage-returning-by-manoeuvre2copybuffertoimage4dragon: " + str(response2manouvre4copybuffertoimage) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #1 {'MESSAGE2SEND4DRAGON': '<roger4xi>roger2copybuffertoimage4dragon</roger4xi>',
    #   'ACK2INFO4DRAGON': False,
    #   'MESSAGE2GET4DRAGON': '<default4xi>default2copybuffertoimage4dragon</default4xi>',
    #   'MESSAGE2RECEIVE4DRAGON': '<copy4xi>copy2copybuffertoimage4dragon</copy4xi>',
    #   'CODE2ACK4DRAGON': ' 0D-1C-',
    #   'MESSAGE2PUT4DRAGON': '<default4xi>default2copybuffertoimage4dragon</default4xi>',
    #   'CODE2NACK4DRAGON': ' ',
    #   'STATE2MANOEUVRE4DRAGON': 1}; time: 02:18:55 10/04/11 Pacific Daylight Time}

    
    #2 {'MESSAGE2SEND4DRAGON': '<ready4xi>ready2copybuffertoimage4dragon</ready4xi>',
    #   'ACK2INFO4DRAGON': False,
    #   'MESSAGE2GET4DRAGON': '<default4xi>default2copybuffertoimage4dragon</default4xi>',
    #   'MESSAGE2RECEIVE4DRAGON': '<ready4xi>ready2copybuffertoimage4dragon</ready4xi>',
    #   'CODE2ACK4DRAGON': ' 0D-2C-',
    #   'MESSAGE2PUT4DRAGON': '<default4xi>default2copybuffertoimage4dragon</default4xi>',
    #   'CODE2NACK4DRAGON': ' ',
    #   'STATE2MANOEUVRE4DRAGON': 2}; time: 02:18:55 10/04/11 Pacific Daylight Time}

    #3 {'MESSAGE2SEND4DRAGON': '<ack4xi>ack2imageinfo4dragon</ack4xi>',
    #   'ACK2INFO4DRAGON': True,
    #   'MESSAGE2GET4DRAGON': '<default4xi>default2copybuffertoimage4dragon</default4xi>',
    #   'MESSAGE2RECEIVE4DRAGON': '<ready4xi>ready2copybuffertoimage4dragon</ready4xi>',
    #   'CODE2ACK4DRAGON': ' 0D-3C-4E-',
    #   'MESSAGE2PUT4DRAGON': '<default4xi>default2copybuffertoimage4dragon</default4xi>',
    #   'CODE2NACK4DRAGON': ' ',
    #   'STATE2MANOEUVRE4DRAGON': 3}; time: 02:18:55 10/04/11 Pacific Daylight Time}
 

    #4 {'MESSAGE2SEND4DRAGON': '<default4xi>default2copybuffertoimage4dragon</default4xi>',
    #   'ACK2INFO4DRAGON': False,
    #   'MESSAGE2GET4DRAGON': '<default4xi>default2copybuffertoimage4dragon</default4xi>',
    #   'MESSAGE2RECEIVE4DRAGON': '<default4xi>default2copybuffertoimage4dragon</default4xi>',
    #   'CODE2ACK4DRAGON': ' 0D-',
    #   'MESSAGE2PUT4DRAGON': '<default4xi>default2copybuffertoimage4dragon</default4xi>',
    #   'CODE2NACK4DRAGON': ' ',
    #   'STATE2MANOEUVRE4DRAGON': 0}

    return response2manouvre4copybuffertoimage           
          
#</function4xi>



# -----------------------------------------------------------------------------------------------------------------------
#<function4xi function = "manoeuvre2readimage4dragon" usage = "control function for read2image4dragon function in this module">

# ----------------------------------------------------------------------------------------------------------------------- 
def manoeuvre2readimage4dragon(queue2readmanoeuvre4dragon, consumerchannel2readmanoeuvre4dragon, publisherchannel2readmanoeuvre4dragon):
    '''
    manoeuvres read control and command
    '''
    global state2read4dragon                                                                                           # STATE FOR COMMAND AND CONTROL
    global max2readstate4dragon                                                                                        # MAX FOR STATE
    global ack2readinfo4dragon                                                                                         # ACK FOR IMAGE INFO
    global message2readget4dragon                                 
    global message2readput4dragon
    global message2readreceive4dragon
    global message2readsend4dragon
    
    # -----------------------------------------------------------------------------------------
    # (Step 0/33) PARAMETERS            
    # -----------------------------------------------------------------------------------------          
    class2print  = "manoeuvre2readimage4dragon" 
    method2print = "manoeuvre2readimage4dragon" 

    code2ack  = ' '
    code2nack = ' '
    
    message2print = "{(Step 0A/33) started" + "; manoeuvre2readimage4dragon-started: " + str(manoeuvre2readimage4dragon)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function manoeuvre2readimage4dragon at 0x0000000003DB8C48>  
    
    message2print = "{(Step 0B/33) passed" + "; queue2readmanoeuvre4dragon-to-manoeuvre2readimage4dragon: " + str(queue2readmanoeuvre4dragon) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <queue4xi.Queue4Xi object at 0x0000000003AD9400>
    
    message2print = "{(Step 0C/33) passed" + "; consumerchannel2readmanoeuvre4dragon-to-manoeuvre2readimage4dragon: " + str(consumerchannel2readmanoeuvre4dragon) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <read-write PipeConnection, handle 408>
    
    message2print = "{(Step 0D/33) passed" + "; publisherchannel2readmanoeuvre4dragon-to-manoeuvre2readimage4dragon: " + str(publisherchannel2readmanoeuvre4dragon) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <read-write PipeConnection, handle 412>     
    
    code2ack  = code2ack + '0D-' 
    response2manouvre4read = {}
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # SETTINGS
    # ------------------------------------------------------------------------------------------------------------------------    
    message2print = "{(Step 0E/70) try" + "; state2read4dragon-try-by-manoeuvre2readimage4dragon"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        if state2read4dragon is None:
            state2read4dragon = 0
            message2print = "{(Step 0F/33) initialized" + "; state2read4dragon-initialized-by-manoeuvre2readimage4dragon to: " + str(state2read4dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        else:
            message2print = "{(Step 0G/33) used" + "; state2read4dragon-used-by-manoeuvre2readimage4dragon: " + str(state2read4dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1; 2; 3;  
    except:
        state2read4dragon = 0
        message2print = "{(Step 0Z1/33) initialized" + "; state2read4dragon-initialized-by-manoeuvre2readimage4dragon to: " + str(state2read4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2readinfo4dragon        = False
        message2print = "{(Step 0Z2/33) initialized" + "; ack2readinfo4dragon-initialized-by-manoeuvre2readimage4dragon to: " + str(ack2readinfo4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2readget4dragon     = '<default4xi>default2read4dragon</default4xi>'
        message2print = "{(Step 0Z3/33) initialized" + "; message2readget4dragon-initialized-by-manoeuvre2readimage4dragon to: " + str(message2readget4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2read4dragon</default4xi>
        message2readput4dragon     = '<default4xi>default2read4dragon</default4xi>'
        message2print = "{(Step 0Z4/33) initialized" + "; message2readput4dragon-initialized-by-manoeuvre2readimage4dragon to: " + str(message2readput4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2read4dragon</default4xi>
        message2readreceive4dragon = '<default4xi>default2read4dragon</default4xi>'
        message2print = "{(Step 0Z5/33) initialized" + "; message2readreceive4dragon-initialized-by-manoeuvre2readimage4dragon to: " + str(message2readreceive4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2read4dragon</default4xi>
        message2readsend4dragon    = '<default4xi>default2read4dragon</default4xi>'
        message2print = "{(Step 0Z6/33) initialized" + "; message2readsend4dragon-initialized-by-manoeuvre2readimage4dragon to: " + str(message2readsend4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2read4dragon</default4xi>
        max2readstate4dragon = 3
        message2print = "{(Step 0Z7/33) initialized" + ";  max2readstate4dragon-initialized-by-manoeuvre2readimage4dragon to: " + str( max2readstate4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE UPDATE
    # ------------------------------------------------------------------------------------------------------------------------                      
    state2read4dragon = state2read4dragon + 1                                                                   
    message2print = "{(Step 0H/33) added" + "; state2read4dragon-added-by-manoeuvre2readimage4dragon: " + str(state2read4dragon) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 1; 2; 3; 4  
    if state2read4dragon > max2readstate4dragon:
        state2read4dragon = 0
        message2print = "{(Step 0I/33) set" + "; state2read4dragon-set-by-manoeuvre2readimage4dragon to: " + str(state2read4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2readinfo4dragon        = False
        message2print = "{(Step 0J/33) set" + "; ack2readinfo4dragon-set-by-manoeuvre2readimage4dragon to: " + str(ack2readinfo4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2readget4dragon     = '<default4xi>default2read4dragon</default4xi>'
        message2print = "{(Step 0K/33) set" + "; message2readget4dragon-set-by-manoeuvre2readimage4dragon to: " + str(message2readget4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2read4dragon</default4xi>
        message2readput4dragon     = '<default4xi>default2read4dragon</default4xi>'
        message2print = "{(Step 0L/33) set" + "; message2readput4dragon-set-by-manoeuvre2readimage4dragon to: " + str(message2readput4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2read4dragon</default4xi>
        message2readreceive4dragon = '<default4xi>default2read4dragon</default4xi>'
        message2print = "{(Step 0M/33) set" + "; message2readreceive4dragon-set-by-manoeuvre2readimage4dragon to: " + str(message2readreceive4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2read4dragon</default4xi>
        message2readsend4dragon    = '<default4xi>default2read4dragon</default4xi>'
        message2print = "{(Step 0N/33) set" + "; message2readsend4dragon-set-by-manoeuvre2readimage4dragon to: " + str(message2readsend4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2read4dragon</default4xi>
        max2readstate4dragon = 3
        message2print = "{(Step 0O/33) initialized" + ";  max2readstate4dragon-set-by-manoeuvre2readimage4dragon to: " + str( max2readstate4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #     
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE I - RECEIVE CONTROL AND COMMAND COPY MESSAGE VIA PIPE AND SEND ROGER OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2read4dragon == 1:
        message2print = "{(Step 1A/70) try" + "; receive-try-by-manoeuvre2readimage4dragon"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2readreceive4dragon = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2readreceive4dragon = consumerchannel2readmanoeuvre4dragon.recv()
            message2print = "{(Step 1B/33) received" + "; message2readreceive4dragon-received-by-manoeuvre2readimage4dragon: " + str(message2readreceive4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # <copy4xi>copy2read4dragon</copy4xi>
                         
            if message2readreceive4dragon == '<copy4xi>copy2read4dragon</copy4xi>':
                message2readsend4dragon = '<roger4xi>roger2read4dragon</roger4xi>'
            else:
                message2readsend4dragon = '<negative4xi><negative2read4dragon</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2readmanoeuvre4dragon.send(message2readsend4dragon)
            
            message2print = "{(Step 1C/33) sent" + "; message2readsend4dragon-sent-by-manoeuvre2readimage4dragon: " + str(message2readsend4dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <roger4xi>roger2read4dragon</roger4xi>
            
            code2ack  = code2ack + '1C-'
        except:
            message2print = "{(Step 1Z/33) failed" + "; message2readreceive4dragon-failed-by-manoeuvre2readimage4dragon: " + str(message2readreceive4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '1Z-' 
    else:
        pass

                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE II - RECEIVE CONTROL AND COMMAND READY MESSAGE VIA PIPE AND SEND READY OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2read4dragon == 2:
        message2print = "{(Step 2A/70) try" + "; receive-try-by-manoeuvre2readimage4dragon"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2readreceive4dragon = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2readreceive4dragon = consumerchannel2readmanoeuvre4dragon.recv()
            message2print = "{(Step 2B/33) received" + "; message2readreceive4dragon-received-by-manoeuvre2readimage4dragon: " + str(message2readreceive4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # <ready4xi>ready2read4dragon</ready4xi> 
                         
            if message2readreceive4dragon == '<ready4xi>ready2read4dragon</ready4xi>':
                message2readsend4dragon = '<ready4xi>ready2read4dragon</ready4xi>'
            else:
                message2readsend4dragon = '<negative4xi><negative2ready4dragon</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2readmanoeuvre4dragon.send(message2readsend4dragon)
            
            message2print = "{(Step 2C/33) sent" + "; message2readsend4dragon-sent-by-manoeuvre2readimage4dragon: " + str(message2readsend4dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <ready4xi>ready2read4dragon</ready4xi>
            
            code2ack  = code2ack + '2C-'
        except:
            message2print = "{(Step 2Z/33) failed" + "; message2readreceive4dragon-failed-by-manoeuvre2readimage4dragon: " + str(message2readreceive4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '2Z-' 
    else:
        pass        
                
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE III - GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE AND SEND ACK OR NACK VIA PIPE
    # ------------------------------------------------------------------------------------------------------------------------
    if state2read4dragon == 3:    
        info2image4dragon = None
        message2print = "{(Step 3A/70) try" + "; info2image4dragon-try-by-manoeuvre2readimage4dragon"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        try: 
            # ----------------------------------------------------------------------------------------------------------------
            # GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE
            # ----------------------------------------------------------------------------------------------------------------        
            info2image4dragon = queue2readmanoeuvre4dragon.get()
            
            message2print = "{(Step 3B/33) got" + "; info2image4dragon-got-by-manoeuvre2readimage4dragon: " + str(info2image4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # ..//..//..//..//..//web//images//png//dragonxi.png
            if info2image4dragon is None:
                code2nack  = code2nack + '3Z1-'
                ack2readinfo4dragon = False 
            else:    
                code2ack  = code2ack + '3C-'
                ack2readinfo4dragon = True
        except:
            message2print = "{(Step 3Z1/33) failed" + "; queue2readmanoeuvre4dragon-failed-by-manoeuvre2readimage4dragon: " + str(queue2readmanoeuvre4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z1-'
            
        message2readsend4dragon = ' '
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # SEND CONTROL INFO ABOUT IMAGE OPERATION - EITHER ACK OR NACK  
            # ----------------------------------------------------------------------------------------------------------------         
            if ack2readinfo4dragon is True:
                message2readsend4dragon = '<ack4xi>ack2imageinfo4dragon</ack4xi>'
            else:
                message2readsend4dragon = '<nack4xi>nack2imageinfo4dragon</nack4xi>'
            message2print = "{(Step 4B/33) set" + "; message2readsend4dragon-set-by-manoeuvre2readimage4dragon to: " + str(message2readsend4dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # <ack4xi>ack2imageinfo4dragon</ack4xi>       
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2readsend4dragon-failed-by-manoeuvre2readimage4dragon: " + str(message2readsend4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            code2nack  = code2nack + '4Z1-'    
        try:                
            message2print = "{(Step 4C/70) using" + "; publisherchannel2readmanoeuvre4dragon-calling-by-manoeuvre2readimage4dragon: " + str(publisherchannel2readmanoeuvre4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # <read-write PipeConnection, handle 412>
            message2print = "{{(Step 4D/70) passing" + "; message2readsend4dragon-passing-to-send-by-manoeuvre2readimage4dragon: " + str(message2readsend4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2readsend4dragon-failed-by-manoeuvre2readimage4dragon: " + str(message2readsend4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z2-'                
        try:    
            publisherchannel2readmanoeuvre4dragon.send(message2readsend4dragon)
            
            message2print = "{(Step 4E/33) sent" + "; message2readsend4dragon-sent-by-manoeuvre2readimage4dragon: " + str(message2readsend4dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <ack4xi>ack2imageinfo4dragon</ack4xi>            
            code2ack  = code2ack + '4E-'
        except:
            message2print = "{(Step 4Z3/33) failed" + "; message2readsend4dragon-failed-by-manoeuvre2readimage4dragon: " + str(message2readsend4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z3-'                
    else:
        pass
        

            
    # ------------------------------------------------------------------------------------------------------------------------
    # RETURN 
    # ------------------------------------------------------------------------------------------------------------------------
    response2manouvre4read['ACK2INFO4DRAGON']        = ack2readinfo4dragon
    response2manouvre4read['CODE2ACK4DRAGON']        = code2ack
    response2manouvre4read['CODE2NACK4DRAGON']       = code2nack
    response2manouvre4read['MESSAGE2GET4DRAGON']     = message2readget4dragon
    response2manouvre4read['MESSAGE2PUT4DRAGON']     = message2readput4dragon
    response2manouvre4read['MESSAGE2RECEIVE4DRAGON'] = message2readreceive4dragon
    response2manouvre4read['MESSAGE2SEND4DRAGON']    = message2readsend4dragon
    response2manouvre4read['STATE2MANOEUVRE4DRAGON'] = state2read4dragon
                          
    message2print = "{(Step 33/33) returning" + "; response2manouvre4read-returning-by-manoeuvre2readimage4dragon: " + str(response2manouvre4read) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #1 {'MESSAGE2SEND4DRAGON': '<roger4xi>roger2read4dragon</roger4xi>',
    #  'ACK2INFO4DRAGON': False,
    #  'MESSAGE2GET4DRAGON': '<default4xi>default2read4dragon</default4xi>',
    #  'MESSAGE2RECEIVE4DRAGON': '<copy4xi>copy2read4dragon</copy4xi>',
    #  'CODE2ACK4DRAGON': ' 0D-1C-',
    #  'MESSAGE2PUT4DRAGON': '<default4xi>default2read4dragon</default4xi>',
    #  'CODE2NACK4DRAGON': ' ', 'STATE2MANOEUVRE4DRAGON': 1}; time: 01:44:40 09/29/11 Pacific Daylight Time}
 
    #2 {'MESSAGE2SEND4DRAGON': '<ready4xi>ready2read4dragon</ready4xi>',
    #  'ACK2INFO4DRAGON': False,
    #  'MESSAGE2GET4DRAGON': '<default4xi>default2read4dragon</default4xi>',
    #  'MESSAGE2RECEIVE4DRAGON': '<ready4xi>ready2read4dragon</ready4xi>',
    #  'CODE2ACK4DRAGON': ' 0D-2C-',
    #  'MESSAGE2PUT4DRAGON': '<default4xi>default2read4dragon</default4xi>',
    #  'CODE2NACK4DRAGON': ' ', 'STATE2MANOEUVRE4DRAGON': 2}; time: 01:44:40 09/29/11 Pacific Daylight Time}
    
    #3 {'MESSAGE2SEND4DRAGON': '<ack4xi>ack2imageinfo4dragon</ack4xi>',
    #  'ACK2INFO4DRAGON': True,
    #  'MESSAGE2GET4DRAGON': '<default4xi>default2read4dragon</default4xi>',
    #  'MESSAGE2RECEIVE4DRAGON': '<ready4xi>ready2read4dragon</ready4xi>',
    #  'CODE2ACK4DRAGON': ' 0D-3C-4E-',
    #  'MESSAGE2PUT4DRAGON': '<default4xi>default2read4dragon</default4xi>',
    #  'CODE2NACK4DRAGON': ' ',
    #  'STATE2MANOEUVRE4DRAGON': 3}; time: 01:44:40 09/29/11 Pacific Daylight Time}
    
    #4 {'MESSAGE2SEND4DRAGON': '<default4xi>default2read4dragon</default4xi>',
    #  'ACK2INFO4DRAGON': False,
    #  'MESSAGE2GET4DRAGON': '<default4xi>default2read4dragon</default4xi>',
    #  'MESSAGE2RECEIVE4DRAGON': '<default4xi>default2read4dragon</default4xi>',
    #  'CODE2ACK4DRAGON': ' 0D-',
    #  'MESSAGE2PUT4DRAGON': '<default4xi>default2read4dragon</default4xi>',
    #  'CODE2NACK4DRAGON': ' ',
    #  'STATE2MANOEUVRE4DRAGON': 0}; time: 02:11:49 09/29/11 Pacific Daylight Time}


 
    
    
    return response2manouvre4read           
          
#</function4xi>


# -------------------------------------------------------------------------------------------------------------------------
#<function4xi function = "manoeuvre2writeimage4dragon" usage = "control function for write2image4dragon function in this module">

# ------------------------------------------------------------------------------------------------------------------------- 
def manoeuvre2writeimage4dragon(queue2writemanoeuvre4dragon, consumerchannel2writemanoeuvre4dragon, publisherchannel2writemanoeuvre4dragon):
    '''
    manoeuvres control and command
    '''
    global state2write4dragon                                                       # STATE FOR COMMAND AND CONTROL
    global max2writestate4dragon                                                    # MAX FOR STATE
    global ack2writeinfo4dragon                                                     # ACK FOR IMAGE INFO
    global message2writeget4dragon                                 
    global message2writeput4dragon
    global message2writereceive4dragon
    global message2writesend4dragon
    
    # -----------------------------------------------------------------------------------------
    # (Step 0/33) PARAMETERS            
    # -----------------------------------------------------------------------------------------          
    class2print  = "manoeuvre2writeimage4dragon" 
    method2print = "manoeuvre2writeimage4dragon" 

    code2ack  = ' '
    code2nack = ' '
    
    message2print = "{(Step 0A/33) started" + "; manoeuvre2writeimage4dragon-started: " + str(manoeuvre2writeimage4dragon)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                      
    # <function manoeuvre2writeimage4dragon at 0x0000000003DB8C48>  
    
    message2print = "{(Step 0B/33) passed" + "; queue2writemanoeuvre4dragon-to-manoeuvre2writeimage4dragon: " + str(queue2writemanoeuvre4dragon) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <queue4xi.Queue4Xi object at 0x000000000422D9E8>
    
    message2print = "{(Step 0C/33) passed" + "; consumerchannel2writemanoeuvre4dragon-to-manoeuvre2writeimage4dragon: " + str(consumerchannel2writemanoeuvre4dragon) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # read-write PipeConnection, handle 568>
    
    message2print = "{(Step 0D/33) passed" + "; publisherchannel2writemanoeuvre4dragon-to-manoeuvre2writeimage4dragon: " + str(publisherchannel2writemanoeuvre4dragon) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <read-write PipeConnection, handle 572> 
    
    code2ack  = code2ack + '0D-' 
    response2manouvre4write = {}
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # SETTINGS
    # ------------------------------------------------------------------------------------------------------------------------    
    message2print = "{(Step 0E/70) try" + "; state2write4dragon-try-by-manoeuvre2writeimage4dragon"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        if state2write4dragon is None:
            state2write4dragon = 0
            message2print = "{(Step 0F/33) initialized" + "; state2write4dragon-initialized-by-manoeuvre2writeimage4dragon to: " + str(state2write4dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        else:
            message2print = "{(Step 0G/33) used" + "; state2write4dragon-used-by-manoeuvre2writeimage4dragon: " + str(state2write4dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1; 2; 3;  
    except:
        state2write4dragon = 0
        message2print = "{(Step 0Z1/33) initialized" + "; state2write4dragon-initialized-by-manoeuvre2writeimage4dragon to: " + str(state2write4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2writeinfo4dragon        = False
        message2print = "{(Step 0Z2/33) initialized" + "; ack2writeinfo4dragon-initialized-by-manoeuvre2writeimage4dragon to: " + str(ack2writeinfo4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2writeget4dragon     = '<default4xi>default2write4dragon</default4xi>'
        message2print = "{(Step 0Z3/33) initialized" + "; message2writeget4dragon-initialized-by-manoeuvre2writeimage4dragon to: " + str(message2writeget4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2write4dragon</default4xi>
        message2writeput4dragon     = '<default4xi>default2write4dragon</default4xi>'
        message2print = "{(Step 0Z4/33) initialized" + "; message2writeput4dragon-initialized-by-manoeuvre2writeimage4dragon to: " + str(message2writeput4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2write4dragon</default4xi>
        message2writereceive4dragon = '<default4xi>default2write4dragon</default4xi>'
        message2print = "{(Step 0Z5/33) initialized" + "; message2writereceive4dragon-initialized-by-manoeuvre2writeimage4dragon to: " + str(message2writereceive4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2write4dragon</default4xi>
        message2writesend4dragon    = '<default4xi>default2write4dragon</default4xi>'
        message2print = "{(Step 0Z6/33) initialized" + "; message2writesend4dragon-initialized-by-manoeuvre2writeimage4dragon to: " + str(message2writesend4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2write4dragon</default4xi>
        max2writestate4dragon = 3
        message2print = "{(Step 0Z7/33) initialized" + ";  max2writestate4dragon-initialized-by-manoeuvre2writeimage4dragon to: " + str( max2writestate4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE UPDATE
    # ------------------------------------------------------------------------------------------------------------------------                      
    state2write4dragon = state2write4dragon + 1                                                                   
    message2print = "{(Step 0H/33) added" + "; state2write4dragon-added-by-manoeuvre2writeimage4dragon: " + str(state2write4dragon) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 1; 2; 3; 4  
    if state2write4dragon > max2writestate4dragon:
        state2write4dragon = 0
        message2print = "{(Step 0I/33) set" + "; state2write4dragon-set-by-manoeuvre2writeimage4dragon to: " + str(state2write4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2writeinfo4dragon        = False
        message2print = "{(Step 0J/33) set" + "; ack2writeinfo4dragon-set-by-manoeuvre2writeimage4dragon to: " + str(ack2writeinfo4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2writeget4dragon     = '<default4xi>default2write4dragon</default4xi>'
        message2print = "{(Step 0K/33) set" + "; message2writeget4dragon-set-by-manoeuvre2writeimage4dragon to: " + str(message2writeget4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2write4dragon</default4xi>
        message2writeput4dragon     = '<default4xi>default2write4dragon</default4xi>'
        message2print = "{(Step 0L/33) set" + "; message2writeput4dragon-set-by-manoeuvre2writeimage4dragon to: " + str(message2writeput4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2write4dragon</default4xi>
        message2writereceive4dragon = '<default4xi>default2write4dragon</default4xi>'
        message2print = "{(Step 0M/33) set" + "; message2writereceive4dragon-set-by-manoeuvre2writeimage4dragon to: " + str(message2writereceive4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2write4dragon</default4xi>
        message2writesend4dragon    = '<default4xi>default2write4dragon</default4xi>'
        message2print = "{(Step 0N/33) set" + "; message2writesend4dragon-set-by-manoeuvre2writeimage4dragon to: " + str(message2writesend4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2write4dragon</default4xi>
        max2writestate4dragon = 3
        message2print = "{(Step 0O/33) initialized" + ";  max2writestate4dragon-set-by-manoeuvre2writeimage4dragon to: " + str( max2writestate4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #     
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE I - RECEIVE CONTROL AND COMMAND COPY MESSAGE VIA PIPE AND SEND ROGER OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2write4dragon == 1:
        message2print = "{(Step 1A/70) try" + "; receive-try-by-manoeuvre2writeimage4dragon"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2writereceive4dragon = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2writereceive4dragon = consumerchannel2writemanoeuvre4dragon.recv()
            message2print = "{(Step 1B/33) received" + "; message2writereceive4dragon-received-by-manoeuvre2writeimage4dragon: " + str(message2writereceive4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # <copy4xi>copy2write4dragon</copy4xi>
                         
            if message2writereceive4dragon == '<copy4xi>copy2write4dragon</copy4xi>':
                message2writesend4dragon = '<roger4xi>roger2write4dragon</roger4xi>'
            else:
                message2writesend4dragon = '<negative4xi><negative2write4dragon</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2writemanoeuvre4dragon.send(message2writesend4dragon)
            
            message2print = "{(Step 1C/33) sent" + "; message2writesend4dragon-sent-by-manoeuvre2writeimage4dragon: " + str(message2writesend4dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <roger4xi>roger2write4dragon</roger4xi>
            
            code2ack  = code2ack + '1C-'
        except:
            message2print = "{(Step 1Z/33) failed" + "; message2writereceive4dragon-failed-by-manoeuvre2writeimage4dragon: " + str(message2writereceive4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '1Z-' 
    else:
        pass

                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE II - RECEIVE CONTROL AND COMMAND READY MESSAGE VIA PIPE AND SEND READY OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2write4dragon == 2:
        message2print = "{(Step 2A/70) try" + "; receive-try-by-manoeuvre2writeimage4dragon"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2writereceive4dragon = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2writereceive4dragon = consumerchannel2writemanoeuvre4dragon.recv()
            message2print = "{(Step 2B/33) received" + "; message2writereceive4dragon-received-by-manoeuvre2writeimage4dragon: " + str(message2writereceive4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # <ready4xi>ready2write4dragon</ready4xi>
                         
            if message2writereceive4dragon == '<ready4xi>ready2write4dragon</ready4xi>':
                message2writesend4dragon = '<ready4xi>ready2write4dragon</ready4xi>'
            else:
                message2writesend4dragon = '<negative4xi><negative2ready4dragon</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2writemanoeuvre4dragon.send(message2writesend4dragon)
            
            message2print = "{(Step 2C/33) sent" + "; message2writesend4dragon-sent-by-manoeuvre2writeimage4dragon: " + str(message2writesend4dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <ready4xi>ready2write4dragon</ready4xi>
            
            code2ack  = code2ack + '2C-'
        except:
            message2print = "{(Step 2Z/33) failed" + "; message2writereceive4dragon-failed-by-manoeuvre2writeimage4dragon: " + str(message2writereceive4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '2Z-' 
    else:
        pass        
                
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE III - GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE AND SEND ACK OR NACK VIA PIPE
    # ------------------------------------------------------------------------------------------------------------------------
    if state2write4dragon == 3:    
        info2image4dragon = None
        message2print = "{(Step 3A/70) try" + "; info2image4dragon-try-by-manoeuvre2writeimage4dragon"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        try: 
            # ----------------------------------------------------------------------------------------------------------------
            # GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE
            # ----------------------------------------------------------------------------------------------------------------        
            info2image4dragon = queue2writemanoeuvre4dragon.get()
            
            message2print = "{(Step 3B/33) got" + "; info2image4dragon-got-by-manoeuvre2writeimage4dragon: " + str(info2image4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # ..//..//..//..//..//web//images//png//dragonxi.png
            if info2image4dragon is None:
                code2nack  = code2nack + '3Z1-'
                ack2writeinfo4dragon = False 
            else:    
                code2ack  = code2ack + '3C-'
                ack2writeinfo4dragon = True
        except:
            message2print = "{(Step 3Z1/33) failed" + "; queue2writemanoeuvre4dragon-failed-by-manoeuvre2writeimage4dragon: " + str(queue2writemanoeuvre4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z1-'
            
        message2writesend4dragon = ' '
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # SEND CONTROL INFO ABOUT IMAGE OPERATION - EITHER ACK OR NACK  
            # ----------------------------------------------------------------------------------------------------------------         
            if ack2writeinfo4dragon is True:
                message2writesend4dragon = '<ack4xi>ack2imageinfo4dragon</ack4xi>'
            else:
                message2writesend4dragon = '<nack4xi>nack2imageinfo4dragon</nack4xi>'
            message2print = "{(Step 4B/33) set" + "; message2writesend4dragon-set-by-manoeuvre2writeimage4dragon to: " + str(message2writesend4dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # <ack4xi>ack2imageinfo4dragon</ack4xi>       
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2writesend4dragon-failed-by-manoeuvre2writeimage4dragon: " + str(message2writesend4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            code2nack  = code2nack + '4Z1-'    
        try:                
            message2print = "{(Step 4C/70) using" + "; publisherchannel2writemanoeuvre4dragon-calling-by-manoeuvre2writeimage4dragon: " + str(publisherchannel2writemanoeuvre4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # <read-write PipeConnection, handle 412>
            message2print = "{{(Step 4D/70) passing" + "; message2writesend4dragon-passing-to-send-by-manoeuvre2writeimage4dragon: " + str(message2writesend4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2writesend4dragon-failed-by-manoeuvre2writeimage4dragon: " + str(message2writesend4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z2-'                
        try:    
            publisherchannel2writemanoeuvre4dragon.send(message2writesend4dragon)
            
            message2print = "{(Step 4E/33) sent" + "; message2writesend4dragon-sent-by-manoeuvre2writeimage4dragon: " + str(message2writesend4dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <ack4xi>ack2imageinfo4dragon</ack4xi>          
            code2ack  = code2ack + '4E-'
        except:
            message2print = "{(Step 4Z3/33) failed" + "; message2writesend4dragon-failed-by-manoeuvre2writeimage4dragon: " + str(message2writesend4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z3-'                
    else:
        pass
        

            
    # ------------------------------------------------------------------------------------------------------------------------
    # RETURN 
    # ------------------------------------------------------------------------------------------------------------------------
    response2manouvre4write['ACK2INFO4DRAGON']        = ack2writeinfo4dragon
    response2manouvre4write['CODE2ACK4DRAGON']        = code2ack
    response2manouvre4write['CODE2NACK4DRAGON']       = code2nack
    response2manouvre4write['MESSAGE2GET4DRAGON']     = message2writeget4dragon
    response2manouvre4write['MESSAGE2PUT4DRAGON']     = message2writeput4dragon
    response2manouvre4write['MESSAGE2RECEIVE4DRAGON'] = message2writereceive4dragon
    response2manouvre4write['MESSAGE2SEND4DRAGON']    = message2writesend4dragon
    response2manouvre4write['STATE2MANOEUVRE4DRAGON'] = state2write4dragon
                          
    message2print = "{(Step 33/33) returning" + "; response2manouvre4write-returning-by-manoeuvre2writeimage4dragon: " + str(response2manouvre4write) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #1
    # {'MESSAGE2SEND4DRAGON': '<roger4xi>roger2write4dragon</roger4xi>', 
    #  'ACK2INFO4DRAGON': False, 'MESSAGE2GET4DRAGON': '<default4xi>default2write4dragon</default4xi>', 
    #  'MESSAGE2RECEIVE4DRAGON': '<copy4xi>copy2write4dragon</copy4xi>', 
    #  'CODE2ACK4DRAGON': ' 0D-1C-', 
    #  'MESSAGE2PUT4DRAGON': '<default4xi>default2write4dragon</default4xi>', 
    #  'CODE2NACK4DRAGON': ' ', 
    #  'STATE2MANOEUVRE4DRAGON': 1}; time: 00:11:53 10/03/11 Pacific Daylight Time}
    #2
    # {'MESSAGE2SEND4DRAGON': '<ready4xi>ready2write4dragon</ready4xi>', 
    #  'ACK2INFO4DRAGON': False, 'MESSAGE2GET4DRAGON': '<default4xi>default2write4dragon</default4xi>', 
    #  'MESSAGE2RECEIVE4DRAGON': '<ready4xi>ready2write4dragon</ready4xi>', 
    #  'CODE2ACK4DRAGON': ' 0D-2C-', 
    #  'MESSAGE2PUT4DRAGON': '<default4xi>default2write4dragon</default4xi>', 
    #  'CODE2NACK4DRAGON': ' ', 
    #  'STATE2MANOEUVRE4DRAGON': 2}; time: 00:11:53 10/03/11 Pacific Daylight Time}
    #3
    # {'MESSAGE2SEND4DRAGON': '<ack4xi>ack2imageinfo4dragon</ack4xi>', 
    #  'ACK2INFO4DRAGON': True, 'MESSAGE2GET4DRAGON': '<default4xi>default2write4dragon</default4xi>', 
    #  'MESSAGE2RECEIVE4DRAGON': '<ready4xi>ready2write4dragon</ready4xi>', 
    #  'CODE2ACK4DRAGON': ' 0D-3C-4E-', 
    #  'MESSAGE2PUT4DRAGON': '<default4xi>default2write4dragon</default4xi>', 
    #  'CODE2NACK4DRAGON': ' ', 
    #  'STATE2MANOEUVRE4DRAGON': 3}; time: 00:11:53 10/03/11 Pacific Daylight Time}

    return response2manouvre4write           
          
#</function4xi>

# -------------------------------------------------------------------------------------------------------------------------
#<function4xi function = "manoeuvre2mapimage4dragon" usage = "control function for map2image4dragon function in this module">

# ------------------------------------------------------------------------------------------------------------------------- 
def manoeuvre2mapimage4dragon(queue2mapmanoeuvre4dragon, consumerchannel2mapmanoeuvre4dragon, publisherchannel2mapmanoeuvre4dragon):
    '''
    manoeuvres control and command
    '''
    global state2map4dragon                                                       # STATE FOR COMMAND AND CONTROL
    global max2mapstate4dragon                                                    # MAX FOR STATE
    global ack2mapinfo4dragon                                                     # ACK FOR IMAGE INFO
    global message2mapget4dragon                                 
    global message2mapput4dragon
    global message2mapreceive4dragon
    global message2mapsend4dragon
    
    # -----------------------------------------------------------------------------------------
    # (Step 0/33) PARAMETERS            
    # -----------------------------------------------------------------------------------------          
    class2print  = "manoeuvre2mapimage4dragon" 
    method2print = "manoeuvre2mapimage4dragon" 

    code2ack  = ' '
    code2nack = ' '
    
    message2print = "{(Step 0A/33) started" + "; manoeuvre2mapimage4dragon-started: " + str(manoeuvre2mapimage4dragon)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                      
    #  
    
    message2print = "{(Step 0B/33) passed" + "; queue2mapmanoeuvre4dragon-to-manoeuvre2mapimage4dragon: " + str(queue2mapmanoeuvre4dragon) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0C/33) passed" + "; consumerchannel2mapmanoeuvre4dragon-to-manoeuvre2mapimage4dragon: " + str(consumerchannel2mapmanoeuvre4dragon) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0D/33) passed" + "; publisherchannel2mapmanoeuvre4dragon-to-manoeuvre2mapimage4dragon: " + str(publisherchannel2mapmanoeuvre4dragon) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    code2ack  = code2ack + '0D-' 
    response2manouvre4map = {}
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # SETTINGS
    # ------------------------------------------------------------------------------------------------------------------------    
    message2print = "{(Step 0E/70) try" + "; state2map4dragon-try-by-manoeuvre2mapimage4dragon"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        if state2map4dragon is None:
            state2map4dragon = 0
            message2print = "{(Step 0F/33) initialized" + "; state2map4dragon-initialized-by-manoeuvre2mapimage4dragon to: " + str(state2map4dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        else:
            message2print = "{(Step 0G/33) used" + "; state2map4dragon-used-by-manoeuvre2mapimage4dragon: " + str(state2map4dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1; 2; 3;  
    except:
        state2map4dragon = 0
        message2print = "{(Step 0Z1/33) initialized" + "; state2map4dragon-initialized-by-manoeuvre2mapimage4dragon to: " + str(state2map4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2mapinfo4dragon        = False
        message2print = "{(Step 0Z2/33) initialized" + "; ack2mapinfo4dragon-initialized-by-manoeuvre2mapimage4dragon to: " + str(ack2mapinfo4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2mapget4dragon     = '<default4xi>default2map4dragon</default4xi>'
        message2print = "{(Step 0Z3/33) initialized" + "; message2mapget4dragon-initialized-by-manoeuvre2mapimage4dragon to: " + str(message2mapget4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2map4dragon</default4xi>
        message2mapput4dragon     = '<default4xi>default2map4dragon</default4xi>'
        message2print = "{(Step 0Z4/33) initialized" + "; message2mapput4dragon-initialized-by-manoeuvre2mapimage4dragon to: " + str(message2mapput4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2map4dragon</default4xi>
        message2mapreceive4dragon = '<default4xi>default2map4dragon</default4xi>'
        message2print = "{(Step 0Z5/33) initialized" + "; message2mapreceive4dragon-initialized-by-manoeuvre2mapimage4dragon to: " + str(message2mapreceive4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2map4dragon</default4xi>
        message2mapsend4dragon    = '<default4xi>default2map4dragon</default4xi>'
        message2print = "{(Step 0Z6/33) initialized" + "; message2mapsend4dragon-initialized-by-manoeuvre2mapimage4dragon to: " + str(message2mapsend4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2map4dragon</default4xi>
        max2mapstate4dragon = 3
        message2print = "{(Step 0Z7/33) initialized" + ";  max2mapstate4dragon-initialized-by-manoeuvre2mapimage4dragon to: " + str( max2mapstate4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE UPDATE
    # ------------------------------------------------------------------------------------------------------------------------                      
    state2map4dragon = state2map4dragon + 1                                                                   
    message2print = "{(Step 0H/33) added" + "; state2map4dragon-added-by-manoeuvre2mapimage4dragon: " + str(state2map4dragon) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 1; 2; 3; 4  
    if state2map4dragon > max2mapstate4dragon:
        state2map4dragon = 0
        message2print = "{(Step 0I/33) set" + "; state2map4dragon-set-by-manoeuvre2mapimage4dragon to: " + str(state2map4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2mapinfo4dragon        = False
        message2print = "{(Step 0J/33) set" + "; ack2mapinfo4dragon-set-by-manoeuvre2mapimage4dragon to: " + str(ack2mapinfo4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2mapget4dragon     = '<default4xi>default2map4dragon</default4xi>'
        message2print = "{(Step 0K/33) set" + "; message2mapget4dragon-set-by-manoeuvre2mapimage4dragon to: " + str(message2mapget4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2map4dragon</default4xi>
        message2mapput4dragon     = '<default4xi>default2map4dragon</default4xi>'
        message2print = "{(Step 0L/33) set" + "; message2mapput4dragon-set-by-manoeuvre2mapimage4dragon to: " + str(message2mapput4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2map4dragon</default4xi>
        message2mapreceive4dragon = '<default4xi>default2map4dragon</default4xi>'
        message2print = "{(Step 0M/33) set" + "; message2mapreceive4dragon-set-by-manoeuvre2mapimage4dragon to: " + str(message2mapreceive4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2map4dragon</default4xi>
        message2mapsend4dragon    = '<default4xi>default2map4dragon</default4xi>'
        message2print = "{(Step 0N/33) set" + "; message2mapsend4dragon-set-by-manoeuvre2mapimage4dragon to: " + str(message2mapsend4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2map4dragon</default4xi>
        max2mapstate4dragon = 3
        message2print = "{(Step 0O/33) initialized" + ";  max2mapstate4dragon-set-by-manoeuvre2mapimage4dragon to: " + str( max2mapstate4dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #     
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE I - RECEIVE CONTROL AND COMMAND COPY MESSAGE VIA PIPE AND SEND ROGER OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2map4dragon == 1:
        message2print = "{(Step 1A/70) try" + "; receive-try-by-manoeuvre2mapimage4dragon"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2mapreceive4dragon = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2mapreceive4dragon = consumerchannel2mapmanoeuvre4dragon.recv()
            message2print = "{(Step 1B/33) received" + "; message2mapreceive4dragon-received-by-manoeuvre2mapimage4dragon: " + str(message2mapreceive4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message2mapreceive4dragon == '<copy4xi>copy2mapimage4dragon</copy4xi>':
                message2mapsend4dragon = '<roger4xi>roger2mapimage4dragon</roger4xi>'
            else:
                message2mapsend4dragon = '<negative4xi><negative2mapimage4dragon</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2mapmanoeuvre4dragon.send(message2mapsend4dragon)
            
            message2print = "{(Step 1C/33) sent" + "; message2mapsend4dragon-sent-by-manoeuvre2mapimage4dragon: " + str(message2mapsend4dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code2ack  = code2ack + '1C-'
        except:
            message2print = "{(Step 1Z/33) failed" + "; message2mapreceive4dragon-failed-by-manoeuvre2mapimage4dragon: " + str(message2mapreceive4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '1Z-' 
    else:
        pass

                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE II - RECEIVE CONTROL AND COMMAND READY MESSAGE VIA PIPE AND SEND READY OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2map4dragon == 2:
        message2print = "{(Step 2A/70) try" + "; receive-try-by-manoeuvre2mapimage4dragon"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2mapreceive4dragon = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2mapreceive4dragon = consumerchannel2mapmanoeuvre4dragon.recv()
            message2print = "{(Step 2B/33) received" + "; message2mapreceive4dragon-received-by-manoeuvre2mapimage4dragon: " + str(message2mapreceive4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message2mapreceive4dragon == '<ready4xi>ready2mapimage4dragon</ready4xi>':
                message2mapsend4dragon = '<ready4xi>ready2mapimage4dragon</ready4xi>'
            else:
                message2mapsend4dragon = '<negative4xi><negative2ready4dragon</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2mapmanoeuvre4dragon.send(message2mapsend4dragon)
            
            message2print = "{(Step 2C/33) sent" + "; message2mapsend4dragon-sent-by-manoeuvre2mapimage4dragon: " + str(message2mapsend4dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code2ack  = code2ack + '2C-'
        except:
            message2print = "{(Step 2Z/33) failed" + "; message2mapreceive4dragon-failed-by-manoeuvre2mapimage4dragon: " + str(message2mapreceive4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '2Z-' 
    else:
        pass        
                
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE III - GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE AND SEND ACK OR NACK VIA PIPE
    # ------------------------------------------------------------------------------------------------------------------------
    if state2map4dragon == 3:    
        info2image4dragon = None
        message2print = "{(Step 3A/70) try" + "; info2image4dragon-try-by-manoeuvre2mapimage4dragon"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        try: 
            # ----------------------------------------------------------------------------------------------------------------
            # GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE
            # ----------------------------------------------------------------------------------------------------------------        
            info2image4dragon = queue2mapmanoeuvre4dragon.get()
            
            message2print = "{(Step 3B/33) got" + "; info2image4dragon-got-by-manoeuvre2mapimage4dragon: " + str(info2image4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
            if info2image4dragon is None:
                code2nack  = code2nack + '3Z1-'
                ack2mapinfo4dragon = False 
            else:    
                code2ack  = code2ack + '3C-'
                ack2mapinfo4dragon = True
        except:
            message2print = "{(Step 3Z1/33) failed" + "; queue2mapmanoeuvre4dragon-failed-by-manoeuvre2mapimage4dragon: " + str(queue2mapmanoeuvre4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z1-'
            
        message2mapsend4dragon = ' '
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # SEND CONTROL INFO ABOUT IMAGE OPERATION - EITHER ACK OR NACK  
            # ----------------------------------------------------------------------------------------------------------------         
            if ack2mapinfo4dragon is True:
                message2mapsend4dragon = '<ack4xi>ack2imageinfo4dragon</ack4xi>'
            else:
                message2mapsend4dragon = '<nack4xi>nack2imageinfo4dragon</nack4xi>'
            message2print = "{(Step 4B/33) set" + "; message2mapsend4dragon-set-by-manoeuvre2mapimage4dragon to: " + str(message2mapsend4dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            #      
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2mapsend4dragon-failed-by-manoeuvre2mapimage4dragon: " + str(message2mapsend4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            code2nack  = code2nack + '4Z1-'    
        try:                
            message2print = "{(Step 4C/70) using" + "; publisherchannel2mapmanoeuvre4dragon-calling-by-manoeuvre2mapimage4dragon: " + str(publisherchannel2mapmanoeuvre4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # 
            message2print = "{{(Step 4D/70) passing" + "; message2mapsend4dragon-passing-to-send-by-manoeuvre2mapimage4dragon: " + str(message2mapsend4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2mapsend4dragon-failed-by-manoeuvre2mapimage4dragon: " + str(message2mapsend4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z2-'                
        try:    
            publisherchannel2mapmanoeuvre4dragon.send(message2mapsend4dragon)
            
            message2print = "{(Step 4E/33) sent" + "; message2mapsend4dragon-sent-by-manoeuvre2mapimage4dragon: " + str(message2mapsend4dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            #          
            code2ack  = code2ack + '4E-'
        except:
            message2print = "{(Step 4Z3/33) failed" + "; message2mapsend4dragon-failed-by-manoeuvre2mapimage4dragon: " + str(message2mapsend4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z3-'                
    else:
        pass
        

            
    # ------------------------------------------------------------------------------------------------------------------------
    # RETURN 
    # ------------------------------------------------------------------------------------------------------------------------
    response2manouvre4map['ACK2INFO4DRAGON']        = ack2mapinfo4dragon
    response2manouvre4map['CODE2ACK4DRAGON']        = code2ack
    response2manouvre4map['CODE2NACK4DRAGON']       = code2nack
    response2manouvre4map['MESSAGE2GET4DRAGON']     = message2mapget4dragon
    response2manouvre4map['MESSAGE2PUT4DRAGON']     = message2mapput4dragon
    response2manouvre4map['MESSAGE2RECEIVE4DRAGON'] = message2mapreceive4dragon
    response2manouvre4map['MESSAGE2SEND4DRAGON']    = message2mapsend4dragon
    response2manouvre4map['STATE2MANOEUVRE4DRAGON'] = state2map4dragon
                          
    message2print = "{(Step 33/33) returning" + "; response2manouvre4map-returning-by-manoeuvre2mapimage4dragon: " + str(response2manouvre4map) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #1 {'MESSAGE2SEND4DRAGON': '<roger4xi>roger2mapimage4dragon</roger4xi>', 
    #  'ACK2INFO4DRAGON': False, 
    #  'MESSAGE2GET4DRAGON': '<default4xi>default2map4dragon</default4xi>', 
    #  'MESSAGE2RECEIVE4DRAGON': '<copy4xi>copy2mapimage4dragon</copy4xi>', 
    #  'CODE2ACK4DRAGON': ' 0D-1C-', 
    #  'MESSAGE2PUT4DRAGON': '<default4xi>default2map4dragon</default4xi>', 
    #  'CODE2NACK4DRAGON': ' ', 
    #  'STATE2MANOEUVRE4DRAGON': 1}
     
    #2 {'MESSAGE2SEND4DRAGON': '<ready4xi>ready2mapimage4dragon</ready4xi>', 
    #  'ACK2INFO4DRAGON': False, 
    #  'MESSAGE2GET4DRAGON': '<default4xi>default2map4dragon</default4xi>', 
    #  'MESSAGE2RECEIVE4DRAGON': '<ready4xi>ready2mapimage4dragon</ready4xi>', 
    #  'CODE2ACK4DRAGON': ' 0D-2C-', 
    #  'MESSAGE2PUT4DRAGON': '<default4xi>default2map4dragon</default4xi>', 
    #  'CODE2NACK4DRAGON': ' ', 
    #  'STATE2MANOEUVRE4DRAGON': 2}; time: 06:49:48 10/04/11 Pacific Daylight Time}
     
    #3 {'MESSAGE2SEND4DRAGON': '<ack4xi>ack2imageinfo4dragon</ack4xi>', 
    #  'ACK2INFO4DRAGON': True, 
    #  'MESSAGE2GET4DRAGON': '<default4xi>default2map4dragon</default4xi>', 
    #  'MESSAGE2RECEIVE4DRAGON': '<ready4xi>ready2mapimage4dragon</ready4xi>', 
    #  'CODE2ACK4DRAGON': ' 0D-3C-4E-', 
    #  'MESSAGE2PUT4DRAGON': '<default4xi>default2map4dragon</default4xi>', 
    #  'CODE2NACK4DRAGON': ' ', 
    #  'STATE2MANOEUVRE4DRAGON': 3}; time: 06:49:48 10/04/11 Pacific Daylight Time}
    
    #4 {'MESSAGE2SEND4DRAGON': '<default4xi>default2map4dragon</default4xi>', 
    #  'ACK2INFO4DRAGON': False, 
    #  'MESSAGE2GET4DRAGON': '<default4xi>default2map4dragon</default4xi>', 
    #  'MESSAGE2RECEIVE4DRAGON': '<default4xi>default2map4dragon</default4xi>', 
    #  'CODE2ACK4DRAGON': ' 0D-', 
    #  'MESSAGE2PUT4DRAGON': '<default4xi>default2map4dragon</default4xi>', 
    #  'CODE2NACK4DRAGON': ' ', 'STATE2MANOEUVRE4DRAGON': 0}; time: 06:49:48 10/04/11 Pacific Daylight Time}

    return response2manouvre4map           
          
#</function4xi>

# -------------------------------------------------------------------------------------------------
#<function4xi function = "write2image4dragon" usage = "
#                                                                                                              
#   # ---------------------------------------------------------------------------------------------
#   # (16) ESTABLISH CONTROL & COMMAND WITH READ2IMAGE4DRAGON                      
#   # ---------------------------------------------------------------------------------------------    
#   print('(Step 16A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL & COMMAND WITH READ2IMAGE4DRAGON ', time4xi())    
#   respon4write2image = {} 
#   task4write2image = None
#   try:                               
#       response4write2image = write2image4dragon(task4write2image)                                                                                                          
#       print('(Step 16B/700) OF DRAGON2IMAGE4XI5TEST - ESTABLISHED CONTROL & COMMAND WITH READ2IMAGE4DRAGON : ', response4write2image)
#       queue2write4dragon      = response4write2image['QUEUE']
#       channel2publish4dragon = response4write2image['PUBLISHERCHANNEL']
#       channel2consume4dragon = response4write2image['CONSUMERCHANNEL']
#   except:
#       print('(Step 16Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL & COMMAND WITH READ2IMAGE4DRAGON ', time4xi()) 
#   # --------------------------------------------------------------------------------------------------
#   # (17) SEND CONTROL & COMMAND COPY VIA PIPE FOR MANOEUVRE2IMAGE4DRAGON TO RECEIVE                 
#   # --------------------------------------------------------------------------------------------------      
#   print('(Step 17A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4DRAGON ', time4xi())     
#   try:                               
#       message2writesend4dragon = "<copy4xi>copy2write4dragon</copy4xi>"
#       channel2publish4dragon = response4write2image['PUBLISHERCHANNEL']
#       channel2publish4dragon.send(message2writesend4dragon)
#       print("(Step 17B/700) OF DRAGON2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4DRAGON: ", channel2publish4dragon, " MESSAGE2SEND4DRAGON: " , message2writesend4dragon)                                                                                                
#   except:
#       print('(Step 17Z1/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4DRAGON ', time4xi())                                                
#   # ------------------------------------------------------------------------------------------
#   # (18) LET MANOEUVRE2IMAGE4DRAGON TO RECEIVE COPY AND AND SEND ROGER VIA PIPE                
#   # ------------------------------------------------------------------------------------------       
#   print("(Step 18C/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4DRAGON ", time4xi()) 
#   try:        
#       response2manoeuvre4dragon = manoeuvre2readimage4dragon(queue2write4dragon, channel2consume4dragon, channel2publish4dragon)
#       print("(Step 18D/700) OF DRAGON2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4DRAGON ", manoeuvre2readimage4dragon)
#   except:
#       print("(Step 18Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4DRAGON ", time4xi())     
#   else:
#       pass # trust activated thread    
#   # ---------------------------------------------------------------------------------------------
#   # (19) RECEIVE CONTROL & COMMAND ROGER VIA PIPE                   
#   # ---------------------------------------------------------------------------------------------          
#   print('(Step 19A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4DRAGON ', time4xi())     
#   try:
#       channel2consume4dragon = response4write2image['CONSUMERCHANNEL']                              
#       message2writereceive4dragon = channel2consume4dragon.recv()
#       print("(Step 19B/700) OF DRAGON2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4DRAGON: ", channel2consume4dragon, " CONTROL & COMMAND: ", message2writereceive4dragon)                                                                                                  
#       # <roger4xi>roger2write4dragon</roger4xi>
#   except:
#       print('(Step 19Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH READ2IMAGE4DRAGON ', time4xi())         
#   # ---------------------------------------------------------------------------------------------------
#   # (20) SEND CONTROL & COMMAND READY VIA PIPE FOR MANOEUVRE2IMAGE4DRAGON TO RECEIVE                
#   # ---------------------------------------------------------------------------------------------------      
#   print('(Step 20A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4DRAGON ', time4xi())     
#   try:                               
#       message2writesend4dragon = "<write4xi>write2write4dragon</write4xi>"
#       channel2publish4dragon = response4write2image['PUBLISHERCHANNEL']
#       channel2publish4dragon.send(message2writesend4dragon)
#       print("(Step 20B/700) OF DRAGON2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4DRAGON: ", channel2publish4dragon, " MESSAGE2SEND4DRAGON: " , message2writesend4dragon)                                                                                                
#       # <write4xi>write2write4dragon</write4xi> 
#   except:
#        print('(Step 20Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4DRAGON ', time4xi())                                                
#   # -----------------------------------------------------------------------------------
#   # (21) LET MANOEUVRE2IMAGE4DRAGON TO RECEIVE READY AND TO SEND READY                
#   # -----------------------------------------------------------------------------------       
#   print("(Step 21A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4DRAGON ", time4xi()) 
#   try:        
#       response2manoeuvre4dragon = manoeuvre2readimage4dragon(queue2write4dragon, channel2consume4dragon, channel2publish4dragon)
#       print("(Step 21B/700) OF DRAGON2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4DRAGON ", manoeuvre2readimage4dragon)
#   except:
#       print("(Step 21Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4DRAGON ", time4xi())     
#   # ---------------------------------------------------------------------------------------
#   # (22) RECEIVE CONTROL & COMMAND READY VIA PIPE                      
#   # ---------------------------------------------------------------------------------------          
#   print('(Step 22A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4DRAGON ', time4xi())     
#   try:
#       channel2consume4dragon = response4write2image['CONSUMERCHANNEL']                              
#       message2writereceive4dragon = channel2consume4dragon.recv()
#       print("(Step 22B/700) OF DRAGON2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4DRAGON: ", channel2consume4dragon, " CONTROL & COMMAND: ", message2writereceive4dragon)                                                                                                  
#       # <write4xi>write2write4dragon</write4xi> 
#   except:
#       print('(Step 22Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH READ2IMAGE4DRAGON ', time4xi())          
#   # -----------------------------------------------------------------------------------------
#   # (24) PUT IMAGE PATH TO QUEUE
#   # -----------------------------------------------------------------------------------------                          
#   print("(Step 24A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO PUT IMAGE PATH TO QUEUE ", time4xi())      
#   try:    
#       queue2write4dragon.put(path2dragon)             
#       print("(Step 24B/700) OF DRAGON2IMAGE4XI5TEST - PUT TO QUEUE IMAGE PATH: ", path2dragon) 
#       # ..//..//..//..//..//web//images//png//dragonxi.png  
#   except:
#       print("(Step 24Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO PUT IMAGE PATH TO QUEUE ", time4xi()) 
#   # ------------------------------------------------------------------------------------------------------
#   # (25) LET MANOEUVRE2IMAGE4DRAGON TO GET FROM QUEUE AND TO SEND EITHER ACK OR NACK VIA PIPE               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 25A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4DRAGON ", time4xi()) 
#   try:        
#       response2manoeuvre4dragon = manoeuvre2readimage4dragon(queue2write4dragon, channel2consume4dragon, channel2publish4dragon)
#       print("(Step 25B/700) OF DRAGON2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4DRAGON ", manoeuvre2readimage4dragon)
#   except:
#       print("(Step 25Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4DRAGON ", time4xi())     
#   # -----------------------------------------------------------------------------------------------
#   # (26) RECEIVE ACK OR NACK ABOUT IMAGE OPERATION VIA PIPE                
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 26A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())      
#   try: 
#       message2writereceive4dragon = channel2consume4dragon.recv()
#       print("(Step 26D/700) OF DRAGON2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4DRAGON: ", channel2consume4dragon, " CONTROL & COMMAND: ", message2writereceive4dragon)                                                                                                   
#       # <ack4xi>ack2imageinfo4dragon</ack4xi>
#   except:
#       print('(Step 26Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())               
#   # -------------------------------------------------------------------------------------------------------
#   # (27) LET MANOEUVRE2IMAGE4DRAGON GO TO STATE 0               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 27A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4DRAGON ", time4xi()) 
#   try:        
#       response2manoeuvre4dragon = manoeuvre2readimage4dragon(queue2write4dragon, channel2consume4dragon, channel2publish4dragon)
#       print("(Step 27B/700) OF DRAGON2IMAGE4XI5TEST - GOT RESPONSE2MANOEUVRE4DRAGON ", response2manoeuvre4dragon)
#   except:
#       print("(Step 27Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4DRAGON ", time4xi())   

#">

# -------------------------------------------------------------------------------------------------   
def write2image4dragon(task4imagewrite2dragon=None):

    '''
    writes 2D images for DRAGON
    '''
    global dragon2image4xi                                                     # 2D-IMAGES FOR DRAGON  
    global control2writeimage4dragon                                              # COMMAND AND CONTROL FOR DRAGON IMAGES 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS        
    # -----------------------------------------------------------------------------------------          
    class2print = "write2image4dragon"
    method2print = "write2image4dragon"

    message2print = "{(Step 1A/33) started" + "; write2image4dragon-started: " + str(write2image4dragon)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function write2image4dragon at 0x00000000038E8EC8>
    
    message2print = "{(Step 1B/33) passed" + "; task4imagewrite2dragon-to-write2image4dragon: " + str(task4imagewrite2dragon) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None
            
    code2ack  = ' '
    code2nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # CREATE DRAGON2IMAGE4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if dragon2image4xi is None:
            dragon2image4xi = create2image4dragon()                                                                              # INITIALIZE IMAGE FOR DRAGON 
            message2print = "{(Step 1C/70) created" + "; dragon2image4xi-created-by-write2image4dragon to: " + str(dragon2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create2image4dragon-skipped-by-write2image4dragon: " + str(write2image4dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # create2image4dragon-skipped-by-write2image4dragon: <function write2image4dragon at 0x00000000038E8EC8>                                 
    except:
        try:
            dragon2image4xi = create2image4dragon()
            message2print = "{(Step 1E/70) set" + "; dragon2image4xi-set-by-write2image4dragon to: " + str(dragon2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            dragon2image4xi = {}
            message2print = "{(Step 1Z/70) yielded" + "; dragon2image4xi-yielded-by-write2image4dragon: " + str(dragon2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z-' 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    try:             
        if task4imagewrite2dragon is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER FOR READ - BUILD TASK! 
            # -----------------------------------------------------------------------------------------    
            
            task4imagewrite2dragon = {}                                                                                                                                                                                       
            
            message2print = "{(Step 1A/70) initialized" + "; task4imagewrite2dragon-created-by-write2image4dragon to: " + str(task4imagewrite2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 
            
            # ---------------------------------------------------------------------------------------------
            # (1B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4dragon = trace2image4dragon                                                                 # CALLBACK FUNCTION FOR DRAGON
            
            message2print = "{(Step 1B1/70) initialized" + "; function2callback4dragon-created-by-write2image4dragon to: " + str(function2callback4dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # <function trace2image4dragon at 0x00000000038E2A48>
            
            code2ack  = code2ack + '1B1-'
           
            args2callback = {}                                                                                      # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "write2image4dragon"                                                            # MODULE OF DRAGONIZED OPENCL MODULE
            args2callback['METHOD']  = "write2image4dragon"                                                            # FUNCTION OF DRAGONIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; write2image4dragon-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR DRAGONIZED OPENCL TO RETURN    

            message2print = "{(Step 1B2/70) initialized" + "; args2callback-created-by-write2image4dragon to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {'MESSAGE': '{(Step 1Z1/33) failed; write2image4dragon-failed ; time: 02:32:29 10/05/11 Pacific Daylight Time}', 
            #  'CLASS': 'write2image4dragon', 
            #  'METHOD': 'write2image4dragon'}; time: 02:32:29 10/05/11 Pacific Daylight Time}

            code2ack  = code2ack + '1B2-'
            
            # ---------------------------------------------------------------------------------------------
            # (1C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imagewrite2dragon['DATA2CALLBACK'] = args2callback                                                    # USER DATA PASSED TO DRAGONIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagewrite2dragon['FUNCTION2CALLBACK'] = function2callback4dragon                                        # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '1C-'
            message2print = "{(Step 1C/70) initialized" + "; task4imagewrite2dragon-created-by-write2image4dragon to: " + str(task4imagewrite2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # {'FUNCTION2CALLBACK': <function trace2image4dragon at 0x00000000038E2A48>,
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; write2image4dragon-failed ; time: 02:32:29 10/05/11 Pacific Daylight Time}',
            #  'CLASS': 'write2image4dragon',
            #  'METHOD': 'write2image4dragon'}}; time: 02:32:29 10/05/11 Pacific Daylight Time}

            
            # ---------------------------------------------------------------------------------------------
            # (2) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-write2image4dragon to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # {}
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-write2image4dragon: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}; time: 02:32:29 10/05/11 Pacific Daylight Time}

                code2ack  = code2ack + '2B-'
            except:
                message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-write2image4dragon: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '2Z-'
                 
            task4imagewrite2dragon['PLATFORM'] = platform2xi                                                           # PLATFORM OF SYSTEM 
              
            
            # ---------------------------------------------------------------------------------------------
            # (3) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2dragon = {}
            message2print = "{(Step 3A/70) trying" + "; context2dragon-trying-by-write2image4dragon: " + str(context2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2dragon = get2context4dragon()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 3B/70) got" + "; context2dragon-got-by-write2image4dragon: " + str(context2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4dragon-failed ; time: 02:32:29 10/05/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2context4dragon', ...
   
                code2ack  = code2ack + '3B-'         
            except:
                message2print = "{(Step 3Z/70) failed" + "; context2dragon-failed-by-write2image4dragon: " + str(context2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '3Z-'
                
            task4imagewrite2dragon ['CONTEXT'] = context2dragon           
            
            # ---------------------------------------------------------------------------------------------
            # (4) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2dragon = {}
            
            message2print = "{(Step 4A/70) trying" + "; device2dragon-trying-by-write2image4dragon: " + str(device2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2dragon = get2dragon4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 4B/70) got" + "; device2dragon-got-by-write2image4dragon: " + str(device2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568' ...
               
                code2ack  = code2ack + '4B-'  
            except:
                message2print = "{(Step 4Z/70) failed" + "; device2dragon-failed-by-write2image4dragon: " + str(device2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '4Z-'
            
            task4imagewrite2dragon ['DEVICE'] = device2dragon
            
                   
            # ---------------------------------------------------------------------------------------------
            # (5) IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            imageinfo2dragon = {}
             
            message2print = "{(Step 5A/70) trying" + "; imageinfo2dragon-trying-by-write2image4dragon: " + str(imageinfo2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imageinfo2dragon = get2imageinfo4dragon()                                                                         
                
                message2print = "{(Step 5B/70) got" + "; imageinfo2dragon-got-by-write2image4dragon: " + str(imageinfo2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'SIZE2QUEUE': 2147483646, 
                #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
                #  'ORDER': 'CL_RGBA'}, 
                #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004494E10>, 
                #  'WIDTH': 800, 
                #  'CONTEXT': {'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4dragon-failed ; time: 02:32:29 10/05/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2context4dragon', 
                #  'METHOD': 'create2context4dragon'}, 
                #  'FUNCTION2CALLBACK': <function trace2context4dragon at 0x000000000387E348>, 
                #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-', 
                #  'INFO2PRIVATE': None, 
                #  'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
                #  'NACK2RETURN': ' ', 
                #  'SIZE2PRIVATE': 0, 
                #  'PROPERTIES': None}, 
                #  'PIPE': (<read-write PipeConnection, handle 520>, <read-write PipeConnection, handle 524>), 
                #  'COUNT2REFERENCE': 1, 
                #  'FUNCTION2CALLBACK': <function trace2image4dragon at 0x00000000038E2A48>, 
                #  'COMMANDQUEUE': ...

                code2ack  = code2ack + '5B-'  
            except:
                message2print = "{(Step 5Z/70) failed" + "; imageinfo2dragon-failed-by-write2image4dragon: " + str(imageinfo2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '5Z-'          
                      
            task4imagewrite2dragon ['IMAGE'] = imageinfo2dragon         
                        

            # ---------------------------------------------------------------------------------------------
            # (6) QUEUE - REQUIRED BY DRAGONXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2dragon = None
            
            message2print = "{(Step 6A/70) trying" + "; queue2dragon-trying-by-write2image4dragon: " + str(queue2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2dragon = Queue4Xi()                                                                              # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 6B/70) got" + "; queue2dragon-got-by-write2image4dragon: " + str(queue2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # <queue4xi.Queue4Xi object at 0x0000000003D4FA90>
                
                code2ack  = code2ack + '6B-'  
            except:
                message2print = "{(Step 6Z/70) failed" + "; queue2dragon-failed-by-write2image4dragon: " + str(queue2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '6Z-'
                
            task4imagewrite2dragon ['QUEUE'] = queue2dragon

            
            # ---------------------------------------------------------------------------------------------
            # (7) PIPE - REQUIRED BY DRAGONXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2dragon = None
           
            message2print = "{(Step 7A/70) trying" + "; pipe2dragon-trying-by-write2image4dragon: " + str(pipe2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2dragon = Pipe4Xi()                                                                                # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 7B/70) got" + "; pipe2dragon-got-by-write2image4dragon: " + str(pipe2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # (<read-write PipeConnection, handle 568>, <read-write PipeConnection, handle 572>)
                
                code2ack  = code2ack + '7B-'  
            except:
                message2print = "{(Step 7Z/70) failed" + "; pipe2dragon-failed-by-write2image4dragon: " + str(pipe2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '7Z-'
                
            task4imagewrite2dragon ['PIPE'] = pipe2dragon 
            
            # ---------------------------------------------------------------------------------------------
            # (8) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2dragon = {}
           
            message2print = "{(Step 8A/70) trying" + "; commandqueueinfo2dragon-trying-by-write2image4dragon: " + str(commandqueueinfo2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2dragon = get2commandqueueinfo4dragon()                                                   # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 8B/70) got" + "; commandqueueinfo2dragon-got-by-write2image4dragon: " + str(commandqueueinfo2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 
                #  'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4dragon-failed ; time: 02:32:29 10/05/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2commandqueue4dragon', 
                #  'METHOD': 'create2commandqueue4dragon'}, 
                #  'FUNCTION2CALLBACK': <function trace2commandqueue4dragon at 0x000000000388DA48> ...
                
                code2ack  = code2ack + '8B-'  
            except:
                message2print = "{(Step 8Z/70) failed" + "; commandqueueinfo2dragon-failed-by-write2image4dragon: " + str(commandqueueinfo2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '8Z-'
        
            task4imagewrite2dragon ['COMMANDQUEUE'] = commandqueueinfo2dragon
            
            
            
            # ---------------------------------------------------------------------------------------------
            # (9) BUFFER - AUTOMATIC BY DRAGONXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2dragon = {}
           
            message2print = "{(Step 9A/70) trying" + "; bufferinfo2dragon-trying-by-write2image4dragon: " + str(bufferinfo2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2dragon = get2bufferinfo4dragon()                                                               # SYSTEM CREATES BUFFER FOR DRAGON, IF NECESSARY
                message2print = "{(Step 9B/70) got" + "; bufferinfo2dragon-got-by-write2image4dragon: " + str(bufferinfo2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 
                #  'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4dragon-failed ; time: 02:32:28 10/05/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2buffer4dragon', 
                #  'METHOD': 'create2buffer4dragon'}, ...
                
                code2ack  = code2ack + '9B-' 
            except:
                message2print = "{(Step 9Z/70) failed" + "; bufferinfo2dragon-failed-by-write2image4dragon: " + str(bufferinfo2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '9Z-'
        
            task4imagewrite2dragon ['BUFFER'] = bufferinfo2dragon 
            
            # ---------------------------------------------------------------------------------------------
            # (10) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2dragon = {}
           
            message2print = "{(Step 10A/70) trying" + "; eventinfo2dragon-trying-by-write2image4dragon: " + str(eventinfo2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2dragon = get2eventinfo4dragon()                                                                    
                message2print = "{(Step 10B/70) got" + "; eventinfo2dragon-got-by-write2image4dragon: " + str(eventinfo2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #  {'CODE2ERROR': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2event4dragon-failed ; time: 02:32:28 10/05/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2event4dragon', 
                #  'METHOD': 'create2event4dragon'}, 
                #  'FUNCTION2CALLBACK': <function trace2event4dragon at 0x00000000038A3F48>, ...
                
                code2ack  = code2ack + '10B-' 
            except:
                message2print = "{(Step 10Z/70) failed" + "; eventinfo2dragon-failed-by-write2image4dragon: " + str(eventinfo2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '10Z-'
        
            task4imagewrite2dragon ['EVENT'] = eventinfo2dragon 
            
            
            # ---------------------------------------------------------------------------------------------
            # (11) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2dragon = {}
           
            message2print = "{(Step 11A/70) trying" + "; eventwaitlist2dragon-trying-by-write2image4dragon: " + str(eventwaitlist2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2dragon = {}                                                                   
                message2print = "{(Step 11B/70) set" + "; eventwaitlist2dragon-set-by-write2image4dragon: " + str(eventwaitlist2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {} 
                
                code2ack  = code2ack + '11B-' 
            except:
                message2print = "{(Step 11Z/70) failed" + "; eventwaitlist2dragon-failed-by-write2image4dragon: " + str(eventwaitlist2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '11Z-'
        
            task4imagewrite2dragon ['EVENTWAITLIST'] = eventwaitlist2dragon                     

            
            # ---------------------------------------------------------------------------------------------
            # (12) EVENTWAITLISTQUANTITY - REQUIRED                               
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2dragon = 0
           
            message2print = "{(Step 12A/70) trying" + "; eventwaitlistquantity2dragon-trying-by-write2image4dragon: " + str(eventwaitlistquantity2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2dragon.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2dragon = eventwaitlistquantity2dragon + 1                                                                   
                message2print = "{(Step 12B/70) set" + "; eventwaitlistquantity2dragon-set-by-write2image4dragon: " + str(eventwaitlistquantity2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 
                
                code2ack  = code2ack + '12B-' 
            except:
                message2print = "{(Step 12Z/70) failed" + "; eventwaitlistquantity2dragon-failed-by-write2image4dragon: " + str(eventwaitlistquantity2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '12Z-'
        
            task4imagewrite2dragon ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2dragon       
                                                 
            
            # ---------------------------------------------------------------------------------------------
            # (13) BLOCKING - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            blocking2dragon = ' '                                                                   
     
            message2print = "{(Step 13A/70) trying" + "; blocking2dragon-trying-by-write2image4dragon: " + str(blocking2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                blocking2dragon = 'CL_FALSE'                                                                           # the write command is non-blocking                                                              
                message2print = "{(Step 13B/70) set" + "; blocking2dragon-set-by-write2image4dragon to: " + str(blocking2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # CL_FALSE 
                code2ack  = code2ack + '13B-' 
            except:
                message2print = "{(Step 13Z/70) failed" + "; blocking2dragon-failed-by-write2image4dragon: " + str(blocking2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '13Z-'
        
            task4imagewrite2dragon ['BLOCKING'] = blocking2dragon             
            
            # ---------------------------------------------------------------------------------------------
            # (14) ORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin2dragon = {}
            
            message2print = "{(Step 14A/70) trying" + "; origin2dragon-trying-by-write2image4dragon: " + str(origin2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin2dragon['X'] = 0 
                origin2dragon['Y'] = 0  
                origin2dragon['Z'] = 0                                                                                                                          
                message2print = "{(Step 14B/70) set" + "; origin2dragon-set-by-write2image4dragon to: " + str(origin2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'Y': 0, 'X': 0, 'Z': 0} 
                code2ack  = code2ack + '14B-' 
            except:
                message2print = "{(Step 14Z/70) failed" + "; origin2dragon-failed-by-write2image4dragon: " + str(origin2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '14Z-'
        
            task4imagewrite2dragon ['ORIGIN'] = origin2dragon
            
            
            # ---------------------------------------------------------------------------------------------
            # (15) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2dragon = {}
            
            message2print = "{(Step 15A/70) trying" + "; region2dragon-trying-by-write2image4dragon: " + str(region2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                region2dragon['X'] = 160                                                                               # WIDTH  IN PIXELS TO READ 
                region2dragon['Y'] = 90                                                                                # HEIGHT IN PIXELS TO READ 
                region2dragon['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO READ                                                                                                                          
                message2print = "{(Step 15B/70) set" + "; region2dragon-set-by-write2image4dragon to: " + str(region2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'Y': 90, 'X': 160, 'Z': 0}
                code2ack  = code2ack + '15B-' 
            except:
                message2print = "{(Step 15Z/70) failed" + "; region2dragon-failed-by-write2image4dragon: " + str(region2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '15Z-'
        
            task4imagewrite2dragon ['REGION'] = region2dragon
            
            
            # ---------------------------------------------------------------------------------------------
            # (16) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.

            height2dragon = 800
            width2dragon = 800
            rowpitch2dragon = 0
            size2element = 1                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity2layer = 1                                                                                 # MAX NUMBER OF VOXELS IN EACH 2D LAYER 
            quantity2direction = 1                                                                                  # ONE DIRECTIONS IN 2D
            
            message2print = "{(Step 16A/70) trying" + "; rowpitch2dragon-trying-by-write2image4dragon: " + str(rowpitch2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            #  
                        
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch2dragon = int(width2dragon * size2element * voxelquantity2layer / quantity2direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 16B/70) set" + "; rowpitch2dragon-set-by-write2image4dragon to: " + str(rowpitch2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 800                                                                                            
                 
                code2ack  = code2ack + '16B-' 
            except:
                message2print = "{(Step 16Z/70) failed" + "; rowpitch2dragon-failed-by-write2image4dragon: " + str(rowpitch2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '16Z-'
        
            task4imagewrite2dragon ['ROWPITCH'] = rowpitch2dragon     
            
            # ----------------------------------------------------------------------------------------------------
            # (17) POINTER - REQUIRED - THE POINTER TO A BUFFER IN HOST MEMORY WHERE IMAGE DATA IS TO BE READ FROM                          
            # ----------------------------------------------------------------------------------------------------             
            pointer2dragon = None
            size2dragon = int(width2dragon * height2dragon)                                                                  # CALCULATED MEMORY
            if size2dragon == 0:
                size2dragon = 4400                                                                                     # MINIMUM MEMORY
            
            message2print = "{(Step 17A/70) trying" + "; pointer2dragon-trying-by-write2image4dragon: " + str(pointer2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:
                arena2dragon = Arena4Xi(size2dragon)                                                                      # SYSTEM RESERVER MEMORY FOR IMAGE                             
                                                                                                                                         
                message2print = "{(Step 17B/70) got" + "; arena2dragon-got-by-write2image4dragon: " + str(arena2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # <arena4xi.Arena4Xi object at 0x00000000041348D0>
                
                size2arena = arena2dragon.size                                                                         # 
                              
                message2print = "{(Step 17C/70) got" + "; size2arena-got-by-write2buffer4dragon: " + str(size2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 640000 
                
                buffer2arena = arena2dragon.buffer
                
                message2print = "{(Step 17D/70) got" + "; buffer2arena-got-by-write2buffer4dragon: " + str(buffer2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # <mmap.mmap object at 0x000000000421A450> 
                
                code2ack  = code2ack + '17D-' 
                
                if size2arena < size2dragon:
                    size2dragon = size2arena                                                                           # AVAILABLE MEMORY FOR IMAGE
                    message2print = "{(Step 17E/70) set" + "; size2dragon-set-by-write2buffer4dragon to: " + str(size2dragon)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    
                    code2ack  = code2ack + '17E-'
                else:
                    pass
                    
                pointer2dragon = buffer2arena
                message2print = "{(Step 17F/70) set" + "; pointer2dragon-set-by-write2buffer4dragon to: " + str(pointer2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # <mmap.mmap object at 0x000000000421A450>
                
                code2ack  = code2ack + '17F-'       
            except:
                message2print = "{(Step 17Z/70) failed" + "; pointer2dragon-failed-by-write2image4dragon: " + str(pointer2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '17Z-'
        
            task4imagewrite2dragon ['POINTER'] = pointer2dragon                            
            task4imagewrite2dragon ['SIZE']    = size2dragon                                                               # AUTOMATIC BY DRAGONXI
            
            
            # --------------------------------------------------------------------------------------------------------
            # (18) SLICEPITCH - REQUIRED - THE SIZE IN BYTES OF THE 2D SLICE OF THE 3D REGION OF A 3D IMAGE BEING READ                         
            # --------------------------------------------------------------------------------------------------------             
            slicepitch2dragon = None                                                                                   # SLICEPITCH IS ZERO FOR 2D IMAGES                           
           
            message2print = "{(Step 18A/70) trying" + "; slicepitch2dragon-trying-by-write2image4dragon: " + str(slicepitch2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                slicepitch2dragon = 0                                                          
                message2print = "{(Step 18B/70) set" + "; slicepitch2dragon-set-by-write2image4dragon to: " + str(slicepitch2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0

                code2ack  = code2ack + '18B-'                
                 
            except:
                message2print = "{(Step 18Z/70) failed" + "; slicepitch2dragon-failed-by-write2image4dragon: " + str(slicepitch2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '18Z-'
            
            task4imagewrite2dragon ['SLICEPITCH'] = slicepitch2dragon                  
            
            # ---------------------------------------------------------------------------------------------
            # EOF PARAMETER PROVIDING                
            # ---------------------------------------------------------------------------------------------                                                                                                                               
        else:
            pass
            
        # ---------------------------------------------------------------------------------------------
        # CHECK PARAMETERS - WHETHER AUTOMATICALLY GENERATED OR USER GIVEN 
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imagewrite2dragon.keys()
        message2print = "{(Step 20/70) using" + "; keys2task-using-by-write2image4dragon: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BLOCKING',
        #  'BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGE',
        #  'ORIGIN',
        #  'PIPE',
        #  'PLATFORM',
        #  'POINTER',
        #  'QUEUE',
        #  'REGION',
        #  'ROWPITCH',
        #  'SIZE',
        #  'SLICEPITCH']; time: 02:45:31 10/05/11 Pacific Daylight Time}

        if (('DATA2CALLBACK' in keys2task) and ('FUNCTION2CALLBACK' in keys2task)):
            pass
        else:                
            # ---------------------------------------------------------------------------------------------
            # (21B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4dragon = trace2image4dragon                                                             # CALLBACK FUNCTION FOR DRAGON
            
            message2print = "{(Step 21B1/70) initialized" + "; function2callback4dragon-created-by-write2image4dragon to: " + str(function2callback4dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code2ack  = code2ack + '21B1-'
           
            args2callback = {}                                                                                  # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "write2image4dragon"                                                        # MODULE OF DRAGONIZED OPENCL MODULE
            args2callback['METHOD']  = "write2image4dragon"                                                        # FUNCTION OF DRAGONIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; write2image4dragon-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR DRAGONIZED OPENCL TO RETURN    
            message2print = "{(Step 21B2/70) initialized" + "; args2callback-created-by-write2image4dragon to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 
    
            code2ack  = code2ack + '21B2-'
        
            # ---------------------------------------------------------------------------------------------
            # (21C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imagewrite2dragon['DATA2CALLBACK'] = args2callback                                                # USER DATA PASSED TO DRAGONIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagewrite2dragon['FUNCTION2CALLBACK'] = function2callback4dragon                                    # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '21C-'
            message2print = "{(Step 21C/70) initialized" + "; task4imagewrite2dragon-created-by-write2image4dragon to: " + str(task4imagewrite2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #
             
        if 'PLATFORM' in keys2task:
            pass
        else:  
            # ---------------------------------------------------------------------------------------------
            # (22) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 22A/70) initialized" + "; platform2xi-created-by-write2image4dragon to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
                
            try:
                platform2xi = get2platformids4xi()                                                              # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 22B/70) got" + "; platform2xi-got-by-write2image4dragon: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                     
                code2ack  = code2ack + '22B-'
            except:
                message2print = "{(Step 22Z/70) failed" + "; platform2xi-failed-by-write2image4dragon: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '22Z-'
                     
            task4imagewrite2dragon['PLATFORM'] = platform2xi                                                       # PLATFORM OF SYSTEM 
              
        if 'CONTEXT' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (23) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2dragon = {}
            message2print = "{(Step 23A/70) trying" + "; context2dragon-trying-by-write2image4dragon: " + str(context2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2dragon = get2context4dragon()                                                                 # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 23B/70) got" + "; context2dragon-got-by-write2image4dragon: " + str(context2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '23B-'         
            except:
                message2print = "{(Step 23Z/70) failed" + "; context2dragon-failed-by-write2image4dragon: " + str(context2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '23Z-'
                
            task4imagewrite2dragon ['CONTEXT'] = context2dragon           
        
        if 'DEVICE' in keys2task:
            pass
        else:                   
            # ---------------------------------------------------------------------------------------------
            # (24) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2dragon = {}
            
            message2print = "{(Step 24A/70) trying" + "; device2dragon-trying-by-write2image4dragon: " + str(device2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2dragon = get2dragon4xi()                                                                       # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 24B/70) got" + "; device2dragon-got-by-write2image4dragon: " + str(device2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '24B-'  
            except:
                message2print = "{(Step 24Z/70) failed" + "; device2dragon-failed-by-write2image4dragon: " + str(device2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '24Z-'
            
            task4imagewrite2dragon ['DEVICE'] = device2dragon
        
        if 'IMAGE' in keys2task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (25) IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            imageinfo2dragon = {}
             
            message2print = "{(Step 25A/70) trying" + "; imageinfo2dragon-trying-by-write2image4dragon: " + str(imageinfo2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imageinfo2dragon = get2imageinfo4dragon()                                                                         
                
                message2print = "{(Step 25B/70) got" + "; imageinfo2dragon-got-by-write2image4dragon: " + str(imageinfo2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code2ack  = code2ack + '25B-'  
            except:
                message2print = "{(Step 25Z/70) failed" + "; imageinfo2dragon-failed-by-write2image4dragon: " + str(imageinfo2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '25Z-'          
                      
            task4imagewrite2dragon ['IMAGE'] = imageinfo2dragon         
                    
        if 'QUEUE' in keys2task:
            pass
        else:   
            # ---------------------------------------------------------------------------------------------
            # (26) QUEUE - REQUIRED BY DRAGONXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2dragon = None
            
            message2print = "{(Step 26A/70) trying" + "; queue2dragon-trying-by-write2image4dragon: " + str(queue2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2dragon = Queue4Xi()                                                                          # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 26B/70) got" + "; queue2dragon-got-by-write2image4dragon: " + str(queue2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '26B-'  
            except:
                message2print = "{(Step 26Z/70) failed" + "; queue2dragon-failed-by-write2image4dragon: " + str(queue2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '26Z-'
                
            task4imagewrite2dragon ['QUEUE'] = queue2dragon

        if 'PIPE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (27) PIPE - REQUIRED BY DRAGONXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2dragon = None
           
            message2print = "{(Step 27A/70) trying" + "; pipe2dragon-trying-by-write2image4dragon: " + str(pipe2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2dragon = Pipe4Xi()                                                                            # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 27B/70) got" + "; pipe2dragon-got-by-write2image4dragon: " + str(pipe2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '27B-'  
            except:
                message2print = "{(Step 27Z/70) failed" + "; pipe2dragon-failed-by-write2image4dragon: " + str(pipe2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '27Z-'
                
            task4imagewrite2dragon ['PIPE'] = pipe2dragon 
            
        if 'COMMANDQUEUE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2dragon = {}
           
            message2print = "{(Step 28A/70) trying" + "; commandqueueinfo2dragon-trying-by-write2image4dragon: " + str(commandqueueinfo2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2dragon = get2commandqueueinfo4dragon()                                               # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 28B/70) got" + "; commandqueueinfo2dragon-got-by-write2image4dragon: " + str(commandqueueinfo2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '28B-'  
            except:
                message2print = "{(Step 28Z/70) failed" + "; commandqueueinfo2dragon-failed-by-write2image4dragon: " + str(commandqueueinfo2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '28Z-'
        
            task4imagewrite2dragon ['COMMANDQUEUE'] = commandqueueinfo2dragon
            
            
        if 'BUFFER' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (29) BUFFER - AUTOMATIC BY DRAGONXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2dragon = {}
           
            message2print = "{(Step 29A/70) trying" + "; bufferinfo2dragon-trying-by-write2image4dragon: " + str(bufferinfo2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2dragon = get2bufferinfo4dragon()                                                           # SYSTEM CREATES BUFFER FOR DRAGON, IF NECESSARY
                message2print = "{(Step 29B/70) got" + "; bufferinfo2dragon-got-by-write2image4dragon: " + str(bufferinfo2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '29B-' 
            except:
                message2print = "{(Step 29Z/70) failed" + "; bufferinfo2dragon-failed-by-write2image4dragon: " + str(bufferinfo2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '29Z-'
        
            task4imagewrite2dragon ['BUFFER'] = bufferinfo2dragon
             
        if 'EVENT' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (30) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2dragon = {}
           
            message2print = "{(Step 30A/70) trying" + "; eventinfo2dragon-trying-by-write2image4dragon: " + str(eventinfo2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2dragon = get2eventinfo4dragon()                                                                    
                message2print = "{(Step 30B/70) got" + "; eventinfo2dragon-got-by-write2image4dragon: " + str(eventinfo2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '30B-' 
            except:
                message2print = "{(Step 30Z/70) failed" + "; eventinfo2dragon-failed-by-write2image4dragon: " + str(eventinfo2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '30Z-'
        
            task4imagewrite2dragon ['EVENT'] = eventinfo2dragon 
        
             
        if 'EVENTWAITLIST' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (31) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2dragon = {}
           
            message2print = "{(Step 31A/70) trying" + "; eventwaitlist2dragon-trying-by-write2image4dragon: " + str(eventwaitlist2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2dragon = {}                                                                   
                message2print = "{(Step 31B/70) set" + "; eventwaitlist2dragon-set-by-write2image4dragon: " + str(eventwaitlist2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '31B-' 
            except:
                message2print = "{(Step 31Z/70) failed" + "; eventwaitlist2dragon-failed-by-write2image4dragon: " + str(eventwaitlist2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '31Z-'
        
            task4imagewrite2dragon ['EVENTWAITLIST'] = eventwaitlist2dragon                     

                 
        if 'EVENTWAITLISTQUANTITY' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (32) EVENTWAITLISTQUANTITY - REQUIRED                               
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2dragon = 0
           
            message2print = "{(Step 32A/70) trying" + "; eventwaitlistquantity2dragon-trying-by-write2image4dragon: " + str(eventwaitlistquantity2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2dragon.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2dragon = eventwaitlistquantity2dragon + 1                                                                     
                message2print = "{(Step 32B/70) set" + "; eventwaitlistquantity2dragon-set-by-write2image4dragon: " + str(eventwaitlistquantity2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '32B-' 
            except:
                message2print = "{(Step 32Z/70) failed" + "; eventwaitlistquantity2dragon-failed-by-write2image4dragon: " + str(eventwaitlistquantity2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '32Z-'
        
            task4imagewrite2dragon ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2dragon       
                                                 
                 
        if 'BLOCKING' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (33) BLOCKING - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            blocking2dragon = ' '                                                                   
            message2print = "{(Step 33A/70) trying" + "; blocking2dragon-trying-by-write2image4dragon: " + str(blocking2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                blocking2dragon = 'CL_FALSE'                                                                       # the write command is non-blocking                                                              
                message2print = "{(Step 33B/70) set" + "; blocking2dragon-set-by-write2image4dragon to: " + str(blocking2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '33B-' 
            except:
                message2print = "{(Step 33Z/70) failed" + "; blocking2dragon-failed-by-write2image4dragon: " + str(blocking2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '33Z-'
        
            task4imagewrite2dragon ['BLOCKING'] = blocking2dragon             
                 
        if 'ORIGIN' in keys2task:
            pass
        else:                    
            # ---------------------------------------------------------------------------------------------
            # (34) ORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin2dragon = {}
            
            message2print = "{(Step 34A/70) trying" + "; origin2dragon-trying-by-write2image4dragon: " + str(origin2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin2dragon['X'] = 0 
                origin2dragon['Y'] = 0  
                origin2dragon['Z'] = 0                                                                                                                          
                message2print = "{(Step 34B/70) set" + "; origin2dragon-set-by-write2image4dragon to: " + str(origin2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '34B-' 
            except:
                message2print = "{(Step 34Z/70) failed" + "; origin2dragon-failed-by-write2image4dragon: " + str(origin2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '34Z-'
        
            task4imagewrite2dragon ['ORIGIN'] = origin2dragon
                
                 
        if 'REGION' in keys2task:
            pass
        else:            
            # ---------------------------------------------------------------------------------------------
            # (35) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2dragon = {}
            
            message2print = "{(Step 35A/70) trying" + "; region2dragon-trying-by-write2image4dragon: " + str(region2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                region2dragon['X'] = 160                                                                               # WIDTH  IN PIXELS TO WRITE 
                region2dragon['Y'] = 90                                                                                # HEIGHT IN PIXELS TO WRITE
                region2dragon['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO WRITE                                                                                                                          
                message2print = "{(Step 35B/70) set" + "; region2dragon-set-by-write2image4dragon to: " + str(region2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '35B-' 
            except:
                message2print = "{(Step 35Z/70) failed" + "; region2dragon-failed-by-write2image4dragon: " + str(region2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '35Z-'
        
            task4imagewrite2dragon ['REGION'] = region2dragon
            
                
                 
        if 'ROWPITCH' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (36) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.
     
            rowpitch2dragon = 0
            size2element = 1                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity2layer = 1                                                                                 # MAX NUMBER OF VOXELS IN EACH 2D LAYER 
            quantity2direction = 1                                                                                  # ONE DIRECTIONS IN 2D
            
            message2print = "{(Step 36A/70) trying" + "; rowpitch2dragon-trying-by-write2image4dragon: " + str(rowpitch2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch2dragon = int(width2dragon * size2element * voxelquantity2layer / quantity2direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 36B/70) set" + "; rowpitch2dragon-set-by-write2image4dragon to: " + str(rowpitch2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #                                                                                             
                 
                code2ack  = code2ack + '36B-' 
            except:
                message2print = "{(Step 36Z/70) failed" + "; rowpitch2dragon-failed-by-write2image4dragon: " + str(rowpitch2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '36Z-'
        
            task4imagewrite2dragon ['ROWPITCH'] = rowpitch2dragon     
                 
        if 'POINTER' in keys2task:
            pass
        else:                
            # ----------------------------------------------------------------------------------------------------
            # (37) POINTER - REQUIRED - THE POINTER TO A BUFFER IN HOST MEMORY WHERE IMAGE DATA IS TO BE READ FROM                          
            # ----------------------------------------------------------------------------------------------------             
            pointer2dragon = None
            size2dragon = int(width2dragon * height2dragon)                                                                  # CALCULATED MEMORY
            if size2dragon == 0:
                size2dragon = 4400                                                                                     # MINIMUM MEMORY
            
            message2print = "{(Step 37A/70) trying" + "; pointer2dragon-trying-by-write2image4dragon: " + str(pointer2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:
                arena2dragon = Arena4Xi(size2dragon)                                                                      # SYSTEM RESERVER MEMORY FOR IMAGE                             
                                                                                                                                         
                message2print = "{(Step 37B/70) got" + "; arena2dragon-got-by-write2image4dragon: " + str(arena2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                size2arena = arena2dragon.size                                                                         # 
                              
                message2print = "{(Step 37C/70) got" + "; size2arena-got-by-write2buffer4dragon: " + str(size2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                
                buffer2arena = arena2dragon.buffer
                
                message2print = "{(Step 37D/70) got" + "; buffer2arena-got-by-write2buffer4dragon: " + str(buffer2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # 
                
                code2ack  = code2ack + '37D-' 
                
                if size2arena < size2dragon:
                    size2dragon = size2arena                                                                           # AVAILABLE MEMORY FOR IMAGE
                    message2print = "{(Step 37E/70) set" + "; size2dragon-set-by-write2buffer4dragon to: " + str(size2dragon)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    
                    code2ack  = code2ack + '37E-'
                else:
                    pass
                    
                pointer2dragon = buffer2arena
                message2print = "{(Step 37F/70) set" + "; pointer2dragon-set-by-write2buffer4dragon to: " + str(pointer2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                
                code2ack  = code2ack + '37F-'       
            except:
                message2print = "{(Step 37Z/70) failed" + "; pointer2dragon-failed-by-write2image4dragon: " + str(pointer2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '37Z-'
        
            task4imagewrite2dragon ['POINTER'] = pointer2dragon                            
            task4imagewrite2dragon ['SIZE']    = size2dragon                                                               # AUTOMATIC BY DRAGONXI
        
             
        if 'SLICEPITCH' in keys2task:
            pass
        else:               
            # --------------------------------------------------------------------------------------------------------
            # (38) SLICEPITCH - REQUIRED - THE SIZE IN BYTES OF THE 2D SLICE OF THE 3D REGION OF A 3D IMAGE BEING READ                         
            # --------------------------------------------------------------------------------------------------------             
            slicepitch2dragon = None                                                                                   # SLICEPITCH IS ZERO FOR 2D IMAGES                           
           
            message2print = "{(Step 38A/70) trying" + "; slicepitch2dragon-trying-by-write2image4dragon: " + str(slicepitch2dragon)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                slicepitch2dragon = 0                                                          
                message2print = "{(Step 38B/70) set" + "; slicepitch2dragon-set-by-write2image4dragon to: " + str(slicepitch2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '38B-'                
                 
            except:
                message2print = "{(Step 38Z/70) failed" + "; slicepitch2dragon-failed-by-write2image4dragon: " + str(slicepitch2dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '38Z-'
            
            task4imagewrite2dragon ['SLICEPITCH'] = slicepitch2dragon                  

        # ---------------------------------------------------------------------------------------------
        # EOF PARAMETER CHECKING AND PROVIDING FOR MISSING ONES
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imagewrite2dragon.keys()
        message2print = "{(Step 39/70) set" + "; keys2task-set-by-write2image4dragon to: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BLOCKING',
        #  'BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGE',
        #  'ORIGIN',
        #  'PIPE',
        #  'PLATFORM',
        #  'POINTER',
        #  'QUEUE',
        #  'REGION',
        #  'ROWPITCH',
        #  'SIZE',
        #  'SLICEPITCH']; time: 02:45:31 10/05/11 Pacific Daylight Time}
             
    except: 
        message2print = "{(Step 39Z/70) yielded" + "; task4imagewrite2dragon-yielded-by-write2image4dragon: " + str(task4imagewrite2dragon) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '39Z-' 

    # ---------------------------------------------------------------------------------------------
    # CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    control2writeimage4dragon = {}       
                                
    # ---------------------------------------------------------------------------------------------
    # CONSUMER AND PUBLISHER CHANNEL FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4channels2dragon = False
    channels2dragon = ()
    message2print = "{(Step 40A/70) trying" + "channels2dragon-trying-by-write2image4dragon: " + str(channels2dragon)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        # 
        consumerchannel2dragon, publisherchannel2dragon = task4imagewrite2dragon ['PIPE']

        message2print = "{(Step 40B/70) got" + "; consumerchannel2dragon-got-by-write2image4dragon: " + str(consumerchannel2dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 568>

        message2print = "{(Step 40C/70) got" + "; publisherchannel2dragon-got-by-write2image4dragon: " + str(publisherchannel2dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 572>                     
        ack4channels2dragon = True
        
        control2writeimage4dragon['CONSUMERCHANNEL']  = consumerchannel2dragon
        
        
        control2writeimage4dragon['PUBLISHERCHANNEL'] = publisherchannel2dragon
        
        code2ack  = code2ack + '40D-' 
        
    except:
        ack4channels2dragon = False
        code2nack  = code2nack + '40Z-'      
                        
    # ---------------------------------------------------------------------------------------------
    # QUEUE FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4queue2dragon = False
    queue2dragon = None
    message2print = "{(Step 41A/70) trying" + "queue2dragon-trying-by-write2image4dragon: " + str(queue2dragon)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        queue2dragon = task4imagewrite2dragon ['QUEUE']

        message2print = "{(Step 41B/70) got" + "; queue2dragon-got-by-write2image4dragon: " + str(queue2dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <queue4xi.Queue4Xi object at 0x0000000003C83A90> 
              
        ack4queue2dragon = True
        control2writeimage4dragon['QUEUE']  = queue2dragon        
        code2ack  = code2ack + '41C-'
    except:
        ack4queue2dragon = False  
        code2nack  = code2nack + '41Z-'
    
    
    # -----------------------------------------------------------------------------------------    
    # DUMMY PROCESS OBJECT = THREAD CREATION
    # -----------------------------------------------------------------------------------------           
    # dummy2image4dragon = None
    # message2print = "{(Step 69A/70) trying" + "dummy2image4dragon-trying-by-write2image4dragon: " + str(dummy2image4dragon)  + "; time: " + time4xi() + "}"   
    # print4xi(class2print, method2print, message2print)     
    # try:                            
        # dummy2image4dragon = DummyProcess4Xi(target=manoeuvre2readimage4dragon,                                               # in this module to handle queue and pipe channels
        #                                   args =(queue2dragon,                                                         # caller puts and gets - does NOT call any functions
        #                                          consumerchannel2dragon, publisherchannel2dragon))                        # caller sends and receives - does NOT call any functions
        # message2print = "{(Step 69B/70) created" + "dummy2image4dragon-created-by-write2image4dragon: " + str(dummy2image4dragon)  + "; time: " + time4xi() + "}" 
        # print4xi(class2print, method2print, message2print)  
        # code2ack  = code2ack + '69B-'
        
    # except:
        # code2nack  = code2nack + '69Z-'
        
    # ---------------------------------------------------------------------------------------------
    # RETURN CONTROL AND COMMAND DICTIONARY
    # ---------------------------------------------------------------------------------------------          
    message2print = "{(Step 70A/70) set" + "; code2ack-set-by-write2image4dragon: " + str(code2ack) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                             
    # 1B1-1B2-1C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-12B-13B-14B-15B-16B-17D-17F-18B-40D-41C-
     
    message2print = "{(Step 70B/70) set" + "; code2nack-set-by-write2image4dragon: " + str(code2nack) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #  
        
    message2print = "{(Step 70C/70) returning" + "; control2writeimage4dragon-returning-by-write2image4dragon: " + str(control2writeimage4dragon) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003C83A90>,
    #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 572>,
    #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 568>}; time: 02:45:31 10/05/11 Pacific Daylight Time}

    return control2writeimage4dragon
                    
#</function4xi> 


 
# -------------------------------------------------------------------------------------------------
#<function4xi function = "release2image4dragon" usage = "
#
#   task4release2image  = {} 
#   task4release2image  ['COUNT2REFERENCE'] = 1 
#   response4release2image = {}
#    
#   print('(Step 12A/33) OF DRAGON2IMAGE4XI5TEST - TRYING TO RELEASE IMAGE OF DRAGON DEVICE ', time4xi())   
#   try:                               
#       response4release2image = release2image4dragon(task4release2image)                                                                                             
#       print('(Step 12B/33) OF DRAGON2IMAGE4XI5TEST - RESPONSE4RELEASE2IMAGE : ', response4release2image)                           
#   except:
#       print('(Step 12Z1/33) OF DRAGON2IMAGE4XI5TEST *** FAILED TO RELEASE IMAGE OF DRAGON DEVICE ', time4xi())  
#   
#                 
#">

# -------------------------------------------------------------------------------------------------   
def release2image4dragon(task4release2dragon=None):

    '''
    releases dragon2image4xi
    '''
    global dragon2image4xi  
    global code2ack
    global code2nack
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "release2image4dragon"
    method2print = "release2image4dragon" 

    message2print = "{(Step 1A/33) started" + "; release2image4dragon-started: " + str(release2image4dragon)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function release2image4dragon at 0x0000000003A9E8C8>
    
    message2print = "{(Step 1B/33) passed" + "; task4release2dragon-to-release2image4dragon: " + str(task4release2dragon) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # {'COUNT2REFERENCE': 1} 

             
    code2ack  = ' '
    code2nack = ' '

    try:
        if dragon2image4xi is None:
            dragon2image4xi = create2image4dragon()                                                                       # INITIALIZE IMAGE                 
    except:
        try:
            dragon2image4xi = create2image4dragon()
        except:
            dragon2image4xi = {}
            message2print = "{(Step 2Z0/70) yielded" + "; dragon2image4xi-yielded-by-release2image4dragon: " + str(dragon2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '2Z0-' 
                
    message2print = "{(Step 1A/70) try" + "; release-try-by-release2image4dragon: None"  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    
    if task4release2dragon is None:
        task4release2dragon = {}
        task4release2dragon['COUNT2REFERENCE'] = 1
           
    try:
        # -----------------------------------------------------------------------------------------
        # RELEASE
        # -----------------------------------------------------------------------------------------               
        dragon2image4xi['COUNT2REFERENCE'] = dragon2image4xi['COUNT2REFERENCE'] - task4release2dragon['COUNT2REFERENCE']
        
        message2print = "{(Step 1A/70) calculated" + "; count2reference-calculated-by-release2image4dragon: " + str(dragon2image4xi['COUNT2REFERENCE']) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
         
        if dragon2image4xi['COUNT2REFERENCE'] < 0:
            dragon2image4xi['COUNT2REFERENCE'] = 0
            
            message2print = "{(Step 1B/70) warning" + "; count2reference-warning-by-release2image4dragon: " + " ** " + str(dragon2image4xi['COUNT2REFERENCE'])  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)      
            # { release2image4dragon : release2image4dragon ; (Step 8Z1/70) warning; count2reference-warning-by-release2image4dragon:  ** 0 ** ; time: 23:41:37 09/06/11 Pacific Daylight Time}    
            code2nack  = code2nack + '1B-'
        else:
            code2ack  = code2ack + '1B-'
    except:
        try:
            dragon2image4xi['COUNT2REFERENCE'] = 0
            
            message2print = "{(Step 8Z1/70) warning" + "; count2reference-warning-by-release2image4dragon: " + " ** " + str(dragon2image4xi['COUNT2REFERENCE'])  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            code2nack  = code2nack + '8Z1-'
            
        except:
            message2print = "{(Step 8Z2/70) yielded" + "; dragon2image4xi-yielded-by-release2image4dragon: " + str(dragon2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '8Z2-'
                                                                                          
            
    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    dragon2image4xi ['ACK2RETURN']  = code2ack
    dragon2image4xi ['NACK2RETURN'] = code2nack  
    
    message2print = "{(Step 70/70) returning" + "; dragon2image4xi-returning-by-release2image4dragon: " + str(dragon2image4xi) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'CODE2ERROR': 0,
    #  'ACK2RETURN': ' ',
    #  'ROWPITCH': 800,
    #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4dragon-failed ; time: 23:41:37 09/06/11 Pacific Daylight Time}',
    #  'CLASS': 'create2image4dragon',
    #  'METHOD': 'create2image4dragon'},
    #  'WIDTH': 800,
    #  'BUFFER': {'CODE2ERROR': 0, 
    #  'COUNT2REFERENCE': 0,
    
    #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4dragon-failed ; time: 23:41:37 09/06/11 Pacific Daylight Time}', 'CLASS': 'create2buffer4dragon', 'METHOD': 'create2buffer4dragon'},
    #  'FUNCTION2CALLBACK': <function trace2buffer4dragon at 0x0000000003A97348>,
    #  'POINTER2BUFFER': <mmap.mmap object at 0x0000000003CDE298>,
    #  'FLAGS': 0, 'CONTEXT': {'COUNT2REFERENCE': 0, ...
    
    
    return dragon2image4xi

#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "retain2image4dragon" usage = "
#   task4retain2image  = {}
#   task4retain2image  ['DIMENSION'] = 2 
#   task4retain2image  ['COUNT2REFERENCE'] = 1 
#   response4retain2Dimage = {}
#   print('(Step 13A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO RETAIN 2D IMAGE OF DRAGON DEVICE ', time4xi())  
#   try:                               
#       response4retain2Dimage = retain2image4dragon(task4retain2image)                                                                                                 
#       print('(Step 13B/700) OF DRAGON2IMAGE4XI5TEST - RESPONSE4RETAIN2DIMAGE : ', response4retain2Dimage)                          
#       # 
#   except:
#       print('(Step 13Z1/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO RETAIN 2D IMAGE OF DRAGON DEVICE ', time4xi()) 
#
#">

# -------------------------------------------------------------------------------------------------   
def retain2image4dragon(task4retain2dragon=None):

    '''
    retains either dragon2image4xi
    '''
    global dragon2image4xi 
    
     
    global code2ack
    global code2nack
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "retain2image4dragon"
    method2print = "retain2image4dragon"

    message2print = "{(Step 1A/33) started" + "; retain2image4dragon-started: " + str(retain2image4dragon)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function retain2image4dragon at 0x0000000003A9E948>   
    
    message2print = "{(Step 1B/33) passed" + "; task4retain2dragon-to-retain2image4dragon: " + str(task4retain2dragon) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # {'COUNT2REFERENCE': 1, 'DIMENSION': 2}
    # {'COUNT2REFERENCE': 1, 'DIMENSION': 3}
    code2ack  = ' '
    code2nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # (Step 2/70) - PROVIDE TASKS, IF NECESSARY AND CHECK GIVEN TASKS 
    # -----------------------------------------------------------------------------------------    
    if task4retain2dragon is None:
        task4retain2dragon = {}
        task4retain2dragon['DIMENSION'] = 2
        task4retain2dragon['COUNT2REFERENCE'] = 1
        message2print = "{(Step 2A/33) set" + "; task4retain2dragon-set-by-retain2image4dragon to: " + str(task4retain2dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #           
    else:
        pass
        
    if 'DIMENSION' in task4retain2dragon.keys():
        if task4retain2dragon['DIMENSION'] == 2:
            pass
        else:
            task4retain2dragon['DIMENSION'] = 2 
            message2print = "{(Step 2B/33) set" + "; dimension-set-by-retain2image4dragon to: " + str(task4retain2dragon['DIMENSION'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #                       
    else:
        task4retain2dragon['DIMENSION'] = 2
    
    # -----------------------------------------------------------------------------------------
    # (Step 3/70) - CHECK COUNT2REFERENCE
    # -----------------------------------------------------------------------------------------         
    if 'COUNT2REFERENCE' in task4retain2dragon.keys():
        pass
    else:
        task4retain2dragon['COUNT2REFERENCE'] = 1
        message2print = "{(Step 3A/33) set" + "; count2reference-set-by-retain2image4dragon to: " + str(task4retain2dragon['COUNT2REFERENCE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
                                   
    if task4retain2dragon['COUNT2REFERENCE'] < 1:
        task4retain2dragon['COUNT2REFERENCE'] = 1
        message2print = "{(Step 3B/33) set" + "; count2reference-set-by-retain2image4dragon to: " + str(task4retain2dragon['COUNT2REFERENCE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #           
        
    # -----------------------------------------------------------------------------------------
    # (Step 4/70) - INITIALIZE 2D IMAGE IF NECESSARY
    # -----------------------------------------------------------------------------------------
    try:
        if dragon2image4xi is None:
            dragon2image4xi = create2image4dragon()
            message2print = "{(Step 4A/70) created" + "; dragon2image4xi-created-by-retain2image4dragon: " + str(dragon2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                                                                                          
    except:
        try:
            dragon2image4xi = create2image4dragon()
            message2print = "{(Step 4B/70) created" + "; dragon2image4xi-created-by-retain2image4dragon: " + str(dragon2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                 
        except:
            dragon2image4xi = {}
            message2print = "{(Step 4Z/70) yielded" + "; dragon2image4xi-yielded-by-retain2image4dragon: " + str(dragon2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '4Z-' 


    # -----------------------------------------------------------------------------------------
    # (Step 5/70) - RETAIN
    # -----------------------------------------------------------------------------------------             
    try:
        dragon2image4xi['COUNT2REFERENCE'] = dragon2image4xi['COUNT2REFERENCE'] + task4retain2dragon['COUNT2REFERENCE']
        message2print = "{(Step 6A/70) calculated" + "; count2reference-calculated-by-retain2image4dragon: " + str(dragon2image4xi['COUNT2REFERENCE']) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #              
        if dragon2image4xi['COUNT2REFERENCE'] <= 0:
            dragon2image4xi['COUNT2REFERENCE'] = 1
            message2print = "{(Step 6Z1/70) warning" + "; count2reference-warning-by-retain2image4dragon: " + " ** " + str(dragon2image4xi['COUNT2REFERENCE'])  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)      
            code2nack  = code2nack + '6Z1-'
        else:
            code2ack  = code2ack + '6C-'

    except:
        try:
            dragon2image4xi['COUNT2REFERENCE'] = 1
        
            message2print = "{(Step 7Z1/70) warning" + "; count2reference-warning-by-retain2image4dragon: " + " ** " + str(dragon2image4xi['COUNT2REFERENCE'])  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            code2nack  = code2nack + '7Z1-'
        
        except:
            message2print = "{(Step 7Z2/70) yielded" + "; dragon2image4xi-yielded-by-retain2image4dragon: " + str(dragon2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '7Z2-'
        
                                                                                          
            
    # -----------------------------------------------------------------------------------------
    # (Step 70/70) - RETURN           
    # -----------------------------------------------------------------------------------------
    
    dragon2image4xi ['ACK2RETURN']  = code2ack
    dragon2image4xi ['NACK2RETURN'] = code2nack     
    message2print = "{(Step 70A/70) returning" + "; dragon2image4xi-returning-by-retain2image4dragon: " + str(dragon2image4xi) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'CODE2ERROR': 0, 'ACK2RETURN': ' 6C-', 'ROWPITCH': 800, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4dragon-failed ; time: 23:25:02 10/05/11 Pacific Daylight Time}', 'CLASS': 'create2image4dragon', 'METHOD': 'create2image4dragon'}, 'WIDTH': 800, 'BUFFER': {'CODE2ERROR': 0, 
    #                   'COUNT2REFERENCE': 0, ...
    # 'COUNT2REFERENCE': 1, 
    # 'FLAGS': 0, 'CONTEXT': ...
    
    return dragon2image4xi

#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "get2imageinfo4dragon" usage = "
#   #1
#   print('(Step 3A/33) OF DRAGON2IMAGE4XI5TEST - TRYING TO GET 2D IMAGE INFO FOR DRAGON', time4xi())  
#   try:
#       imageinfo2dragon = get2imageinfo4dragon()                                                                       
#       print('(Step 3B/33) OF DRAGON2IMAGE4XI5TEST - GOT 2D IMAGE INFO FOR DRAGON: ', imageinfo2dragon) 
#   except:
#       print('(Step 3Z/33) OF DRAGON2IMAGE4XI5TEST *** FAILED TO GET 2D IMAGE INFO FOR DRAGON', time4xi())
#   #2
#   task4getinfo2image = {}
#   task4getinfo2image ['DIMENSION'] = 2
#   task4getinfo2image ['WIDTH'] = 0
#   task4getinfo2image ['HEIGHT'] = 0
#   response4getinfo2image = {}
#   print('(Step 12C/33) OF DRAGON2IMAGE4XI5TEST - TRYING TO GET SELECTED 3D IMAGE INFO OF DRAGON DEVICE ', time4xi())    
# 
#   try:                               
#       response4getinfo2image = get2imageinfo4dragon (task4getinfo2image)                                                                                             
#       print('(Step 12D/33) OF DRAGON2IMAGE4XI5TEST - RESPONSE4RELEASE2IMAGE : ', response4getinfo2image)                           
#       #
#   except:
#       print('(Step 12Z2/33) OF DRAGON2IMAGE4XI5TEST *** FAILED TO GET SELECTED 3D IMAGE INFO OF DRAGON DEVICE ', time4xi()) 
#            
#">

# -------------------------------------------------------------------------------------------------   
def get2imageinfo4dragon(task4getinfo2image=None):

    '''
    gets dragon2image4xi
    '''
    
    global dragon2image4xi
    
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33)           
    # -----------------------------------------------------------------------------------------          
    class2print = "get2imageinfo4dragon"
    method2print = "get2imageinfo4dragon"

    message2print = "{(Step 0A/33) started" + "; get2imageinfo4dragon-started: " + str(get2imageinfo4dragon)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # <function get2imageinfo4dragon at 0x0000000003A9E9C8>                                                              
    
    message2print = "{(Step 0A/33) passed" + "; task4getinfo2image-passed-to-get2imageinfo4dragon: " + str(task4getinfo2image) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)   
    # {'WIDTH': 0, 'DIMENSION': 2, 'HEIGHT': 0}
    # {'WIDTH': 0, 'DEPTH': 0, 'DIMENSION': 3, 'HEIGHT': 0}
    # {'DOMAIN': 'ALL', 'DIMENSION': 2}
    
    image2dragon = {}
    try: 
        if dragon2image4xi is None:
            # -------------------------------------------------------------------------------------
            # INITIALIZE 2D IMAGE FOR DRAGON           
            # -------------------------------------------------------------------------------------           
            image2dragon = create2image4dragon()
            message2print = "{(Step 1B2/33) initialized" + "; image2dragon-initialized-by-get2imageinfo4dragon to: " + str(image2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
        else:
            message2print = "{(Step 1B2/33) using" + "; dragon2image4xi-using-by-get2imageinfo4dragon: " + str(dragon2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CODE2ERROR': 0, 
            #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-14B-15B-16B-17B-17D-18B-18C-21A1-22B1-22C1-23B-24B-25B-26B-27B-28B-29B-30B-31B-34H-35H-36B-37B-38B-', 
            #  'ROWPITCH': 800, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4dragon-failed ; time: 09:05:31 10/05/11 Pacific Daylight Time}', 
            #  'CLASS': 'create2image4dragon', 
            #  'METHOD': 'create2image4dragon'}, 
            #  'WIDTH': 800, 
            #  'BUFFER': ...
            
            
        # -------------------------------------------------------------------------------------
        # RETURN THE WHOLE OR PART OF THE IMAGE INFO OF DRAGON          
        # -------------------------------------------------------------------------------------     
        dimension2xi = 0
        
        if task4getinfo2image is None:
            task4getinfo2image = {}
            task4getinfo2image ['DIMENSION'] = 2 
            task4getinfo2image ['DOMAIN'] = 'ALL'
            image2dragon = dragon2image4xi
            message2print = "{(Step 1C/33) using" + "; image2dragon-got-by-get2imageinfo4dragon: " + str(image2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'CODE2ERROR': 0, 
            #  'DOMAIN': 'ALL', 
            #  'CONTEXT': ...
            
        else:
            pass    

        keys2task4imageinfo = task4getinfo2image.keys()
        if 'DIMENSION' in keys2task4imageinfo:
            dimension2xi = task4getinfo2image['DIMENSION']
            if dimension2xi == 2:
                pass
            else:
                dimension2xi = 2    
        else:
            pass
            
        if 'DOMAIN' in keys2task4imageinfo:
            domain2xi = task4getinfo2image['DOMAIN']
            if domain2xi == 'ALL':
                pass
            else:
                domain2xi = 'ALL'

            keys2domain = dragon2image4xi.keys()
            for key2domain in keys2domain:
                task4getinfo2image[key2domain] = dragon2image4xi[key2domain]
                    
            message2print = "{(Step 1C/33) using" + "; task4getinfo2image-using-by-get2imageinfo4dragon: " + str(task4getinfo2image)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                          
        else:
            pass            
            
                    

        keys2task = task4getinfo2image.keys()
        for key2task in keys2task:
            try:
                message2print = "{(Step 2D2/33) using" + "; key2task-using-by-get2image4dragon: " + str(key2task)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                # WIDTH; DIMENSION; HEIGHT  
                image2dragon [key2task] = dragon2image4xi[key2task]
                
                message2print = "{(Step 2D2/33) set" + "; info-of-image2dragon-set-by-get2imageinfo4dragon to: " + str(image2dragon [key2task])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 800; 800  
            except:
                message2print = "{(Step 2Z/70) yielded" + "; key2task-yielded-by-get2image4dragon: " + str(key2task) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                               
                # DIMENSION;
                                   
                message2print = "{(Step 2Z2/70) using" + "; task4getinfo2image-using-by-get2image4dragon: " + str(task4getinfo2image)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                # 
                
                message2print = "{(Step 2Z3/70) using" + "; dragon2image4xi-using-by-get2image4dragon: " + str(dragon2image4xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #                           
    except:
        try:
            # -------------------------------------------------------------------------------------
            # INITIALIZE           
            # -------------------------------------------------------------------------------------           
            image2dragon = create2image4dragon()
            message2print = "{(Step 1F/33) initialized" + "; image2dragon-initialized-by-get2imageinfo4dragon to: " + str(image2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #
            if task4getinfo2image is None:
                image2dragon = dragon2image4xi
                message2print = "{(Step 1G/33) got" + "; image2dragon-got-by-get2imageinfo4dragon: " + str(image2dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
            else:
                keys2task = task4getinfo2image.keys()
                for key2task in keys2task:
                    try:
                        message2print = "{(Step HD2/33) using" + "; key2task-using-by-get2image4dragon: " + str(key2task)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)  
                        # 
                        image2dragon [key2task] = dragon2image4xi[key2task]
                    
                        message2print = "{(Step HD2/33) set" + "; info-of-image2dragon-set-by-get2imageinfo4dragon to: " + str(image2dragon [key2task])  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        #   
                    except:
                        message2print = "{(Step HZ/70) yielded" + "; key2task-yielded-by-get2image4dragon: " + str(key2task) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)                            
        except:
            image2dragon = {}
            message2print = "{(Step 1Z3/70) yielded" + "; image2dragon-yielded-by-get2image4dragon: " + str(image2dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
                        
    message2print = "{(Step 33/33) returning" + "; image2dragon-returning-by-get2imageinfo4dragon: " + str(image2dragon) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)
    # 2D: {'WIDTH': 800, 'HEIGHT': 800}
    # 3D: {'WIDTH': 800, 'DEPTH': 800, 'HEIGHT': 800}
    # 2D ALL: {'CODE2ERROR': 0, 
    #  'ROWPITCH': 800, 
    #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4dragon-failed ; time: 09:05:31 10/05/11 Pacific Daylight Time}', 
    #  'CLASS': 'create2image4dragon', 
    #  'METHOD': 'create2image4dragon'}, 
    #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
    #  'ORDER': 'CL_RGBA'}, 
    #  'BUFFER': ...
    
    # 3D ALL: {'PIPE': (<read-write PipeConnection, handle 520>, <read-write PipeConnection, handle 524>), 
    #  'ROWPITCH': 800, 
    #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create3image4dragon-failed ; time: 09:05:31 10/05/11 Pacific Daylight Time}', 
    #  'CLASS': 'create3image4dragon', 
    #  'METHOD': 'create3image4dragon'}, 
    #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
    #  'ORDER': 'CL_RGBA'}, 
    #  'BUFFER': ....
    #  'HEIGHT': 800, 
    #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004504400>, 
    #  'WIDTH': 800, 
    #  'DEPTH': 800, 
    #  'FLAGS': 0, 
    #  'CONTEXT': ...
    
    return image2dragon
            
#</function4xi> 
 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "get2supportedimageformats4dragon" usage = "
#
#   print('(Step 3A/33) OF DRAGON2IMAGE4XI5TEST - TRYING TO GET SUPPORTED IMAGE FORMATS FOR DRAGON', time4xi())  
#   try:
#       imageformats = get2supportedimageformats4dragon()                                                                       
#       print('(Step 3B/33) OF DRAGON2IMAGE4XI5TEST - GOT SUPPORTED IMAGE FORMATS FOR DRAGON: ', imageformats) 
#   except:
#       print('(Step 3Z/33) OF DRAGON2IMAGE4XI5TEST *** FAILED TO GET SUPPORTED IMAGE FORMATS FOR DRAGON', time4xi())
#            
#">

# -------------------------------------------------------------------------------------------------   
def get2supportedimageformats4dragon(task4supportedimageformats2dragon=None):  

    '''
    gets supported image formats for dragon
    '''
    
    global dragon2image4xi

    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33)           
    # -----------------------------------------------------------------------------------------          
    class2print = "get2supportedimageformats4dragon"
    method2print = "get2supportedimageformats4dragon"

    message2print = "{(Step 0A/33) started" + "; get2supportedimageformats4dragon-started: " + str(get2supportedimageformats4dragon)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print) 
    # <function get2supportedimageformats4dragon at 0x0000000003BBE8C8>                                                          
    
    message2print = "{(Step 0B/33) passed" + "; task4supportedimageformats2dragon-passed-to-get2supportedimageformats4dragon: " + str(task4supportedimageformats2dragon) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)   
    # None; {'DIMENSION': 3} 
    
    
    if task4supportedimageformats2dragon is None:
        task4supportedimageformats2dragon = {} 
        task4supportedimageformats2dragon['DIMENSION'] = 2                                                             # 2D image is the default
    
    dimension2xi = 2
    try:
        keys2supportedimageformats = task4supportedimageformats2dragon.keys()    
        if 'DIMENSION' in keys2supportedimageformats:
            dimension2xi = task4supportedimageformats2dragon['DIMENSION']
            if ((dimension2xi <=0) or (dimension2xi > 2)): 
                dimension2xi = 2
        else:
            dimension2xi = 2
    except:
        pass        
    
    message2print = "{(Step 1A/33) using" + "; dimension2xi-using-by-dimension2xi: " + str(dimension2xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
    # 2; 3
                        
    supportedimageformats2dragon = {} 
    try: 
        if dragon2image4xi is None:
            # -------------------------------------------------------------------------------------
            # INITIALIZE           
            # -------------------------------------------------------------------------------------           
            image2dragon = create2image4dragon()                                                                          # create dragon2image4xi
            message2print = "{(Step 1B/33) initialized" + "; image2dragon-initialized-by-get2supportedimageformats4dragon to: " + str(image2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #                
        else:
            pass

                        
        # -------------------------------------------------------------------------------------
        # SUPPORTED IMAGE FORMATS FOR DRAGON       
        # -------------------------------------------------------------------------------------     
        supportedimageformats2dragon = dragon2image4xi['FORMAT']
        message2print = "{(Step 1E/33) got" + "; supportedimageformats2dragon-got-by-get2supportedimageformats4dragon: " + str(supportedimageformats2dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)           
        # {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'} 
            
    except:
        try:
            # -------------------------------------------------------------------------------------
            # INITIALIZE           
            # -------------------------------------------------------------------------------------           
            image2dragon = create2image4dragon()
            message2print = "{(Step 1F1/33) initialized" + "; image2dragon-initialized-by-get2supportedimageformats4dragon to: " + str(image2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
     
            
            # -------------------------------------------------------------------------------------
            # SUPPORTED IMAGE FORMATS FOR DRAGON       
            # -------------------------------------------------------------------------------------     
            supportedimageformats2dragon = dragon2image4xi['FORMAT']
            message2print = "{(Step 1H/33) got" + "; supportedimageformats2dragon-got-by-get2supportedimageformats4dragon: " + str(supportedimageformats2dragon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)           
            #  
                                              
        except:
            message2print = "{(Step 1Z/70) yielded" + "; supportedimageformats2dragon-yielded-by-get2image4dragon: " + str(supportedimageformats2dragon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
                        
    message2print = "{(Step 33/33) returning" + "; supportedimageformats2dragon-returning-by-get2supportedimageformats4dragon: " + str(supportedimageformats2dragon) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # 2: {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'}
    # 3: {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'}
    
    return supportedimageformats2dragon
            
#</function4xi> 

 



# -------------------------------------------------------------------------------------------------
#<function4xi function = "wait2image4dragon" usage = "
#   print('=' *33)          
#   print('(Step 16A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO WAIT 2D IMAGE FOR DRAGON ', time4xi())   
#   print('=' *33)      
#   try:
#       response4wait2dragon = wait2image4dragon()
#       keys2response = response4wait2dragon.keys()
#       print('(Step 16B/700) OF DRAGON2IMAGE4XI5TEST - WAITED 2D IMAGE FOR DRAGON ', time4xi()) 
#       for key2response in sorted(keys2response):
#           print(key2response, ' : ', response4wait2dragon[key2response])
#           # ACK2RETURN  :   6C-
#           # BUFFER  :  {'CODE2ERROR': 0 ...
#           # CALLBACK  :  <function trace2image4dragon at 0x0000000004E93B48>
#           # CODE2ERROR  :  0
#           # COMMANDQUEUE  :  {'CODE2ERROR': 0, ...
#           # CONTEXT  :  {'COUNT2REFERENCE': 0 ...
#           # COUNT2REFERENCE  :  1
#           # DATA2CALLBACK  :  {'MESSAGE': '{(Step 1Z1/33) failed; create2image4dragon-failed ; time: 02:00:43 10/06/11 Pacific Daylight Time}', 'CLASS': 'create2image4dragon', 'METHOD': 'create2image4dragon'}
#           # DEPTH  :  0
#           # DEVICE  :  {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568', 'PROFILING2QUEUE': 'ACK',...
#           # EMPTY2QUEUE  :  True
#           # EVENT  :  {'CODE2ERROR': 0, 'DATA2CALLBACK':...
#           # FLAGS  :  0
#           # FORMAT  :  {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'}
#           # FULL2QUEUE  :  False
#           # FUNCTION2CALLBACK  :  <function trace2image4dragon at 0x0000000004E93B48>
#           # HEIGHT  :  800
#           # NACK2RETURN  :   
#           # PIPE  :  (<read-write PipeConnection, handle 556>, <read-write PipeConnection, handle 560>)
#           # PLATFORM  :  {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}
#           # POINTER  :  <mmap.mmap object at 0x0000000004351500>
#           # QUEUE  :  <queue4xi.Queue4Xi object at 0x00000000044E95C0>
#           # ROWPITCH  :  800
#           # SIZE2QUEUE  :  2147483646 
#   except:
#       print('(Step 16Z1/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO WAIT 2D IMAGE FOR DRAGON: ', wait2dragon)            
#">

# -------------------------------------------------------------------------------------------------   
def wait2image4dragon(task4wait2image=None):

    '''
    waits dragon2image4dragon
    '''
    
    global dragon2image4xi
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33)           
    # -----------------------------------------------------------------------------------------          
    class2print = "wait2image4dragon"
    method2print = "wait2image4dragon"

    message2print = "{(Step 1A1/33) started" + "; wait2image4dragon-started: " + str(wait2image4dragon)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    #  
    
    message2print = "{(Step 1A2/33) passed" + "; task4wait2image-passed-by-wait2image4dragon to: " + str(task4wait2image)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 
    
    try: 
        if dragon2image4xi is None:
            # -------------------------------------------------------------------------------------
            # INITIALIZE - KEEP SYSTEM RUNNING           
            # -------------------------------------------------------------------------------------           
            dragon2image4xi = create2image4dragon()
            
            message2print = "{(Step 1B/33) created" + "; dragon2image4xi-created-by-wait2image4dragon: " + str(dragon2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #
                                                
            # -------------------------------------------------------------------------------------
            # WAIT         
            # -------------------------------------------------------------------------------------             
            queue2image4dragon = None
            try:
                queue2image4dragon = dragon2image4xi['QUEUE']
                message2print = "{(Step 1C/33) got" + "; queue2image4dragon-got-by-wait2image4dragon to: " + str(queue2image4dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #
            except:     
                message2print = "{(Step 1Z1/70) yielded" + "; dragon2xi-yielded-by-wait2image4dragon: " + str(dragon2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
            
            size2image4dragon = 0
            try:                    
                size2image4dragon = queue2image4dragon.qsize()
                message2print = "{(Step 1D/33) got" + "; size2image4dragon-got-by-wait2image4dragon to: " + str(size2image4dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                #
                               
                
            except:     
                message2print = "{(Step 1Z2/70) yielded" + "; dragon2xi-yielded-by-wait2image4dragon: " + str(dragon2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
             
            empty2image4dragon = True       
            try:                    
                empty2image4dragon= queue2image4dragon.empty()
                message2print = "{(Step 1E/33) got" + "; empty2image4dragon-got-by-wait2image4dragon to: " + str(empty2image4dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                #
            except:     
                message2print = "{(Step 1Z3/70) yielded" + "; empty2image4dragon-yielded-by-wait2image4dragon: " + str(empty2image4dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
             
            full2image4dragon = False       
            try:                                
                full2image4dragon = queue2image4dragon.full()
                message2print = "{(Step 1F/33) got" + "; full2image4dragon-got-by-wait2image4dragon to: " + str(full2image4dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                #                
            except:     
                message2print = "{(Step 1Z4/70) yielded" + "; full2image4dragon-yielded-by-wait2image4dragon: " + str(full2image4dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)   
                            
            message2print = "{(Step 1G/33) finished" + "; dragon2image4xi-finished-by-wait2image4dragon to: " + str(dragon2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #
                            
        else:
            # -------------------------------------------------------------------------------------
            # USE THE EXISTING IMAGE FOR DRAGON          
            # -------------------------------------------------------------------------------------     
            
            message2print = "{(Step 1H/33) used" + "; dragon2image4xi-used-by-wait2image4dragon: " + str(dragon2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
            
            # -------------------------------------------------------------------------------------
            # WAIT         
            # -------------------------------------------------------------------------------------             
            queue2image4dragon = None
            try:
                queue2image4dragon = dragon2image4xi['QUEUE']
                message2print = "{(Step 1I/33) got" + "; queue2image4dragon-got-by-wait2image4dragon to: " + str(queue2image4dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
            except:     
                message2print = "{(Step 1Z5/70) yielded" + "; dragon2xi-yielded-by-wait2image4dragon: " + str(dragon2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
            if task4wait2image is None:
                pass
            else:
                # -------------------------------------------------------------------------------------
                # IMAGE GIVEN AS PARAMETER          
                # -------------------------------------------------------------------------------------               
                message2print = "{(Step 1J1/33) passed" + "; task4wait2image-passed-by-wait2image4dragon to: " + str(task4wait2image)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                queue2task = {}    
                try:
                    queue2task = task4wait2image ['QUEUE']
                    message2print = "{(Step 1J2/33) set" + "; queue2task-set-by-wait2image4dragon to: " + str(queue2task)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                except:     
                    message2print = "{(Step 1Z61/70) yielded" + "; queue2task-yielded-by-wait2image4dragon: " + str(queue2task) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                
                try:                        
                    if queue2task == queue2image4dragon:
                        # -------------------------------------------------------------------------------------
                        # ACCEPTED - GIVEN QUEUE IS THE SAME          
                        # -------------------------------------------------------------------------------------                     
                        message2print = "{(Step 1K/33) accepted" + "; queue2task-accepted-by-wait2image4dragon: " + str(queue2task) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        # 
                    else:
                        try:
                            message2print = "{(Step 1L/33) warned" + "; queue2task-warned-by-wait2image4dragon: " + " ** " + str(queue2task)  + " ** " + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print) 
                            # 
                        
                            message2print = "{(Step 1M/33) warned" + "; queue2image4dragon-warned-by-wait2image4dragon: " + " ** " + str(queue2image4dragon)  + " ** " + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)    
                            # 
                        except:
                            pass
                                                     
                except:     
                    message2print = "{(Step 1Z62/70) yielded" + "; queue2task-yielded-by-wait2image4dragon: " + str(queue2task) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)                   
            
            size2queue4dragon = 0        
            try:                    
                size2queue4dragon = queue2image4dragon.qsize()
                message2print = "{(Step 1N1/33) got" + "; size2queue4dragon-got-by-wait2image4dragon to: " + str(size2queue4dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # 
                
                dragon2image4xi ['SIZE2QUEUE'] = size2queue4dragon
                message2print = "{(Step 1N2/33) set" + "; size2queue4image-got-by-wait2image4dragon to: " + str(dragon2image4xi ['SIZE2QUEUE'])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                  
                # 
            except:     
                message2print = "{(Step 1Z7/70) yielded" + "; size2queue4dragon-yielded-by-wait2image4dragon: " + str(size2queue4dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
            
            empty2queue4dragon = False
            try:                    
                empty2queue4dragon = queue2image4dragon.empty()
                message2print = "{(Step 1O1/33) got" + "; empty2queue4dragon-got-by-wait2image4dragon to: " + str(empty2queue4dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # 
                
                dragon2image4xi ['EMPTY2QUEUE'] = empty2queue4dragon
                message2print = "{(Step 1O2/33) set" + "; empty2queue4image-got-by-wait2image4dragon to: " + str(dragon2image4xi ['EMPTY2QUEUE'])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #               
            except:     
                message2print = "{(Step 1Z8/70) yielded" + "; empty2queue4dragon-yielded-by-wait2image4dragon: " + str(empty2queue4dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
            
            full2queue4dragon = False
            try:                                
                full2queue4dragon = queue2image4dragon.full()
                message2print = "{(Step 1P1/33) got" + "; full2queue4dragon-got-by-wait2image4dragon to: " + str(full2queue4dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # 
                 
                dragon2image4xi ['FULL2QUEUE'] = full2queue4dragon
                message2print = "{(Step 1P2/33) set" + "; full2queue4image-got-by-wait2image4dragon to: " + str(dragon2image4xi ['FULL2QUEUE'])  + "; time: " + time4xi() + "}" 
                print4xi(class2print, method2print, message2print)
                #                          
            except:     
                message2print = "{(Step 1Z9/70) yielded" + "; full2queue4dragon-yielded-by-wait2image4dragon: " + str(full2queue4dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
            
            state2queue4dragon = None
            try:                                
                state2queue4dragon = queue2image4dragon.__getstate__()
                message2print = "{(Step 1P3/33) got" + "; state2queue4dragon-got-by-wait2image4dragon to: " + str(state2queue4dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # 
                 
                dragon2image4xi ['STATE2QUEUE'] = state2queue4dragon
                message2print = "{(Step 1P4/33) set" + "; state2queue4image-set-by-wait2image4dragon to: " + str(dragon2image4xi ['STATE2QUEUE'])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #           
            except:     
                message2print = "{(Step 1Z10/70) yielded" + "; state2queue4dragon-yielded-by-wait2image4dragon: " + str(state2queue4dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)               
                
                                                            
            message2print = "{(Step 1Q/33) finished" + "; dragon2image4xi-finished-by-wait2image4dragon: " + str(dragon2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
    except:
        try:
            # -------------------------------------------------------------------------------------
            # INITIALIZE           
            # -------------------------------------------------------------------------------------           
            dragon2image4dragon = create2image4dragon()
            message2print = "{(Step 1R/33) got" + "; dragon2image4xi-got-by-wait2image4dragon to: " + str(dragon2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #  
            
            # -------------------------------------------------------------------------------------
            # WAIT         
            # -------------------------------------------------------------------------------------             
            queue2image4dragon = None
            try:
                queue2image4dragon = dragon2image4xi['QUEUE']
                message2print = "{(Step 1S/33) got" + "; queue2image4dragon-got-by-wait2image4dragon to: " + str(queue2image4dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #
            except:     
                message2print = "{(Step 1Z10/70) yielded" + "; queue2image4dragon-yielded-by-wait2image4dragon: " + str(queue2image4dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
            if task4wait2image is None:
                pass
            else:
                # -------------------------------------------------------------------------------------
                # IMAGE QUEUE GIVEN AS PARAMETER          
                # -------------------------------------------------------------------------------------               
                queue2task = None    
                try:
                    queue2task = task4wait2image ['QUEUE']
                    message2print = "{(Step 1T/33) passed" + "; queue2task-passed-by-wait2image4dragon to: " + str(queue2task)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    #
                    
                    if queue2task == queue2image4dragon:
                        # -------------------------------------------------------------------------------------
                        # ACCEPTED - GIVEN QUEUE IS THE SAME          
                        # -------------------------------------------------------------------------------------                     
                        message2print = "{(Step 1U/33) accepted" + "; queue2task-accepted-by-wait2image4dragon: " + str(queue2task) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        # 
                    else:
                        message2print = "{(Step 1X/33) warned" + "; queue2task-warned-by-wait2image4dragon: " + " ** " + str(queue2task)  + " ** " + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print) 
                        # 
                        
                        message2print = "{(Step 1Y1/33) warned" + "; queue2image4dragon-warned-by-wait2image4dragon: " + " ** " + str(queue2image4dragon)  + " ** " + "; time: " + time4xi() + "}" 
                        print4xi(class2print, method2print, message2print)    
                        #                      
                except:     
                    message2print = "{(Step 1Z11/70) yielded" + "; queue2task-yielded-by-wait2image4dragon: " + str(queue2task) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 
            
            size2image4dragon = 0                             
            try:                    
                size2image4dragon = queue2image4dragon.qsize()
                message2print = "{(Step 1Y2/33) got" + "; size2image4dragon-got-by-wait2image4dragon to: " + str(size2image4dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                #
            except:     
                message2print = "{(Step 1Z12/70) yielded" + "; size2image4dragon-yielded-by-wait2image4dragon: " + str(size2image4dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
             
            empty2image4dragon = True          
            try:                    
                empty2image4dragon = queue2image4dragon.empty()
                message2print = "{(Step 1Y3/33) got" + "; empty2image4dragon-got-by-wait2image4dragon to: " + str(empty2image4dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                #
            except:     
                message2print = "{(Step 1Z13/70) yielded" + "; dragon2xi-yielded-by-wait2image4dragon: " + str(dragon2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
             
            full2image4dragon = False          
            try:                                
                full2image4dragon= queue2image4dragon.full()
                message2print = "{(Step 1Y4/33) got" + "; full2image4dragon-got-by-wait2image4dragon to: " + str(full2image4dragon)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                #                
            except:     
                message2print = "{(Step 1Z14/70) yielded" + "; full2image4dragon-yielded-by-wait2image4dragon: " + str(full2image4dragon) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                                                        
            message2print = "{(Step 1Y5/33) finished" + "; dragon2image4xi-finished-by-wait2image4dragon: " + str(dragon2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                    
        except:
            dragon2image4dragon= {}
            message2print = "{(Step 1Z15/70) yielded" + "; dragon2image4xi-yielded-by-wait2image4dragon: " + str(dragon2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
                        
    message2print = "{(Step 33/33) returning" + "; dragon2image4xi-returning-by-wait2image4dragon: " + str(dragon2image4xi) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # {'SIZE2QUEUE': 2147483646, 
    #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
    #  'ORDER': 'CL_RGBA'}, 
    #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004420E10>, 
    #  'WIDTH': 800, 
    #  'CONTEXT': ...
    
    return dragon2image4xi
    
   
#</function4xi> 
                                               
# =================================================================================================
#<class4xi class = "DRAGON2Image4Xi" base = "object" child = "None"
#   synopsis = "offers methods for DRAGON 2D images" usage = "2D Image4Xi of DRAGON"
#>

# =================================================================================================
class DRAGON2Image4Xi(object): 
    ''' 
    DRAGON2Image4Xi offers methods for DRAGON 2D images
    '''            

    #                                        
    # ---------------------------------------------------------------------------------------------
    # sharable class variables
    # ---------------------------------------------------------------------------------------------      
    #<share4xi share = " 
    classvariable2dragon = cpu_corecount4xi()
    #"
    #/>
        
    # ---------------------------------------------------------------------------------------------
    # Public API of this class
    # ---------------------------------------------------------------------------------------------    
    #<expose4xi expose = "
    _exposed_ = (
            'create2image',                    # ACK 5.31   p76    
            'copy2image',                      # ACK 5.31   p88
            'copy2imagetobuffer',              # ACK 5.3.4  p90
            'copy2buffertoimage',              # ACK 5.3.4  p92
            'read2image',                      # ACK 5.3.3  p84
            'write2image',                     # ACK 5.3.3  p84            
            'map2image',                       # ACK 5.3.5  p94            
            'get2imageinfo',                   # ACK 5.3.6  p97
            'get2supportedimageformats',       # ACK 5.3.2  p82 nb: a separate method to get format whereas using get2imageinfo you can access any element by key such as 'FORMAT'   
            'retain2image',                    # ACK 5.4.1  p99 nb: for compability to OpenCL - you can get any element of dictionary by get2imageinfo
            'release2image',                   # ACK 5.4.1  p99
            'trace2image',                     # ACK modernized 
            'set2imagecallback',               # ACK modernized
            'set2image',                       # ACK modernized nb: with memorymap features you can process 2D images as files!                    
            'wait2image',                      # ACK modernized nb: checks Queue object and thereby generates delay   
            'sense',                           # ACK modernized  
            'reason',                          # ACK modernized  
            'ace')                             # ACK modernized           
    #"
    #/>
 
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "init-of-DRAGON2Image4Xi" usage = "
    #   print('(Step 5A/33) OF DRAGON2IMAGE4XI5TEST - CREATING IMAGE4DRAGON OBJECT ', time4xi())     
    #   try:
    #       imageobject2dragon = DRAGON2Image4Xi()
    #       print('(Step 5B/33) OF DRAGON2IMAGE4XI5TEST - CREATED IMAGE4DRAGON OBJECT: ', imageobject2dragon)                                  
    #   except:
    #       print('(Step 5Z/33) OF DRAGON2IMAGE4XI5TEST *** FAILED TO CREATE IMAGE4DRAGON OBJECT')
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def __init__(self, task4imagecreate2dragon = None):
        '''
        _init__ method initializes DRAGON2Image4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------
                    
        class2print = "DRAGON2Image4Xi"
        method2print = "__init__"
                     
        message2print = "{(Step 1A/33) started" + "; init-of-DRAGON2Image4Xi-started: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # 23:41:37 09/06/11 Pacific Daylight Time 

        message2print = "{(Step 1B/33) borning" + "; dragon2image2xi-of-DRAGON2Image4Xi-borning: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None 
                
        message2print = "{(Step 1C/33) inheriting" + "; object-inheriting-by-dragon2image2xi-of-DRAGON2Image4Xi: " + str(object)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # <class 'object'>
        
        message2print = "{(Step 1D/33) sharing" + "; classvariable2dragon-sharing-by-dragon2image2xi-of-DRAGON2Image4Xi: " + str(DRAGON2Image4Xi.classvariable2dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 4             
                                
        message2print = "{(Step 1E/33) passed" + "; task4imagecreate2dragon-passed-to-init-of-DRAGON2Image4Xi: " + str(task4imagecreate2dragon) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None                            
        
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)             
        # -----------------------------------------------------------------------------------------
        
        message2print = "{(Step 2A/33) passing" + "; task4imagecreate2dragon-passing-to-create2image4dragon-by-init-of-DRAGON2Image4Xi: " + str(task4imagecreate2dragon) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None  
        
        dragon2image2xi = create2image4dragon(task4imagecreate2dragon)
        
        message2print = "{(Step 2B/33) performed" + "; create2image4dragon-performed-by-init-of-DRAGON2Image4Xi: " + str(create2image4dragon) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # <function create2image4dragon at 0x0000000003A9E7C8>
                        
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)             
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 3A/33) activated" + "; dragon2image2xi-of-DRAGON2Image4Xi-activated: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # { DRAGON2Image4Xi : __init__ ; (Step 3A/33) activated; dragon2image2xi-of-DRAGON2Image4Xi-activated: ; time: 00:14:05 09/07/11 Pacific Daylight Time} 
                
        message2print = "{(Step 3B/33) inherited" + "; object-inherited-by-dragon2image2xi-of-DRAGON2Image4Xi: " + str(object)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # <class 'object'> 
        
        message2print = "{(Step 3C/33) shared" + "; classvariable2dragon-shared-by-dragon2image2xi-of-DRAGON2Image4Xi: " + str(DRAGON2Image4Xi.classvariable2dragon)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 4           
       
        message2print = "{(Step 3D/33) returning" + "; init-of-DRAGON2Image4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # 00:14:05 09/07/11 Pacific Daylight Time
        

    #</methodize4xi>
    

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "create2image-of-DRAGON2Image4Xi" usage = "
    #   print('(Step 23A/33) OF DRAGON2IMAGE4XI5TEST - TRYING TO CREATE DRAGON IMAGE', time4xi()) 
    #   try:
    #       xi2dragon = DRAGON2Image4Xi()                            
    #       image2dragon = xi2dragon.create2image()                                  
    #       print('(Step 23B/33) OF DRAGON2IMAGE4XI5TEST - CREATED DRAGON IMAGE: ', image2dragon)
    #   except:
    #       print('(Step 23Z/33) OF DRAGON2IMAGE4XI5TEST *** FAILED TO CREATE DRAGON IMAGE ***', time4xi())                  
    #                     
    #">   
    
    # ---------------------------------------------------------------------------------------------   
    def create2image(self, task4image2create = None):
        '''
        create2image method creates 2D image objects of DRAGON2Image4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "DRAGON2Image4Xi"
        method2print = "create2image"
                     
        message2print = "{(Step 1A/33) started" + "; create2image-of-DRAGON2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 00:14:05 09/07/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4image2create-passed-to-create-of-DRAGON2Image4Xi: " + str(task4image2create) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                      
        # None 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4image2create-passing-to-create2image4dragon-by-create2image-of-DRAGON2Image4Xi: " + str(task4image2create) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # None      
  
        dragon2image2xi = create2image4dragon(task4image2create)

        message2print = "{(Step 2B/33) performed" + "; create2image4dragon-performed-by-create2image-of-DRAGON2Image4Xi: " + str(create2image4dragon) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function create2image4dragon at 0x00000000039DF7C8>
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
        message2print = "{(Step 3/33) returning" + "; dragon2image2xi-returning-by-create2image-of-DRAGON2Image4Xi: " + str(dragon2image2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          # 
        # {'CODE2ERROR': 0,
        #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-14B-15B-16B-17B-17D-18B-18C-21A1-22B1-22C1-23B-24B-25B-26B-27B-28B-29B-30B-31B-34H-35H-36B-37B-38B-',
        #  'ROWPITCH': 800,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4dragon-failed ; time: 00:53:55 10/05/11 Pacific Daylight Time}',
        #  'CLASS': 'create2image4dragon',
        #  'METHOD': 'create2image4dragon'},
        #  'WIDTH': 800 ...

        return dragon2image2xi
                                
    #                        
    #</methodize4xi>
    

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "set2image-of-DRAGON2Image4Xi" usage = "
    #   print('(Step 1A/33) OF DRAGON2IMAGE4XI - TRYING TO SET DRAGON IMAGE')
    #   try:
    #       cc2xi = DRAGON2Image4Xi()                            
    #       image2dragon = cc2xi.set2image()                                  
    #       print('(Step 1B/33) OF DRAGON4XI5TEST - SET DRAGON IMAGE: ', image2dragon)                                                  
    #   except:
    #       print('(Step 1Z/33) OF DRAGON4XI5TEST *** FAILED TO SET DRAGON IMAGE ***')  
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def set2image(self, task4set2image = None):
        '''
        sets 2D image DRAGON objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "DRAGON2Image4Xi"
        method2print = "set2image"
                     
        message2print = "{(Step 1A/33) started" + "; set2image-of-DRAGON2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:02:22 09/07/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4set2image-passed-to-set2image-of-DRAGON2Image4Xi: " + str(task4set2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # {'CODE2ERROR': 0,
        #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004191E10>,
        #  'CONTEXT': {'COUNT2REFERENCE': 0, ...
        
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4set2image-passing-to-set2image4dragon-by-set2image-of-DRAGON2Image4Xi: " + str(task4set2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # {'CODE2ERROR': 0, 
        #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004191E10>, 
        #  'CONTEXT': {'COUNT2REFERENCE': 0 ...

        dragon2image2xi = set2image4dragon(task4set2image)

        message2print = "{(Step 2B/33) performed" + "; set2image4dragon-performed-by-set2image-of-DRAGON2Image4Xi: " + str(set2image4dragon) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function set2image4dragon at 0x0000000003AB36C8> 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; dragon2image2xi-returning-by-set2image-of-DRAGON2Image4Xi: " + str(dragon2image2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0,
        #  'ACK2RETURN': ' 23B-24B-25B-26B-27B-28B-',
        #  'ROWPITCH': 800,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4dragon-failed ; time: 01:02:22 09/07/11 Pacific Daylight Time}', 'CLASS': 'create2image4dragon', 'METHOD': 'create2image4dragon'},
        #  'WIDTH': 800,
        #  'BUFFER': {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4dragon-failed ; time: 01:02:22 09/07/11 Pacific Daylight Time}',
        #  'CLASS': 'create2buffer4dragon',
        #  'METHOD': 'create2buffer4dragon'},
        #  'FUNCTION2CALLBACK': <function trace2buffer4dragon at 0x0000000003AA81C8>,
        #  'POINTER2BUFFER': <mmap.mmap object at 0x0000000003CC92F0>,
        #  'FLAGS': 0,
        #  'CONTEXT': {'COUNT2REFERENCE': 0 ...
                    
                
        return dragon2image2xi
                                
                            
    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "get2imageinfo-of-DRAGON2Image4Xi" usage = "
    #   print('(Step 1A/33) OF DRAGON2IMAGE4XI - TRYING TO GET DRAGON 2D IMAGE')
    #   try:
    #       cc2xi = DRAGON2Image4Xi()                            
    #       image2dragon = cc2xi.get2imageinfo()                                  
    #       print('(Step 1B/33) OF DRAGON4XI5TEST - GOT DRAGON IMAGE: ', image2dragon)                                                  
    #   except:
    #       print('(Step 1Z/33) OF DRAGON4XI5TEST *** FAILED TO GET DRAGON 2D IMAGE ***')   
    # ">
    
    # ---------------------------------------------------------------------------------------------   
    def get2imageinfo(self, task4getinfo2image=None):
        '''
        get2imageinfo method gets 2D objects of DRAGON2Image4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "DRAGON2Image4Xi"
        method2print = "get2imageinfo"
                     
        message2print = "{(Step 1A/33) started" + "; get2imageinfo-of-DRAGON2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 00:14:05 09/07/11 Pacific Daylight Time 

        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # ----------------------------------------------------------------------------------------- 
        dragon2image2xi = get2imageinfo4dragon(task4getinfo2image)
        
        message2print = "{(Step 2A/33) returning" + "; dragon2image2xi-returning-by-get2imageinfo-of-DRAGON2Image4Xi: " + str(dragon2image2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0,
        #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-14B-15B-16B-17B-17D-18B-18C-21A1-22B1-22C1-23B-24B-25B-26B-27B-28B-29B-30B-31B-34H-35H-36B-37B-38B-',
        #  'ROWPITCH': 800,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4dragon-failed ; time: 00:14:05 09/07/11 Pacific Daylight Time}', 'CLASS': 'create2image4dragon', 'METHOD': 'create2image4dragon'},
        #  'WIDTH': 800,
        #  'BUFFER': {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4dragon-failed ; time: 00:14:05 09/07/11 Pacific Daylight Time}',
        #  'CLASS': 'create2buffer4dragon',
        #  'METHOD': 'create2buffer4dragon'},
        #  'FUNCTION2CALLBACK': <function trace2buffer4dragon at 0x00000000039D8348>,
        #  'POINTER2BUFFER': <mmap.mmap object at 0x00000000037D32F0>,
        #  'FLAGS': 0,
        #  'CONTEXT': {'COUNT2REFERENCE': 0, ...
        # or when only two elemets required
        # {'WIDTH': 800, 'HEIGHT': 800}
        
        return dragon2image2xi
                            
    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "get2supportedimageformats-of-DRAGON2Image4Xi" usage = "
    #  print('(Step 6B1/33) OF DRAGON2IMAGE4XI5TEST - TRYING TO GET SUPPORTED IMAGE FORMATS FOR DRAGON')
    #  try:
    #      cc2xi = DRAGON2Image4Xi()                            
    #      supportedimageformats2dragon = cc2xi.get2supportedimageformats()                                  
    #      print('(Step 6B2/33) OF DRAGON4XI5TEST - GOT SUPPORTED IMAGE FORMATS FOR DRAGON: ', supportedimageformats2dragon)
    #                                                   
    #  except:
    #      print('(Step 2Z/33) OF DRAGON4XI5TEST *** FAILED TO GET SUPPORTED IMAGE FORMATS FOR DRAGON***')     
    # ">
    
    # ---------------------------------------------------------------------------------------------   
    def get2supportedimageformats(self):
        '''
        get2supportedimageformats method gets supported imageformats by objects of DRAGON2Image4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "DRAGON2Image4Xi"
        method2print = "get2supportedimageformats"
                     
        message2print = "{(Step 1A/33) started" + "; get2supportedimageformats-of-DRAGON2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:43:00 09/07/11 Pacific Daylight Time

        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # ----------------------------------------------------------------------------------------- 
        supportedimageformats2dragon = get2supportedimageformats4dragon()
        
        message2print = "{(Step 2A/33) returning" + "; supportedimageformats2dragon-returning-by-get2imageinfo-of-DRAGON2Image4Xi: " + str(supportedimageformats2dragon) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'} 
        
        return supportedimageformats2dragon
                            
    #</methodize4xi>    
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "read2image-of-DRAGON2Image4Xi" usage = "
    #   print('(Step 127A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL AND COMMAND WITH READ IMAGE OBJECT ', time4xi()) 
    #   response4read2dragon = {}
    #   try:
    #       xi2dragon = DRAGON2Image4Xi()    
    #       response4read2dragon = xi2dragon.read2image()
    #       print('(Step 127B/700) OF DRAGON2IMAGE4XI5TEST - ESTABLISHED CONTROL AND COMMAND WITH READ IMAGE OBJECT, RESPONSE4READ2DRAGON: ', response4read2dragon) 
    #   except:
    #       print('(Step 127Z1/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL AND COMMAND WITH READ IMAGE OBJECT ', time4xi())
    #   ... continue as with read2image4dragon function
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def read2image(self, task4read2image = None):
        '''
        reads DRAGON2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "DRAGON2Image4Xi"
        method2print = "read2image"
                     
        message2print = "{(Step 1A/33) started" + "; read2image-of-DRAGON2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 00:20:02 10/05/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4read2image-passed-to-read2image-of-DRAGON2Image4Xi: " + str(task4read2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4read2image-passing-to-read2image4dragon-by-read2image-of-DRAGON2Image4Xi: " + str(task4read2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None
  
        response4readimage2dragon = read2image4dragon(task4read2image)

        message2print = "{(Step 2B/33) performed" + "; read2image4dragon-performed-by-read2image-of-DRAGON2Image4Xi: " + str(read2image4dragon) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function read2image4dragon at 0x000000000385DA48> 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4readimage2dragon-returning-by-read2image-of-DRAGON2Image4Xi: " + str(response4readimage2dragon) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'QUEUE': <queue4xi.Queue4Xi object at 0x000000000417DD68>,
        #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 936>,
        #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 932>}; time: 00:20:02 10/05/11 Pacific Daylight Time}
  
        return response4readimage2dragon
                                                           
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "map2image-of-DRAGON2Image4Xi" usage = "
    #   print('(Step 127A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL AND COMMAND WITH MAP IMAGE OBJECT ', time4xi()) 
    #   xi2dragon = DRAGON2Image4Xi() 
    #   ...
    #   response4map2dragon = {}
    #   try:
    #       response4map2dragon = xi2dragon.map2image()
    #       print('(Step 127B/700) OF DRAGON2IMAGE4XI5TEST - ESTABLISHED CONTROL AND COMMAND WITH MAP IMAGE OBJECT, RESPONSE4READ2DRAGON: ', response4map2dragon) 
    #   except:
    #       print('(Step 127Z1/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL AND COMMAND WITH MAP IMAGE OBJECT ', time4xi())
    #   ... continue as with map2image4dragon function
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def map2image(self, task4map2image = None):
        '''
        maps DRAGON2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "DRAGON2Image4Xi"
        method2print = "map2image"
                     
        message2print = "{(Step 1A/33) started" + "; map2image-of-DRAGON2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:28:44 10/05/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4map2image-passed-to-map2image-of-DRAGON2Image4Xi: " + str(task4map2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4map2image-passing-to-map2image4dragon-by-map2image-of-DRAGON2Image4Xi: " + str(task4map2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None
  
        response4mapimage2dragon = map2image4dragon(task4map2image)

        message2print = "{(Step 2B/33) performed" + "; map2image4dragon-performed-by-map2image-of-DRAGON2Image4Xi: " + str(map2image4dragon) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function map2image4dragon at 0x000000000390BAC8> 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4mapimage2dragon-returning-by-map2image-of-DRAGON2Image4Xi: " + str(response4mapimage2dragon) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'QUEUE': <queue4xi.Queue4Xi object at 0x00000000050E9C18>,
        #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 1100>,
        #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 1096>}; time: 01:28:44 10/05/11 Pacific Daylight Time}

        
        return response4mapimage2dragon
                                                           
    #</methodize4xi>
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "release2image-of-DRAGON2Image4Xi" usage = "
    #
    # ---------------------------------------------------------------------------------------------
    # RELEASE 2D IMAGE BY DECREMENTING REFERENCE COUNT WITH 1             
    # ---------------------------------------------------------------------------------------------    
    #   task4release2image  = {} 
    #   task4release2image  ['COUNT2REFERENCE'] = 1 
    #   response4release2image = {}
    #   print('(Step 133A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO RELEASE 2D IMAGE OF DRAGON DEVICE ', time4xi())    
    # 
    #   try:                               
    #       response4release2Dimage = xi2dragon.release2image(task4release2image)                                                                                             
    #       print('(Step 133B/700) OF DRAGON2IMAGE4XI5TEST - RELEASED 2D IMAGE OF DRAGON DEVICE ', time4xi())                           
    #       keys2response = response4release2Dimage.keys()
    #       for key2response in sorted(keys2response):
    #           print(key2response, ' : ', response4release2Dimage[key2response])
    #           #
    #   except:
    #       print('(Step 133Z1/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO RELEASE IMAGE OF DRAGON DEVICE ', time4xi())  
    #   
    #   task4getinfo2image = {}
    #   task4getinfo2image ['COUNT2REFERENCE'] = 0
    #   response4getinfo2image = {}
    #   print('(Step 133C/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO GET SELECTED 2D IMAGE INFO OF DRAGON DEVICE ', time4xi())     
    #   # 
    #   try:                               
    #       response4getinfo2Dimage = xi2dragon.get2imageinfo (task4getinfo2image)                                                                                             
    #       keys2response = response4getinfo2Dimage.keys()
    #       print('(Step 133D/700) OF DRAGON2IMAGE4XI5TEST - GOT SELECTED 2D IMAGE INFO OF DRAGON DEVICE ', time4xi())   
    #       for key2response in sorted(keys2response):
    #           print(key2response, ' : ', response4getinfo2Dimage[key2response])
    #           # 
    #   except:
    #       print('(Step 133Z2/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO GET SELECTED 2D IMAGE INFO OF DRAGON DEVICE ', time4xi())  
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def release2image(self, task4release2image = None):
        '''
        releases DRAGON2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "DRAGON2Image4Xi"
        method2print = "release2image"
                     
        message2print = "{(Step 1A/33) started" + "; release2image-of-DRAGON2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
            
        message2print = "{(Step 1B/33) passed" + "; task4release2image-passed-to-release2image-of-DRAGON2Image4Xi: " + str(task4release2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4release2image-passing-to-release2image4dragon-by-release2image-of-DRAGON2Image4Xi: " + str(task4release2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # 
  
        dragon2image2xi = release2image4dragon(task4release2image)

        message2print = "{(Step 2B/33) performed" + "; release2image4dragon-performed-by-release2image-of-DRAGON2Image4Xi: " + str(release2image4dragon) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; dragon2image2xi-returning-by-release2image-of-DRAGON2Image4Xi: " + str(dragon2image2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        #                          
                
        return dragon2image2xi
                                                           
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "retain2image-of-DRAGON2Image4Xi" usage = "
    #
    # ---------------------------------------------------------------------------------------------
    # RETAIN 2D IMAGE BY DECREASING REFERENCE WITH 1             
    # ---------------------------------------------------------------------------------------------    
    #   task4retain2image  = {}
    #   task4retain2image  ['COUNT2REFERENCE'] = 1 
    #   response4retain2Dimage = {}
    #   print('(Step 134A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO RETAIN 2D IMAGE OF DRAGON DEVICE ', time4xi())   
    #   try:                               
    #       response4retain2Dimage = xi2dragon.retain2image(task4retain2image)                                                                                                 
    #       print('(Step 134B/700) OF DRAGON2IMAGE4XI5TEST - RESPONSE4RETAIN2DIMAGE : ', time4xi())                         
    #       keys2response = response4retain2Dimage.keys()
    #       for key2response in sorted(keys2response):
    #           print(key2response, ' : ', response4retain2Dimage[key2response])
    #           # 
    #   except:
    #       print('(Step 134Z1/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO RETAIN 2D IMAGE OF DRAGON DEVICE ', time4xi())
    #   task4getinfo2image = {}
    #   task4getinfo2image ['COUNT2REFERENCE'] = 0
    #   response4getinfo2image = {}
    #   print('(Step 134C/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO GET SELECTED 2D IMAGE INFO OF DRAGON DEVICE ', time4xi())     
    #   # 
    #   try:                               
    #       response4getinfo2Dimage = xi2dragon.get2imageinfo (task4getinfo2image)                                                                                             
    #       keys2response = response4getinfo2Dimage.keys()
    #       print('(Step 134D/700) OF DRAGON2IMAGE4XI5TEST - GOT SELECTED 2D IMAGE INFO OF DRAGON DEVICE ', time4xi())   
    #       for key2response in sorted(keys2response):
    #           print(key2response, ' : ', response4getinfo2Dimage[key2response])
    #           # 
    #   except:
    #       print('(Step 134Z2/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO GET SELECTED 2D IMAGE INFO OF DRAGON DEVICE ', time4xi())  
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def retain2image(self, task4retain2image = None):
        '''
        retains DRAGON2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "DRAGON2Image4Xi"
        method2print = "retain2image"
                     
        message2print = "{(Step 1A/33) started" + "; retain2image-of-DRAGON2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 00:14:05 09/07/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4retain-passed-to-retain2image-of-DRAGON2Image4Xi: " + str(task4retain2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # {'COUNT2REFERENCE': 1}
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4retain-passing-to-retain2image4dragon-by-retain-of-DRAGON2Image4Xi: " + str(task4retain2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # {'COUNT2REFERENCE': 1}
  
        dragon2image2xi = retain2image4dragon(task4retain2image)

        message2print = "{(Step 2B/33) performed" + "; retain2image4dragon-performed-by-retain2image-of-DRAGON2Image4Xi: " + str(retain2image4dragon) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function retain2image4dragon at 0x00000000039DF948>
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; dragon2image2xi-returning-by-retain2image-of-DRAGON2Image4Xi: " + str(dragon2image2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0, 
        #  'ACK2RETURN': ' 1D-', 
        #  'ROWPITCH': 800, 
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4dragon-failed ; time: 00:14:05 09/07/11 Pacific Daylight Time}', 
        #  'CLASS': 'create2image4dragon', 
        #  'METHOD': 'create2image4dragon'}, 
        #  'WIDTH': 800, 
        #  'BUFFER': {'CODE2ERROR': 0, 
        #  ...
        #  'COUNT2REFERENCE': 1,
        #  'FLAGS': 0,
        #  'CONTEXT': {'COUNT2REFERENCE': 0 ...
                         
                
        return dragon2image2xi
                                                           
    #</methodize4xi>   
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "wait2image-of-DRAGON2Image4Xi" usage = "
    #   print('=' *33)          
    #   print('(Step 126A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO WAIT 2D IMAGE OPERATION FOR DRAGON ', time4xi()) 
    #   print('=' *33)      
    #   wait2dragon = {}
    #   try:
    #       response4wait2Dimage = xi2dragon.wait2image()
    #       print('(Step 122B/700) OF DRAGON2IMAGE4XI5TEST - WAITED 2D IMAGE OPERATION FOR DRAGON ', time4xi()) 
    #       keys2response = response4wait2Dimage.keys()
    #       for key2response in sorted(keys2response):
    #           print(key2response, ' : ', response4wait2Dimage[key2response])
    #           # 
    #   except:
    #       print('(Step 126Z1/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO 2D IMAGE OPERATION FOR DRAGON ', time4xi())    
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def wait2image(self, task4wait2image = None):
        '''
        waits DRAGON2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "DRAGON2Image4Xi"
        method2print = "wait2image"
                     
        message2print = "{(Step 1A/33) started" + "; wait2image-of-DRAGON2Image4Xi-started: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)   
        # 00:14:05 09/07/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4wait2image-passed-to-wait2image-of-DRAGON2Image4Xi: " + str(task4wait2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4wait-passing-to-wait2image4dragon-by-wait2image-of-DRAGON2Image4Xi: " + str(task4wait2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None
  
        dragon2image2xi = wait2image4dragon(task4wait2image)

        message2print = "{(Step 2B/33) performed" + "; wait2image4dragon-performed-by-wait2image-of-DRAGON2Image4Xi: " + str(wait2image4dragon) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # semaphore and lock analysis by multiprocessing4xi
        # <function wait2image4dragon at 0x00000000039DFA48> 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; dragon2image2xi-returning-by-wait2image-of-DRAGON2Image4Xi: " + str(dragon2image2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'SIZE2QUEUE': 2147483646,
        #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'},
        #  'QUEUE': <queue4xi.Queue4Xi object at 0x000000000434DDA0>,
        #  'WIDTH': 800,
        #  'CONTEXT': ...
                
        return dragon2image2xi
                                                           
    #</methodize4xi> 
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "write2image-of-DRAGON2Image4Xi" usage = "
    #   print('(Step 127A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL AND COMMAND WITH WRITE IMAGE OBJECT', time4xi()) 
    #   response4write2dragon = {}
    #   try:
    #       response4write2dragon = xi2dragon.write2image()
    #       print('(Step 127B/700) OF DRAGON2IMAGE4XI5TEST - ESTABLISHED CONTROL AND COMMAND WITH WRITE IMAGE OBJECT, RESPONSE4WRITE2DRAGON: ', response4write2dragon) 
    #   except:
    #       print('(Step 127Z1/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL AND COMMAND WITH WRITE IMAGE OBJECT', time4xi()) 
    #
    #   ... continue as with write2image4dragon function
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def write2image(self, task4write2image = None):
        '''
        writes DRAGON2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "DRAGON2Image4Xi"
        method2print = "write2image"
                     
        message2print = "{(Step 1A/33) started" + "; write2image-of-DRAGON2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 02:45:31 10/05/11 Pacific Daylight Time
            
        message2print = "{(Step 1B/33) passed" + "; task4write2image-passed-to-write2image-of-DRAGON2Image4Xi: " + str(task4write2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4write2image-passing-to-write2image4dragon-by-write2image-of-DRAGON2Image4Xi: " + str(task4write2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None
  
        response4writeimage2dragon = write2image4dragon(task4write2image)

        message2print = "{(Step 2B/33) performed" + "; write2image4dragon-performed-by-write2image-of-DRAGON2Image4Xi: " + str(write2image4dragon) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function write2image4dragon at 0x000000000380AEC8>
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4writeimage2dragon-returning-by-write2image-of-DRAGON2Image4Xi: " + str(response4writeimage2dragon) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004D10E10>,
        #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 968>,
        #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 964>}; time: 02:45:31 10/05/11 Pacific Daylight Time}

                     
                
        return response4writeimage2dragon
                                                           
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "copy2image-of-DRAGON2Image4Xi" usage = "
    #   print('(Step 127A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL AND COMMAND WITH WRITE IMAGE OBJECT', time4xi()) 
    #   response4copy2dragon = {}
    #   try:
    #       response4copy2dragon = xi2dragon.copy2image()
    #       print('(Step 127B/700) OF DRAGON2IMAGE4XI5TEST - ESTABLISHED CONTROL AND COMMAND WITH WRITE IMAGE OBJECT, RESPONSE4WRITE2DRAGON: ', response4copy2dragon) 
    #   except:
    #       print('(Step 127Z1/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL AND COMMAND WITH WRITE IMAGE OBJECT', time4xi()) 
    #
    #   ... continue as with copy2image4dragon function
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def copy2image(self, task4copy2image = None):
        '''
        copies DRAGON2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "DRAGON2Image4Xi"
        method2print = "copy2image"
                     
        message2print = "{(Step 1A/33) started" + "; copy2image-of-DRAGON2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:28:44 10/05/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4copy2image-passed-to-copy2image-of-DRAGON2Image4Xi: " + str(task4copy2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4copy2image-passing-to-copy2image4dragon-by-copy2image-of-DRAGON2Image4Xi: " + str(task4copy2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None 
  
        response4copyimage2dragon = copy2image4dragon(task4copy2image)

        message2print = "{(Step 2B/33) performed" + "; copy2image4dragon-performed-by-copy2image-of-DRAGON2Image4Xi: " + str(copy2image4dragon) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function copy2image4dragon at 0x000000000390B8C8>
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4copyimage2dragon-returning-by-copy2image-of-DRAGON2Image4Xi: " + str(response4copyimage2dragon) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004EC4128>,
        #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 1000>,
        #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 996>}; time: 01:28:44 10/05/11 Pacific Daylight Time}
        
        return response4copyimage2dragon
                                                           
    #</methodize4xi>
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "copy2imagetobuffer-of-DRAGON2Image4Xi" usage = "
    #   print('(Step 127A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL AND COMMAND WITH COPY IMAGE TO BUFFER OBJECT', time4xi()) 
    #   response4copyimagetobuffer = {}
    #   try:
    #       response4copyimagetobuffer = xi2dragon.copy2imagetobuffer()
    #       print('(Step 127B/700) OF DRAGON2IMAGE4XI5TEST - ESTABLISHED CONTROL AND COMMAND WITH COPY IMAGE TO BUFFER  OBJECT, RESPONSE4WRITE2DRAGON: ', response4copyimagetobuffer) 
    #   except:
    #       print('(Step 127Z1/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL AND COMMAND WITH COPY IMAGE TO BUFFER ', time4xi()) 
    #
    #   ... continue as with copy2imagetobuffer4dragon function
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def copy2imagetobuffer(self, task4copy2imagetobuffer = None):
        '''
        copies 2D images to buffer
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "DRAGON2Image4Xi"
        method2print = "copy2imagetobuffer"
                     
        message2print = "{(Step 1A/33) started" + "; copy2imagetobuffer-of-DRAGON2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:28:44 10/05/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4copy2imagetobuffer-passed-to-copy2imagetobuffer-of-DRAGON2Image4Xi: " + str(task4copy2imagetobuffer) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4copy2imagetobuffer-passing-to-copy2imagetobuffer4dragon-by-copy2imagetobuffer-of-DRAGON2Image4Xi: " + str(task4copy2imagetobuffer) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None
  
        response4copyimage2dragon = copy2imagetobuffer4dragon(task4copy2imagetobuffer)

        message2print = "{(Step 2B/33) performed" + "; copy2imagetobuffer4dragon-performed-by-copy2imagetobuffer-of-DRAGON2Image4Xi: " + str(copy2imagetobuffer4dragon) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function copy2imagetobuffer4dragon at 0x000000000390B948>
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4copyimage2dragon-returning-by-copy2imagetobuffer-of-DRAGON2Image4Xi: " + str(response4copyimage2dragon) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004FE5FD0>,
        #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 1036>,
        #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 1032>}; time: 01:28:44 10/05/11 Pacific Daylight Time}
          
        return response4copyimage2dragon
                                                           
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "copy2buffertoimage-of-DRAGON2Image4Xi" usage = "
    #   print('(Step 127A/700) OF DRAGON2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL AND COMMAND WITH COPY BUFFER TO IMAGE OBJECT', time4xi()) 
    #   response4copybuffertoimage = {}
    #   try:
    #       response4copybuffertoimage = xi2dragon.copy2buffertoimage()
    #       print('(Step 127B/700) OF DRAGON2IMAGE4XI5TEST - ESTABLISHED CONTROL AND COMMAND WITH COPY BUFFER TO IMAGE  OBJECT, RESPONSE4WRITE2DRAGON: ', response4copybuffertoimage) 
    #   except:
    #       print('(Step 127Z1/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL AND COMMAND WITH COPY BUFFER TO IMAGE ', time4xi()) 
    #
    #   ... continue as with copy2buffertoimage4dragon function
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def copy2buffertoimage(self, task4copy2buffertoimage = None):
        '''
        copies 2D images to buffer
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "DRAGON2Image4Xi"
        method2print = "copy2buffertoimage"
                     
        message2print = "{(Step 1A/33) started" + "; copy2buffertoimage-of-DRAGON2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:28:44 10/05/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4copy2buffertoimage-passed-to-copy2buffertoimage-of-DRAGON2Image4Xi: " + str(task4copy2buffertoimage) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4copy2buffertoimage-passing-to-copy2buffertoimage4dragon-by-copy2buffertoimage-of-DRAGON2Image4Xi: " + str(task4copy2buffertoimage) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None
  
        response4copyimage2dragon = copy2buffertoimage4dragon(task4copy2buffertoimage)

        message2print = "{(Step 2B/33) performed" + "; copy2buffertoimage4dragon-performed-by-copy2buffertoimage-of-DRAGON2Image4Xi: " + str(copy2buffertoimage4dragon) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function copy2buffertoimage4dragon at 0x000000000390B9C8> 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4copyimage2dragon-returning-by-copy2buffertoimage-of-DRAGON2Image4Xi: " + str(response4copyimage2dragon) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'QUEUE': <queue4xi.Queue4Xi object at 0x000000000505A6D8>,
        #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 1068>,
        #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 1064>}; time: 01:28:44 10/05/11 Pacific Daylight Time}

                
        return response4copyimage2dragon
                                                           
    #</methodize4xi>    
                    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "trace2image-of-DRAGON2Image4Xi" usage = "
    #   print('=' *33)          
    #   print('(Step 15A/25) OF DRAGON2IMAGE4XI5TEST - TRYING TO TRACE DRAGON: ', time4xi())      
    #   print('=' *33)      
    #   trace2dragon = None
    #   try:
    #       trace2dragon = trace2image4dragon() 
    #       print('(Step 15B/25) OF DRAGON2IMAGE4XI5TEST - TRACE2DRAGON: ', trace2dragon)  
    #       # {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4dragon-failed {}; time: 01:39:15 10/06/11 Pacific Daylight Time}', 'CLASS': 'trace2image4dragon', 'METHOD': 'trace2image4dragon'}
    #    except:
    #       print('(Step 15Z1/25) OF DRAGON2IMAGE4XI5TEST *** FAILED TO TRACE DRAGON: ', trace2dragon)     
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def trace2image(self, task4trace2image = None):
        '''
        traces DRAGON2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "DRAGON2Image4Xi"
        method2print = "trace2image"
                     
        message2print = "{(Step 1A/33) started" + "; trace2image-of-DRAGON2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 00:14:05 09/07/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4trace2image-passed-to-trace-of-DRAGON2Image4Xi: " + str(task4trace2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None   
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4trace2image-passing-to-trace2image4dragon-by-trace2image-of-DRAGON2Image4Xi: " + str(task4trace2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None  
  
        trace2xi = trace2image4dragon(task4trace2image)

        message2print = "{(Step 2B/33) performed" + "; trace2image4dragon-performed-by-trace2image-of-DRAGON2Image4Xi: " + str(trace2image4dragon) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function trace2image4dragon at 0x00000000039DF348>
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; trace2xi-returning-by-trace2image-of-DRAGON2Image4Xi: " + str(trace2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4dragon-failed {}; time: 01:05:20 09/08/11 Pacific Daylight Time}',
        #  'CLASS': 'trace2image4dragon',
        #  'METHOD': 'trace2image4dragon'}; time: 01:05:20 09/08/11 Pacific Daylight Time}
                 
                
        return trace2xi
                                                           
    #</methodize4xi>       


    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "set2imagecallback-of-DRAGON2Image4Xi" usage = "
    #   print('(Step 1A/33) OF DRAGON2IMAGE4XI - TRYING TO SET DRAGON IMAGE')
    #   try:
    #       cc2xi = DRAGON2Image4Xi()                            
    #       image2dragon = cc2xi.set2imagecallback()                                  
    #       print('(Step 1B/33) OF DRAGON4XI5TEST - SET DRAGON IMAGE: ', image2dragon)                                                  
    #   except:
    #       print('(Step 1Z/33) OF DRAGON4XI5TEST *** FAILED TO SET DRAGON IMAGE ***')  
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def set2imagecallback(self, task4set2imagecallback = None): 
        '''
        set2imagecallback of DRAGON2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "DRAGON2Image4Xi"
        method2print = "set2imagecallback"
                     
        message2print = "{(Step 1A/33) started" + "; set2imagecallback-of-DRAGON2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 00:14:05 09/07/11 Pacific Daylight Time
            
        message2print = "{(Step 1B/33) passed" + "; task4set2imagecallback-passed-to-set2imagecallback-of-DRAGON2Image4Xi: " + str(task4set2imagecallback) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None    
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4set2imagecallback-passing-to-set2imagecallback -by-set2imagecallback-of-DRAGON2Image4Xi: " + str(task4set2imagecallback) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None
        
        callback2dragon4image = {} 
        try:
            callback2dragon4image = set2imagecallback4dragon (task4set2imagecallback)

            message2print = "{(Step 2B/33) callback2dragon4image" + "; callback2dragon4image-set-by-set2imagecallback-of-DRAGON2Image4Xi to: " + str(callback2dragon4image) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            # <function trace2image4dragon at 0x00000000039DF348>
        except:
            message2print = "{(Step 4A/33) failed" + "; callback2dragon4image-failed-by-set2imagecallback-of-DRAGON2Image4Xi: " + str(callback2dragon4image) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
                                            
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; callback2dragon4image-returning-by-set2imagecallback-of-DRAGON2Image4Xi: " + str(callback2dragon4image) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # <function trace2image4dragon at 0x0000000003AB31C8> 
                                             
        return callback2dragon4image
                                                           
    #</methodize4xi>     
    
                                                              
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "sense-of-DRAGON2Image4Xi" usage = "
    #   try:
    #       print(xi2dragon.sense())
    #       print('(Step 24B/33) OF DRAGON2IMAGE4XI5TEST - SENSED ', time4xi()) 
    #   except:
    #       print('(Step 24Z/33) OF DRAGON2IMAGE4XI5TEST *** FAILED TO SENSE ', time4xi())  
    # ">
    
    # ---------------------------------------------------------------------------------------------
    def sense(self):
        '''
        method sense: senses DRAGON2Image4Xi class objects
        '''
        
        # -----------------------------------------------------------------------------------------
        #
        # ----------------------------------------------------------------------------------------- 
        class2print = 'DRAGON2Image4Xi'
        method2print = 'sense'
        
                     
        message2print = "{(Step 1A/33) started" + "; sense-of-DRAGON2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:02:22 09/07/11 Pacific Daylight Time 
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------  
        
        message2print = "{(Step 2A/33) sensing" + "; DRAGON2Image4Xi-sensing-by-sense-of-DRAGON2Image4Xi: " + str(DRAGON2Image4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        # <class 'dragon2image4xi.DRAGON2Image4Xi'>         
        
        # -----------------------------------------------------------------------------------------
        # get sensed knowledge for this object
        # -----------------------------------------------------------------------------------------        
        
        kb2xi = sense_object4xi(self)
        
        message2print = "{(Step 3A/33) got" + "; kb2xi-got-by-sense-of-DRAGON2Image4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'object': <dragon2image4xi.DRAGON2Image4Xi object at 0x0000000004188400>}   
        
        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb2xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------         
        try:
            kb2xi['classvariable2dragon-of-DRAGON2Image4Xi'] = DRAGON2Image4Xi.classvariable2dragon
        except:
            message2print = "{(Step 4A/33) yielded" + "; kb2xi-yielded-by-sense-of-DRAGON2Image4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            
      
        message2print = "{(Step 5A/33) sensed" + "; DRAGON2Image4Xi-sensed-by-sense-of-DRAGON2Image4Xi: " + str(DRAGON2Image4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        # <class 'dragon2image4xi.DRAGON2Image4Xi'>
        
        # -----------------------------------------------------------------------------------------
        # 
        # -----------------------------------------------------------------------------------------         
        
        message2print = "{(Step 6A/33) returning" + "; kb2xi-returning-by-sense-of-DRAGON2Image4Xi: " + str(kb2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # {'object': <dragon2image4xi.DRAGON2Image4Xi object at 0x0000000004188400>, 'classvariable2dragon-of-DRAGON2Image4Xi': 4}             
          
        return kb2xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
    #</methodize4xi>
    
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "reason-of-DRAGON2Image4Xi" usage = "
    #   print('(Step 25A/33) OF DRAGON2IMAGE4XI5TEST - REASONING ', time4xi())
    #   try:
    #       print(xi2dragon.reason())
    #       print('(Step 25B/33) OF DRAGON2IMAGE4XI5TEST - REASONED ', time4xi())
    #   except:
    #       print('(Step 25Z/33) OF DRAGON2IMAGE4XI5TEST *** FAILED TO REASON ', time4xi())   
    #">
    
    # ---------------------------------------------------------------------------------------------
    def reason(self):
        '''
        method reason: reasons DRAGON2Image4Xi class objects
        '''
        
        # -----------------------------------------------------------------------------------------
        #
        # ----------------------------------------------------------------------------------------- 
        class2print = 'DRAGON2Image4Xi'
        method2print = 'reason'

                     
        message2print = "{(Step 1A/33) started" + "; reason-of-DRAGON2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:02:22 09/07/11 Pacific Daylight Time 
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        
        message2print = "{(Step 2A/33) reasoning" + "; DRAGON2Image4Xi-reasoning-by-reason-of-DRAGON2Image4Xi: " + str(DRAGON2Image4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # <class 'dragon2image4xi.DRAGON2Image4Xi'>  
        
        # -----------------------------------------------------------------------------------------
        # get reasoned knowledge for this object
        # -----------------------------------------------------------------------------------------         
        
        kb4xi = reason_object4xi(self)
        
        
        message2print = "{(Step 3A/33) got" + "; kb4xi-got-by-reason-of-DRAGON2Image4Xi: " + str(kb4xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'object-reason-object4xi': <dragon2image4xi.DRAGON2Image4Xi object at 0x0000000004188400>}      
        
        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb4xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------          
        try:
            kb4xi['time-reason-of-DRAGON2Image4Xi'] = time4xi() #adding time
        except:
            message2print = "{(Step 4Z/33) yielded" + "; kb4xi-yielded-by-reason-of-DRAGON2Image4Xi: " + str(kb4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
        
          
        
        message2print = "{(Step 5A/33) reasoned" + "; DRAGON2Image4Xi-reasoned-by-reason-of-DRAGON2Image4Xi: " + str(DRAGON2Image4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        # <class 'dragon2image4xi.DRAGON2Image4Xi'>
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        
        message2print = "{(Step 6A/33) returning" + "; kb4xi-returning-by-reason-of-DRAGON2Image4Xi: " + str(kb4xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # {'object-reason-object4xi': <dragon2image4xi.DRAGON2Image4Xi object at 0x0000000004188400>, 'time-reason-of-DRAGON2Image4Xi': '01:02:22 09/07/11 Pacific Daylight Time'}                
          
        return kb4xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------            
    #</methodize4xi>
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "ace-of-DRAGON2Image4Xi" usage = "
    #   print('(Step 26A/33) OF DRAGON2IMAGE4XI5TEST - ACING ', time4xi())   
    #   try:
    #       print(xi2dragon.ace())
    #       print('(Step 26B/33) OF DRAGON2IMAGE4XI5TEST - ACED ', time4xi())                                                 
    #   except:
    #       print('(Step 26Z/33) OF DRAGON2IMAGE4XI5TEST *** FAILED TO ACE ', time4xi())                                       
    #    
    #">
    
    # ---------------------------------------------------------------------------------------------
    def ace(self):
        '''
        method ace: aces DRAGON2Image4Xi class objects 
        '''
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------       
        
        
        class2print = 'DRAGON2Image4Xi'
        method2print = 'ace'
        
                     
        message2print = "{(Step 1A/33) started" + "; ace-of-DRAGON2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:02:22 09/07/11 Pacific Daylight Time   
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        
        message2print = "{(Step 2A/33) acing" + "; DRAGON2Image4Xi-acing-by-ace-of-DRAGON2Image4Xi: " + str(DRAGON2Image4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        # <class 'dragon2image4xi.DRAGON2Image4Xi'>           
        
        # -----------------------------------------------------------------------------------------
        # get aced knowledge for this object
        # -----------------------------------------------------------------------------------------        
        
        kb8xi = ace_object4xi(self)  
        
        message2print = "{(Step 3A/33) got" + "; kb8xi-got-by-ace-of-DRAGON2Image4Xi: " + str(kb8xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'object8xi': <dragon2image4xi.DRAGON2Image4Xi object at 0x0000000004188400>}   


        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb8xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------              
        try:
             kb8xi['cores-of-DRAGON2Image4Xi'] = DRAGON2Image4Xi.classvariable2dragon
        except:
             message2print = "{(Step 4A/33) yielded" + "; kb8xi-yielded-by-ace-of-DRAGON2Image4Xi: " + str(kb8xi)  + "; time: " + time4xi() + "}"
             print4xi(class2print, method2print, message2print) 
  
        message2print = "{(Step 5A/33) aced" + "; DRAGON2Image4Xi-aced-by-ace-of-DRAGON2Image4Xi: " + str(DRAGON2Image4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        # <class 'dragon2image4xi.DRAGON2Image4Xi'>  
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        message2print = "{(Step 6A/33) returning" + "; kb8xi-returning-by-ace-of-DRAGON2Image4Xi: " + str(kb8xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # {'cores-of-DRAGON2Image4Xi': 4, 'object8xi': <dragon2image4xi.DRAGON2Image4Xi object at 0x0000000004188400>}
        
        return kb8xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------            
    #</methodize4xi>
#</class4xi>
#</program4xi>

# -------------------------------------------------------------------------------------------------
# Recorded History                             
# ------------------------------------------------------------------------------------------------- 
#<record4xi record = "
#   dragon2image4xi5test.txt
#   sense-dragon2image4xi5test.xml
#   reason-dragon2image4xi5test.xml
#   ace-dragon2image4xi5test.xml
#"
#>

#<acknowledge4xi acknowledgement = "33" tester = "Small & Smart"/>

# -------------------------------------------------------------------------------------------------
# to-be-done                              
# -------------------------------------------------------------------------------------------------    
#<do4xi do = "
# nothing
#"
#/>

#<note4xi note = "
# parse4xi.py - to parse this module
# "/>

# -------------------------------------------------------------------------------------------------
# Legal Notice                             
# ------------------------------------------------------------------------------------------------- 
#<protect4xi protection = "copyright"
#   owner = "Small & Smart"
#   rights = "All rights reserved"
#   state = "California"
#   country = "United States of America"
#   codes = "Codes established in California and applicable to corporations in California"
#   compliance = "The California Law"
#   court = "Supreme Court of California, 350 McAllister Street, San Francisco, CA 94102-4783"
#/>

#<end4xi end = "
# -------------------------------------------------------------------------------------------------
# end of dragon2image4xi.py                              
# ------------------------------------------------------------------------------------------------- 
#"
#/>
